/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.bert;



/**
 * A GUI to configure agents in the Push Agent Manager(PAM).
 * Will work standalone or built as part of PAM.
 *
 * @version 1, 10 December 1997
 * @version 2, 1 Septemeber 1998
 * @author Sam Gerstein
 * @author Nazario Irizarry, Jr.
 */
import java.awt.Component;
import java.awt.event.AdjustmentListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.WindowEvent;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.MenuBar;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.MenuShortcut;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowListener;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Scrollbar;
import java.awt.PopupMenu;
import java.awt.Graphics;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Enumeration;
import java.io.*;
import java.net.*;
import org.mitre.pam.interfaces.Juggler;
import org.mitre.pam.interfaces.Memento;
import org.mitre.pam.interfaces.ScheduleAttributes;
import org.mitre.pam.interfaces.HasProperties;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.ExecToController;
import org.mitre.pam.interfaces.AdminTool;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pdif.eventParser.PDIFWriter;
import org.mitre.pdif.eventParser.ParseEvent3;
import org.mitre.pdif.eventParser.BadPDIFException;
import org.mitre.pam.store.SourceSinkInfo;
import org.mitre.pam.util.pdifHashtables.PDIFConn;
import org.mitre.pam.util.pdifHashtables.DuplicateIDException;
import org.mitre.pam.util.draw.DrawPanel;
import org.mitre.pam.util.draw.DrawListener;
import org.mitre.pam.util.draw.DrawParent;
import org.mitre.pam.util.draw.BlockConnector;
import org.mitre.pam.util.draw.DrawEvent;
import org.mitre.pam.util.draw.BlockGroup;
import org.mitre.pam.util.alarm.AlarmReporter;
import org.mitre.pam.util.SortedHash;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.monitor.MultipleTextBoxDlg;
import org.mitre.pam.monitor.MessageDlg;
import org.mitre.pam.monitor.YesNoDlg;
import org.mitre.pam.monitor.ThreeButtonDlg;
import org.mitre.pam.util.pdifHashtables.ParentServices;
import org.mitre.pam.util.pdifHashtables.NodeEditor;


public final class Bert extends DrawPanel implements ActionListener, ParentServices, WindowListener, AdminTool, LoadCallbacks {
	
	
	public static final String POST_FIELD_NAME = "changeText";

	public static final String ICON_MAP_FILE_NAME = "iconmap";

	private static final int VERSION = 2;
	
	private static final String ADMIN_NAME = "Administrator";
	
	private static final MenuItem fileMenuOPEN_PAM_SERV = new MenuItem("Open PAM Server...");
	private static final MenuItem fileMenuSAVE_PAM_SERV = new MenuItem("Save To Server...");
	private static final MenuItem fileMenuOPEN_PAM_FILE = new MenuItem("Open PAM State File...");
	private static final MenuItem fileMenuSAVE_PAM_FILE = new MenuItem("Save To File...");
	private static final MenuItem fileMenuAUTOLAYOUT = new MenuItem("Autolayout");
	private static final MenuItem fileMenuQUIT = new MenuItem("Quit");
	
	private Vector ignoreAttributes=new Vector();
	private static boolean debug;
	private PopupMenu fMenu;
	private PopupMenu edgePopup;
	private PopupMenu nodePopup;
	private AlarmReporter reporter = new AlarmReporter("bert");

	DrawParent drawParent;
	BertPDIFRead PAMstate;
	private int nStandardDoPopUpItems; // number of standard items in the create popup menu

	
//	Hashtable imagesByType;

	
	private int initNum = 1;
	private boolean layoutChanged = false;
	
	private Vector displayColumns;
	private int nodeWidth = 100;
	private int nodeHeight = 100;
	private int maxXpos;
	private int maxYpos;
	private int maxDepth;
	private HandleEdgePopup handleEdgePopup;
	private HandleNodePopup handleNodePopup;
	private PopupMenu doPopUp;

	private PerformOnNew listenForNewHandler = new PerformOnNew();
	private PerformOnExisting listenForExistingHandler = new PerformOnExisting();
	
	private boolean needExistingRebuild;
	private boolean needAvailableRebuild;
	
	private Thread loadPostThread = null;
	private boolean ignoreRightMouse;
	private BlockGroup defaultBlockGroup = new BlockGroup();
	private ConnectionInfo connectionInfo = new ConnectionInfo();
		
	public Bert(String title) {
		this();
		setTitle(title);
	}
	
	public Bert() {
		setReporter(reporter);
//		listThreads(" -- at start");
		connectionInfo.exec = ExecGlobal.getExec();
		connectionInfo.localMode = connectionInfo.exec != null;
		if (connectionInfo.localMode) {
		    try {
		    	URL url = new URL(ExecGlobal.getProtocolHostAndPort());
		    	connectionInfo.host = url.getHost();
		    	connectionInfo.port = url.getPort();
		    	connectionInfo.localHost = connectionInfo.host;
		    	connectionInfo.localPort = connectionInfo.port;
		    }
		    catch(MalformedURLException ignore){}
		    connectionInfo.localHost = connectionInfo.host;
		} else {
			try {
				connectionInfo.localHost = InetAddress.getLocalHost().getHostName();
				connectionInfo.localPort = 19090;
			}
			catch (UnknownHostException e) {}
	    	connectionInfo.host = connectionInfo.localHost;
	    	connectionInfo.port = 19090;
		}

		

		connectionInfo.bertVersion = VERSION;

		BertGraphicBlock.setParentServices(this);
		
		ignoreAttributes.addElement("runState");
		ignoreAttributes.addElement(Memento.MEMENTO_UID);
		ignoreAttributes.addElement("commonName");
		ignoreAttributes.addElement("javaClass");
		ignoreAttributes.addElement(ScheduleAttributes.LAST_UPDATE_TIME);
		ignoreAttributes.addElement(ScheduleAttributes.LAST_UPDATE_LONG);
		
		drawParent = getDrawParent();
		drawParent.addDrawListener(new EditorListener());

		buildMenus();
		buildPopups();
		addWindowListener(this);
	}	

/**
* Interface AdminTool -- not used here
*/
	public void setExec(ExecToController exec) {
	}
	
	public void setTitle(String title) {
		super.setTitle("Bert, "+title);
	}
/**
 * handles the menus.
 */
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
			
		if (command == fileMenuOPEN_PAM_FILE.getLabel()) { // open state file
			readStateFile();
		} else if (command == fileMenuOPEN_PAM_SERV.getLabel()) { // open PAM server
			readStateServer();
		} else if (command == fileMenuSAVE_PAM_FILE.getLabel()) { // save state file
			postStateFile();
		} else if (command == fileMenuSAVE_PAM_SERV.getLabel()) { // send to PAM server
			postState();
		} else if (command == fileMenuQUIT.getLabel()) { // exit the GUI
			windowClosing(null);
		} else if (command == fileMenuAUTOLAYOUT.getLabel()) { 
			autoLayout(true);
		}
	}	
/** 
 * builds the menus
 */
	private void buildMenus() {
		fMenu = new PopupMenu("Do...");
		fMenu.addActionListener(this);
		setCommandPopup(fMenu);
		
		fMenu.add(fileMenuOPEN_PAM_SERV);
		fMenu.add(fileMenuOPEN_PAM_FILE);
		fMenu.add(fileMenuSAVE_PAM_SERV);
		fMenu.add(fileMenuSAVE_PAM_FILE); 
		fMenu.add(fileMenuAUTOLAYOUT);
		fMenu.add(fileMenuQUIT);		
	}	

	public void cleanSlate() {
		drawParent.removeAll();
	}	
	public BlockConnector connect (BertGraphicBlock fromNode, BertGraphicBlock toNode){
			if (!connectionInfo.isAdmin && !toNode.isOwnedBy(connectionInfo.userID) ) {
				if (!toNode.isOwned()) {
					// the node to be connected _to_ is not owned- offer ownership.
					if (confirmMsg( "This node is unowned. Do you wish to take ownership and complete the connection?"))
						toNode.getData().setOwner(connectionInfo.userID);
					else
						return null;
				} else {
					showMessageDlg( "You cannot modify the inputs to this node. (you do not own it)");
					return null;
				}	
			} 
			// the two ends are both nodes, so the underlying data representation must be synched.
			return connectNodes(fromNode, toNode);
	}	
/**
 * this function does not check the ownership of a node before making a connection.
 * call connect() to get that functionality.
 */

	private Edge connectNodes (BertGraphicBlock from, BertGraphicBlock to)
	{
		// find an existing connection or make a connection between the nodes.
		BertPDIFNode Pfrom, Pto;
		if ((Pfrom = from.getData()) == null)
			throw new IllegalArgumentException("connect: from node without PDIFNode");
				
		if ((Pto = to.getData()) == null)
			throw new IllegalArgumentException("connect: to node without PDIFNode");


		PDIFConn newConn = PAMstate.makeNewConn(2);
		newConn.roleList[0] = BertPDIFRead.FROM;
		newConn.roleList[1] = BertPDIFRead.TO;
		newConn.nodeList[0] = from.getData();
		newConn.nodeList[1] = to.getData();
		PAMstate.addConnection(newConn);
		
//		Pfrom.touch();
//		Pto.touch();
		Edge e = new Edge(from, to, newConn);
		drawParent.add(e);
		return e;
	}
	
/**
 * searches for an image for a node which has been instantiated
 * on the PDIFNode side.  It looks back up the parent tree for
 * any image which could represent this node, if this particular
 * one doesn't have its own.
 */
/*	Image findImageForType(String typeName) {
		PDIFNode n = PAMstate.get(typeName);
		Image img;

		while (n != null) {
			img = (Image)imagesByType.get(typeName);
			if (img != null)
				return img;
			else {
				n = PAMstate.get(typeName);
				if (n != null)
					typeName = n.type; 
			}	
		}	
		return null;
	}
*/
/**
 * finds the best location for a node.
 */
	private java.awt.Point findOptimalLocation(BertGraphicBlock n) {
		// so far, it's not too smart :)
		return new java.awt.Point(0, 0);
	}	
/**
 * removes a node's visual representation, but does not delete
 * the data representation (so it will still show up in the
 * instance list
 */
 /*
	void hideNode(BertGraphicBlock n) {
		drawParent.remove(n);
	}	
*/

	public static void main(String args[]){
		debug = false;
		if (args != null && args.length > 0 && args[0] != null) {
			String com1 = args[0];
			if (com1.equalsIgnoreCase("-debug") ) {
				debug = true;
			}
			if (com1.startsWith("-h") || com1.equals("-?") || com1.equals("-version")) {
				System.out.println("Bert Build 2");
				return;
			}
		}
		if (DRAW_PANEL_VERSION < 1){
			System.out.println("You need a newer version of pamCore.zip in order to run this gui");
			return;
		}
		Bert theApp = new Bert();
		theApp.setVisible(true);
	}	
/**
 * creates a new BertPDIFNode with the given name and type, then calls
 * the other newNode to create the graphical node.
 */
 
	public BertGraphicBlock createNode(String name, String type, int x, int y) {
		BertPDIFNode pnode = (BertPDIFNode) PAMstate.get(type);
		if (pnode == null) {
			System.err.println("Internal error: type \"" + type + "\" is not declared.");
			return null;
		}
		if (pnode.getType() != SourceSinkInfo.PIM_STR && 
				pnode.getType() != SourceSinkInfo.DIM_STR && 
				pnode.getType() != SourceSinkInfo.JUGGLER_STR) {
			System.err.println("Internal error: node \"" + pnode.getName() + "\" is not a type (its type is \"" + pnode.getType() + "\").");
			return null;
		}	

		layoutChanged = true;
		needExistingRebuild = true;

		/*
		* copy the type the new node is an instance of
		*/
		BertPDIFNode pnodeClone = (BertPDIFNode) PAMstate.makeNewNode(type, name); 
		pnodeClone.copyAttributesNotConnections(pnode); 
		pnodeClone.setOwner(connectionInfo.userID);
		pnodeClone.setProperty(BertPDIFNode.EXPOSURE_VERB, BertPDIFNode.EXPOSURE_PRIVATE); // set the exposure to private.
		pnodeClone.getGraphicNode().setPos(x,y);
		pnodeClone.touch();
		/*
		* get rid of a few unnecessary (unwanted) properties
		*/
		Enumeration ignored = ignoreAttributes.elements();
		while (ignored.hasMoreElements())
			pnodeClone.removeProperty((String)ignored.nextElement());

		try {
			PAMstate.addNode(pnodeClone);
		} catch (DuplicateIDException e) {
			reporter.warning("node name \"" + name + "\" is already taken.");
			reporter.reportException(e);
			return null;
		}
		showGraphicNode(pnodeClone);
		return pnodeClone.getGraphicNode();
	}
/**
 * call this to create a graphical node object for a PDIFNode which
 * already exists in the state. (this includes those just added...)
 */

	public void showGraphicNode(BertPDIFNode pdifNode) {
		BertGraphicBlock node = pdifNode.getGraphicNode();
		node.updatePresentation();
		if (node.isOnScreen()) {
			return;
		}

		node.setPosFromPDIF();
		drawParent.add(node);
		node.setOnScreen(true);
		
		/* 
		* now look through the list of connections and existing nodes to 
		* see if any lines are necessary to Nodes already on the screen
		*/
		Enumeration enum = pdifNode.connections();
		
		while (enum.hasMoreElements()) {
			PDIFConn tmpConn = (PDIFConn) enum.nextElement();

			if (tmpConn.nodeList[0] != pdifNode) {
				BertPDIFNode otherNode = (BertPDIFNode)tmpConn.nodeList[0];
				if( otherNode.getGraphicNode().isOnScreen()) {
					if (tmpConn.roleList[0] == BertPDIFRead.TO) {
						drawParent.add(new Edge(node, otherNode.getGraphicNode(), tmpConn));
					}
					else if (tmpConn.roleList[0] == BertPDIFRead.FROM) {
						drawParent.add(new Edge(otherNode.getGraphicNode(), node, tmpConn));
					}
				}
			} 
			else if (tmpConn.nodeList[1] != pdifNode) {
				BertPDIFNode otherNode = (BertPDIFNode)tmpConn.nodeList[1];
				if( otherNode.getGraphicNode().isOnScreen()) {
					if (tmpConn.roleList[1] == BertPDIFRead.TO) {
						drawParent.add(new Edge(node, otherNode.getGraphicNode(), tmpConn));
					}
					else if (tmpConn.roleList[1] == BertPDIFRead.FROM) {
						drawParent.add(new Edge(otherNode.getGraphicNode(), node, tmpConn));
					}
				}
			} 
		}	
		return;
	}	
/**
 * offers to save the layout,
 * returns false if they Cancel the operation,
 * true otherwise (save or not)
 */
	boolean optionalSave() {
		return true;
	}	
/**
* Save state to the same server we loaded from.  If less
* than ten minutes have elapsed since we last loaded the configuration then
* save without prompting the user.  Otherwise, prompt the user for the save info.
*/
	void save() {
		if (connectionInfo.loadedByURL) {
			if (connectionInfo.lastLoadSaveTime != 0 && System.currentTimeMillis() < connectionInfo.lastLoadSaveTime + 10*60*1000) {
				boolean postAll = false;
				if (!PAMstate.hasChangedElements()) {
					if (confirmMsg( "There are no changes to the state.  Post entire state anyway?"))
						postAll = true;
					else
						return;
				}
				postStateToServer(postAll);
			}
			else {
				postState();
			}
		}
		else if (connectionInfo.loadedByFile)
			postStateFile();
	}
/**
* Post state either by sending to last server used or by asking the user
*/
	void postState() {
		if (loadPostThread != null)
			return;
		boolean postAll = false;
		if (!PAMstate.hasChangedElements()) {
			if (confirmMsg( "There are no changes to the state.  Post entire state anyway?"))
				postAll = true;
			else
				return;
		}
			
		String defaultHost = connectionInfo.host;
		int defaultPort = connectionInfo.port;
		int userPort = defaultPort;

		String labels[] = {"Password", "Host name:", "Port:"};
		String servInfo[];
		
		do {
			String defs[] = {"", defaultHost, Integer.toString(defaultPort)};
			servInfo = MultipleTextBoxDlg.showMultipleTextBoxDlg(this, labels, defs, "Save configuration "+connectionInfo.userID+" to server");
			if (servInfo == null)
				return;
			
			try {
				userPort = Integer.parseInt(servInfo[2]);
			}
			catch (NumberFormatException e) {
				userPort = -2;
			}
		}
		while (userPort == -2 || servInfo[1].length()==0);

		connectionInfo.userPassword = servInfo[0];
		connectionInfo.host = servInfo[1];
		connectionInfo.port = userPort;
		
		postAll = postAll || !connectionInfo.host.equals(defaultHost) || (connectionInfo.port != defaultPort);
		postStateToServer(postAll);
	}
/**
* Post state to a server
*/
	void postStateToServer(boolean postAll) {
		
		if (loadPostThread != null)
			return;
		loadPostThread = new PostStateToServer(postAll, this, connectionInfo, this, reporter);
		loadPostThread.start();
	}	
/**
* Post state to a file
*/
	void postStateFile() {
		if (loadPostThread != null)
			return;
		loadPostThread = new PostStateToFile(this, connectionInfo, this, reporter);
		loadPostThread.setPriority(Thread.NORM_PRIORITY+1);
		loadPostThread.start();

	}

/**
* Loads state from a file
*/
	void readStateFile() {
		if (loadPostThread != null)
			return;
		loadPostThread = new ReadStateFile(this, connectionInfo, this, reporter);
		loadPostThread.start();
	}	
/**
* Loads state from a PAM server
*/
	void readStateServer() {
		if (loadPostThread != null)
			return;
		loadPostThread = new ReadStateFromServer(this, connectionInfo, this, reporter);
		loadPostThread.setPriority(Thread.NORM_PRIORITY+1);
		loadPostThread.start();
	}
/*
* Reload from a file or a server, based
* on whichever was used last.  If none was used
* then load from a server
*/	
	void reload() {
		if (connectionInfo.loadedByFile) {
			readStateFile();
		}
		else {
			readStateServer();
		}
	}
	
/**
* this function reads the names and locations of
* the nodes from a file.  It then checks the existence of the
* nodes against the state it got from PAM.
* @param fileName - the name of the file to read (without the extention)
*/
	public void readLayout() {
			java.util.Enumeration enum = PAMstate.elements();
			while(enum.hasMoreElements()) {
				BertPDIFNode pNode = (BertPDIFNode) enum.nextElement();
				if ((pNode.isOwnedBy(connectionInfo.userID) || pNode.isPublic()) && pNode.hasPos()) {
					showGraphicNode(pNode);
				}
			}
			drawParent.needsRefresh();
			createExistingList();
	}	
	
	public void autoLayout(boolean markAsMoved) {
		
		Vector PIMList = new Vector();
		Vector DIMList = new Vector();
		Vector jugglerList = new Vector();		
		java.util.Enumeration enum = PAMstate.elements();
		BertPDIFNode n, m;
		Vector listToAddTo;
		
		nodeWidth = 0;
		nodeHeight = 30;
		while (enum.hasMoreElements()) {
			n = (BertPDIFNode)enum.nextElement();
			n.autoPlaced = -1;
			// check that it's an instance of a pimType, dimType, or jugglerType
			if (connectionInfo.isAdmin || n.isPublic() || n.isOwnedBy(connectionInfo.userID)) {
				m = (BertPDIFNode)PAMstate.get(n.getType());
				if (m == null)
					continue;
				if (m.getType() == SourceSinkInfo.PIM_STR)
					listToAddTo = PIMList;
				else if(m.getType() == SourceSinkInfo.DIM_STR)
					listToAddTo = DIMList;
				else if(m.getType() == SourceSinkInfo.JUGGLER_STR)
					listToAddTo = jugglerList;
				else if(m.getType() == SourceSinkInfo.MODULE_STR)
					listToAddTo = jugglerList;
				else if(m.getType() == SourceSinkInfo.DAEMON_STR)
					listToAddTo = jugglerList;					
				else
					continue;
					
				listToAddTo.addElement(n);
				int t = drawParent.getBlockWidth(n.getName(), defaultBlockGroup); 
				if (t > nodeWidth)
					nodeWidth = t;
			}
		}
		
		//draw the existing PAM modules
		if (nodeWidth < 20)
			nodeWidth = 20;
		nodeWidth = nodeWidth + 25;

		
		maxXpos = 0;
		maxYpos = 0;
		maxDepth = 2;
		displayColumns = new Vector(5);
		for (int i=0;i<PIMList.size();i++) {
			buildNetwork((BertPDIFNode)PIMList.elementAt(i),0, markAsMoved);
		}

		for (int i=0;i<jugglerList.size();i++) {
			BertPDIFNode pNode = (BertPDIFNode)jugglerList.elementAt(i);
			if (pNode.autoPlaced == -1)
				buildNetwork(pNode, 1, markAsMoved);
		}

		for (int i=0;i<DIMList.size();i++) {
			BertPDIFNode pNode = (BertPDIFNode)DIMList.elementAt(i);
			if (pNode.autoPlaced == -1)
				buildNetwork(pNode, maxDepth, markAsMoved);
		}

		// resize the canvas so that we can see all of it!
		Dimension size = drawParent.getSize();
		if ((size.width < maxXpos+200) || (size.height < maxYpos+200))
			drawParent.setSize(maxXpos+500, maxYpos+500);
		drawParent.invalidate();
		validate();
		drawParent.needsRefresh();
		createExistingList();
	}
	
	private void buildNetwork(BertPDIFNode node,int depth, boolean markAsMoved) {
		if (node.autoPlaced >= depth)
			return;
		while (displayColumns.size() < depth+1) {
			displayColumns.addElement(new Vector());
		} 
		Vector column = (Vector) displayColumns.elementAt(depth);
			
		column.addElement(node);
		
		int x = 35 + depth*nodeWidth;	
		int y = column.size()*nodeHeight;
		


		if (x > maxXpos)
			maxXpos = x;
		if (y > maxYpos)
			maxYpos = y;
		if (depth > maxDepth)
			maxDepth = depth;
		
//		System.out.println(node.getName()+" "+x+" "+y);
		node.autoPlaced = depth;
		node.getGraphicNode().setPos(x,y);
		showGraphicNode(node);
		
		PDIFConn tmpConn = null;
		BertPDIFNode tmpNode = null;
		
		Enumeration conns = node.connections();
		while (conns.hasMoreElements()) {
			tmpConn = (PDIFConn) conns.nextElement();

			if (node != tmpConn.nodeList[0] && tmpConn.roleList[0] == BertPDIFRead.TO) {
				tmpNode = (BertPDIFNode) tmpConn.nodeList[0];
			} 
			else if (node != tmpConn.nodeList[1] && tmpConn.roleList[1] == BertPDIFRead.TO) {
				tmpNode = (BertPDIFNode) tmpConn.nodeList[1];
			} 
			else {
				continue;
			}						
			buildNetwork(tmpNode,depth+1, markAsMoved);
		}
	}
	public void removeEdge(Edge e) {
		drawParent.remove(e);

		PDIFConn conn;
		if ((conn = e.getData()) != null) {
			PAMstate.removeConnection(conn);
			e.setData(null);
			}
	}
/**
 * removes the underlying PDIFNode
 */
	public void removeNode(BertGraphicBlock n) {
		drawParent.remove(n); // remove graphic entry
		n.setOnScreen(false);
		PAMstate.removeNode(n.getData()); // remove data entry
		needExistingRebuild = true;
	}	

/**
 * hides the node from this view 
 */
	public void hideNode(BertGraphicBlock n) {
		drawParent.remove(n); // remove graphic entry
		n.setOnScreen(false);
	}	

	
/**
* We don't do anything with this event
*/
    public void windowOpened(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowClosed(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowIconified(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowDeiconified(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowActivated(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowDeactivated(WindowEvent e) {}

/**
* Close the window.  If in standalone mode then also call exit() cause
* something is causing Bert to stay alive...  NI.
*/
	public void windowClosing(WindowEvent e) {
		
		if (!optionalSave()) //save if necessary, cancel closing if requested.
			return;

		if (PAMstate != null && PAMstate.hasChangedElements()) {
			Integer saveP = ThreeButtonDlg.showThreeButtonDlg(this, "State changed.  Post to server?");
			if (saveP == null) 
				return;
			else if (saveP.intValue() == 0)
					postState();
		}

		if (connectionInfo.localMode) {
			setVisible(false);
			return;
		}

		cleanupReferences();
		finish();
		dispose();
		
//		listThreads(" -- at end");
		Runtime.getRuntime().exit(1);
	}
	
	private void cleanupReferences() {
		fMenu.removeActionListener(this);
		edgePopup.removeActionListener(handleEdgePopup);
		nodePopup.removeActionListener(handleNodePopup);
		drawParent.removeDrawListener(new EditorListener());
		removeWindowListener(this);
		drawParent.setOpenAreaPopup(null);
		BertGraphicBlock.setParentServices(null);

		newPIM.removeActionListener(listenForNewHandler);
		newDIM.removeActionListener(listenForNewHandler);
		newJuggler.removeActionListener(listenForNewHandler);
		existingPIM.removeActionListener(listenForExistingHandler);
		existingDIM.removeActionListener(listenForExistingHandler);
		existingJuggler.removeActionListener(listenForExistingHandler);
	}
/*
	static private void listThreads(String title) {
		System.out.println(title);
		
		ThreadGroup daddy = Thread.currentThread().getThreadGroup();
		ThreadGroup granddad = daddy;
		do {
			daddy = granddad;
			granddad = daddy.getParent();
		}
		while (granddad != null);
		listThreadGroup(daddy);
	}
	
	static private void listThreadGroup(ThreadGroup daddy) {
		System.out.println("ThreadGroup "+daddy.getName());
		Thread[] ta = new Thread[20];
		int n = daddy.enumerate(ta, false);
		for (int i = 0; i < n; i++) {
			boolean alive = ta[i].isAlive();
			boolean daemon = ta[i].isDaemon();
			String infoString = "";
			if (alive)
				infoString += " alive";
			if (daemon)
				infoString += " daemon";
			System.out.println("  --> thread ["+i+"] "+ta[i].getName()+" prio "+ta[i].getPriority()+infoString);
		}
		ThreadGroup[] tg = new ThreadGroup[20];
		n = daddy.enumerate(tg, false);
		for (int i = 0; i < n; i++) {
			boolean destroyed = tg[i].isDestroyed();
			boolean daemon = tg[i].isDaemon();
			String infoString = "";
			if (destroyed)
				infoString += " destroyed";
			if (daemon)
				infoString += " daemon";
			System.out.println("  --> subgroup ["+i+"] "+tg[i].getName()+infoString);
			listThreadGroup(tg[i]);
		}
		
	}

*/
	
	/**
	 * this function writes the names and locations of
	 * the nodes to a file.
	 * @param fileName - the name of the file to write to 
	 */
	void writeLayout(String fileName) throws IOException {
		PDIFWriter pOut = new PDIFWriter(new PrintWriter(new FileOutputStream(fileName)), "PAM", "GUI");
		pOut.writeHeaderAttribute("owner", connectionInfo.userID);
		java.util.Enumeration en = drawParent.getListOfNodes();
		while (en.hasMoreElements()) {
			// print out each node
			BertGraphicBlock n = (BertGraphicBlock)en.nextElement();
			java.awt.Point p = n.getLocation();
			if (n.getData() == null)
				continue;
		
			pOut.writeNode(n.getData().getType(), n.getName());
			pOut.writeAttributeValue("x", p.x);
			pOut.writeAttributeValue("y", p.y);
		}	
		pOut.close();
	}	

	public void showMessageDlg(String msg) {
		MessageDlg.showMessageDlg(this, msg);
	}

	private boolean confirmMsg(String msg) {
		return YesNoDlg.showYesNoDlg(this, msg);
	}
	
	Menu newPIM;
	Menu newJuggler;
	Menu newDIM;
	Menu existingPIM;
	Menu existingJuggler;
	Menu existingDIM;
	private final static String PROPERTIES = "Properties...";
	private final static String HIDE = "Hide";
	private final static String DELETE = "Delete";

	void buildPopups() {

	/*
	* build the node popup
	*/
		nodePopup = new PopupMenu("BertGraphicBlock");
		nodePopup.add(new MenuItem(PROPERTIES));
		nodePopup.add(new MenuItem(HIDE));
		nodePopup.add(new MenuItem(DELETE));
		drawParent.setBlockPopup(nodePopup);
		nodePopup.addActionListener(handleNodePopup = new HandleNodePopup());
		
	/*
	* build the edge popup
	*/
		edgePopup = new PopupMenu("Connection");
		edgePopup.add(new MenuItem(DELETE));
		drawParent.setConnectorPopup(edgePopup);
		edgePopup.addActionListener(handleEdgePopup = new HandleEdgePopup());

	/*
	* build the open area agent popup
	*/
		doPopUp = new SpecialPopupMenu("");

		newPIM = new Menu("New Source");
		newJuggler = new Menu("New Handler");
		newDIM = new Menu("New Output");
		existingPIM = new Menu("Existing Source");
		existingJuggler = new Menu("Existing Handler");
		existingDIM = new Menu("Existing Output");
		nStandardDoPopUpItems = 6;

		doPopUp.add(newPIM);
		doPopUp.add(newJuggler);
		doPopUp.add(newDIM);
		doPopUp.add(existingPIM);
		doPopUp.add(existingJuggler);
		doPopUp.add(existingDIM);
		drawParent.setOpenAreaPopup(doPopUp);
		
		newPIM.addActionListener(listenForNewHandler);
		newDIM.addActionListener(listenForNewHandler);
		newJuggler.addActionListener(listenForNewHandler);
		existingPIM.addActionListener(listenForExistingHandler);
		existingDIM.addActionListener(listenForExistingHandler);
		existingJuggler.addActionListener(listenForExistingHandler);
	}

/**
* Update the first part of the popup
*/
	private void createExistingList() {
			
		SortedHash PIMArray = new SortedHash();
		SortedHash DIMArray = new SortedHash();
		SortedHash jugglerArray = new SortedHash();			

		existingPIM.removeAll();
		existingJuggler.removeAll();
		existingDIM.removeAll();
		
	/*
	* figure out what each element is and whether is should be available
	*/
		java.util.Enumeration enum = PAMstate.elements();
		SortedHash listToAddTo;
		while (enum.hasMoreElements()) {
			BertPDIFNode pNode = (BertPDIFNode)enum.nextElement();
			BertPDIFNode p;
			// check that it's an instance of a pimType, dimType, or jugglerType
			if ((p = (BertPDIFNode)PAMstate.get(pNode.getType())) == null) 
				continue;
			if (p.getType() == SourceSinkInfo.PIM_STR)
				listToAddTo = PIMArray;
			else if(p.getType() == SourceSinkInfo.DIM_STR)
				listToAddTo = DIMArray;
			else if(p.getType() == SourceSinkInfo.JUGGLER_STR)
				listToAddTo = jugglerArray;
			else
				continue;


			if (connectionInfo.isAdmin || pNode.isOwnedBy(connectionInfo.userID) || pNode.isPublic()) { 
				listToAddTo.put(pNode.getName(), pNode);
			}
		}
		
	/*
	* Create the items for "existing" 
	*/
		Enumeration pEnum = PIMArray.elements();
		while(pEnum.hasMoreElements()) {
			BertPDIFNode pNode = (BertPDIFNode) pEnum.nextElement();
			existingPIM.add(pNode.getGraphicNode().getMenuItem());
		}

		pEnum = DIMArray.elements();
		while(pEnum.hasMoreElements()) {
			BertPDIFNode pNode = (BertPDIFNode) pEnum.nextElement();
			existingDIM.add(pNode.getGraphicNode().getMenuItem());
		}

		pEnum = jugglerArray.elements();
		while(pEnum.hasMoreElements()) {
			BertPDIFNode pNode = (BertPDIFNode) pEnum.nextElement();
			existingJuggler.add(pNode.getGraphicNode().getMenuItem());
		}
		

		needExistingRebuild = false;
	}

/**
* Create the second part of the popup
*/
	public void createAvailableList() {	 	
		Menu[] standardMenus = {newPIM, newDIM, newJuggler};
		String[] requests = {SourceSinkInfo.PIM_STR, SourceSinkInfo.DIM_STR, SourceSinkInfo.JUGGLER_STR};
		SortedHash extraMenus = new SortedHash(); // a vector of MenuItems that will be added to create popup
	/*
	* purge of existing extra menus
	*/
		Enumeration extraMenuKeys = extraMenus.keys();
	 	for (int i = doPopUp.getItemCount()-1; i>= nStandardDoPopUpItems;i--) {
	 		Menu menu = (Menu) doPopUp.getItem(i);
 			menu.removeActionListener(listenForNewHandler);
 			menu.removeAll();
	 		doPopUp.remove(i);
	 	}
	/*
	* Create a menu list of available agents.
	* Also recognizes special menus 
	*/
		
		SortedHash ary = new SortedHash();
		
		for (int i = 0; i< 3; i++) {
			ary.clear();
			Enumeration enum = PAMstate.elements(requests[i]);
			while (enum.hasMoreElements()) {
				BertPDIFNode bpn = (BertPDIFNode)enum.nextElement();
				if (bpn.getProperty(Agent.META_GUI_GROUP) == null) {
					ary.put(bpn.getName(), bpn);
				}
				else {
					String label1 = bpn.getProperty(Agent.META_GUI_GROUP);
					SortedHash submenuHash = (SortedHash)extraMenus.get(label1);
					if (submenuHash == null) {
						submenuHash = new SortedHash();
						extraMenus.put(label1, submenuHash);
					}
					submenuHash.put(bpn.getName(), new MenuItem(bpn.getName()));
				}
				
			}
			
			standardMenus[i].removeAll();
			enum = ary.keys();
			while (enum.hasMoreElements()) {
				standardMenus[i].add(new MenuItem((String)enum.nextElement()));
			}
		}	
		
	/*
	* add required extra menus
	*/
		Enumeration extras = extraMenus.keys();
		while (extras.hasMoreElements()) {
			String label1 = (String) extras.nextElement();
			Menu level1Menu = new Menu(label1);
			doPopUp.add(level1Menu);
			level1Menu.addActionListener(listenForNewHandler);
			SortedHash sh = (SortedHash) extraMenus.get(label1);
			Enumeration level2enum = sh.elements();
			while (level2enum.hasMoreElements()) {
				level1Menu.add((MenuItem) level2enum.nextElement());
			}
		}
		needAvailableRebuild = false;
	}

	public boolean isAdminSession() {
		return connectionInfo.isAdmin;
	}

	public boolean isDebugSession() {
		return debug;
	}
	
	public HasProperties getPDIFNode(String name){
		return PAMstate.get(name);
	}
	
	public String getUserID(){
		return connectionInfo.userID;
	}
	public boolean isRenameSupported(){
		return true;
	}

	public void renamePDIFNode(String oldName, String newName){
	
		BertPDIFNode pn = (BertPDIFNode) PAMstate.get(oldName);
		PAMstate.renameNode(pn, newName); // rename the data node 
		pn.getGraphicNode().setName(newName); // rename the graphic node
	}
/**
* From interface AdminTool. Not implemented for Bert.
*/
	public void displayAgent(String agent){}

	private String userPrefix(String userName) {
		StringBuffer prefix = new StringBuffer(20);
		int nout = 0;
		for (int i = 0; i< userName.length(); i++) {
			char c = userName.charAt(i);
			if (Character.isLetter(c)) {
				prefix.append(c);
			}
			else {
				prefix.append("stvwxyz".charAt(i));
			}
			if (++nout >= 5)
				return prefix.toString(); // return with 5 chars
		}
		nout = 0;
		while(prefix.length() < 6)
			prefix.append("dfghjklm".charAt(nout++));
		return prefix.toString();
	}
	
/**
* Enable or disable user controls such as menus and right mouse button sensing.
* If enabled then loadPostThread is cleared.
*/
	public void setUserControls(boolean enable) {
		setEnabled(enable);
		buttonSubmit.setEnabled(enable);
		buttonReload.setEnabled(enable);
		buttonDoFunc.setEnabled(enable);
		ignoreRightMouse = !enable;
		if (enable) {
			loadPostThread = null;
		}
	}
/**
* Set a new state structure
*/
	public void setNewStateStructure(BertPDIFRead stateStructure) {
		PAMstate = stateStructure;
	}
/**
* Set a new state structure
*/
	public final BertPDIFRead getStateStructure() {
		return PAMstate;
	}
/**
* The EditorListener handles connections, moving blocks,
* and save and load commands
*/
	class EditorListener implements DrawListener
	{
		
		public void blockConnectRequest(DrawEvent evt){
			BertGraphicBlock from = (BertGraphicBlock) evt.getFromBlock();
			BertGraphicBlock to = (BertGraphicBlock)evt.getToBlock();
//			System.out.println("connect nodes: "+from+" -to- "+to);
			connectNodes(from, to);
		}

		public void blockMoveTo(DrawEvent evt){
			BertGraphicBlock block = (BertGraphicBlock)evt.getBlock();
			block.getData().setPos(block.getX(), block.getY());
		}

		public void doCommand(DrawEvent evt){
//			System.out.println("cmd: "+evt.getId());
			String cmd = evt.getId();
			if (cmd == DrawEvent.SAVE) {
				save();
			}
			else if (cmd == DrawEvent.LOAD) {
				reload();
			}
		}
	}

/**
* Show, hide, and edit nodes
*/
	class HandleNodePopup implements ActionListener {
		public void actionPerformed(ActionEvent evt) {
			BertGraphicBlock node = (BertGraphicBlock) drawParent.getBlockThatIsPoped();
			if (evt.getActionCommand() == DELETE) {
				if (connectionInfo.isAdmin || node.isOwnedBy(connectionInfo.userID))
					removeNode(node);
				else
					hideNode(node);
			}
			else if (evt.getActionCommand() == HIDE) {
				hideNode(node);
			}
			else if (evt.getActionCommand() == PROPERTIES) {
				node.editProperties();
			}

		}
	}

/**
* Delete connections between nodes
*/
	class HandleEdgePopup implements ActionListener {
		public void actionPerformed(ActionEvent evt) {
			BlockConnector l = drawParent.getConnectorThatIsPoped();
//			System.out.println("connector request: "+evt.getActionCommand()+" "+l.toString());
			if (evt.getActionCommand() == DELETE) {
				removeEdge((Edge) l);
			}
		}
	}
/**
* Handle button press to create new node
*/	
	class PerformOnNew implements ActionListener {
		public void actionPerformed(ActionEvent e) {
//			System.out.println("evt new ="+e+e.getSource());
			if (ignoreRightMouse) // ignore when loading or saving (we are disabled then)
				return;
			if (!(e.getSource() instanceof MenuItem))
				return;
				
			String selection = e.getActionCommand();
//			System.out.println("selection="+selection);
			int initNum = 1;
			String testName = userPrefix(connectionInfo.userID) + selection;
			while (PAMstate.get(testName+initNum) != null)
				initNum++;			
			createNode(testName+initNum, selection, drawParent.getOpenAreaClickX(), drawParent.getOpenAreaClickY());
		}
	}
/**
* Handle button press to show existing node
*/	
	class PerformOnExisting implements ActionListener {
		
		public void actionPerformed(ActionEvent e) {
//			System.out.println("evt exist="+e+e.getSource());
			if (ignoreRightMouse) // ignore when loading or saving (we are disabled then)
				return;
			if (!(e.getSource() instanceof MenuItem))
				return;
			if (e.getSource() instanceof Menu)
				return;
				
			String selection = e.getActionCommand();
//			System.out.println("selection="+selection);
			BertPDIFNode pNode = (BertPDIFNode)PAMstate.get(selection);
			if (!pNode.getGraphicNode().isOnScreen())
				showGraphicNode(pNode);
		}
	}
	
/**
* Special PopupMenu type that allows us to know when the popup
* goes away.
*/
	class SpecialPopupMenu extends PopupMenu {
		public SpecialPopupMenu(String name) {
			super(name);
		}
	/**
	* Override parent so we can know when the popup goes away.
	* At this point we can then rebuild the popup menu.  If we
	* rebuild the popup menu while it is showing we can get into
	* trouble.
	*/
   		public void show(Component origin, int x, int y) {
			super.show(origin, x, y);
			if (needExistingRebuild) {
				createExistingList();
			}
			if (needAvailableRebuild) {
				createAvailableList();
			}
   		}
	}


/*	
	class PostStateToXXX extends Thread {
		public void run() {
			try {
				setEnabled(false);
			}
			finally {
				loadPostThread = null;
				setEnabled(true);
			}
		}
	}
*/

	
}