/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.bert;
import java.util.Enumeration;
import java.io.Reader;
import org.mitre.pam.util.pdifHashtables.PDIFRead;
import org.mitre.pam.util.pdifHashtables.PDIFNode;
import org.mitre.pam.util.pdifHashtables.PDIFConn;
import org.mitre.pam.store.SourceSinkInfo;
import org.mitre.pam.interfaces.ScheduleAttributes;
import org.mitre.pam.interfaces.Reporter;

import org.mitre.pam.util.pdifHashtables.DuplicateIDException;
import org.mitre.pam.executive.PAMSave;
import org.mitre.pdif.eventParser.BadPDIFException;

/**
 * Overrides the factory methods
 *
 * @version 1, 28 May 1998
 * @author Nazario Irizarry
 */
class BertPDIFRead extends PDIFRead {
	String layoutId;
	public static String FROM = "from";	
	public static String TO = "to";	
	Reporter reporter;
	int inputBuildNumber; // set to the psvsnb pdif header attribute
	private boolean headersChecked;
	private boolean useMetaData;

	BertPDIFRead(Reader inStream, String layoutId, Reporter reporter) {
		super(inStream);
		this.reporter = reporter;
		this.layoutId = layoutId;
	}

	
	public PDIFNode makeNewNode(String type, String id) {
		if (!headersChecked)
			checkHeaderData();
		if (type == null)
			throw new IllegalArgumentException("type == null");
		if (id == null)
			throw new IllegalArgumentException("id == null");
		BertPDIFNode bp = new BertPDIFNode(type, id, reporter);
		bp.setUseMetadata(useMetaData);
		bp.setLayoutId(layoutId);
		new BertGraphicBlock(bp);
		return bp;
	}

	public PDIFNode makeNewBaseNode(String type, String id) {
		if (type == null)
			throw new IllegalArgumentException("type == null");
		if (id == null)
			throw new IllegalArgumentException("id == null");
		BertPDIFNode bp = new BertPDIFNode(type, id, reporter);
		bp.setUseMetadata(useMetaData);
		return bp;
	}

/*
* Override the basic go method so that we can look for the build
* number of the server iPAM after the load.
*/
	private void checkHeaderData()  {
		headersChecked = true;
	/*
	* Look for the build identifier
	*/
		String buildVers = (String) headerAttributes.get(PAMSave.BULID_NUMBER_PREFIX);
		if (buildVers != null) {
			inputBuildNumber = 0;
			try {
				inputBuildNumber = Integer.parseInt(buildVers);
			}
			catch (Exception ignore){
			}
		}
		if (inputBuildNumber >= 64) {
			useMetaData = true;
		}
	}

}