/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */

package org.mitre.pam.bert;


/**
 * A line component for use with awt.
 * 
 * @version 1, 4 December 1997
 * @author Sam Gerstein
 */
class Line extends java.awt.Component {
	int x1, y1, x2, y2;
	double thresh = 3.0;



	public Line() {
		super();
	}
	/**
	 * computes the distance from the given point to the line to
	 * determine whether the point should be considered "in" the
	 * line for mouse event purposes.
	 * @param x - x position
	 * @param y - y position
	 * @return true if given point is within threshold
	 */
	public boolean contains (int x, int y) {
		return (dist(x, y) <= thresh) ;		
	}	
	/**
	 * computes the distance from the given point to the line.
	 * @param xp - the x position
	 * @param yp - the y position
	 * @return distance from point (xp, yp) to line
	 */
	public double dist(int xp, int yp){
		int dx, dy; // the vector rep. of the line
		int cx, cy; // the vector rep. of the line from the beginning of the
							// line to the point
		double ax, ay; 
		double len_d2, scal;//, len_d;
		
		// move back into world space
		xp = xp + getLocation().x;
		yp = yp + getLocation().y;

		// compute the vector components of the line		
		dx = x2 - x1;
		dy = y2 - y1;

		// compute the vector components of the line from 
		// the beginning of the line to the point being checked
		cx = xp - x1;
		cy = yp - y1;
		
		// compute the length of the line
		len_d2 = dx * dx + dy * dy;
//		len_d = java.lang.Math.sqrt(len_d2);

		// compute the scalar projection of c onto d
		scal = (cx * dx + cy * dy)/len_d2;
		if (scal > 1.0) {
			// we're off one end of the segment
			ax = xp - x2;
			ay = yp - y2;
		} else if (scal < 0.0) {
			// we're off the other end
			ax = xp - x1;
			ay = yp - y1;
		}	else {
			// compute the vector components of the shortest line from the line
			// to the point
			ax = cx - scal * dx;
			ay = cy - scal * dy;
		}	
		return (java.lang.Math.sqrt(ax * ax + ay * ay));
	}	
	/**
	 * draws the line.
	 * @param graphics - the Graphics object to draw on.
	 */
	public void paint(java.awt.Graphics graphics) {
		graphics.setColor(getForeground());
		graphics.drawLine(x1, y1, x2, y2);
	}	
	/**
	 * sets the endpoints of the line, then
	 * resets the bounds of the object.
	 * @param x1 and
	 * @param y1 - the first point
	 * @param x2 and
	 * @param y2 - the second point
	 */
	public void setEnds (int x1, int y1, int x2, int y2) {
		this.x1 = x1;
		this.y1 = y1;
		this.x2 = x2;
		this.y2 = y2;	
		setBounds(-5 + ((x1 < x2)?x1:x2), -5+((y1 < y2)?y1:y2), 10+((x1 > x2)?x1-x2:x2-x1), 10+((y1 > y2)?y1-y2:y2-y1));
	}	
}