/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.bert;
import java.awt.Frame;
import java.awt.FileDialog;
import java.net.*;
import java.io.*;

import org.mitre.pam.monitor.MultipleTextBoxDlg;
import org.mitre.pam.util.Codecs;
import org.mitre.pam.interfaces.Reporter;

/**
* Post state to a server. 
* This could take a few seconds so we don't want to lock up
* the AWT thread that caught the GUI event to request this.
*/
class PostStateToServer extends Thread {
	boolean postAll;
	LoadCallbacks callbacks;
	ConnectionInfo connectionInfo;
	Frame parentFrame;
	Reporter reporter;
	
	PostStateToServer (boolean postAll, LoadCallbacks callbacks, ConnectionInfo connectionInfo, Frame parentFrame, Reporter reporter) {
		this.postAll = postAll;
		this.callbacks = callbacks;
		this.connectionInfo = connectionInfo;
		this.parentFrame = parentFrame;
		this.reporter = reporter;
	}

	public void run() {
		try {
			callbacks.setUserControls(false);
			StringWriter stateAsString = new StringWriter();
			callbacks.setStatus("Writing to server, please wait...");
			HttpURLConnection urlc=null;
			BertPDIFRead PAMstate = callbacks.getStateStructure();
			try {
				PrintWriter printout = new PrintWriter(stateAsString);
				PAMstate.writeState(printout, true, postAll);
				printout.flush();
				printout.close();
				
				URL url = new URL("http", connectionInfo.host, connectionInfo.port, ConnectionInfo.POST_FILE_NAME);
				urlc = (HttpURLConnection) url.openConnection();
				urlc.setRequestMethod("POST");
				urlc.setDoOutput(true);
				urlc.setUseCaches(false);
				urlc.setRequestProperty("Authorization","Basic "+Codecs.base64Encode(connectionInfo.userID+":"+connectionInfo.userPassword));
				PrintWriter dos = new PrintWriter(urlc.getOutputStream());
				dos.print("changeText="+URLEncoder.encode(stateAsString.toString()));
				dos.flush();
	
				int code = urlc.getResponseCode();
				String response = urlc.getResponseMessage();
				
				if (callbacks.isDebugSession()) {
					System.out.println("\nCONTENT:\n" +stateAsString.toString());
					System.out.println("\nRESPONSE CODE:" +code);
					System.out.println("\nRESPONSE:\n" +response);
				}
	
				if (code >= 400) {
					callbacks.setStatus( "Error: "+response);
					return;
				}
			} catch (Exception e) {
				callbacks.setStatus( "Failed to post (see log).");
				reporter.reportException(e);
				return;
			} finally {
				if (urlc != null)
					urlc.disconnect();
			}
	
			PAMstate.clearChangedFlags();
			callbacks.setStatus("Post done.");
			connectionInfo.lastLoadSaveTime = System.currentTimeMillis();

		}
		finally {
			callbacks.setUserControls(true);
		}
	}
}
