/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.bert;
import java.awt.Frame;
import java.awt.FileDialog;
import java.net.URL;
import java.net.URLEncoder;
import java.io.*;

import org.mitre.pam.monitor.MultipleTextBoxDlg;
import org.mitre.pam.util.Codecs;
import org.mitre.pam.interfaces.Reporter;

/**
* Class to read state from a file. 
* This probably would not take that long but this
* way it is consistent with the threads that
* read and write to a server.
*/
class ReadStateFile extends Thread {
	LoadCallbacks callbacks;
	ConnectionInfo connectionInfo;
	Frame parentFrame;
	Reporter reporter;

	ReadStateFile(LoadCallbacks callbacks, ConnectionInfo connectionInfo, Frame parentFrame, Reporter reporter) {
		this.callbacks = callbacks;
		this.connectionInfo = connectionInfo;
		this.parentFrame = parentFrame;
		this.reporter = reporter;
	}

	public void run() {
		try {
			callbacks.setUserControls(false);
	
			String labels[] = {"User or configuration name:"};
			String defs[] = {connectionInfo.userID};
			String tmpID = "";
			while (tmpID.length() == 0 ) {
				String servInfo[] = MultipleTextBoxDlg.showMultipleTextBoxDlg(parentFrame, labels, defs, "Open local file");
				if (servInfo == null)
					return;
				
				tmpID = servInfo[0];
			}
	
	
			FileDialog askStateFileName = new FileDialog(parentFrame, "Open State File", FileDialog.LOAD);
			if (connectionInfo.stateFileName != null) {
				askStateFileName.setFile(connectionInfo.stateFileName);
				askStateFileName.setDirectory(connectionInfo.stateDirectoryName);
			}	
			askStateFileName.setVisible(true);
			if (askStateFileName.getFile() == null)
				return;
			
			String tmpStateFileName = askStateFileName.getDirectory() + askStateFileName.getFile();
			BertPDIFRead tmpState;
			try {
				callbacks.setStatus("Loading...");
				tmpState = new BertPDIFRead(new BufferedReader(new FileReader(tmpStateFileName), 400), tmpID, reporter);
			} catch (IOException e) {
				callbacks.setStatus("Load error");
				callbacks.showMessageDlg( "Unable to open specified state file.");
				return;
			}
			try {
				tmpState.go();
			} catch (Exception e) {
				callbacks.setStatus("Load error "+e.getMessage());
				callbacks.showMessageDlg( "State file not valid.");
				reporter.reportException(e);
				return;
			}
			
			// make no changes to the app or state before this point (allowing for recovery)
			callbacks.cleanSlate();
			connectionInfo.stateFileName = askStateFileName.getFile();
			connectionInfo.stateDirectoryName = askStateFileName.getDirectory();
			callbacks.setNewStateStructure(tmpState);
			connectionInfo.loadedByURL = false;
			connectionInfo.loadedByFile = true;
			connectionInfo.userID = tmpID;
	
			String remoteAdminName = (String)tmpState.get("exec").getStringProperty("bertAdministratorName","Administrator");
			connectionInfo.isAdmin = connectionInfo.userID.equals(remoteAdminName) ;
/*
			if (connectionInfo.isAdmin)
				fileMenuSAVE_PAM_FILE.setEnabled(true);
			fileMenuSAVE_PAM_SERV.setEnabled(true);
			fileMenuAUTOLAYOUT.setEnabled(true);
*/
			callbacks.createAvailableList();
			if (connectionInfo.isAdmin) {
				callbacks.autoLayout(false);
			}
			else {
				callbacks.readLayout();
			}
			callbacks.setStatus("Load done");
			connectionInfo.lastLoadSaveTime = System.currentTimeMillis();


		}
		finally {
			callbacks.setUserControls(true);
		}
	}
}
