/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.bert;
import java.awt.Frame;
import java.net.URL;
import java.net.URLEncoder;
import java.io.*;

import org.mitre.pam.monitor.MultipleTextBoxDlg;
import org.mitre.pam.util.Codecs;
import org.mitre.pam.interfaces.Reporter;

/**
* Class to read state from a server. 
* This could take a few seconds so we don't want to lock up
* the AWT thread that caught the GUI event to request this.
*/
class ReadStateFromServer extends Thread{
	LoadCallbacks callbacks;
	ConnectionInfo connectionInfo;
	Frame parentFrame;
	Reporter reporter;

	ReadStateFromServer(LoadCallbacks callbacks, ConnectionInfo connectionInfo, Frame parentFrame, Reporter reporter) {
		this.callbacks = callbacks;
		this.connectionInfo = connectionInfo;
		this.parentFrame = parentFrame;
		this.reporter = reporter;
	}
	
	public void run() {
		try {
			callbacks.setUserControls(false);

			String tmpHost = "";
			int tmpPort = -2;
			String tmpID = connectionInfo.userID;
			String tmpPassword = "";
	
			String defaultHost = null;
			int defaultPort = 19090;
	
			defaultHost = connectionInfo.host;
			defaultPort = connectionInfo.port;
	
			String labels[] = {"User or configuration name:", "Password", "Host name:", "Port:"};
			String defs[] = {connectionInfo.userID, connectionInfo.userPassword, defaultHost, Integer.toString(defaultPort)};
			while (tmpID.length() == 0 || tmpPort== -2 || tmpHost.length()==0) {
				String servInfo[] = MultipleTextBoxDlg.showMultipleTextBoxDlg(parentFrame, labels, defs, "Open server");
				if (servInfo == null)
					return;
				
				tmpID = servInfo[0];
				tmpPassword = servInfo[1];
				tmpHost = servInfo[2];
				try {
					tmpPort = Integer.parseInt(servInfo[3]);
				}
				catch (NumberFormatException e) {
					tmpPort = -2;
				}
			}
			
			connectionInfo.host = tmpHost;
			connectionInfo.port = tmpPort;
			connectionInfo.userPassword = tmpPassword;
			connectionInfo.userID = tmpID;

			URL tmpURL;
			java.io.InputStream tmpStream;
	//		URLConnection  tmpConnection;
			try {
				callbacks.setStatus("Loading...");
				String queryString = ConnectionInfo.QUERY_FILE_NAME;
				queryString += "?editor="+URLEncoder.encode(connectionInfo.SELF_NAME);
				queryString += "&Authorization="+Codecs.base64Encode(tmpID+":"+tmpPassword);
				queryString += "&version="+connectionInfo.bertVersion;
				queryString += "&configuration="+URLEncoder.encode(tmpID);
				queryString += "&noRedundant=true";
	//	System.out.println("request url= "+queryString);
				tmpURL = new URL("http", tmpHost, tmpPort, queryString);
				tmpURL.openConnection().setUseCaches(false);
				tmpStream = tmpURL.openStream();
			}
			catch	 (java.net.MalformedURLException e) {
				callbacks.setStatus("Load error");
				callbacks.showMessageDlg( "invalid URL.");
				return;
			} 
			catch (java.io.IOException e) {
				callbacks.setStatus("Load error: "+e.getMessage());
				callbacks.showMessageDlg( "couldn't open URL.");
				return;
			}		
	
			BertPDIFRead tmpState;
			tmpState = new BertPDIFRead(new BufferedReader(new InputStreamReader(tmpStream), 400), tmpID, reporter);
	
	
			try {
				tmpState.go();
			} catch (Exception e) {
				callbacks.showMessageDlg( "failed to parse PAM state. (" + e + ").");
				callbacks.setStatus("Load error");
				reporter.reportException(e);
				return;
			}		
	
	
			// make no changes to the app or state before this point (allowing for recovery)
			callbacks.cleanSlate();
			callbacks.setNewStateStructure(tmpState);
			connectionInfo.loadedByURL = true;
			connectionInfo.loadedByFile = false;
	
			String remoteAdminName = (String)tmpState.get("exec").getStringProperty("bertAdministratorName","Administrator");
			connectionInfo.isAdmin = connectionInfo.userID.equals(remoteAdminName) ;
	
			BertPDIFNode p = (BertPDIFNode)tmpState.get("exec");
			int minimumVersion =  p.getIntegerProperty("minimumBertVersion",0);
				
			if (connectionInfo.bertVersion < minimumVersion)
				callbacks.showMessageDlg( "Warning: need a newer version of this software ("+minimumVersion+")");
			String serverID = (String) p.getStringProperty("id","");
			if (serverID!="")
				serverID = " - "+serverID;
			//
			
				
			parentFrame.setTitle(tmpHost+":"+tmpPort+serverID+" ["+connectionInfo.userID+"]");
	
			callbacks.createAvailableList();
			if (connectionInfo.isAdmin) {
				callbacks.autoLayout(false);
			}
			else {
				callbacks.readLayout();
			}
			
			callbacks.setStatus("Load done");
			connectionInfo.lastLoadSaveTime = System.currentTimeMillis();
		}
		finally {
			callbacks.setUserControls(true);
		}
	}

}
