/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.aggregator;

import java.util.*;
import java.io.File;

import org.mitre.pam.interfaces.*;

import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.executive.Module;
import org.mitre.pam.util.EnabledTable;
import org.mitre.pam.daemon.DaemonBase;
import org.mitre.pam.executive.Command;
import org.mitre.pam.executive.CommandParallelInputs;
import org.mitre.pam.executive.CommandParallelOutputs;

/**
* A base class for handlers that do not trigger their inputs
* when they are asked for their input.  In other words
* they implement some kind of buffer. Inputs contribute to
* the buffer.  Outputs pull from the buffer.  This is not a queue
* because once something is in the buffer the buffer is not empty
* no matter how many times it is pulled.
*
* @author Nazario Irizarry, Jr.
*/
abstract public class BufferBase extends NoSchedBase {

	protected BufferBase () {
		super(false,false);
		initBufferBase(false);
	}

/**
* This form of the constructor does not invoke initBufferBase()
*/
	protected BufferBase (boolean ignore, boolean ignore2) {
//		super(false,false);
	}

/**
* Init actually fills in the Memento and sets default attributes
*/
	protected void initBufferBase(boolean initParent) {
		setProperty(HAS_OUTPUTS,true);
		setProperty(HAS_INPUTS,true);
		setProperty(Schedulable.HAS_SCHEDULE, false);
	}

	
/**
* Callback to receive a command, a juggler must always return to
* state= ExecGlobal.idle when it is not waiting for pims or dims
*/
	public void doCommittedTask(){
		Task task = taskHelper.getCommittedTask();
		String command = task.getCommand();

		reporter.debugMsg("task:",task,null,null,ExecGlobal.TRACE_MSG_FLOW);
		if ( command == Command.SEND_PRODUCT ) {
			declareStart(ExecGlobal.WORKING);
			pleasePutProduct( task.getProduct(), task);
			declareEnd(false);
		} 
		else if ( command == Command.ACTIVATE ) {
			doPutProductToAll(gAgentsProduct, getSinkList().elements());
		} 
		else if ( command == Command.GET_PRODUCT ) {
			doPutExistingProductToOne(gAgentsProduct, task.getSink(), task.getPutToken());
		} 
		else {
			warning("unexpected task, "+task);
			declareEnd(false); // clears committed task
		}
	}
	

/**
* 
*/
	public void putProduct(Product product){
		gAgentsProduct = buildProduct(product);
	}
}
