/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.daemon;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.util.smtp.SendMail;
import org.mitre.pam.util.smtp.MIMEc;
import org.mitre.pam.interfaces.ExecToController;

//import org.mitre.pam.util.alarm.Alarm;

/**
* This class creates a Daemon that filters
* the log file and, based on date, replaces the log.txt with a newer
* edited version and emails a copy to the administrator.
*
* @author   John Fantry, Technical Co-op (D510)
* @author   Jason Mathews use default smtp server and use mailing list
*/
public class LogFilter extends DaemonBase {
  protected final static String MAIL_TO = "mailLogTo";
  protected final static String DELETE_ALL_AFTER_N_DAYS = "deleteAllAfterNDays";
  protected final static String SAVE_EXCEPTIONS_ONLY_AFTER_N_DAYS = "saveExceptionsOnlyAfterNDays";

  public LogFilter() {
    setProperty(MAIL_TO, "");
    setProperty(DELETE_ALL_AFTER_N_DAYS, "5");
    setProperty(SAVE_EXCEPTIONS_ONLY_AFTER_N_DAYS, "3");
    setPeriod(3600*12);
  }

  public void doDaemonTask() {

	try {
		Integer deleteAll = new Integer(getProperty(DELETE_ALL_AFTER_N_DAYS));
		Integer saveExceptions = new Integer(getProperty(SAVE_EXCEPTIONS_ONLY_AFTER_N_DAYS));
    	((ExecToController)exec).filterLog(deleteAll.intValue(), saveExceptions.intValue());
	} catch(NumberFormatException e) {
	  reportException(e);
	  return;
	}
    
/*    File logFile = new File(System.getProperty("user.dir") + System.getProperty("file.separator") +
                            "0checkpts" + System.getProperty("file.separator") + "log.txt");*/

    String mailHost = getSmtpHostName();

    Vector mailingList = SendMail.buildMailingList(getProperty(MAIL_TO), mailHost);

    if (mailingList.size() == 0) {
      return;
    }

		SendMail mailer = new SendMail(mailHost);
		
		File logFile = new File(ExecGlobal.getCheckPointDir(), "log.txt");
		
			MIMEc message = new MIMEc();
			message.setFrom("LogFilter");
			message.setSubject("New log.txt");
			message.setBody("Enclosed");
			message.addAttachment("text/plain", logFile);

    for (int i=0; i < mailingList.size(); i++) {
      String to = (String) mailingList.elementAt(i);
      message.setTo(to);
      try {
			mailer.sendMail(message);
		} catch(IOException ioe) {
        reporter.reportException(ioe);
      }
    }//for
  }

}
