/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.executive;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.rmi.RMISecurityManager;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Observable;
import java.util.Properties;
import java.util.Calendar;

import org.mitre.pam.interfaces.*;
import org.mitre.pdif.eventParser.ValueList;
import org.mitre.pam.store.FileRef;
import org.mitre.pam.store.FileRefList;
import org.mitre.pam.store.IntegrityException;
import org.mitre.pam.executive.PropertySet;
import org.mitre.pam.store.SourceSinkInfo;
import org.mitre.pam.util.PAMDate;
import org.mitre.pam.util.alarm.Alarm;
import org.mitre.pam.util.alarm.AlarmFrame;
import org.mitre.pam.util.alarm.AlarmReporter;
import org.mitre.pdif.eventParser.PDIFWriter;
import org.mitre.pdif.eventParser.ParseEvent3;
import org.mitre.pdif.eventParser.BadPDIFException;
import org.mitre.pam.product.view.ProductShell;
import org.mitre.pam.product.view.ProductFactoryImpl;

import org.mitre.pam.daemon.Dict;
import org.mitre.pam.product.view.DictionaryProduct;
import org.mitre.pam.store.FileRef;
import org.mitre.pam.util.SortedHash;
import org.mitre.pam.util.StreamCopy;

import Acme.Serve.Serve;


/**
* Exec level implements core scheduling and resource
* allocation.<P>
*
* @author Nazario Irizarry, Jr.
*/
public class Exec00 extends ExecGlobal implements Runnable, ExecCallbacks{
	protected final static String DEFAULT_PORT = "19090";
/**
* The thing that handles disptching for the exec
*/
	protected FlowDispatcher queueDispatcher;
/**
* The reporter provides for logging and reporting for the exec
*/
	protected Reporter reporter = new AlarmReporter("exec");
/**
* Reference to a class that allows the exec to masquerade as an agent
*/
	protected Agent agentFace;
/**
* Web Service
*/
	Serve serve;
/**
* Checkpoint thread
*/
	protected Thread checkPointThread;
/**
* Flag indicates that a checkpoint has been requested.
*/
	protected boolean checkPointRequested;
/**
* Flag indicates that a shutdown has been requested.
*/
	protected boolean shutdownRequested;
	
/**
* The Juggler commander
*/
	protected Scheduler jugglerScheduler;
/**
* The PIM executor
*/
	protected Executer PIMExecuter;
/**
* The DIM executor
*/
	protected Executer DIMExecuter;
/**
* List of  Daemons
*/
	protected SortedHash daemonList;
/**
* List of  PIMs
*/
	protected SortedHash PIMList;
/**
* List of  Jugglers
*/
	protected SortedHash jugglerList;
/**
* List of  DIMs
*/
	protected SortedHash DIMList;
/**
* List of  Asynchronous agents
*/
	protected Hashtable asynchronousAgentList;
/**
* An array that points to the individual module lists
*/
	protected SortedHash[] moduleList;
/**
* A list of things that need to be persistently saved
*/
	protected SortedHash[] listOfThingsWithProduct;
/**
* A list of things that have input connections
*/
	protected SortedHash[] listOfThingsWithInputs;
/**
* A list of things that have no persistent product
*/
	protected SortedHash[] listOfThingsWithoutProduct;
/**
* A "table" of source and sink info
*/
	protected SourceSinkInfo sourceSinkInfo;
/**
* Our local product sweeper
*/
	protected SweepAndGC sweeper;
/**
* Alarm frame used only if debugging
*/
	protected AlarmFrame alarmFrame;
/*
* A duplicate of execOptions.runState
*/
	protected String execState;
/**
* Default authorization list
*/
	private String[] DEFAULT_AUTHLIST = {"Administrator","*:*"};
	
/**
* identifies the last day that a day-named checkpoint file was created
* (in addition to normal periodic backup)
*/
	protected int lastCheckpointDay = -1;
/**
* Flag that indicates that the exec has already started the async agents
*/
	protected boolean asyncAgentsBulkStarted;
/**
* A reference to the product factory, not created until needed.
*/
	protected ProductFactory productFactory;

/**
* Build the exec and all its parts
*/
	protected void init() {
		Runtime.runFinalizersOnExit(true);

		Module.exec = this;
		ProductShell.setExec(this, reporter);
		Command.setExec(this);

		execOptions = makeNewMemento("exec","exec");

		execOptions.setProperty(SUPPORT_DIR_ROOT,"0supportDir");
		execOptions.setProperty(WORKING_DIR_ROOT,"0workingDir");
		execOptions.setProperty(CHECK_POINT_DIR,"0checkpts");
//		execOptions.setProperty(PRODUCT_STORE_DIR,"0checkpts");
//		execOptions.setProperty(PRODUCT_DIR,"products");
		execOptions.setProperty(PIM_THREAD_COUNT,"2");
		execOptions.setProperty(DIM_THREAD_COUNT,"1");
		execOptions.setProperty(ACTIVE_FILEREF_COUNT,"-1");
		execOptions.setProperty(PROXY_HOST,System.getProperty("proxyHost",""));
		execOptions.setProperty(PROXY_PORT,System.getProperty("proxyPort",""));
		execOptions.setProperty(LAST_CHECKPOINT_LONG,"-1");
		execOptions.setProperty(SERVER_PORT_NUMBER,"19090");
		execOptions.setProperty(EXEC_ID,"");
		execOptions.setProperty(CONCURRENCY_LIMIT, "1");
		execOptions.setProperty(DEFAULT_SMTP_HOST_NAME, "");

		execOptions.setProperty(MIN_BERT_VERSION, "2");
		execOptions.setProperty(BERT_ADMINS_NAME, "Administrator");
		execOptions.setPropertyValues(IPAM_ACCESS_FOLKS, new ValueList(DEFAULT_AUTHLIST));
		execOptions.setProperty(LOG_DELETED_FILES, "false");


		
		execOptions.setProperty(PROTOCOL_AND_HOST, "");
		execOptions.setProperty(PEER_PROTOCOL_HOST_PORT, "");
		execOptions.setProperty(WORKING_DIR_URL_ROOT_NAME,"/PAMWorkingDir");
		execOptions.setProperty(SUPPORT_DIR_URL_ROOT_NAME,"/PAMSupportDir");


	/*
	* create the lists but the async agent list 
	* is not mutually exclusive with other 4 lists
	*/

		daemonList = new SortedHash();
		PIMList = new SortedHash();
		jugglerList = new SortedHash();
		DIMList = new SortedHash();
		asynchronousAgentList = new Hashtable(50); 

		moduleList = new SortedHash[4];
		moduleList[ExecToMonitor.DIM] = DIMList;
		moduleList[ExecToMonitor.PIM] = PIMList;
		moduleList[ExecToMonitor.DAEMON] = daemonList;
		moduleList[ExecToMonitor.JUGGLER] = jugglerList;
		
		listOfThingsWithProduct = new SortedHash[4];
		listOfThingsWithProduct[0] = jugglerList;
		listOfThingsWithProduct[1] = DIMList;
		listOfThingsWithProduct[2] = daemonList;
		listOfThingsWithProduct[3] = PIMList;
		
		listOfThingsWithoutProduct = new SortedHash[0];

		listOfThingsWithInputs = new SortedHash[2];
		listOfThingsWithInputs[0] = jugglerList;
		listOfThingsWithInputs[1] = DIMList;



		setState(STARTING);
		
		Alarm.setLogFile(new File(getCheckPointDir(),"log.txt"));
		
		jugglerScheduler = new Scheduler(JUGGLER_SCHEDULER, this);
		PIMExecuter = new Executer(this, PIM_EXECUTER, 2);
		DIMExecuter = new Executer(this, DIM_EXECUTER,1);
		queueDispatcher = new QueueDispatcher(PIMExecuter, DIMExecuter, reporter);

		
		FileRef.setExec(this);
		sourceSinkInfo = new SourceSinkInfo(this);
		sweeper = new SweepAndGC(listOfThingsWithProduct, this);
		
//		startFromConfig(configFile);
		checkPointThread = new Thread(this,"iPAM Exec");
		checkPointThread.setPriority(Thread.NORM_PRIORITY -1);
//		checkPointThread.start(); // moved to Exec06 at the end of startFromConfig()
		
		
		try {
			System.setSecurityManager(new RMISecurityManager());
	    } catch (Exception e) {
	    	reporter.reportException(e);
	    }

		
	}
	protected final void startAsyncAgents() {
		boolean somethingStarted = false;
		synchronized (asynchronousAgentList) {
			Enumeration e = asynchronousAgentList.elements();
			while (e.hasMoreElements()) {
				Module agent = (Module)e.nextElement();
				agent.execSetAsync(true);
				somethingStarted = true;
			}
			asyncAgentsBulkStarted = true;
		}
		if (somethingStarted)
			reporter.log("Async services started");
		else
			reporter.log("No async services to start");
	}
	
/**
* Get a reference to the product factory.
*/
	public ProductFactory getProductFactory(){
		if (productFactory == null) {
			productFactory = new ProductFactoryImpl(reporter);
		}
		return productFactory;
	}

/**
* The exec thread executes checkpoint and product cleanup
*/
	private long nextSweepTime = System.currentTimeMillis() + 60000;
	public void run() {
		try {
			Thread.sleep(5000);	// sleep initially to let things start up
		}
		catch (InterruptedException e) {}

		setState(WORKING);
		startAsyncAgents();
		
		try {
			while (true) {
				setState(WORKING);
				try {
					Thread.sleep(3030);	// sleep for 3 seconds
				}
				catch (InterruptedException e) {}

				boolean lShutdownRequested = shutdownRequested;
				boolean lCheckPointRequested = checkPointRequested;
				
			/*
			* Boost priority on shutdown
			*/
				if (lShutdownRequested) {
					checkPointThread.setPriority(Thread.NORM_PRIORITY + 1);
				}
				
				if (nextSweepTime < System.currentTimeMillis()) {
					setState(IDLE);
					sweeper.sweep();
					nextSweepTime = System.currentTimeMillis() + 120013;
				}
				if (!lShutdownRequested && !checkPointRequested)
					continue;
				
				if (lShutdownRequested) {
					lCheckPointRequested = true;
					reporter.log("Executive shutdown commences");
				}
	
			/*
			* Notify asynchronous agents and services
			*/
				
				if (lCheckPointRequested || lShutdownRequested) {
					update(null, ExecGlobal.START_GC);
					boolean aStopIssued = false;
					
					Enumeration asyncs = asynchronousAgentList.elements();
					while (asyncs.hasMoreElements()){
						try {
							if (lShutdownRequested) {
								((Module)asyncs.nextElement()).stopService();
								aStopIssued = true;
							}
							else {
								((Module)asyncs.nextElement()).pauseService();
							}
						}
						catch (Exception e) {
							reporter.reportException(e);
						}
					}
					if (aStopIssued) {
						reporter.log("Waiting for asynchronous processes to wind down");
						try {
							Thread.sleep(1071); // allow some time
						}
						catch (InterruptedException e) {}
					}
				}
			/*
			* 
			*/
				if (lShutdownRequested) {
					setState(SHUTTING_DOWN);
					serve.killWebService();
					try {
						Thread.sleep(4530);
					}
					catch (InterruptedException e) {}
				}
				else if (lCheckPointRequested) {
					setState(IDLE);
				}
				
			/*
			* Checkpoint and
			* notify async agents to resume if we are not shutting down
			*/
				if (lCheckPointRequested) {
					reporter.log("checkpoint begin");
					saveExecState();
					if (!lShutdownRequested) {
						Enumeration asyncs = asynchronousAgentList.elements();
						while (asyncs.hasMoreElements()){
							try {
								((AsynchronousAgent)asyncs.nextElement()).resumeService();
							}
							catch (Exception e) {
								reporter.reportException(e);
							}
						}
					}
					reporter.log("checkpoint end");
				}

			/*
			* clean up products
			*/
				if (lShutdownRequested) {
					
					reporter.log("Expired file reference sweep...");
					sweeper.markUsedFileRefs();
					FileRefList.purgeAll(2); // delete files that we have not marked in two passes. 
					reporter.log("Done expired file reference sweep");

					reporter.log("Working dir(s) sweep");
					FileRefList.doSweep(g_getWorkingDir());
					reporter.log("Done working dir(s) sweep");
				
				/*
				* the following items all stop automatically when
				* exec goes into SHUTTING_DOWN (except for lingering
				* threads that may have been in the middle of I/O)
				*
				*	sweeper
				*	jugglerScheduler
				*	PIMExecuter
				*	DIMExecuter
				*/

					jugglerList = null;
					PIMList = null;
					DIMList = null;
					Runtime.getRuntime().gc();
					reporter.log("Executive shutdown complete");
					
					try {
						Thread.sleep(1030);
					}
					catch (InterruptedException e) {}
					checkPointThread = null;
					Runtime.getRuntime().exit(1);
				}
			checkPointRequested = false;
			shutdownRequested = false;
			}
		}
		catch (Exception e) {
			reporter.reportException(e);
		}
	}

/**
* update list of available and enabled debug keys
*/
	protected void updateDebugKeyList() {
		setPropertyValues(AVAILABLE_DEBUG_KEYS, new ValueList(Alarm.getAvailableDebugKeyList()));
		setPropertyValues(DEBUG_KEY, new ValueList(Alarm.getEnabledDebugKeyList()));
		
	}
/**
* change exec state
*/
	final void setState(String state) {
		if (execState != state) {
			execState = state;
			execOptions.setProperty(STATE, state);
			update(null, state);	
		}
	}
/**
* Get the agent type of an agent.
* 
* @param id of requested agent
* @return null if not found
*/
	public String getAgentsKind(Agent agent){
		return sourceSinkInfo.getNameFromJavaClass(agent.getClass().getName());
	}

/**
* Looks up a source based on its kind and id
* @param kind  either "juggler" or "PIM" (unused as of iPAM build 31)
* @param id must match what you get when you do Source.getId()
*/
	public Source getActiveSourceById(String kind, String id) {
		Source source = null;
		
		source =  (Source) PIMList.get(id);
		if (source != null)
			return source;
			
		source =  (Source) jugglerList.get(id);
		if (source != null)
			return source;
/* this does not look like it should be here... where'd it come from
		Object generic = sourceSinkInfo.getUnusedInstanceFromCommonName(kind);
		if (generic == null)
			return null;
		if (generic instanceof PIM)
			return (Source)PIMList.get(id);
		if (generic instanceof Juggler)
			return (Source)jugglerList.get(id);
*/
		return source;
	}
/**
* Looks up a sink based on its kind and id
*
* @param kind  either "juggler" or "DIM" (unused as of iPAM build 31)
* @param id must match what you get when you do Source.getId()
*/
	public Sink getActiveSinkById(String kind, String id) {
		Sink sink = null;
		
		sink = (Sink) jugglerList.get(id);
		if (sink != null)
			return sink;

		sink = (Sink) DIMList.get(id);
		if (sink != null)
			return sink;

/* this does not look like it should be here... where'd it come from

		Object generic = sourceSinkInfo.getUnusedInstanceFromCommonName(kind);
		if (generic == null)
			return null;
		if (generic instanceof DIM)
			return (Sink)DIMList.get(id);
		if (generic instanceof Juggler)
			return (Sink)jugglerList.get(id);
*/
		
		return sink;
	}
/**
* Looks up item in PIM, DIM, Daemon, and Juggler List
*/
	public Agent getActiveAgent(String id) {
		if (id.equals("exec"))
			return agentFace;
			
		for (int i = 0; i<moduleList.length; i++) {
			Object item = moduleList[i].get(id);
			if (item != null)
				return (Agent) item;
		}
		
		return null;
	}
/**
* Search for the specified Agent and generate
* an IntegrityException if it can't be found
*/
	protected Agent findAgentWithIntegrity(String id) throws IntegrityException{
		try {
			Agent item = (Agent) getActiveAgent(id);
			if (item == null) {
				throw new IntegrityException("Can't locate agent ["+id+"]");
			}
			return item;
		}
		catch (ClassCastException e) {
			throw new IntegrityException("Agent ["+id+"] is not Agent");
		}
	}

/**
* Search for the specified Source and generate
* an IntegrityException if it can't be found
*/
	protected ConnectedAgent findSourceWithIntegrity(String id) throws IntegrityException{
		try {
			Agent source = getActiveAgent(id);
			if (source == null) {
				throw new IntegrityException("Can't locate agent ["+id+"]");
			}
			if (!(source instanceof Source)) {
				throw new IntegrityException("Agent ["+id+"] is not a product source");
			}
			return (ConnectedAgent)source;
		}
		catch (ClassCastException e) {
			throw new IntegrityException("Agent ["+id+"] is of unexpected type.");
		}
	}

/**
* Search for the specified Sink and generate
* an IntegrityException if it can't be found
*/
	protected ConnectedAgent findSinkWithIntegrity(String id) throws IntegrityException{
		try {
			Agent sink =  getActiveAgent(id);
			if (sink == null) {
				throw new IntegrityException("Can't locate agent ["+id+"]");
			}
			if (!(sink instanceof Sink)) {
				throw new IntegrityException("Agent ["+id+"] is not a product sink");
			}
			return (ConnectedAgent) sink;
		}
		catch (ClassCastException e) {
			throw new IntegrityException("Agent ["+id+"] is of unexpected type.");
		}
	}
/**
* Set the attributes from those in a hashtable
*/
	public void setAttributesFromHashtable(Hashtable hash) {
		
		for (Enumeration e = hash.keys() ; e.hasMoreElements() ;) {
		    String attribute = (String)e.nextElement();
			
			Object val = hash.get(attribute);
			if (val == null) {
				setProperty(attribute, "");
			}
			else if (val instanceof ValueList) {
				setPropertyValues(attribute, (ValueList)val);
			}
			else {
				String sval = val.toString();
				if (sval.equals(PropertySet.DELETED)) {
					removeProperty(attribute);
				}
				else {
					setProperty(attribute, sval);
				}
			}
		}
	}

/**
* Copy the property and value potentially with a different name
*/
	final public void copyPropertyFromAs(String fromName, HasProperties fromOther, String newName){
		int dim = fromOther.getCurrentDimension(fromName);

		if (dim == 0)
			removeProperty(fromName);
		else if (dim == 1)
			setProperty(newName, fromOther.getProperty(fromName));
		else
			setPropertyValues(newName, fromOther.getPropertyValues(fromName));
	}

/**
* set alarm frame title
*/
	protected void setAlarmFrameTitle(String title) {
		if (alarmFrame != null) {
			alarmFrame.setTitle(title);
		}
	}
/**
* Set pim thread count
*/
	private void setPIMThreadCount(int n) {
		if (n < 1)
			n = 1;
		if (n > 100)
			n = 100;
		execOptions.setProperty(PIM_THREAD_COUNT, Integer.toString(n));
		PIMExecuter.setDesiredThreadCount(n);
	}
/**
* Set dim thread count
*/
	private void setDIMThreadCount(int n) {
		if (n < 1)
			n = 1;
		if (n > 100)
			n = 100;
		execOptions.setProperty(DIM_THREAD_COUNT, Integer.toString(n));
		DIMExecuter.setDesiredThreadCount(n);
	}
/**
* Set dim thread count
*/
	private void setConcurrencyLimit(int n) {
		if (n < 1)
			n = 1;
		if (n > 100)
			n = 100;
		execOptions.setProperty(CONCURRENCY_LIMIT, Integer.toString(n));
		DIMExecuter.setConcurrencyLimit(n);
	}

/**
* Checkpoint writes out the current configuration but not necessarily right away.
*/
	public void checkPoint() {
		reporter.log("checkpoint requested");
		checkPointRequested = true;
	}
	
	protected void saveExecState() {
		String holdState = getState();
		setState(START_CHECKPOINT);
		updateDebugKeyList();
	/*
	* clean up our memory of PropertySet used ids
	*/
//		PropertySet.forgetUsedIds();
	/*
	* Create backup of checkpoint file
	*/
		
		File ckpt = getCheckPointFile();
		File ckptOld = new File(getCheckPointDir(), ckpt.getName()+".old");
		if (ckpt.exists()) {
			ckptOld.delete();
			ckpt.renameTo(ckptOld);
		}
		FileOutputStream fout = null;
		PDIFWriter pout = null;
		BufferedOutputStream bout = null;
		PrintWriter prodOut = null;

	/*
	* wait for queues to complete
	*/
		PIMExecuter.waitTillIdle();
		DIMExecuter.waitTillIdle();

	/*
	* save state
	*/
		boolean firstSucceeded = false;
		try {

			fout = new FileOutputStream(ckpt);
			bout = new BufferedOutputStream(fout, 1024);
			pout = new PDIFWriter(new PrintWriter(bout),"CDA", "ipam");
			prodOut = new PrintWriter(
				new FileOutputStream(
				new File(ckpt.getParent(), makeProductStoreName(ckpt.getName()))));			

			PAMSave pamSave = new PAMSave(this, execOptions, sourceSinkInfo, listOfThingsWithoutProduct, listOfThingsWithProduct);
			pamSave.setSuppressRedundantAttributes(true);
			pamSave.save(pout, prodOut);			
			pout.close();
			prodOut.close();

			PAMDate now = new PAMDate();
			execOptions.setProperty(LAST_CHECKPOINT_LONG, Long.toString(now.getTime()));
			execOptions.setProperty(LAST_CHECKPOINT_DATE, now.toString());
			execOptions.setProperty(ACTIVE_FILEREF_COUNT, Integer.toString(FileRefList.getLivingFileCount()));
			
			firstSucceeded = true;
		}
		catch (Exception e) {
			reporter.reportException(e);
			if (prodOut != null) {
				prodOut.close();
			}
			if (pout != null) {
				pout.close();
			}
			if (ckptOld.exists()) {
				ckpt.delete();
				ckptOld.renameTo(ckpt);
			}
		}

	/*
	* make a second copy based on day, but only once per day.
	*/
		if (firstSucceeded) {
			int day =  Calendar.getInstance().get(Calendar.DAY_OF_MONTH);
			if (day != lastCheckpointDay){
				saveExecState2(day);
				lastCheckpointDay = day;
			}
		}		
		
		update(null, END_CHECKPOINT);
		setState(holdState);
	}

	protected void saveExecState2(int theDay) {
	/*
	* Create backup of checkpoint file
	*/
		File existingCkpt = getCheckPointFile();
		File ckpt = new File(getCheckPointDir(), existingCkpt.getName()+".day"+Integer.toString(theDay));
		if (ckpt.exists()) {
			ckpt.delete();
		}
		try {
			StreamCopy.copyFile(existingCkpt, ckpt);
		}
		catch (IOException e) {
			reporter.reportException(e);
		}
	}

/**
* make the name of the product store file
*/
	protected String makeProductStoreName(String configName) {
		int ind = configName.lastIndexOf('.');
		if (ind >= 0) {
			configName = configName.substring(0,ind);
		}
		return configName+".ps";
	}
	
/**
* get notification of a change
*/
	public void update(Observable o, Object arg) {
		setChanged();
		notifyObservers(arg);
	}


/**
* Stop everything
*/
	public void halt() {
		shutdownRequested = true;
	}

	public void finalize() {
		reporter.log("Exec ends");
	}
/**
* Prepare sources for read.  PIMs get queued.  Jugglers get notified
* if they have no schedule of their own. 
*/
	public void executeMySources(ConnectedAgent requestor, Task taskSet) {
		queueDispatcher.executeMySources(requestor, taskSet);
	}
/**
* Notify sinks that there is new info. 
*/
	public void notifyMyClients(ConnectedAgent requestor, Task taskSet) {
		queueDispatcher.notifyMyClients(requestor, taskSet);
	}
/**
* Get the exec id.  These numbers are guaranteed
* to increase as new versions of the exec come out.  Users
* are encouraged to use whichNumber==0
*
* @param whichNumber 0 - get the build number
* @param whichNumber 1 - get the major version
* @param whichNumber 2 - get the minor version
*/
	public int getBuildNumber(int whichNumber){
		return VersionInfo.getBuildNumber(whichNumber);
	}
/**
* Tell the exec that an agents schedule has changed
*/
	public void myScheduleChanged(Schedulable whoChanged){
		if (whoChanged instanceof Juggler || whoChanged instanceof Daemon) {
			jugglerScheduler.scheduleChanged(whoChanged);
		}
	}
/**
* Factory service to make new memento
* @param parentKind of the parent
* @param parentId of the parent
*/
	public Memento makeNewMemento(String parentKind, String parentId) {
		return new PropertySet(parentKind, parentId, reporter);
	}
/**
* The exec does not implement functionality here.  It redirects all
* setProperty() to setPropertyValues();
*/
	final public void setProperty(String attribute, String val){
		setPropertyValues(attribute, new ValueList(val));
	}
/**
* Gets an attribute of the exec.
*
*/
	final public String getProperty(String attribute){
		return execOptions.getProperty(attribute);
	}
/**
* Remove an attribute of the agent.
*/
	final public void removeProperty(String attribute){
		execOptions.removeProperty(attribute);
	}
/**
* Add a property-value list 
*/
	final public void setPropertyValues(String property, ValueList values){
		if (property.equalsIgnoreCase(DEBUG_KEY) || property.equalsIgnoreCase(DEBUG_VERBOSE_KEY)) {
			execOptions.setPropertyValues(DEBUG_KEY, values);
			Enumeration keys = values.elements();
			while (keys.hasMoreElements())
				Alarm.setDebug((String)keys.nextElement(), true);
			return;
		}

		try {
			
			if (values.size() == 0) {
				return;
			}
			else {
				String val = values.elementAt(0);
				if (val == null || val.length() == 0)
					return;
				if ( property.equalsIgnoreCase(PIM_THREAD_COUNT) ) {
					setPIMThreadCount(Integer.parseInt(val));
				}
				else if ( property.equalsIgnoreCase(DIM_THREAD_COUNT) ) {
					setDIMThreadCount(Integer.parseInt(val));
				}
				else if ( property.equalsIgnoreCase(CONCURRENCY_LIMIT) ) {
					setConcurrencyLimit(Integer.parseInt(val));
				}
				else if ( property.equalsIgnoreCase(WORKING_DIR_URL_ROOT_NAME) ) {
				}
				else if ( property.equalsIgnoreCase(SUPPORT_DIR_URL_ROOT_NAME) ) {
				} 
				else if ( property.equalsIgnoreCase(EXEC_ID) ) {
					execOptions.setProperty(EXEC_ID, val);
					setAlarmFrameTitle(val);
				}
				else {
					execOptions.setPropertyValues(property, values);
				}
			}
		}
		catch (NumberFormatException ex) {
			reporter.assert(false, property,"had a bad value, no changes made, value=",values.toString(), null);
		}
	}
/**
* Get multivalues for exec
*/
	final public ValueList getPropertyValues(String property){
		return execOptions.getPropertyValues(property);
	}
/**
* Get the "dictionary" for a structured data entry (reserved for future growth) <p>
* Warning: The dictionary name should be different than the name of any other agent.
* @param dictionaryName the name of this Dictionary object
* @param agent making the request (can't be null)
* @return               the Dictionary object
*/
    public IDictionary getDictionary(String dictionaryName, Agent agent) {
    	if (dictionaryName == null) 
    		throw new IllegalArgumentException("getDictionary(), dictionary name can't be null");
    	if (agent == null) 
    		throw new IllegalArgumentException("getDictionary), agent name can't be null");
    	String agentId = agent.getId();
    	if (agentId == null || agentId.length() == 0) {
    		reporter.assert(false, "Attempt to make a dictionary by an agent with no id, did you create the dictionary in the agents constructor? don't",null,null,null);
	    	}
	    else {
	    	reporter.log("Dictionary\t"+dictionaryName+"\treferenced by agent\t"+agentId);
	    }
        return Dict.makeDict((ExecToEditor)this, dictionaryName);
    }
/**
* get exec state
*/
	final public String getState() {
		return execState;
	}
/**
* Get a reference to the exec as type Agent.  Useful for
* for objects that need to act on the Exec's behalf and want
* to issue log messages on behalf of the exec.  Also useful
* to access the exec's attributes
*/
	public Agent getExecAsAgent(){
		return agentFace;
	}
/**
* A factory method to create report agents
*/
	public Reporter makeReporter(String contextName){
		return new AlarmReporter(contextName);
	}

}




	