/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.executive;

import java.net.URL;
import java.net.HttpURLConnection;
import java.io.*;
import java.util.Properties;
import java.util.Hashtable;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.lang.reflect.Method;

import org.mitre.pam.interfaces.Daemon;
import org.mitre.pam.interfaces.ExecToController;

import org.mitre.pam.interfaces.ScheduleAttributes;
import org.mitre.pam.store.FileRef;
import org.mitre.pam.store.IntegrityException;
import org.mitre.pam.util.alarm.Alarm;
import org.mitre.pam.util.alarm.AlarmFrame;
import org.mitre.pdif.eventParser.BadPDIFException;
import org.mitre.pam.monitor.ToolPicker;
import org.mitre.pam.monitor.SrvBadReq;
import org.mitre.pam.daemon.Dict;

import javax.servlet.*;
import javax.servlet.http.HttpServlet;

import Acme.Serve.Serve;
import Acme.Serve.SampleServlet;
import Acme.Serve.TestServlet;
import Acme.Serve.CgiServlet;
import Acme.Serve.FileServlet;


/**
* Exec layer to implement controller functions.<P>
*
* @author Nazario Irizarry, Jr.
*/
public class Exec06 extends Exec04 implements ExecToController{
	
	private String execStartedString = "";
	ToolPicker tp;
	private boolean inPlugin; // avoid infinite recursion due to programmer error

	public Exec06() {
		super();
	}
/**
* Build the exec and all its parts
*/
	protected void init() {
		execSingleton = this;
		super.init();

		defineModuleAlias("checkpoint", "org.mitre.pam.daemon.CheckpointDaemon");
		defineModuleAlias("logFilter", "org.mitre.pam.daemon.LogFilter");
		defineModuleAlias("exceptionChecker", "org.mitre.pam.daemon.ExceptionChecker");
		defineModuleAlias("remoteAgentEditor", "org.mitre.pam.rmi.editor.EditorDaemon");
		defineModuleAlias("memo", "org.mitre.pam.daemon.Memo");
		defineModuleAlias(Dict.PAM_ALIAS, "org.mitre.pam.daemon.Dict"); 
		defineModuleAlias("deleteExpiredAgents", "org.mitre.pam.daemon.DeleteOnceAgents");

		return;
	}
/**
* Gets the default checkpoint file
*/
	public File getDefaultCheckPointFile(){
		return getCheckPointFile();
	}
	
/**
* Start exec from a given configuration file.
* @param configFile the plain text file with the configuration
* @param useGui false to disable all gui's
*/
	public void startFromConfig (File configFile, String[] cmdLineArgs){
		startFromConfig(configFile, true, cmdLineArgs);
	}

/**
* Restart the system from the config file.
* If the config file is null then don't use input files, but
* do whatever needs to be done to start.  This includes
* defining the proxyHost and proxyPort parameters.
* (except starting the web service).
*
* @param configFile the plain text file with the configuration
* @param useGui false to disable gui's displayed by the exec
* @param cmdLineArgs the command line, passed to dynamically loaded plugins
*/
	public void startFromConfig (File configFile, boolean useGui, String[] cmdLineArgs) {
		
		if (inPlugin)
			return;
		inPlugin = true;

		if (useGui) {
			makeAlarmFrame();
		}

		scanAndLoadPlugins1(useGui, cmdLineArgs);
		scanAndLoadPlugins2(useGui, cmdLineArgs);
		
		if (configFile == null) {
			execStartedString = "(no configuration file)";
			setProperty(EXEC_STARTED_FROM_FILE, "");
		}
		else {
			execStartedString = "from "+configFile.toString();
			setProperty(EXEC_STARTED_FROM_FILE, configFile.toString());
			reporter.log("Exec starting "+execStartedString);

			Reader configStream=null;
			try {
				File ssInfoFile = new File(getSupportDir(), "sourcesinkinfo.txt");
				if (ssInfoFile.exists())
					sourceSinkInfo.load(new FileReader(ssInfoFile));
	
			/*
			* validate the input stream
			*/
				configStream = new BufferedReader(new FileReader(configFile), 500);				
				PAMLoad ps = new PAMLoad(this);
				ps.validatePDIF(configStream);
				configStream.close();
			/*
			* load the input stream
			*/
				configStream = new BufferedReader(new FileReader(configFile), 500);				
				ps.load(configStream);
				configStream.close();
				configStream = null;
				
			/*
			* restore products
			*/
				ps.loadSavedProduct(new File(configFile.getParent(), makeProductStoreName(configFile.getName())));			
			}
			catch (Exception e){
				reporter.reportException(e);
			}
			finally {
				if (configStream != null)
					try { configStream.close(); } catch (IOException ignore) {}
				inPlugin = false;
			}
			inPlugin = false;
		}
	/*
	* Define proxyHost and proxyPort
	*/
		String proxyHost = getProperty(PROXY_HOST);
		String proxyPort = getProperty(PROXY_PORT);
		if (proxyHost != null && proxyHost.length() > 0) {
			Properties p = System.getProperties();
			p.put(PROXY_HOST,proxyHost);
			p.put(PROXY_PORT,proxyPort);
			p.put("proxySet","true");
			reporter.log("Using proxyHost="+proxyHost+",  proxyPort="+proxyPort);
		}

		reporter.log("Exec started "+execStartedString);

		scanAndLoadPlugins3(useGui, cmdLineArgs);
		scanAndLoadPlugins4(useGui, cmdLineArgs);
		scanAndLoadPlugins5(useGui, cmdLineArgs);
		classList.setSize(0);
		checkPointThread.start();
//		setState(WORKING); // not needed because checkPointThread does this first thing

	}

/*
* Identify plugins and load them
*/
	private Vector classList = new Vector(10);
	private static Class iPAMPluginClass;
/*
* Executed before the configuration file is read
*/
	private void scanAndLoadPlugins1(boolean useGui, String[] cmdLineArgs) {
		Properties sysProps = System.getProperties();
		StringTokenizer st = new StringTokenizer(sysProps.getProperty("java.class.path",""), File.pathSeparator);
	/*
	* Collect list of zip or jar files (but only the base name, not the directory or extension)
	*/
		while (st.hasMoreElements()) {
			String oneFile = st.nextToken();
			String lowerFile = oneFile.toLowerCase();
			int tail = lowerFile.indexOf(".jar");
			int head = lowerFile.lastIndexOf(File.separator);
			if (tail == -1)
				tail = lowerFile.indexOf(".zip");
			if (tail >= 0 && head < tail) {
				String classRoot = oneFile.substring(head+1,tail);
				if (classList.contains(classRoot))
					continue;
				try {
					Class pluginClass = Class.forName(classRoot);
					if (validPlugin(pluginClass)) {
						classList.addElement(pluginClass);
						activatePlugin(pluginClass, useGui, 1, cmdLineArgs, new File(oneFile));
					}
				}
				catch ( ClassNotFoundException ignore ) {}
			}
		}
	}

	private void scanAndLoadPlugins2(boolean useGui, String[] cmdLineArgs) {
		Enumeration e = classList.elements();
		while (e.hasMoreElements()) {
			activatePlugin((Class) e.nextElement(), useGui, 2, cmdLineArgs, null);
		}
	}

	private void scanAndLoadPlugins3(boolean useGui, String[] cmdLineArgs) {
		
		Enumeration e = classList.elements();
		while (e.hasMoreElements()) {
			activatePlugin((Class) e.nextElement(), useGui, 3, cmdLineArgs, null);
		}
	}

	private void scanAndLoadPlugins4(boolean useGui, String[] cmdLineArgs) {		
		Enumeration e = classList.elements();
		while (e.hasMoreElements()) {
			activatePlugin((Class) e.nextElement(), useGui, 4, cmdLineArgs, null);
		}
	}

	private void scanAndLoadPlugins5(boolean useGui, String[] cmdLineArgs) {		
		Enumeration e = classList.elements();
		while (e.hasMoreElements()) {
			activatePlugin((Class) e.nextElement(), useGui, 5, cmdLineArgs, null);
		}
	}

/*
* @return true if the specified Class implements org.mitre.pam.interfaces.IPamPlugin
*/
	private boolean validPlugin(Class item) {
		if (iPAMPluginClass == null) {
			try {
				iPAMPluginClass = Class.forName("org.mitre.pam.interfaces.IPamPlugin");
			} catch (ClassNotFoundException e) {
				reporter.reportException(e);
				return false;
			}
		}
		
		Class[] list = item.getInterfaces();
		for (int i = 0; i< list.length; i++) {
			if (list[i] == iPAMPluginClass) {
				return true;
			}
		}
		return false;
	}
	
	private static final Class STRING_ARRAY_CLASS = (new String[0]).getClass();
	private static final Class FILE_CLASS = (new File("")).getClass();
	
	private void activatePlugin(Class plugin, boolean useGui, int phase, String[] cmdLineArgs, File fileArgument) {
		Class[] noArgClass = new Class[0];
		Class[] oneArgClass = new Class[1];
		Class[] twoArgClass = new Class[2];
		Class[] fileClass = new Class[2];
		Object[] noArg = new Object[0];
		Object[] oneArg = new Object[1];
		Object[] twoArg = new Object[2];
		Object[] fileArgs = new Object[2];
		try {
		/*
		* Build one arg call
		*/
			oneArg[0] = this;
			oneArgClass[0] = Class.forName("org.mitre.pam.interfaces.ExecToController");
		/*
		* Build two arg call, exec and string array
		*/
			twoArgClass[0] = oneArgClass[0];
			twoArgClass[1] = STRING_ARRAY_CLASS;
			twoArg[0] = this;
			twoArg[1] = cmdLineArgs;
		/*
		* Build two arg call, exec and File
		*/
			fileClass[0] = oneArgClass[0];
			fileClass[1] = FILE_CLASS;
			fileArgs[0] = this;
			fileArgs[1] = fileArgument; // in case needed below
		}
		catch (ClassNotFoundException e) {
			System.out.println(e);
		}

		try {
			switch (phase) {
				case 1: {
					/*
					* build the version string
					*/
						String version = "";
						try {
							version = (String) plugin.getDeclaredMethod("getImplementationVersion", noArgClass).invoke(null, noArg);
							version = " ("+version+")";
						}
						catch (Exception e) {
						}
						
		
					/*
					* invoke: 	static public boolean isOkToLoad(ExecToController exec, String[] cmdLineArgs)
					*/
						Boolean validLoad = null;
						try {
							validLoad = (Boolean) plugin.getDeclaredMethod("isOkToLoad", twoArgClass).invoke(null, twoArg);
						}
						catch (Exception e) {
							reporter.reportException(e);
							return;
						}
			
					/*
					* echo the version string and tell user that it was not loaded
					* if the isOkToLoad call says that it is not ok
					*/
						if (validLoad.booleanValue()) {
							reporter.log("loading "+plugin.getName()+version);
						} else {
							reporter.log("not loading "+plugin.getName()+version);
							classList.removeElement(plugin);
							return;
						}

						try {
						/*
						* 	invoke: static public void makeSupportDirIfNeeded(ExecToController exec, File zipFile)		
						*/
							plugin.getDeclaredMethod("makeSupportDirIfNeeded", fileClass).invoke(null, fileArgs);
						}
						catch (NoSuchMethodException ignore) {
//						 System.out.println("makeSupdir not found :"+plugin);
						}
						catch (Exception e) {
							reporter.reportException(e);
						}

						break;
				}
				case 2: {
						try {
						/*
						* 	invoke: public static void defineModules(ExecToController exec)		
						*/
							plugin.getDeclaredMethod("defineModules", oneArgClass).invoke(null, oneArg);
						}
						catch (NoSuchMethodException ignore) {
						}
						catch (Exception e) {
							reporter.reportException(e);
						}
						break;
				}
				case 3: {
						if (useGui) {
							try {
							/*
							* 	invoke: static public void runWizards(ExecToController exec)		
							*/
								plugin.getDeclaredMethod("runWizards", oneArgClass).invoke(null, oneArg);
							}
							catch (NoSuchMethodException ignore) {
							}
							catch (Exception e) {
								reporter.reportException(e);
							}
						}
						break;
				}
				case 4: {
						try {
						/*
						* 	invoke: public static void defineServlets(ExecToController exec)		
						*/
							plugin.getDeclaredMethod("defineServlets", oneArgClass).invoke(null, oneArg);
						}
						catch (NoSuchMethodException ignore) {
						}
						catch (Exception e) {
							reporter.reportException(e);
						}
						break;
				}
				case 5: {
						if (useGui) {
							try {
							/*
							* 	invoke: public static void defineGUIs(ExecToController exec)		
							*/
								plugin.getDeclaredMethod("defineGUIs", oneArgClass).invoke(null, oneArg);
							}
							catch (NoSuchMethodException ignore) {
							}
							catch (Exception e) {
								reporter.reportException(e);
							}
						}
						break;
				}
			}
		}
		catch (Exception e) {
			reporter.reportException(e);
		}
	}
/**
* Modifies the runtime environment from a pdif string
* @return null if successfull
* @return error string if not successfull
*/
	public String modifyFromPDIFString (String modString) {
		File modFile = null;
		File execwd = g_getWorkingDir("exec");
		FileOutputStream fout=null;
		try {
			modFile = new File(execwd, FileRef.getNewFileId());
			fout = new FileOutputStream(modFile);
			PrintWriter p = new PrintWriter(fout);
			p.println(modString);
			p.flush();
			if (fout != null)
				fout.close();
		}
		catch (IOException e){
			return e.toString();
		}
		finally {
		}
		
		
		
		try {
			Reader configStream = new FileReader(modFile);				
			PAMLoad ps = new PAMLoad(this);
			ps.validatePDIF(configStream);
			configStream.close();
			configStream = new FileReader(modFile);				
			ps.load(configStream);
			
//			ps.loadSavedProduct(getJugglerProductFile("productStore.txt"));			
		}
		catch (IntegrityException e2){
			return e2.toString();
		}
		catch (BadPDIFException e){
			return e.toString();
		}
		catch (FileNotFoundException fnf) {
			return fnf.toString();
		}
		catch (IOException e){
			return e.toString();
		}
		finally {
		if (modFile != null)
			modFile.delete();
		}
		
		return null;
	}

/**
* Looks through the log file to see if there are
* any new exceptions.  An exception is considered new if its time is recent
* based on the milliSecondsBack. Exceptions that are new are recorded in file whose handle
* is also supplied as an argument.
*/
	public void filterExceptions(long milliSecondsBack, File file){
		Alarm.filterExceptions( milliSecondsBack,  file);
	}
/**
* Performs two layer filtering of the log file in order to
* keep the file manageable.  Two intervals (in units of days)
* are provided as arguments in order to perform
* the filtering.  Any entry whose date is before the eraseAllBeforeDays interval is disgarded, and any
* entry that is before the eraseAllButExceptionsBeforeDays interval is also disgarded UNLESS the entry
* is for an exception or assertion.  All entries that are made after the eraseAllButException
* date are saved.
*/
	public void filterLog(int eraseAllBeforeDays, int eraseAllButExceptionsBeforeDays){
		Alarm.filterLog( eraseAllBeforeDays,  eraseAllButExceptionsBeforeDays);
	}
/**
* Start web service
*/
	private void initWebService(){
		serve = new Serve( getServerPortNumber(), reporter );


		Servlet ts = new TestServlet();
		serve.addServlet( "/SampleServlet", new SampleServlet() );
		serve.addServlet( "/TestServlet", ts );
		serve.addServlet( "/TestServlet/*", ts );
//		serve.addServlet( "*", new FileServlet() ); // moved to listenForWebConnections()
	}
/**
* Define a URL path for a servlet
*/
	public void addServlet(String path, HttpServlet servlet){
		try {
			if (serve == null)
				initWebService();
			serve.addServlet(path, servlet);
		}
		catch (Exception e) {
			reporter.reportException(e);
		}
	}
/**
* Start listening for connections web service
* @exception IOException if the port is in use or if there is some other problem
*/
	public void listenForWebConnections() throws IOException{
		addServlet( "*", new SrvBadReq() ); 
		String protnhost = execOptions.getProperty(PROTOCOL_AND_HOST);
		if (protnhost == null || protnhost.length() == 0) {
			try {
				execOptions.setProperty(PROTOCOL_AND_HOST, "http://"+InetAddress.getLocalHost().getHostName());
				execOptions.setProperty(PEER_PROTOCOL_HOST_PORT, "http://"+InetAddress.getLocalHost().getHostName()+":"+DEFAULT_PORT);
			}
			catch (Exception e) {
				try {
					String addr = InetAddress.getLocalHost().getHostAddress();
					execOptions.setProperty(PROTOCOL_AND_HOST, "http://"+addr);
					execOptions.setProperty(PEER_PROTOCOL_HOST_PORT, "http://"+InetAddress.getLocalHost().getHostAddress()+":"+DEFAULT_PORT);
					reporter.log("Can't identify symbolic host name, will use numeric net host address "+addr);
				}
				catch (UnknownHostException e2) {
					reporter.log("Problem accessing localhost TCP/IP information");
					reporter.reportException(e2);
				}
			}
		}
		if (serve == null)
			initWebService();
		serve.startAcceptingConnections(getServerPortNumber());
		
	/*
	* Check to see that the supportDir alias is working
	*/
		URL testurl = null;
		boolean success = false;
		try {
			testurl = new URL(getPeerProtocolHostAndPort()+getSupportDirURLRoot()+"/help/doNotDelete.txt");
			reporter.log("Validating access to support directory via the web service");
			HttpURLConnection connection = (HttpURLConnection) testurl.openConnection();
			int code = connection.getResponseCode();
			if (code >= 200 && code < 300)
				success = true;
		}
		catch (Exception e) {
			reporter.reportException(e);
		}
		
		if (success) {
			reporter.log("Web service access ok.");
		}
		else {
			String supportDir = getProperty(SUPPORT_DIR_URL_ROOT_NAME);
			String workingDir = getProperty(WORKING_DIR_URL_ROOT_NAME);
			reporter.warning("Can not access "+testurl+
				"\n\t"+supportDir+" and "+workingDir+" should both be accessible via the web server" );
		}
	}
/**
* Add an admin tool
* @param toolClassName full java class name for a tool that implements org.mitre.pam.interfaces.AdminTool
* @param displayName the short string to be used on the tool picker's gui
*/
	public void addAdminTool(String toolClassName, String displayString){
		makeToolPicker();
		tp.addTool(toolClassName, displayString);
	}
/**
* Show or hide the tool picker
*/
	public void setToolPickerVisibility(boolean visible){
		if (visible) {
			makeToolPicker();
			tp.setVisible(true);
		}
		else {
			if (tp == null)
				return;
			tp.setVisible(false);
		}
	}
	private void makeToolPicker() {
		if (tp == null) {
			tp = new ToolPicker();
			tp.setExec(this);
			tp.setVersionString(getVersionString());
			if (alarmFrame != null) {
				tp.addAdminToolRef(alarmFrame,"Log"); // 0
			}
		}
		tp.setCommonTitle(getExecAsAgent().getStringProperty("id",""));
		tp.updateTitle();
	}
	private String getVersionString() {
						
		int major = getBuildNumber(1);
		int minor = getBuildNumber(2);
		int build = getBuildNumber(0);
		String versionString = "iPAM "+Integer.toString(major)+"."+Integer.toString(minor)+"."+Integer.toString(build);
		return versionString;
	}
/**
* Trap this so we can set the gui state before the save
*/
	protected void saveExecState() {
		if (tp != null) 
			tp.saveStateTo();
		super.saveExecState();
	}
/**
* make alarm frame
*/
	private void makeAlarmFrame() {
		if (alarmFrame == null) {
			String name = getProperty("id");
			if (name == null || name.length() == 0) {
				String host = getProtocolHostNoPort();
				if (host.length() > 8)
					name = getProtocolHostNoPort().substring(7);
				else
					name = "<pam>";
			}
			alarmFrame = new AlarmFrame();
			setAlarmFrameTitle(name);
			alarmFrame.show();
		}
	}
/**
* Checks one directory to see if it exsits as an iPAM support directory and has
* a minimum number of files.
*
* @param dirName the simple name of the directory to check
* @param count the minimum number of files that must exist
* @exception Exception if there is something wrong -- the exception message explains why
*/
	public void isAValidSupportDir( String dirName, int count) throws Exception {
		File supportDirFile = ExecGlobal.getSupportDir(dirName);
		String[] list = supportDirFile.list();
		int fileCount = (list != null) ? list.length : 0;
		if (fileCount == 0)
			throw new Exception("Can't run - support directory "+supportDirFile.getAbsolutePath()+" is empty (check your default directory).");
		if (fileCount < count)
			throw new Exception("Can't run - support directory "+supportDirFile.getAbsolutePath()+" had too few support files, was expecting at least "+count);
		return;
	}
}