/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.executive;

import java.util.Enumeration;

import org.mitre.pam.interfaces.ConnectedAgent;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.Task;
import org.mitre.pam.interfaces.Source;
import org.mitre.pam.interfaces.Doer;
import org.mitre.pam.interfaces.DIM;
import org.mitre.pam.interfaces.Handler;
import org.mitre.pam.interfaces.ManagesOwnInputQueue;



class QueueDispatcher implements FlowDispatcher {

	Executer inputQueue;
	Executer outputQueue;
	Reporter reporter;
	
	public QueueDispatcher(Executer inputQueue, Executer outputQueue, Reporter reporter) {
		this.inputQueue = inputQueue;
		this.outputQueue = outputQueue;
		this.reporter = reporter;
	}

/**
* Prepare sources for read.  PIMs get queued.  Handlers get notified
* if they have no schedule of their own. 
*/
	public void executeMySources(ConnectedAgent requestor, Task taskSet) {
		reporter.debugMsg("QDispatcher.executeMySources begin,\t",taskSet.getTasker().toString(),"task",taskSet, ExecGlobal.TRACE_MSG_FLOW);
		
		Enumeration list = taskSet.getTaskEnumeration();
		if (!list.hasMoreElements()) {
			taskSet.signalDone();
		}
		else {
			while(list.hasMoreElements()) {
				Task aTask = (Task) list.nextElement();
				Doer doer = aTask.getDoer();
			/*
			* Handlers get executed immediately unless they are busy.
			* If they are busy they get queued on the input queue
			*/
				if (doer instanceof Handler && doer.isIdle()) {
					reporter.debugMsg("QDispatcher.executeMySources aTask.perform(),\t",aTask,null,null, ExecGlobal.TRACE_MSG_FLOW);
					aTask.perform();
				}
				else if (doer instanceof DIM) {
					reporter.assert(false,"Unexpected doer:",doer,"task:",aTask);
					aTask.signalDone();
				}
				else {
					reporter.debugMsg("QDispatcher.executeMySources adding to inputQueue,\t",aTask,null,null, ExecGlobal.TRACE_MSG_FLOW);
					inputQueue.add(aTask);
				}
			}
		}
		
		reporter.debugMsg("QDispatcher.executeMySources end,\t",taskSet.getTasker().toString(),null,null, ExecGlobal.TRACE_MSG_FLOW);
	}
/**
* Notify DIMs or other sinks that there is new info. 
*/
	public void notifyMyClients(ConnectedAgent requestor, Task taskSet) {	
		reporter.debugMsg("QDispatcher.notifyMyClients begin,\t",taskSet.getTasker(),"task",taskSet, ExecGlobal.TRACE_MSG_FLOW);
		
		Enumeration list = taskSet.getTaskEnumeration();
		if (!list.hasMoreElements()) {
			taskSet.signalDone();
		}
		else {
			while(list.hasMoreElements()) {
				Task aTask = (Task) list.nextElement();
				Doer doer = aTask.getDoer();
				if (doer instanceof ManagesOwnInputQueue ) {
					reporter.debugMsg("QDispatcher.notifyMyClients aTask.perform(),\t",aTask,null,null, ExecGlobal.TRACE_MSG_FLOW);
					aTask.perform();
				}
				else if (doer instanceof Source && doer.getTaskingPermission(aTask)) {
					reporter.debugMsg("QDispatcher.notifyMyClients aTask.perform(),\t",aTask,null,null, ExecGlobal.TRACE_MSG_FLOW);
					aTask.perform();
				}
				else {
					reporter.debugMsg("QDispatcher.notifyMyClients adding to outputQueue,\t",aTask,null,null, ExecGlobal.TRACE_MSG_FLOW);
					outputQueue.add(aTask); // DIMs and Handlers that are busy get queued
				}
			}
		}
		
		reporter.debugMsg("QDispatcher.notifyMyClients end,\t",taskSet.getTasker(),null,null, ExecGlobal.TRACE_MSG_FLOW);
	}

}