/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.executive;
import java.util.Vector;
import java.util.Enumeration;
import org.mitre.pam.interfaces.Doer;
import org.mitre.pam.interfaces.Task;
import org.mitre.pam.interfaces.Tasker;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.Sink;
import org.mitre.pam.executive.ExecGlobal;


/**
* The set finisher detects when all items of an executer's set are complete and then
* sends a message to its tasker when they are all complete. 
*
* @author Nazario Irizarry, Jr.
*/
abstract class SetFinisher extends Command implements Tasker {
	protected Vector taskList = new Vector(10);
	
	public SetFinisher(Tasker tasker) {
		super(tasker, null, "finisher");
	}
/**
* This object "executes" its task by looking
* at the doer of the task and keeping track of all the
* doers that have completed.  When all the doers are done then
* this object does its thing which is to notify of set completion - signalDone().
*/
	public void taskDone(Task task){
	/*
	* when done, the arg is the PIM that is complete
	*/
		reporter.assert(taskList.contains(task), "SetFinisher.taskDone did not contain ",task.toString()," in taskList",null);
		reporter.debugMsg(this,", ",task,", is done.",ExecGlobal.TRACE_MSG_FLOW);
		taskList.removeElement(task);
		if (taskList.size() <= 0) {
			signalDone();
		}
	}

/**
* Add a an item to the set we are tracking
*/
	protected void waitFor(Task item) {
		taskList.addElement(item);
		reporter.debugMsg(this,"waitFor: ",item,null,ExecGlobal.TRACE_MSG_FLOW);
	}
/**
* Post tasks to executer
*/
	public void postTasksToExecuter(Executer executer) {
		if (taskList.size() == 0) {
			signalDone();
			return;
		}
		for (int i = 0; i< taskList.size(); i++) {
			executer.add((Task) taskList.elementAt(i));
		}
	}
/**
* Signal our tasker that we are done
*/
	public void signalDone(){
		/*
		* clear the wait taskList in case someone other than ourselves
		* calls signalDone()
		*/
		taskList.setSize(0);
		super.signalDone();
	}
/**
* the object as a string
*/
	public String toString(){
		return "SetFinisher"+Integer.toString(idNumber)+" "+command;
	}
}

