/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.executive;

import java.util.Observer;
import org.mitre.pam.util.PAMDate;
import org.mitre.pam.interfaces.Doer;
import org.mitre.pam.interfaces.ScheduledCommand;
import org.mitre.pam.interfaces.Tasker;
import org.mitre.pam.interfaces.Schedulable;
import org.mitre.pam.interfaces.ExecCallbacks;

/**
* A basic unscheduled command for the exec. 
*
* @author Nazario Irizarry, Jr.
*/
public class TimedCommand extends Command implements ScheduledCommand  {
	
	protected long time;

	public TimedCommand (long time, Tasker tasker, Schedulable doer, String command) {
		super(tasker, doer, command);
		this.time = time;
	}

	public TimedCommand (Tasker tasker, Schedulable doer, String command) {
		super(tasker, doer, command);
		this.time = 0;
	}

/**
* @return time when exection is desired
*/
	public long getTime(){
		return time;
	}
/**
* @return time do execute the command in millis
*/
	public void setTime(long time){
		if (this.time != time) {
			this.time = time;
			notifyExec();
		}
		else {
			this.time = time;
		}
	}
/**
* Tell the exec that the time has changed.
*/
	private void notifyExec() {
		exec.myScheduleChanged((Schedulable)getDoer());
	}
	
/**
* delay existing time in seconds
*/
	public void delayFromNow(int timeSeconds) {
		time = System.currentTimeMillis() + 1000*timeSeconds;
		notifyExec();
	}
/**
* a string form suitable for displaying on admin lists
*/
	public String toString(){
		PAMDate date = new PAMDate(time);
		return doer.getId()+", "+command+" @ "+date.toString();
	}

}
