package org.mitre.pam.getter;

import org.mitre.pam.interfaces.ContainerProduct;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ProductFilter;
import org.mitre.pam.interfaces.Reporter;

import org.mitre.pam.product.view.DictionaryProduct;

/**
* This filter removes invalid DictionaryProducts.  It should be used after a
* product is deserialized.
*
* @author John R. Fantry
*/
public class DictProdFilter implements ProductFilter {

  Reporter reporter;

  public DictProdFilter(Reporter reporter) {
    this.reporter = reporter;
  }

  /**
  * @param parent the container product for the product to test
  * @param productToTest the product to test
  * @return true based on the product test, for example to continue iteration (Product.apply())
  */
  public boolean filter(ContainerProduct parent, Product productToTest) {
    if(parent == null || productToTest == null) {
      return true;
    }

    if(productToTest instanceof DictionaryProduct) {
      if(!((DictionaryProduct)productToTest).isValid()) {
        reporter.warning("Received invalid DictionaryProduct, associated dictionary is unavailable.");
        parent.removeElement(productToTest);
      }
    }

    return true;
  }
}