/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.interfaces;

import java.io.File;
import java.util.Hashtable;
import java.util.Enumeration;
import org.mitre.pdif.eventParser.ValueList;

/**
* Defines an active agent. <p>
*
* @author Nazario Irizarry, Jr.
*/
public interface Agent extends Nameable, Summary, Doer, Tasker, HasProperties {
/**
* prefix used for metadata on agent attributes
*/
	public final static String META_ATTRIBUTE_PREFIX = "meta_attr_";
/**
* denotes the type of value in the meta string that describes a particular attribute
*/
	public final static String META_ATTRIBUTE_VALUE_TYPE = "type";
/**
* default value
*/
	public final static String META_ATTRIBUTE_VALUE_DEFAULT = "default";
/**
* denotes integer types
*/
	public final static String META_ATTRIBUTE_TYPE_INTEGER = "int";
/**
* denotes integer types
*/
	public final static String META_ATTRIBUTE_TYPE_LONG = "long";
/**
* denotes boolean types
*/
	public final static String META_ATTRIBUTE_TYPE_BOOLEAN = "bool";
/**
* denotes boolean types
*/
	public final static String META_ATTRIBUTE_MULTIVALUED = "multi";
/**
* denotes string types 
*/
	public final static String META_ATTRIBUTE_TYPE_STRING = "str";
/**
* denotes string types 
*/
	public final static String META_ATTRIBUTE_TYPE_DOUBLE = "doub";
/**
* denotes enumeration types 
*/
	public final static String META_ATTRIBUTE_TYPE_ENUMERATION = "enum";
/**
* identifies help string leadin
*/
	public final static String META_ATTRIBUTE_HELP_SHORT = "shortHelp";
/**
* identifies attributes that are passwords
*/
	public final static String META_ATTRIBUTE_TYPE_PASSWORD = "isPwd";
/**
* The type of algorithm used to encode the password
*/
	public final static String META_PASSWORD_ALGORITHM_DEFAULT = "defPwdAlg";
/**
* the list of default values
*/
	public final static String META_ATTRIBUTE_VALUE_DEFAULTS = "defs";
/**
* lead-in for enumerated values
*/
	public final static String META_ATTRIBUTE_ENUMERATED_VALUES = "enumValues";

/**
* If agents have propertyies starting with the META_ATTRIBUTE_ADD_PREFIX
* then its value represents the GUI's popup label to create a new property
* and the default value for the property name is the string following the
* META_ATTRIBUTE_ADD_PREFIX;  e.g., a agent with the property
* (meta_add_prop_prefix_productAttribute_ "Product Attribute") would show
* a popup menu "Add Product Attribute Property ..." and bring up a dialog
* with the default property name as productAttribute_newAttribute4.
*/
        public final static String META_ATTRIBUTE_ADD_PREFIX = "meta_add_prop_prefix_";
/**
* The name of the attribute that holds the description
*/
	public final static String DESCRIPTION = "description";
/**
* The name of the attribute that identifies whether the module is enabled
*/
	public final static String ENABLED = "enabled";
/**
* The agents creator
*/
	public final static String CREATOR = "creator";
/**
* Whether this agent is exposed to the world or just to the creator
*/
	public static String EXPOSURE_VERB = "exposure";
/**
* Whether this agent is exposed to the public or just to the creator
*/
	public static String EXPOSURE_PRIVATE = "private";
/**
* Whether this agent is exposed to the public or just to the creator
*/
	public static String EXPOSURE_PUBLIC = "public";	
/**
* A meta property that associates the agent with a group on gui displays.
* The intent is that a tool such as Bert can use this name
* to group agents in hierarchical menus
*/
	public static String META_GUI_GROUP = "meta_gui_group";
/**
* A meta property identifies agents that should not be saved
* to persistent storage when the system shutdown. The value of
* this meta property must be "false" to cause the agent not to persist.
* If the attribute does not exist it is taken as "true"
*/
	public static String META_PERSIST = "meta_persistentAgent";

/**
* An agent with this attribute will be allowed to take this
* much time to complete without disabling it.  
* Most agents won't even have this attribute and the executive
* may disable them after 5 to 10 minutes if they have been busy
* that long
*/
	public final static String EXTREME_EXECUTION_TIME = "extremeExecutionTimeIsSecs";
/**
* Get the reporter for the Agent
*/
	public Reporter getReporter();
/**
* Enable or disable this agent
*/
	public void setEnabled(boolean enableState);
/**
* @return true if this module is enabled
*/
	public boolean isEnabled();
/**
* Get the working directory
*/
	public File getWorkingDir();
/**
* Set the kind of agent
*/
	public void setKind(String kind);
/**
* This is intended for read only reference to the memento. To set
* a property use setProperty()
*/
	public Memento getMemento ();
/**
* Identifies the persistent product for an item, this
* is the method used by the exec to find out what to save
* on system shutdown.
*/
	public Product getPersistentProduct();
/**
* Identifies the persistent product for an item, this
* is the method used by the exec to restore product during
* system restart
*/
	public void setPersistentProduct(Product p);
/**
* An activator is the thing that gets an agent to do something.  
* The activator may be a schedule.
*/
	public Object getActivator();
}
