/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.interfaces;
/**
* Defines an interface to identify and work with asynchronous agents.
* These agents respond to the external world without the executive
* knowing about it.  Therefore the exec must coordinate with these
* agents other than via the normal doCommand approach.  Each agent must keep
* track of whether it has been started and whether it has been paused.  The
* exec will not keep track of this.
*
* <p> AsynchronousAgents usually have output data. 
* However, when an agent implements AsynchronousAgent
* the exec calls  startService() and stopService()
* when appropriate.  Some DIMs may find this useful to know whether any additional
* thread need to be started.  For this reason, the exec does not assume
* that just because the agent implements AsynchronousAgent that it has output
* data (as seen from the executives point of view).
*
* @author Nazario Irizarry, Jr.
*/


public interface AsynchronousAgent {

/**
* Called whenever we transition from disabled to enabled.
* Will never be called twice in a row without an interceding
* call to disableService().  When this is called you should
* assume that all relevant properties have been defined.  However,
* this does not mean that a key property may not change while
* the service is running.  If that is important a service
* must also monitor the setProperty() calls on the agent.
*/
	public void enableService();

/**
* Called on shutdown or whenever the agent transitions from enabled to disabled.
* Will never be called twice in a row without an interceding
* call to enableService(). Remember that Thread.stop() is deprecated.
*/
	public void disableService();
	
/**
* Called to request that the agent suspend activity if it makes
* sense.  This is done by the exec just before checkpointing.
* However, it is not mandatory to do so.  A default "do-nothing"
* implementation is provided by the Module base class.
*/
	public void pauseService();
	
/**
* Called to request that the agent resume activity if it makes
* sense.  This is done by the exec just before checkpointing.
* However, it is not mandatory to do so.  A default "do-nothing"
* implementation is provided by the Module base class.
*/
	public void resumeService();
}
