/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */

package org.mitre.pam.interfaces;


/**
* A set of generic attributes for reporting on entities that have location. <p>
*
* Related to MTDSTrackProduct
*
* @author Henry C. Callihan, Jr.
*/
public interface BasicProductAttributes {
	/*
	* Altitude/Depth is in feet, do not assume integer or double.  Depth is
	* represented as negative values
	*/
	public final static String ALTITUDE = "altitude";
	/*
	* Bearing is in degrees and is the positional bearing of an acoustic or EW line of bearing
	*/
	public final static String BEARING = "bearing";
	/*
	* Category has values of unknown, air, surface, subsurface, space,
	* land, point, and ew
	*/
	public final static String CATEGORY = "category";
	/*
	* Course is in degrees,if you are range sensitive then you must convert to the desired range
	*/
	public final static String COURSE = "course";
	/*
	* EmergencyType is a first level classification on entities that have position.
	* Link 16 examples are a downed aircraft or a distressed vessel.
	*/
	public final static String EMERGENCY_TYPE = "emergencyType";
	/*
	* ExistenceType has values of real, exercise, simulation,
	* exerciseAndSimulation, drop
	*/
	public final static String EXISTENCE_TYPE = "existenceType";
	/*
	* FixBearingDescriptor is an integer that indicates if the entity is a fix or bearing
	*/
	public final static String FIX_BEARING_DESCRIPTOR = "fixBearingDescriptor";
	/*
	* FriendOrFoe has values of friend, foe, neutral, and unknown
	*/
	public final static String FRIEND_OR_FOE = "friendOrFoe";
	/*
	* FriendOrFoeAmplification has values of pending, assumed, and general
	*/
	public final static String FRIEND_OR_FOE_AMPLIFICATION = "friendOrFoeAmplification";
	/*
	* Latitude is in double signed degrees, where north is positive
	* and south is negative
	*/
	public final static String LATITUDE = "latitude";
	/*
	* Longitude is in double signed degrees, where east is positive
	* and west is negative
	*/
	public final static String LONGITUDE = "longitude";
	/*
	* PointAmplification is a second level classification on entites that have position.
	* Link 16 examples are distressed vessel or corridor tab.  Flight planning examples
	* are turn-short and turn-early.
	*/ 
	public final static String POINT_AMPLIFICATION = "pointAmpification";
	/*
	* PointType is a first level classification on entities that have position.
	* Link 16 examples are a navigation hazard or a reference point.  Flight planning
	* examples are waypoint and egress point.
	*/
	public final static String POINT_TYPE = "pointType";
	/*
	* From is the source of an email msg.  The role of this field
	* is similar to the SourceTrackIdentifier in tracking systems.
	* So we pick the name of the field to be the same.
	*/
	public final static String FROM = "from";
	/*
	* SourceTrackIdentifier for tracking applications is
	* is in the format of AAOOO where A is either a letter or an octal
	* digit and O is an octal digit.  However, its field name is the same
	* as the "from:" field.
	*/
	public final static String SOURCE_TRACK_IDENTIFIER = FROM;
	/*
	* SourceType is a string that defines the source of the data
	*/
	public final static String SOURCE_TYPE = "sourceType";
	/*
	* Speed is in integer knots (nautical miles/hour)
	*/
	public final static String SPEED = "speed";
	/*
	* Time is in double seconds
	*/
	public final static String TIME = "time";
	/*
	* TrackIdentifier is in the format of AAOOO where A is either a letter or an octal
	* digit and O is an octal digit
	*/
	public final static String TRACK_IDENTIFIER = "streamId";
	/*
	* TrackQuality is a double with range of 0 through 1 and indicates to
	* reliability of the positional values, where 1 is most reliable
	*/
	public final static String TRACK_QUALITY = "streamQuality";
	/*
	* XPositionWGS84 is a double that gives the X coordinate of a space track
	*/
	public final static String X_POSITION_WGS84 = "xPositionWGS84";
	/*
	* XVelocityWGS84 is a double that gives the X velocity component of a space track
	*/
	public final static String X_VELOCITY_WGS84 = "xVelocityWGS84";
	/*
	* YPositionWGS84 is a double that gives the Y coordinate of a space track
	*/
	public final static String Y_POSITION_WGS84 = "yPositionWGS84";
	/*
	* YVelocityWGS84 is a double that gives the Y velocity component of a space track
	*/
	public final static String Y_VELOCITY_WGS84 = "yVelocityWGS84";
	/*
	* ZPositionWGS84 is a double that gives the Z coordinate of a space track
	*/
	public final static String Z_POSITION_WGS84 = "zPositionWGS84";
	/*
	* ZVelocityWGS84 is a double that gives the Z velocity component of a space track
	*/
	public final static String Z_VELOCITY_WGS84 = "zVelocityWGS84";
	
	/*
	* The following constants are used by the above fields that have String values
	*/
	public final static String CATEGORY_ACOUSTIC = "acoustic";
	public final static String CATEGORY_AIR = "air";
	public final static String CATEGORY_EMERGENCY_POINT = "emergencyPoint";
	public final static String CATEGORY_EW = "eW";
	public final static String CATEGORY_LAND_POINT = "landPoint";
	public final static String CATEGORY_LAND_TRACK = "landTrack";
	public final static String CATEGORY_POINT = "point";
	public final static String CATEGORY_POINTER = "pointer";
	public final static String CATEGORY_SPACE = "space";
	public final static String CATEGORY_SURFACE = "surface";
	public final static String CATEGORY_SUBSURFACE = "subsurface";
	public final static String CATEGORY_UNKNOWN = "unknown";
	public final static String EXISTENCE_TYPE_DROP = "drop";
	public final static String EXISTENCE_TYPE_EXERCISE = "exercise";
	public final static String EXISTENCE_TYPE_EXERCISE_AND_SIMULATION = "exerciseAndSimulation";
	public final static String EXISTENCE_TYPE_REAL = "real";
	public final static String EXISTENCE_TYPE_SIMULATION = "simulation";
	public final static String FRIEND_OR_FOE_AMPLIFICATION_ASSUMED = "assumed";
	public final static String FRIEND_OR_FOE_AMPLIFICATION_GENERAL = "general";
	public final static String FRIEND_OR_FOE_AMPLIFICATION_PENDING = "pending";
	public final static String FRIEND_OR_FOE_FOE = "foe";
	public final static String FRIEND_OR_FOE_FRIEND = "friend";
	public final static String FRIEND_OR_FOE_NEUTRAL = "neutral";
	public final static String FRIEND_OR_FOE_UNKNOWN = "unknown";
	public final static String SOURCE_TYPE_LINK_16 = "link16";
}