/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.interfaces;

/**
* An abstraction for things that can execute commands. <P>
* @author Nazario Irizarry, Jr.
*/
public interface Doer  extends Identifiable  {
	public static String DONE = "Doer.DONE";
/**
* Accept a task.  A task is the evolution of commands from prior
* iPAM versions.  A task object describes itself.  A task needs
* to be acknowledged upon completion.
*
* @param task to be performed
*/
	public void doCommittedTask();	

/**
* Get the token to issue a task.
* 
* @param task is what is to be done
* @return false if not idle or if already committed to another task
*/
	public boolean getTaskingPermission(Task task);
/**
* Release the token. For simplicity this does not have
* to be released. The token is "released"
* automatically after the agent is done executing.
*
* @param task is what is to be done
*/
	public void releaseTaskingPermission(Task task);
/**
* Wait until doer is idle
*
*/
	public void waitTillIdle();
/**
* Are you idle
*
*/
	public boolean isIdle();
/**
* Doer's have state, they must return to ExecGlobal.IDLE
* when they are doing nothing
*/
	public String getState();
}
