/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.interfaces;

import java.util.Enumeration;
import java.util.Observer;
import java.util.Hashtable;

/**
* Basic framework methods implemented by the exec.  These are available
* to agents.  Note that there are other calls available via direct
* references to static methods of class org.mitre.pam.executive.ExecGlobal.
*
* @author Nazario Irizarry, Jr.
*/
public interface ExecCallbacks extends Observer
{
/**
* A factory method to create report agents
*/
	Reporter makeReporter(String contextName);
/**
* Get the run state of the exec
*/
	public String getState();
/**
* Get the exec id.  These numbers are guaranteed
* to increase as new versions of the exec come out.  Users
* are encouraged to use whichNumber==0
*
* @param whichNumber 0 - get the build number
* @param whichNumber 1 - get the major version
* @param whichNumber 2 - get the minor version
*/
	public int getBuildNumber(int whichNumber);
/**
* Tell the exec that an agents schedule has changed
*/
	public void myScheduleChanged(Schedulable whoChanged);
/**
* Factory service to make new memento, the pair of kind and id must be unique
* @param parentKind for the memento parent
* @param parentId for the parent
*/
	public Memento makeNewMemento(String parentKind, String parentId);
/**
* Get the agent type of an agent.
* 
* @param id of requested agent
* @return null if not found
*/
	public String getAgentsKind(Agent agent);
/**
* Get the "dictionary" for a structured data entry (reserved for future growth) <p>
* Warning: The dictionary name should be different than the name of any other agent.
* @param dictionaryName the name of this Dictionary object
* @param agent making the request (can't be null)
* @return               the Dictionary object
*/
	public IDictionary getDictionary(String dictionaryName, Agent agent);
/**
* Get a reference to the exec as type Agent.  Useful for
* for objects that need to act on the Exec's behalf and want
* to issue log messages on behalf of the exec.  Also useful
* to access the exec's attributes
*/
	public Agent getExecAsAgent();
/**
* Get a reference to the product factory.
*/
	public ProductFactory getProductFactory();
}
