/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.interfaces;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Observer;
import java.io.PrintWriter;
import java.io.IOException;

/**
* These interfaces are typically used to get information
* about the current state of the system <p>
*
* @author Nazario Irizarry, Jr.
*/
public interface ExecToMonitor extends ExecCallbacks {
	static final long serialVersionUID = -5398274395395704408L;
/**
* Retain this constant for backward compatiblity, it is the same
* as OUTPUT_AGENT
*/
	public final int DIM = 1;
/**
* Retain this constant for backward compatiblity, it is the same
* as INPUT_AGENT
*/
	public final int PIM = 2;
/**
* Retain this constant for backward compatiblity, it is the same
* as HANDLER
*/
	public final int JUGGLER = 3;

	public final int DAEMON = 0;
	public final int OUTPUT_AGENT = 1;
	public final int INPUT_AGENT = 2;
	public final int HANDLER = 3;
/**
* A selector for the getPAMState method. The value of
* this selector is the name of the owner of the desired configuration
*/
	public final String SELECTOR_CONFIGURATION_NAME = "configurationName";
/**
* A selector for the getPAMState method. 
* If this selector is present (it can have any value) then
* the returned state will not duplicate any attributes between
* the parent and the node.  Successfully suppresses a lot of meta info.
*/
	public final String SELECTOR_NO_REDUNDANCY_WITH_PARENT = "noRedundancyWithParent";


/**
* Get list of currently active modules of the specified type.
*
* @param type, INPUT_AGENT, OUTPUT_AGENT, HANDLER, DAEMON
* @return an Enumeration of active objects per request
*/
	public Enumeration getListOfActiveModules(int type);
/**
* Get list of currently active agents that belong to the
* group designated by the attribute value pair.  One possible
* attribute would be "group" in which case the value would identify a
* group's name or identifier. <p>
*
* @param attribute the attribute to look for in each agent, if null then all agents are returned
* @param value if null any value is acceptable,
*  if not null then this is the String value that each attribute must have,
* @return an Enumeration of type Agent of active agents that meet the criterion
*/
	public Enumeration getListOfActiveAgents(String attribute, String value);
/**
* Get list of available modules of the specified type.  
* 
* @param type, INPUT_AGENT, OUTPUT_AGENT, HANDLER, DAEMON
* @return an enumeration of unused instances of DIM for use as a template
*/
	public Enumeration getListOfAvailableModules(int type);
/**
* Get item memento
* @param id the it of the item
* @param properties [in] an empty vector; [out] a vector of Strings
* @param values [in] an empty vector; [out] a vector of Strings
*/
	public void getItemStateList(String id, Vector properties, Vector values);
/**
* Get status of queue
*@param whichQueue, INPUT_AGENT, OUTPUT_AGENT, HANDLER
*@param running is empty vector on input, returned as vector of Strings on output
*@param pending is empty vector on input, returned as vector of Strings on output
*/
	public void getQueState(int whichQueue, Vector running, Vector pending);
/**
* Add observer
*/
	public void addObserver(Observer o);
/**
* Get state as a stream
* @param p the print writer to use to write the output
* @param selectors null, or a set of "query" options
*/
	public void getPAMState(PrintWriter p, Hashtable selectors) throws IOException;
/**
* Looks up a sink based on its kind and id
*
* @param kind  either "juggler" or "DIM"
* @param id must match what you get when you do Source.getId()
*/
	public Sink getActiveSinkById(String kind, String id);
/**
* Looks up a source based on its kind and id
* @param kind  either "juggler" or "PIM"
* @param id must match what you get when you do Source.getId()
*/
	public Source getActiveSourceById(String kind, String id);

/**
* Get exec's memento
*/
	public Memento getMemento();
/**
* Get list of an agents sources. 
* 
* @param agentId
* @return an enumeration of agents of type Source, never null
*/
	public Enumeration getListOfAgentSources(String agentId);
/**
* Get list of an agents sinks. 
* 
* @param agentId
* @return an enumeration of agents of type Sink, never null
*/
	public Enumeration getListOfAgentSinks(String agentId);
/**
* Get a specific (unused,unistantiated) agent.
* 
* @param id of requested agent
* @return null if not found
*/
	public Agent getAvailableAgent(String id);
/**
* Get a specific instantiated agent.
* 
* @param id of requested agent
* @return null if not found
*/
	public Agent getActiveAgent(String id);
}
