/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.interfaces;
import org.mitre.pdif.eventParser.ValueList;
import java.util.Enumeration;
import org.mitre.pam.util.AttributeKey;

/**
* Generic interface describing an object with attributes,
* aka, properties. The "name" of the attribute may
* be specified as a String or as a special AttributeKey<p>
*
* @author Nazario Irizarry, Jr.
*/
public interface HasKeyedProperties extends HasProperties {
/**
* Remove a property without regard for other remote
* copies of the data
*
*/
	public void removeProperty(AttributeKey property);
/**
* Remove a property but tag it so that remote, dependent
* data copies can synchronize with this deletion
*
*/
	public void removePropertyAndRemoteCopies(AttributeKey property);
/**
* Get a property
*/
	public ValueList getPropertyValues(AttributeKey propertyKey);
/**
* Set a property
*/
	public void setPropertyValues(AttributeKey propertyKey, ValueList values);
/**
* Get a Double-valued property
* @param propertyKey key for the property
* @param defaultValue value to use if the property does not exist
* @return true or false, if the property does not exist false is returned
*/
	public Double getProperty(AttributeKey propertyKey, Double defaultValue);
/**
* Get a Boolean-valued property
* @param propertyKey key for the property
* @param defaultValue value to use if the property does not exist
* @return true or false, if the property does not exist false is returned
*/
	public Boolean getProperty(AttributeKey propertyKey, Boolean defaultValue);
/**
* Get a String-valued property
* @param propertyKey key for the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	public String getProperty(AttributeKey propertyKey, String defaultValue);
/**
* Get a Integer-valued property
* @param propertyKey key for the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	public Integer getProperty(AttributeKey propertyKey, Integer defaultValue);
/**
* Get a Long-valued property
* @param propertyKey key for the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	public Long getProperty(AttributeKey propertyKey, Long defaultValue);

/**
* Set a Long-valued property
* @param propertyKey key for the property
* @param defaultValue value to use if the property does not exist
* @return true or false, if the property does not exist false is returned
*/
	public void setProperty(AttributeKey propertyKey, Long value);
/**
* Set a String-valued property
* @param propertyKey key for the property
* @param defaultValue value to use if the property does not exist
* @return true or false, if the property does not exist false is returned
*/
	public void setProperty(AttributeKey propertyKey, String value);
/**
* Set a Boolean-valued property
* @param propertyKey key for the property
* @param defaultValue value to use if the property does not exist
* @return true or false, if the property does not exist false is returned
*/
	public void setProperty(AttributeKey propertyKey, Boolean value);
/**
* Set a Integer-valued property
* @param propertyKey key for the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	public void setProperty(AttributeKey propertyKey, Integer value);
/**
* Set a Double-valued property
* @param propertyKey key for the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	public void setProperty(AttributeKey propertyKey, Double value);


/**
* Set a long-valued property
* @param propertyKey key for the property
* @param defaultValue value to use if the property does not exist
* @return true or false, if the property does not exist false is returned
*/
	public void setProperty(AttributeKey propertyKey, long value);
/**
* Set a boolean-valued property
* @param propertyKey key for the property
* @param defaultValue value to use if the property does not exist
* @return true or false, if the property does not exist false is returned
*/
	public void setProperty(AttributeKey propertyKey, boolean value);
/**
* Set a int-valued property
* @param propertyKey key for the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	public void setProperty(AttributeKey propertyKey, int value);
/**
* Set a double-valued property
* @param propertyKey key for the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	public void setProperty(AttributeKey propertyKey, double value);
/**
* @return true if the property exists
*/
	public boolean isProperty(AttributeKey propertyKey);

}
