/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.interfaces;
import java.util.Enumeration;
import org.mitre.pam.util.AttributeKey;
/**
* A dictionary is a "registered" set of attributes.
* The set can be used internally to optimize
* lookups on attributes.
*
*/
public interface IDictionary {
/**
* Returns the name of this dictionary
* @return   the name of this Dictionary object
*/
    public String getDictionaryName();
/**
* Returns type of the specified field
* @param name   the field name
* @param dataType   PropertyMetaData.INTEGER, PropertyMetaData.LONG, PropertyMetaData.FLOAT, etc.
*/
    public Class getDataType(String name);
/**
* Returns type of the specified field
* @param key the key that represents the attribute
* @param dataType   PropertyMetaData.INTEGER, PropertyMetaData.LONG, PropertyMetaData.FLOAT, etc.
*/
    public Class getDataType(AttributeKey key);
/**
* Define and set the type of the specified field
* @param key the key that represents the attribute
* @param type   PropertyMetaData.INTEGER, PropertyMetaData.LONG, PropertyMetaData.FLOAT, etc.
*/
    public void defineProperty(AttributeKey key, Class type);
/**
* Define and set the type of the specified field
* @param name of the attribute
* @param type   PropertyMetaData.INTEGER, PropertyMetaData.LONG, PropertyMetaData.FLOAT, etc.
*/
    public void defineProperty(String name, Class type);
/**
* Returns an Enumeration of field names
* @return   an Enumeration of the field names contained within this Dictionary object
*/
    public Enumeration getNames();
/**
* Returns true, if this "dictionary" is empty.
* @return   true, if this Dictionary does not associate any fieldnames to indexes and datatypes
*/
    public boolean isEmpty();
/**
* Returns true if a field with a given name already exists
* @param name   the field name
* @return   true, if there exists a field with this name
*/
    public boolean propertyExists (String name);    
/**
* A factory method created to facilitate migration from iPAM 2.2
* to iPAM 2.3.  This method takes arguments similar to the defunct
* getIndexOf method. To use this
* change calls of the type dictionaryProduct.getIndexOf("attr", Dict.INTEGER)
* into dictionary.makeAttributeKey("attr", PropertyMetaData.INTEGER).
*/ 
	public AttributeKey makeAttributeKey(String name, Class type);
}