/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.interfaces;
import java.net.URL;
import java.util.Vector;
import org.mitre.pam.store.FileRef;
/**
* Can make the basic product types.
* Note that it can not make a Hashtable product
* because that product class is deprecated.
*
* @author Nazario Irizarry, Jr.
*/
public interface ProductFactory {
/**
* Make a Basic String Product <p>
*
* @param id a mandatory, non-blank identifier
* @param description may be ""; if null then "" is used
* @param content may be ""; if null then "" is used 
*/
	public Product makeProduct(String id, String description, String content);
/**
* Make a Basic URL Product <p>
*
* @param id a mandatory, non-blank identifier
* @param description may be ""; if null then "" is used
* @param content may be null but if you do so you will get a string product with content == "" 
*/
	public Product makeProduct(String id, String description, URL content);
/**
* Make a Basic File Product <p>
*
* @param id a mandatory, non-blank identifier
* @param description may be ""; if null then "" is used
* @param content may be null but if you do so you will get a string product with content == "" 
*/
	public Product makeProduct(String id, String description, FileRef content);
/**
* Make a Product  that contains a byte array<p>
*
* @param id a mandatory, non-blank identifier
* @param description may be ""; if null then "" is used
* @param content may be null but if you do so you will get a string product with content == "" 
*/
	public Product makeProduct(String id, String description, byte[] content);
/**
* Make a Container Product that works like a vector.
* Initial size is 0.
*
* @param id a mandatory, non-blank identifier
* @param description may be ""; if null then "" is used
*/
	public ContainerProduct makeVectorProduct(String id, String description);
/**
* Make a Container Product that works like a vector.
* Initial size is 0.
*
* @param id a mandatory, non-blank identifier
* @param description may be ""; if null then "" is used
* @param vector a Vector of Products to use as the content.
*/
	public ContainerProduct makeVectorProduct(String id, String description, Vector vector);
/**
* A ProductPool is a collection of products of fixed size.
* Product pools can only contain non container products (no vector products).
* Members can only belong to one pool at a time.
* @param key used as a generic reference to the identity of this product pool
* @param nItemsInPool number of items in the pool
*/
	public ProductPool makeProductPool(Object key, Product[] members);
}
