/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.interfaces;

import java.util.Enumeration;
/**
* An abstraction for a task that needs to be performed. <P>
* @author Nazario Irizarry, Jr.
*/
public interface Task {

/**
* Send product requires one argument which is the product to send
*/
	public static String SEND_PRODUCT = "send";

/**
* Get product requires one argument which is the sink to call with a sink.putProduct(Product).
*/
	public static String GET_PRODUCT = "get";
/**
* Activate tells a juggler to update its product and then send to all outputs
*/
	public static String ACTIVATE = "activate";
/**
* @return the agent that will execute this task
*/
	public Doer getDoer();
/**
* @return the tasking agent - the one that gets the done signal
*/
	public Tasker getTasker();

/**
* @return the argument of a SEND_PRODUCT
*/
	public Product getProduct();
/**
* @return the argument of a GET_PRODUCT
*/
	public Sink getSink();

/**
* @return the task string
*/
	public String getCommand();
/**
* perform the task that we represent
*/
	public void perform();

/**
* Signal our tasker that we are done
*/
	public void signalDone();
/**
* a string form suitable for displaying on admin screens lists
*/
	public String getAsDisplayString();
/**
* Get the sequence of tasks
*/
	public Enumeration getTaskEnumeration();
/**
* @return the put token argument for GET_PRODUCT and SEND_PRODUCT
*/
	public Task getPutToken();
/**
* In some cases there may be "proxy" tasks, so this tasks tasker
* may not me the actual Agent that initiated the command.
*
* @return the Agent that created/initiated the task.
*/
	public Agent identifyAgentSource();
}
