/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.monitor;

import java.io.IOException;
import javax.servlet.*;
import javax.servlet.http.*;

import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.interfaces.ExecToMonitor;
import org.mitre.pam.interfaces.ExecToController;
import org.mitre.pam.interfaces.ExecCallbacks;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.util.AuthContext;

/**
* A servlet base class that takes care of fetching references to the executive
*
* @author Nazario Irizarry, Jr.
*/
abstract public class SrvPAMBase extends HttpServlet {

	protected ExecToController exec;
	protected Reporter reporter;
	
    protected void service (HttpServletRequest req, HttpServletResponse resp)
		throws ServletException, IOException {
		getExec();
		
		try {
			super.service(req, resp);
		}
		catch (ServletException se){
			reportException(se);
			throw se;
		}
		catch (IOException ioe){
			reportException(ioe);
			throw ioe;
		}
		catch (RuntimeException ex){
			reportException(ex);
			throw ex;
		}
	
	}

/**
* 
* Writes the last part of servlet as its name, followed
* by the message
*
*/
    public void log(String msg) {
		getServletContext().log(getServletName() + "\t"+ msg);
    }
    
    private String servletName;

    private String getServletName() {
    	if (servletName != null)
    		return servletName;
		servletName = getClass().getName();
		int pos = servletName.lastIndexOf(".");
		if (pos > 0)
			servletName = servletName.substring(pos+1);
		return servletName;
    }
/**
* get a copy of our exec at init time
*/
    public void init(ServletConfig config) throws ServletException {
		super.init(config);
		getExec();
    }
/**
* Get a reference to the exec if we don't already have one
*/
	protected void getExec() {
		if (exec == null) {
			exec = ExecGlobal.getExec();
			reporter = exec.makeReporter(getServletName());
		}
	}
/**
* Contrary to the documentation
* we need to return a current number
* so that we force new content
*/
    protected long getLastModified (HttpServletRequest req)
    {
		return System.currentTimeMillis();
    }
/**
* Used by iPAM servlets to validate access 
*/
	protected AuthContext getAuthContext(HttpServletRequest req, HttpServletResponse res) {
		AuthContext context = new AuthContext();
		
		context.res = res;
		context.req = req;
		context.authList = exec.getExecAsAgent().getPropertyValues("pamAccess");
		context.realm = "exec";
		return context;   
	}
/**
* report an exception, for backward compatiblity with
* code prior to the existence of the reporter
*/
	public void reportException(Throwable e) {
		reporter.reportException(e);
	}
/**
* report an exception, for backward compatiblity with
* code prior to the existence of the reporter
*/
	public void assert(boolean cond, String msg1, String msg2, String msg3, String msg4) {
		reporter.assert(cond, msg1, msg2, msg3, msg4);
	}
}