/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */

package org.mitre.pam.monitor;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentAdapter;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Vector;

import org.mitre.pam.interfaces.ExecToController;
import org.mitre.pam.interfaces.AdminTool;
import org.mitre.pam.interfaces.Memento;

import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.monitor.utils.LineCanvas;
import org.mitre.pdif.eventParser.ValueList;
import org.mitre.pam.util.WindowUtil;

/**
 * This class is a Frame that allows the various iPAM administration tools
 * to be hidden and shown.
 *
 * @author Michael DeCorpo
 */

public class ToolPicker extends Frame
{
	java.awt.Panel	tools = new java.awt.Panel();
	java.awt.Panel	stop = new java.awt.Panel();

	SymAction lSymAction;
	Vector toolClass = new Vector();
	Vector toolRef = new Vector();
	Vector toolLabel = new Vector();
	Vector toolStatus = new Vector();
	Vector toolButton = new Vector();
	private String TITLE_PREFIX = "Picker";
	Memento options;
	static final String PICKER_SAVE_NAME = "toolWindow_toolPicker";

	//{{DECLARE_CONTROLS
	
	ExecToController exec;
	LineCanvas lineCanvas;
	String commonTitle;
	String versionString;
	SymComponent componentListener;
	
	//}}
	
	//{{DECLARE_MENUS
	//}}

  public ToolPicker()
  {
	java.awt.Label toolLabel;
	java.awt.Label statusLabel;
	
	setLayout(new java.awt.BorderLayout());
	add(tools,java.awt.BorderLayout.CENTER);
	
    // This code is automatically generated by Visual Cafe when you add
    // components to the visual environment. It instantiates and initializes
    // the components. To modify the code, only use code syntax that matches
    // what Visual Cafe can generate, or Visual Cafe may be unable to back
    // parse your Java file into its visual environment.
    
    //{{INIT_CONTROLS
    GridBagLayout gridBagLayout;
    gridBagLayout = new GridBagLayout();
    tools.setLayout(gridBagLayout);
    
    setVisible(false);
    setSize(insets().left + insets().right + 230,insets().top + insets().bottom + 105);
    setFont(new Font("Dialog", Font.PLAIN, 10));
    setBackground(new Color(12632256));
    
// -- The column headings 

    toolLabel = new java.awt.Label("Tool");
//    toolLabel.setBounds(insets().left + 1,insets().top + 2,137,26);
    toolLabel.setFont(new Font("TimesRoman", Font.BOLD, 14));  
    GridBagConstraints gbc;
    gbc = new GridBagConstraints();
    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.anchor = GridBagConstraints.WEST;
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.insets = new Insets(0,5,0,0);
    tools.add(toolLabel,gbc);
    
    statusLabel = new java.awt.Label("Status");
//    statusLabel.setBounds(insets().left + 138,insets().top + 2,54,26);
    statusLabel.setFont(new Font("TimesRoman", Font.BOLD, 14));
    gbc = new GridBagConstraints();
    gbc.gridx = 1;
    gbc.gridy = 0;
    gbc.anchor = GridBagConstraints.WEST;
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.insets = new Insets(0,0,0,0);
    tools.add(statusLabel,gbc);

// -- Horizontal line under column headings 

    lineCanvas = new LineCanvas(LineCanvas.HORIZONTAL);
    lineCanvas.setSize(230,2);
    gbc = new GridBagConstraints();
    gbc.gridx = 0;
    gbc.gridy = 1;
    gbc.gridwidth = 3;
    gbc.anchor = GridBagConstraints.NORTH;
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.insets = new Insets(0,0,5,0);
    tools.add(lineCanvas,gbc);


// -------------------------------------------------------------------------
// -- Horizontal line over the lower part with the stop button
	
    lineCanvas = new LineCanvas(LineCanvas.HORIZONTAL);
    lineCanvas.setSize(230,2);
    gbc = new GridBagConstraints();
    gbc.gridx = 0;
    gbc.gridy = 100; // will we every have this many tools?
    gbc.gridwidth = 3;
    gbc.anchor = GridBagConstraints.NORTH;
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.weightx = 1;
    gbc.insets = new Insets(5,0,5,0);
    tools.add(lineCanvas,gbc);

// -- The stop button
	Button 	stopButton = null;
	stopButton = new java.awt.Button();
	stopButton.setLabel("Stop iPAM");
//	stopButton.setBounds(insets().left + 192,insets().top + 30 + 26,36,21);
	stopButton.setBackground(Color.red);
	stopButton.setForeground(Color.white);
	gbc = new GridBagConstraints();
	gbc.gridx = 0;
	gbc.gridy = 101;
    gbc.gridwidth = 3;
    gbc.anchor = GridBagConstraints.EAST;
    gbc.fill = GridBagConstraints.NONE;
	gbc.insets = new Insets(5,5,5,5);
    tools.add(stopButton,gbc);
    
    stopButton.addActionListener(new StopAction());
// -------------------------------------------------------------------------

    //}}
    setTitle(null);
    
    //{{INIT_MENUS
    //}}
    
    //{{REGISTER_LISTENERS
    SymWindow aSymWindow = new SymWindow();
    this.addWindowListener(aSymWindow);
    lSymAction = new SymAction();
    
    componentListener = new SymComponent();
    //}}    



    pack();
	}

	public void addAdminToolRef(AdminTool tool, String displayString) {
		addToolWithRef(tool.getClass().getName(), displayString, tool);
		positionToolWindow(displayString, tool);
	}
	public void addTool(String toolClassName, String displayString) {
		addToolWithRef(toolClassName, displayString, null);
	}
	
	synchronized private void addToolWithRef(String toolClassName, String displayString, AdminTool instanceReference) {
		Label toolnLabel;
		Label toolnStatus;
		Button toolnButton;
		
		int nth = toolClass.size();
		
		toolnLabel = new java.awt.Label(displayString);
//		toolnLabel.setBounds(insets().left + 1,insets().top + 28 + nth*26,97,26);
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 3+nth;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0,5,0,0);
		gbc.weightx = 2;
		tools.add(toolnLabel,gbc);
		
		toolnStatus = new java.awt.Label("Stopped");
//		toolnStatus.setBounds(insets().left + 138,insets().top + 30 + nth*26,54,21);
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 3+nth;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0,0,0,0);
		gbc.weightx = 1;
		tools.add(toolnStatus,gbc);
		
		toolnButton = new java.awt.Button();
		toolnButton.setLabel("Start");
//		toolnButton.setBounds(insets().left + 192,insets().top + 30 + nth*26,36,21);
		toolnButton.setBackground(new Color(12632256));
		gbc = new GridBagConstraints();
		gbc.gridx = 2;
		gbc.gridy = 3+nth;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,0,0,0);
		gbc.weightx = 1;
		tools.add(toolnButton,gbc);
		
		toolClass.addElement(toolClassName);
		toolLabel.addElement(toolnLabel);
		toolStatus.addElement(toolnStatus);
		toolButton.addElement(toolnButton);

		toolRef.addElement(instanceReference);
		if (instanceReference != null) {
			instanceReference.addComponentListener(componentListener);
			toolnButton.setLabel(instanceReference.isVisible()?"Stop":"Start");
			toolnStatus.setText(instanceReference.isVisible()?"Running":"Stopped");
		}
		
		toolnButton.addActionListener(lSymAction);
		tools.invalidate(); // mac does not automatically invalidate in mrj 2.1.4
	}
	
  public ToolPicker(String title)
  {
    this();
    setTitle(title);
  }


  /**
   * This method sets the title that will be used for all of the different
   * admin tools.
   */
  public void setCommonTitle(String commonTitle)
  {
    this.commonTitle = commonTitle;
  }


  /**
   * This method sets the title that will be used for all of the different
   * admin tools.
   */
  public void setVersionString(String versionString)
  {
    this.versionString = versionString;
  }

  /**
   * Sets the frame title to the composite title
   */
  public void updateTitle()
  {
    setTitle(TITLE_PREFIX+", "+commonTitle+", "+versionString);
  }

  /**
   * Set the exec.  Use the exec's memento to store and retrieve
   * bounds info for tools.
   */
  public void setExec(ExecToController exec) {
    this.exec = exec;
    this.options = exec.getExecAsAgent().getMemento();
  }
  
  public synchronized void show()
  {
    move(50, 50);
    super.show();
  }
  
  static public void main(String args[])
  {
    (new ToolPicker()).show();
  }

/**
* Gets called when exec is checkpointing to save state of windows
*/
	public void saveStateTo(){
		
		try {

			WindowUtil.storeWindowBoundsToMemento(PICKER_SAVE_NAME, getBounds(), options);

			String nameToUse;
			for (int i = 0; i<toolRef.size(); i++) {
				AdminTool aTool = (AdminTool) toolRef.elementAt(i);
				if (aTool != null) {
					nameToUse = "toolWindow_"+((Label)toolLabel.elementAt(i)).getText();
					WindowUtil.storeWindowBoundsToMemento(nameToUse, ((AdminTool) toolRef.elementAt(i)).getBounds(), options);
				}
			}
		}
		catch (Exception ignore) {
			exec.getExecAsAgent().getReporter().reportException(ignore);
		}

	}
	
  public void addNotify()
  {
    // Record the size of the window prior to calling parents addNotify.
    Dimension d = getSize();
    
    super.addNotify();
    
    if (fComponentsAdjusted)
      return;
    
    // Adjust components according to the insets
    setSize(insets().left + insets().right + d.width, insets().top + insets().bottom + d.height);
    Component components[] = getComponents();
    for (int i = 0; i < components.length; i++)
      {
	Point p = components[i].getLocation();
	p.translate(insets().left, insets().top);
	components[i].setLocation(p);
      }
    fComponentsAdjusted = true;
  }
  
  // Used for addNotify check.
  boolean fComponentsAdjusted = false;

  /**
   * This method will start a local copy of the tool
   */
  public void startTool(int nth) 
  {
  	try {
		AdminTool tool = (AdminTool)Class.forName((String)toolClass.elementAt(nth)).newInstance();
		toolRef.setElementAt(tool, nth);
		tool.setExec(exec);
		tool.setLocation((this.getBounds().x + this.getBounds().width)/2,this.getBounds().y/2);
		tool.addComponentListener(componentListener);
		tool.setTitle(commonTitle);

		positionToolWindow( ((Label)toolLabel.elementAt(nth)).getText(), tool);

		tool.setVisible(true);
	}
	catch(Exception e) {
		exec.getExecAsAgent().getReporter().reportException(e);
	}
  }

	private void positionToolWindow(String toolName, AdminTool tool) {
		String nameToUse = "toolWindow_"+toolName;
		try {
			Rectangle bounds = WindowUtil.restoreWindowBoundsFromMemento(nameToUse, options);
			if (bounds != null)
				tool.setBounds(bounds);

		}
		catch(Exception e) {
			exec.getExecAsAgent().getReporter().reportException(e);
		}
	}
	
	public void setVisible(boolean visible) {
		super.setVisible(visible);
		if (options == null || !visible)
			return;
		try {
			Rectangle bounds = WindowUtil.restoreWindowBoundsFromMemento(PICKER_SAVE_NAME, options);
			if (bounds != null) {
				setBounds(bounds);
			}
		}
		catch(Exception ignore) {}
		if (visible) {
			validate();
		}
	}
	
  /**
   * This method takes care of what happens when the tool's 
   * stop/start button was pressed. It simply changes the visibility of the
   * tool.
   */
  void toolButtonActionPerformed(int nth)
  {
	AdminTool tool = (AdminTool)toolRef.elementAt(nth);
    if (tool == null) {
       startTool(nth);
       return;
     }
    tool.setVisible(!tool.isVisible());
  }

// -------------------------------------------------------------------------
	void stopBtn_Clicked(java.awt.event.ActionEvent event) {
		// to do: place event handler code here.
		exec.halt();

/* 
* don't hide frame in case there was a problem
* the user will still have access to the gui's
*/
/*
		setVisible(false);   
		dispose();
*/
	}
// -------------------------------------------------------------------------

  class SymWindow extends java.awt.event.WindowAdapter
  {
    public void windowClosing(java.awt.event.WindowEvent event)
    {
		Object object = event.getSource();
		if (object == ToolPicker.this)
			ToolPicker_WindowClosing(event);
    }
  }
  
  /**
   * If the ToolPicker window can't be closed. Don't let it.
   */
  void ToolPicker_WindowClosing(java.awt.event.WindowEvent event)
  {
//    setVisible(false);
  }
  
  /** 
   * This class listens for Component Events that are generated by 
   * the different admin tools when they are hidden and shown.  
   */
  class SymComponent extends java.awt.event.ComponentAdapter
  {
    /**
     * If the tool was hidden, change the labels appropriately.
     */
    public void componentHidden(java.awt.event.ComponentEvent event)
    {
      Object object = event.getSource();
	  int nth = toolRef.indexOf(object);
	  if (nth >= 0) {
	  	Label label = (Label)toolStatus.elementAt(nth);
	  	(label).setText("Stopped");
	  	((Button)toolButton.elementAt(nth)).setLabel("Start");
	  	label.invalidate();
	  	validate();
      }
    }

    /**
     * If the tool was shown, change the labels appropriately.
     */
    public void componentShown(java.awt.event.ComponentEvent event)
    {
      Object object = event.getSource();
  	  int nth = toolRef.indexOf(object);
      if (nth >= 0) {
      	Label label = (Label)toolStatus.elementAt(nth);
      	(label).setText("Running");
      	((Button)toolButton.elementAt(nth)).setLabel("Stop");
      	label.invalidate();
      	validate();
      }

	}
  }
  
  class SymAction implements java.awt.event.ActionListener
  {
    public void actionPerformed(java.awt.event.ActionEvent event)
    {
      Object object = event.getSource();
      if (object instanceof Button) {
      	int nth = toolButton.indexOf(object);
      	if (nth >= 0) {
      		toolButtonActionPerformed(nth);
      	}
      }
    }
  }

// -------------------------------------------------------------------------
	class StopAction implements java.awt.event.ActionListener{
		public void actionPerformed(java.awt.event.ActionEvent event){
			stopBtn_Clicked(event);
		}
	}
// -------------------------------------------------------------------------

  
}
