/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.monitor.utils;

import java.awt.Checkbox;
import java.awt.Panel;
import java.awt.ScrollPane;
import java.awt.ItemSelectable;
import java.awt.GridLayout;
import java.awt.Color;
import java.awt.AWTEventMulticaster;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.lang.String;
import java.util.Vector;

/**
 * This class keeps track of a list of items that can be either true or false.
 *
 * @author Michael DeCorpo
 */

public class ItemScroller extends ScrollPane implements ItemListener, ItemSelectable
{
  Vector items, values;
  
  Panel panel;
  Checkbox boxes[];
  ItemListener itemListener;

  public ItemScroller()
  {
    this(new Vector(),new Vector());
  }

  public ItemScroller(Vector items, Vector values)
  {
    super();
    setBackground(Color.white);
    this.items = items;
    this.values = values;

    panel = new Panel();
    performLayout();
    add(panel);
  }

  /**
   * This method handles the creation and layout of the checkboxes that
   * will go into the scrollable list.
   */
  private void performLayout()
  {
    panel.removeAll();

    // make sure the items and the values vectors are the same size
    int i = items.size();
    if (i != values.size())
      return;

    // Use GridLayout with at least 5 rows and exactly 1 column.
    if (i < 5)
      panel.setLayout(new GridLayout(5,1,0,0));
    else
      panel.setLayout(new GridLayout(i,1,0,0));

    // go through the vectors creating new checkboxes and adding them.
    boxes = new Checkbox[i];
    for (int j=0; j<i; j++) {
      String s = (String) items.elementAt(j);
      Boolean b = (Boolean) values.elementAt(j);
      boxes[j] = new Checkbox(s);
      boxes[j].addItemListener(this);
      boxes[j].setState(b.booleanValue());
      panel.add(boxes[j]);
    }
  }

  /**
   * This method takes a Vector of String items and another Vector of Boolean
   * values and displays this new list.
   */
  public void setItems(Vector items, Vector values)
  {
    this.items = items;
    this.values = values;

    performLayout();
    panel.invalidate();
    validate();

  }

  public Vector getItems()
  {
    return items;
  }

  public Vector getValues()
  {
    return values;
  }

  /**
   * This method sets the value of the item specified to true.
   */
  public void turnOnItem(String item)
  {
    if ((items == null) || (values == null))
      return;
    for(int i=0; i<items.size(); i++) {
      String s = (String) items.elementAt(i);
      if (s.equals(item)) {
	values.setElementAt(new Boolean(true),i);
	boxes[i].setState(true);
	break;
      }
    }
  }

  /**
   * This method sets the value of the item specified to false.
   */
  public void turnOffItem(String item)
  {
    if ((items == null) || (values == null))
      return;
    for(int i=0; i<items.size(); i++) {
      String s = (String) items.elementAt(i);
      if (s.equals(item)) {
	values.setElementAt(new Boolean(true),i);
	boxes[i].setState(false);
      }
    }
  }

  /**
   * If a the state of one of the checkboxes was changed, create a new item
   * event and send it to the registered ItemListeners.
   */
  public void itemStateChanged(ItemEvent event)
  {
    //System.out.println("itemStateChanged");
    if (itemListener != null) {
      String item = ((Checkbox) event.getSource()).getLabel();
      for (int i=0; i<items.size(); i++) {
	if (((String) items.elementAt(i)).equals(item)) 
	  values.setElementAt(new Boolean(!((Boolean) values.elementAt(i)).booleanValue()),i);
      }
      itemListener.itemStateChanged(new ItemEvent(this,event.getID(),((Checkbox) event.getSource()).getLabel(),event.getStateChange()));
    }  
  }

  public void addItemListener(ItemListener l)
  {
    itemListener = AWTEventMulticaster.add(itemListener,l);
  }
  
  public void removeItemListener(ItemListener l)
  {
    itemListener = AWTEventMulticaster.remove(itemListener,l);
  }

  /**
   * This mehtod implements ItemSelectable.  It returns an array of all
   * the checkboxes that are turned on.
   */
  public Object[] getSelectedObjects()
  {
    if (boxes == null)
      return null;
    
    int count = 0;
    for (int i=0; i<boxes.length; i++) {
      if (boxes[i].getState())
	count++;
    }

    int j=0;
    Object rslt[] = new Object[count];
    for (int i=0; i<boxes.length; i++) {
      if (boxes[i].getState())
	rslt[j++] = boxes[j];
    }

    return rslt;
  }
 
}




