/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */

package org.mitre.pam.product.view;

import java.net.URL;
import java.util.Vector;
import java.util.Hashtable;
import org.mitre.pam.store.FileRef;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ContainerProduct;
import org.mitre.pam.interfaces.ProductFactory;
import org.mitre.pam.interfaces.ProductPool;
import org.mitre.pam.interfaces.Reporter;


/**
* Can make the basic product types.
* Note that it can not make a Hashtable product
* because that product class is deprecated.
*
* @author Nazario Irizarry, Jr.
*/
final public class ProductFactoryImpl implements ProductFactory {
	private Reporter reporter;
/**
* Used to keep track of product pools
*/
	static Hashtable productPools;

	public ProductFactoryImpl(Reporter reporter) {
		this.reporter = reporter;
	}
	
/**
* Make a Basic String Product <p>
*
* @param id a mandatory, non-blank identifier
* @param description may be ""; if null then "" is used
* @param content may be ""; if null then "" is used 
*/
	public Product makeProduct(String id, String description, String content){
		BasicProduct p = new BasicProduct(id, description);
		p.setContent(content);
		return p;
	}
/**
* Make a Basic URL Product <p>
*
* @param id a mandatory, non-blank identifier
* @param description may be ""; if null then "" is used
* @param content may be null but if you do so you will get a string product with content == "" 
*/
	public Product makeProduct(String id, String description, URL content){
		BasicProduct p = new BasicProduct(id, description);
		p.setContent(content);
		return p;
	}
/**
* Make a Basic File Product <p>
*
* @param id a mandatory, non-blank identifier
* @param description may be ""; if null then "" is used
* @param content may be null but if you do so you will get a string product with content == "" 
*/
	public Product makeProduct(String id, String description, FileRef content){
		BasicProduct p = new BasicProduct(id, description);
		p.setContent(content);
		return p;
	}
/**
* Make a Product  that contains a byte array<p>
*
* @param id a mandatory, non-blank identifier
* @param description may be ""; if null then "" is used
* @param content may be null but if you do so you will get a string product with content == "" 
*/
	public Product makeProduct(String id, String description, byte[] content){
		BasicProduct p = new BasicProduct(id, description);
		p.setContent(content);
		return p;
	}
/**
* Make a Container Product that works like a vector.
* Initial size is 0.
*
* @param id a mandatory, non-blank identifier
* @param description may be ""; if null then "" is used
*/
	public ContainerProduct makeVectorProduct(String id, String description){
		VectorProduct p = new VectorProduct(id, description);
		return p;
	}
/**
* Make a Container Product that works like a vector.
* Initial size is 0.
*
* @param id a mandatory, non-blank identifier
* @param description may be ""; if null then "" is used
* @param vector a Vector of Products to use as the content.
*/
	public ContainerProduct makeVectorProduct(String id, String description, Vector vector){
		VectorProduct p = new VectorProduct(id, description);
		p.setContent(vector);
		return p;
	}
/**
* A ProductPool is a collection of products of fixed size.
* Product pools can only contain non container products (no vector products).
* Members can only belong to one pool at a time.
* @param poolId used as a generic reference to the identity of this product pool
* @param nItemsInPool number of items in the pool
*/
	public ProductPool makeProductPool(Object poolId, Product[] members){
		if (productPools == null)
			productPools = new Hashtable(20);
			
		if (productPools.containsKey(poolId)) {
			reporter.warning("A product pool with the specified id already exists: "+poolId);
		}
		BasicProductPoolImpl pool = new BasicProductPoolImpl(poolId, members);
		productPools.put(poolId, pool);
		return pool;
	}
}

