/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.product.view;

import java.net.URL;

import java.io.*;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Random;
import java.util.Date;
import java.net.MalformedURLException;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ContainerProduct;
import org.mitre.pam.store.*;
import org.mitre.pdif.eventParser.PDIFItem;
import org.mitre.pdif.eventParser.PDIFWriter;
import org.mitre.pdif.eventParser.PDIFOutManager;
import org.mitre.pdif.eventParser.ParseEvent3;
import org.mitre.pdif.eventParser.BadPDIFException;

import org.mitre.pam.executive.ExecGlobal;

/**
* Save a product to persistent storage. Used by the exec. <P>
*
* @author Nazario Irizarry, Jr.
*/
public class ProductStore {

	static final String PDIF_PRODUCT_ROOT = "productRoot";
	
	public Hashtable needOutput = new Hashtable(100);
	public Hashtable loaded = new Hashtable(100);
	private String productRoot = null;
	private Product theRootProduct;
	
/**
* used for generating unique ids
*/
	static Random rand = new Random(System.currentTimeMillis());
	
	public ProductStore () {
	}
	
/*
	public static void main(String args[]) {
		main();
	}

	public static void main() {
		try {
			ProductStore aStore = new ProductStore();
			
			System.out.println("\noutput\n");
			BasicProduct p1 = new BasicProduct("prod1","desc1");
			p1.setContent("content 1");
			BasicProduct p2 = new BasicProduct("prod2","desc2");
			p2.setContent(new URL("http://www.naz.org/"));
			BasicProduct p3 = new BasicProduct("prod3","desc3");
			p3.setContent(new FileRef(new File("foobar"),11,"the test source file"));
			
			VectorProduct cont = new VectorProduct("cont1","desc cont");
			HashtableProduct cont2 = new HashtableProduct("cont hash","desc hash");
			String rootKey = cont.getUid();
			String rootKey2 = cont2.getUid();
			
			cont.addElement(p1);
			cont.addElement(p2);
			cont.addElement(p3);
			
			cont2.addElement(p2);
			cont2.addElement(p1);
			
			aStore.addToStore(cont);
			aStore.addToStore(cont2);
			aStore.store(new PrintWriter(System.out));
			aStore.store(new PrintWriter(new FileOutputStream("test.out")));
			
			System.out.println("\nload\n");
			
			ProductStore bStore = new ProductStore();
			bStore.load(new FileInputStream("test.out"));
			Product newRoot = bStore.getProduct(rootKey);
			
			System.out.println("\nreoutput 1, root=\n"+rootKey);
			ProductStore cStore = new ProductStore();
			cStore.addToStore(newRoot);
			cStore.store(new PrintWriter(System.out));

			System.out.println("\nreoutput 2, root=\n"+rootKey2);
			Product newRoot2 = bStore.getProduct(rootKey2);
			ProductStore dStore = new ProductStore();
			dStore.addToStore(newRoot2);
			dStore.store(new PrintWriter(System.out));
		}
		catch (Exception e) {
			e.printStackTrace();
		}

	}
*/
	private void addForOutput(PDIFItem productIn) {
		PDIFItem product = (PDIFItem) productIn;
		if (!needOutput.containsKey(product.getUid()))
			needOutput.put(product.getUid(), product);
	}
	
	public void addToStore(Product product) {
		addForOutput((PDIFItem)product);
	}
	
	public void store  (PrintWriter out) throws IOException {
		PDIFOutManager pdifOutManager = new PDIFOutManager(needOutput);
		PDIFWriter pdifOut = new PDIFWriter(out, "CDA", "pam");
		
		if (productRoot != null) 
			pdifOut.writeHeaderAttribute(PDIF_PRODUCT_ROOT, productRoot);
						
	/*
	* the initial product may reference other products
	*/
		while ( pdifOutManager.hasNeedOutputParts()) {
			Enumeration parts = pdifOutManager.getNeedsOutputEnumeration();
			
			while (parts.hasMoreElements()) {
				PDIFItem needsOutput = (PDIFItem) parts.nextElement();
				pdifOutManager.addPartAlreadyOutput(needsOutput.getUid(), this);
				needsOutput.storeParts(pdifOut);
				pdifOutManager.addPartHasRelation(needsOutput); // this part could be simplified based on following loop

				Enumeration contents = needsOutput.getRelatedPDIFItems();
				while(contents != null && contents.hasMoreElements()) {
					pdifOutManager.addNeedsOutput((PDIFItem)contents.nextElement());
				}

			}
			
		}

	/*
	* this loop could be simplified because as written all partsthat
	* have been output are registered as having relationships (even if they don't)
	*/
		if ( pdifOutManager.hasRelations()) {
			Enumeration parts = pdifOutManager.getRelationEnumeration();
			
			while (parts.hasMoreElements()) {
				PDIFItem needsOutput = (PDIFItem) parts.nextElement();
				needsOutput.storeRelations(pdifOut);
			}
			
		}
		
		pdifOut.close();
	}

/**
*
*/
	public Product getProduct(String key) {
		return (Product) loaded.get(key);
	}
/**
* Load and return a product from persistent storage.
*/
	public void load(Reader in) {
		Product root=null;
		
		try {
						
			ParseEvent3 pEvent = new ParseEvent3(in);
			
			try {
				while ( true ) {
					int type = pEvent.next();
					
					if (type == ParseEvent3.EOF)
						break;
					
					if ( type == ParseEvent3.HEADER_ATTRIBUTE && 
						(pEvent.attribute.name.equals(PDIF_PRODUCT_ROOT)) ) {						
						productRoot = pEvent.attribute.getValue();
					}
					else if (type == ParseEvent3.NODE_ENTER ) {
						try {
							ProductShell currentProduct = (ProductShell) Class.forName(pEvent.node.kind).newInstance();
							currentProduct.PDIFSerialIn(pEvent);
							loaded.put(pEvent.node.id, currentProduct);
						} 
						catch (ClassCastException e) {
							reportException(e);
							throw new BadPDIFException("exception while trying to load node product id="+pEvent.node.id);
						}
						catch (ClassNotFoundException e) {
							reportException(e);
							throw new BadPDIFException("exception while trying to load node product id="+pEvent.node.id);
						}
						catch (InstantiationException e) {
							reportException(e);
							throw new BadPDIFException("exception while trying to load node product id="+pEvent.node.id);
						}
						catch (IllegalAccessException e) {
							reportException(e);
							throw new BadPDIFException("exception while trying to load node product id="+pEvent.node.id);
						}
					}
					else if (type == ParseEvent3.CONNECTOR) {
					/*
					* Product.from is always first
					*/
						ContainerProduct p = null;
						Product to=null;
						String role = null;
						String fromId = null;
						String toId = null;
						
						if (pEvent.connectorEnds[0].role.equals("from")) {
							fromId = pEvent.connectorEnds[0].id;
							toId = pEvent.connectorEnds[1].id;
							role = pEvent.connectorEnds[1].role;
						}
						else {
							fromId = pEvent.connectorEnds[1].id;
							toId = pEvent.connectorEnds[0].id;
							role = pEvent.connectorEnds[0].role; 
						}
						try {
							p = (ContainerProduct) loaded.get(fromId);
							to = (Product) loaded.get(toId);
							assert(to != null,"ProductStore null item, in load from "+fromId+" to "+toId);
							assert(role != null,"ProductStore null role, in load from "+fromId+" to "+toId);
							p.addElement(role, to);
						}
						catch (ClassCastException e) {
							reportException(e);
							throw new BadPDIFException("trying to connect container "+fromId+" to "+toId);
						}
					}
				}
			}
			catch (BadPDIFException e) {
				reportException(e);
			}
//		System.out.println("main root "+productRoot);
			if (productRoot != null)
			theRootProduct = (Product) loaded.get(productRoot);
		}
		catch (IOException e) {
			reportException(e);
		}

//		assert(root != null, "ProductStore missed root product on input "+in);
		return ;
		
	}

/**
* Get the root of the products (if there is one)
*/
	public Product getRootProduct() {			
		return theRootProduct;
	}

/**
* Set the root of the products (if there is one)
*/
	public void setRootProduct(Product root) {
		productRoot = root.getUid();
	}
/**
* report an exception
*/
	private void reportException(Throwable e) {
		ExecGlobal.getExec().getExecAsAgent().getReporter().reportException(e);
	}
/**
* test an assertion
*/
	private void assert(boolean cond, String msg) {
		ExecGlobal.getExec().getExecAsAgent().getReporter().assert(cond, msg,null,null,null);
	}
}
