/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util;

import java.io.IOException;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

import javax.servlet.*;
import javax.servlet.http.*;

import org.mitre.pam.util.Codecs;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pdif.eventParser.ValueList;
import org.mitre.pam.interfaces.ExecCallbacks;

/**
* A context record for authorization of http data requests
*
* @author Nazario Irizarry, Jr.
*/
public class AuthContext {
/**
* Required on input
*/
		public HttpServletRequest req;
/**
* Required on input
*/
		public HttpServletResponse res;
/**
* Required on input
*/
		public String realm;
/**
* Required on input, can be of type String or of type ValueList
*/
		public Object authList;
/**
* Extracted from the request
*/
		public String username;
/**
* Extracted from the request
*/
		public String password;
		private String combined;
/**
* Checks to see if the Http request
* had proper authorization (name and password).
* If it does not then the response is made telling the
* user of an authorization failure.
*/
	public boolean verifyAccess (Reporter reporter, HttpServlet servlet, String accessString) throws IOException
	{
		boolean valid = false;
		
		if (authList == null)
			return true;
		else if (authList instanceof String)
			valid = vString((String)authList);
		else if (authList instanceof ValueList)
			valid = vList((ValueList)authList);
			
		if (valid) {
			reporter.log(accessString+"\taccessed by:\t"+username+" [" + req.getRemoteHost()+"]");
		}
		else {
			/*
			* don't report username == null because that is part of the
			* standard challenge protocol
			*/
			if (username != null)
				reporter.log(accessString+"\taccess denied to:\t"+username+" [" + req.getRemoteHost()+"]");
		}
		return valid;
	}

	private void unauthorized(HttpServletResponse res) throws IOException{
		ServletOutputStream out = res.getOutputStream();
		res.setStatus(401, "Unauthorized");
		res.setHeader("WWW-Authenticate", "Basic realm=\""+realm+"\"");
		out.println("<HTML><HEAD><TITLE>AuthorizationRequired</TITLE></HEAD>");
		out.println("<BODY><H1>Authorization failed<H1></BODY></HTML>");
		out.println("");
	}
	
	private boolean getUserNamePassword() throws IOException {
		String userNamePassword = req.getHeader("Authorization");
	/*
	* Extract the authorization string from query line
	*/		
		if (userNamePassword == null || userNamePassword.length() == 0) {
			combined = req.getParameter( "Authorization" );
			if (combined == null || combined.length() == 0) {
				unauthorized(res);
//		System.out.println("bad query stuff --> "+combined);
				return false;
			}
		}
	/*
	* Extract the authorization string from the http header
	*/		
		else {
			StringTokenizer st = new StringTokenizer(userNamePassword);
			try {
				st.nextToken(); // skip the first param in the http header block
				combined = st.nextToken();
			}
			catch (NoSuchElementException e) {
				unauthorized(res);	
//		System.out.println("bad http header stuff --> "+userNamePassword);
				return false;
			}
		}
		combined = Codecs.base64Decode(combined);
	/*
	* Decode the username and password
	*/		
		StringTokenizer up = new StringTokenizer(combined);
		username="";
		password = "";
		
		try {
			username = up.nextToken(":");
			password = up.nextToken();
		}
		catch (NoSuchElementException e) {}
/*
		System.out.println("combined -->"+combined+"<--");
		System.out.println("username -->"+username+"<--");
		System.out.println("password -->"+password+"<--");
*/
		return true;
	}
	
	private boolean vString(String authorizationString) throws IOException {
		if (authorizationString.length() == 0)
			return true;
		if (authorizationString.equals("*:*"))
			return true;
		if (!getUserNamePassword())
			return false;
		
	/*
	* Get the list of authorized users
	*/
		if (authorizationString.equals(combined))
			return true;
		
		unauthorized(res);	
		return false;
	}
	
	private boolean vList(ValueList authorizationList) throws IOException {
		if (authorizationList.size() == 0)
			return true;

		if (!getUserNamePassword())
			return false;

		for (int i = 0; i < authorizationList.size(); i++) {
			String authorizationString = authorizationList.elementAt(i);
//	System.out.println("autstrhing ->"+authorizationString);
			if (authorizationString.equals("*:*"))
				return true;
		/*
		* Get the list of authorized users
		*/
			if (authorizationString.equals(combined))
				return true;
		}
		unauthorized(res);
		return false;
	}

}
