/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util;

import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Calendar;

/**
* Allows customization of certain
* time/date output message formats without
* specific dependence on JDK 1.1. <p>
*
* @author Nazario Irizarry, Jr.
*/

public class PAMDate extends Date{
	static GregorianCalendar calendar = new GregorianCalendar(); // share one calendar to conserve memory

	public PAMDate() {
		super();
	}

	public PAMDate(long time) {
		super(time);
	}

	public String asNumberString() {
		return toNumberString(this);
	}

	public String as8CharString() {
		return to8CharString(this);
	}


/**
* Returns a string like "Aug 27,1997"
*/
	public static String to8CharString(Date when) {
		StringBuffer result = new StringBuffer(20);
		
		String s = when.toString();
		
		result.append(s.substring(4,10)); // Aug 27
		result.append(",");
		result.append(s.substring(23)); // 1997
		return result.toString();
	}

/**
* Returns a string like "5-28"
*/
	public String asMonthDay() {
		return toMonthDay(this);
	}
/**
* Returns a string like "5-28"
*/
	public static String toMonthDay(Date when) {
		StringBuffer result = new StringBuffer(20);
		
		int mo = when.getMonth()+1;	// java months start at 0
		int dy = when.getDate();

		result.append(Integer.toString(mo));
		result.append("-");
		result.append(Integer.toString(dy));
		return result.toString();
	}
/**
* Returns a string like "19971201 1433"
*/
	public static String toNumberString(Date when) {
		StringBuffer result = new StringBuffer(20);
		
		int yr = when.getYear()+1900;	// java years start at 1900
		int mo = when.getMonth()+1;	// java months start at 0
		int dy = when.getDate();

		int hr = when.getHours();
		int mi = when.getMinutes();
		int se = when.getSeconds();

		result.append(Integer.toString(yr));
		
		if (mo <= 9)
			result.append('0');
		result.append(Integer.toString(mo));

		if (dy <= 9)
			result.append('0');
		result.append(Integer.toString(dy));
		
		result.append(' ');
		
		if (hr <= 9)
			result.append('0');
		result.append(Integer.toString(hr));

		if (mi <= 9)
			result.append('0');
		result.append(Integer.toString(mi));

		if (se <= 9)
			result.append('0');
		result.append(Integer.toString(se));

		
		return result.toString();
	}

/**
* Reimplements the deprecated method by same signature
*/
	public void setMinutes(int i) {
		super.setMinutes(i);
	}

/**
* Reimplements the deprecated method by same signature
*/
	public void setHours(int s) {
		super.setHours(s);
	}
/**
* Reimplements the deprecated method by same signature.
* Used by BackWebLocalDIM class.
*/
	public int getYear() {
		return super.getYear();
	}
/**
* Reimplements the deprecated method by same signature
* Used by BackWebLocalDIM class.
*/
	public int getMonth() {
		return super.getMonth();
	}
/**
* Reimplements the deprecated method by same signature
* Used by BackWebLocalDIM class.
*/
	public int getDate() {
		return super.getDate();
	}
/**
* Reimplements the deprecated method by same signature.
* UpdateComputer.nextTime()  uses this call.  This method can be removed
* by changing the algorithm in nextTime() to use the
* incrementOneDay() method of this class.  However,
* this was not done to avoid retest to get to iPAM 2.3
*
*/
	public int getDay() {
		return super.getDay();
	}
/**
* Adds one day to the time
*/
	void incrementOneDays() {
		synchronized (calendar) {
			calendar.setTime(this);
			calendar.roll(Calendar.DAY_OF_WEEK, true);
			setTime(calendar.getTime().getTime());
		}
	}
}