/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
/* -----------------6/30/97 1:51PM--------------------------------------------
 * AlarmFrame.java - A quick-and-dirty display for Alarm messages
 * 
 * This module contains a class that can be used to display the error messages
 * generated by the org.mitre.tjt.alarm.Alarm class.
 * -------------------------------------------------------------------------*/

package org.mitre.pam.util.alarm;

import org.mitre.pam.interfaces.AdminTool;
import org.mitre.pam.interfaces.ExecToController;
import java.awt.Frame;
import java.awt.Font;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;
import java.awt.TextArea;
import java.util.Hashtable;

/**
 * Implements a debug window for the Alarm class.  It can be used to provide
 * a quick debug viewport during software development.
 * 
 * @author Tim Taylor, Lead Software Systems Engineer (D510)
 */
public class AlarmFrame extends Frame implements AlarmListener, AdminTool, WindowListener {
  private static final String DEFAULTNAME = "defaultFrame";
  /** The text component that gets the posted alarm message. */
  private TextArea text;

  private static Hashtable allFrames = new Hashtable(10);

  /**
   * Creates the debug window and registers itself as a listener with the
   * Alarm class so that it will be called whenever an alarm is raised.
   */
  public AlarmFrame() {
    this(DEFAULTNAME);
  }

  /**
   * Creates the debug window and registers itself as a listener with the
   * Alarm class so that it will be called whenever an alarm is raised.
   * @param title   A title for the window.
   */
  public AlarmFrame(String title) {
    super(title);
	text = new TextArea();
	text.setFont(new Font("Helvetica",Font.PLAIN, 9));
    add("Center", text);
    setLocation(20,20);
    setSize(350, 250);

    // Make the text field read-only
    text.setEditable(false);

    // Register as a listener for the alarm class...
    allFrames.put(title, this);
    Alarm.addAlarmListener(this);
    addWindowListener(this);
  }

  /**
   * Entry point for the callbacks from the Alarm class.  The text of the
   * alarm message are appended to the text area.
   * @param ae    The alarm event data.  This contains the severity and
   *  message text, along with the category.
   */
  public void alarmMessageReport(AlarmEvent ae) {
//    String name = ae.getCategory();
//    AlarmFrame frame = null;
//    if(name != null) {
//      frame = (AlarmFrame)allFrames.get(name);
//      if(frame != this) return;
//    }
    /*
    * limit size of stuff in the window
    */
    String currentText = text.getText();
    if (currentText.length() > 20000) {
    	text.setText(currentText.substring(10000));
    }
    text.append(ae.getCategory()+" "+ae.getMessage());
  }
/**
* Set tools title
*/
	public void setTitle(String title){
		super.setTitle("Log, "+title);
	}
/**
* Give the tool a handle to the exec -- unused
*/
	public void setExec(ExecToController exec){
	}
/**
* We don't do anything with this event
*/
    public void windowOpened(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowClosed(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowIconified(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowDeiconified(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowActivated(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowDeactivated(WindowEvent e) {}
/**
* Close the window.
*/
	public void windowClosing(WindowEvent e) {
		setVisible(false);		 // hide the Frame
	}

}