/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util.draw;

import java.awt.*;


public class BlockConnector extends GenericConnector{

	int x1,x2,y1,y2;
	double xv, yv; // unit vector
	
	Rectangle boundingRect = new Rectangle();
	DrawParent parent;
	Color color = Color.black;
	int highlighted;

	BlockConnector() {}
	
	public BlockConnector(Block from, Block to) {
		super(from, to);
		this.parent = parent;
		updateBounds();
		
		to.sources.addElement(this);
		from.sinks.addElement(this);
	}
	
	void setParent(DrawParent parent) {
		this.parent = parent;
	}
	
		
	void updateBounds() {
		if (from == null || to == null)
			return;
		x1 = from.boundingRect.x + from.boundingRect.width;
		x2 = to.boundingRect.x;
		y1 = from.boundingRect.y + from.boundingRect.height/2;
		y2 = to.boundingRect.y + to.boundingRect.height/2;
		
		boundingRect.x = Math.min(x1,x2);
		boundingRect.y = Math.min(y1,y2);
		boundingRect.width = Math.abs(x1-x2);
		
		boundingRect.height = Math.abs(y1-y2);
	/*
	* Horizontal lines result in zero height and
	* Vertical lines result in zero width.
	* This makes it impossible later to select the line
	*/
		if (boundingRect.width < 4) {
			boundingRect.width = 4;
			boundingRect.x -= 2;
		}
		if (boundingRect.height < 4) {
			boundingRect.height = 4;
			boundingRect.y -= 2;
		}
	/*
	* compute constants for distance computations
	*/
		xv =  x2 - x1;
		yv =  y2 - y1;
		double lineLength = Math.sqrt(xv*xv+yv*yv);
		if (lineLength <= 1.0)
			lineLength = 1.0;
		xv /= lineLength;
		yv /= lineLength;
	}
	
	void paint(Graphics g) {
		g.drawLine(	x1, y1, x2, y2);
	}
	
	double distanceTo(int x, int y) {		
		return Math.abs(xv*(y - y1) - yv*(x - x1));
	}
	
	public void editProperties() {
//		System.out.println("edit properties for "+this.toString());
	}
	
	void setColor(Color color) {
		this.color = color;
	}
/**
* To highlight a line we set its highlight attribute to a
* rolling magic number.  Thus to unhighlight we just advance
* the magic number -- no need to find each line
* and unhighlight it.  We can build a clean highlight list
* at the next refresh or rehighlight event.
*/
	final public void setHighlighted(int magicNumber) {
		this.highlighted = magicNumber;
	}

/**
* compare to the highlight magic number
*/
	final public boolean isHighlighted(int magicNumber) {
		return this.highlighted == magicNumber;
	}

}