/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util.draw;

import java.awt.*;


public class GenericConnector {

	public Block from;
	public Block to;
	private int code;
	private static int blockId;
	private int connectorId;

	GenericConnector() {
		connectorId = ++blockId;
	}


	GenericConnector(Block from, Block to) {
		this();
		this.from = from;
		this.to = to;
		if (from == null || to == null) {
			throw new RuntimeException("null from or to: "+toString());
		}
    	code = from.hashCode() ^ to.hashCode();
	}
	

	public String toString() {
		String fromName, toName;
		
		if (from == null)
			fromName = "<null>";
		else
			fromName = from.getName();
		if (to == null)
			toName = "<null>";
		else
			toName = to.getName();
		return ("Connector "+connectorId+" from "+fromName+" -to- "+toName);
	}
	
/**
* The hashcode for this connector is computed
* from the things it connects.
* Between this and equals() we can use BlockConnector
* as a key in a hashtable.
*
* @return  a hash code value for this object. 
*/
    public int hashCode() {
    	return code;
    }

/**
* @return true if the from and to Blocks are == (same).
* Between this and hashCode() we can use BlockConnector
* as a key in a hashtable.
*/
    public boolean equals(Object anObject) {
		if (this == anObject) {
		    return true;
		}

		if (anObject instanceof GenericConnector) {
			GenericConnector conn = (GenericConnector) anObject;
    		return from == conn.from && to == conn.to;

    	}
    	return false;
    }

}