/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util.pdifHashtables;
import java.awt.MenuItem;
import java.net.URLEncoder;
import java.util.Hashtable;
import java.util.Enumeration;

import org.mitre.pam.util.PAMDate;
import org.mitre.pam.store.SourceSinkInfo;
import org.mitre.pam.executive.PropertySet;
import org.mitre.pdif.eventParser.ValueList;
import org.mitre.pam.interfaces.Sink;
import org.mitre.pam.interfaces.ConnectedAgent;
import org.mitre.pam.interfaces.Schedulable;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.Memento;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.HasProperties;

import org.mitre.pam.util.MetaData;

/**
 * iPAM specific get/set methods on top of
 * the PDIFNode class.
 *
 * @author Nazario Irizarry
 */
public class IpamPDIFNode extends PDIFNode {
	public static String EXPOSURE_VERB = Agent.EXPOSURE_VERB;
	public static String EXPOSURE_PRIVATE = Agent.EXPOSURE_PRIVATE;
	public static String EXPOSURE_PUBLIC = Agent.EXPOSURE_PUBLIC;	
	public static String ENABLED = "enabled";	
	public static String OWNER = "creator";	
	public static final String LAST_MOD_VERB = "bert_lastModified";
	
	protected boolean useMetaData;
	protected Hashtable metaData;
	

	public IpamPDIFNode(String type, String id, Reporter reporter) {
		super(type, id, new PropertySet(type,id,reporter));
	}
	public IpamPDIFNode(String type, String id, HasProperties hasProperties) {
		super(type, id, hasProperties);
	}
/**
* does this node have inputs
*/
	public boolean hasInputs() {
		return getBooleanProperty(ConnectedAgent.HAS_INPUTS,false);
	}
/**
* does this node have outputs
*/
	public boolean hasOutputs() {
		return getBooleanProperty(ConnectedAgent.HAS_OUTPUTS,false);
	}

/**
* does this node have a schedule
*/
	public boolean hasSchedule() {
		return getBooleanProperty(Schedulable.HAS_SCHEDULE,false);
	}
/**
* is this enabled
*/
	public boolean isEnabled() {
		return getBooleanProperty(ENABLED,false);
 	}

/**
* Remove a property
*/
/*	public void removeProperty(String property) {
		setProperty(property, PropertySet.DELETED);
	}
*/
/**
* identifies a public node
*/
	public boolean isPublic() {
		return getStringProperty(EXPOSURE_VERB,"").equals(EXPOSURE_PUBLIC);
	}

/**
* 
*/
	public boolean isOwned() {
		return getStringProperty(OWNER,"") != "";
	}
	
/**
* identifies whether the node belongs to the owner under test
*/
	static String lastTestedOwner;
	static String lastTestedCodedOwner;
	
	public boolean isOwnedBy(String testOwner) {
		/*
		* "cache" the coding of the name so we don't excessively
		* URLencode the test name
		*/
		if (lastTestedOwner != testOwner) {
			lastTestedCodedOwner = URLEncoder.encode(testOwner);
			lastTestedOwner = testOwner;
		}
		String owner = getOwner();
		return owner.equals(lastTestedCodedOwner);
	}

/**
* Sets the owner.  The owner may be a name with spaces in it.  The
* string is stored internally using URLencoding
*/
	static String lastSetOwner;
	static String lastSetCodedOwner;
	public void setOwner(String owner) {
		/*
		* "cache" the coding of the name so we don't excessively
		* URLencode the test name
		*/
		if (lastSetOwner != owner) {
			lastSetCodedOwner = URLEncoder.encode(owner);
			lastSetOwner = owner;
		}
		setProperty(OWNER,lastSetCodedOwner);
	}

/**
* Get the owner.  Since the name is stored using
* internally using URLencoding, this is not a public method
*/
	private String getOwner() {
		return getStringProperty(OWNER,"");
	}

	public void touch() {
		setProperty(LAST_MOD_VERB, (new PAMDate()).toString());
	}

	


/**
* Stores the dictionary version of the propertyName
*/
	public void setPropertyValues(String propertyName, ValueList values){
		if (propertyName == Memento.MEMENTO_UID)
			return; 
		if (propertyName == Memento.MEMENTO_KIND) 
			return;
		if (propertyName.startsWith("cache.") )
			return;
		if (propertyName.startsWith("meta_attr_") ) {
			if (useMetaData) {
				if (metaData == null)
					metaData = new Hashtable(30);
				String prop = propertyName.substring("meta_attr_".length());
				metaData.put(prop, new MetaData(prop, values));
			}
			return;
		}
		super.setPropertyValues( propertyName,  values);
	}
/**
* Initialize metadata based on meta attributes already contained.
* This can happen when this object is created using an already populated
* HasProperties object
*/
	public void buildMetaDataFromExistingAttributes() {
		useMetaData = true;
		if (metaData == null)
			metaData = new Hashtable(30);
		Enumeration e = getProperties();
		while (e.hasMoreElements()) {
			String attr = (String) e.nextElement();
			if (attr.startsWith("meta_attr_")) {
				String prop = attr.substring("meta_attr_".length());
				metaData.put(prop, new MetaData(prop, getPropertyValues(attr)));
			}
		}
	}
/**
* Stores the dictionary version of the propertyName
*/
	public void setProperty(String propertyName, String value){
		if (propertyName == Memento.MEMENTO_UID)
			return; 
		if (propertyName == Memento.MEMENTO_KIND) 
			return;
		if (propertyName.startsWith("cache.") )
			return;
		super.setProperty(propertyName, value);
	}
/**
* Set the metadata flag
*/
	public void setUseMetadata(boolean flag) {
		useMetaData = flag;
	}
}