/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */

package org.mitre.pam.util.pdifHashtables;


/**
 * Enumerates the nodes of a NodeHashtable based on a specified type.
 * 
 * @version 1, 4 December 97
 * @author Sam Gerstein
 */
import java.util.*;
class NodeHashtableEnumerator implements Enumeration {
/**
 * the type to filter for
*/
	private String type;
/**
 * the enumeration of all elements in the table-
 * these are checked for the desired type as they're processed.
*/
	private Enumeration hashEnum;
/**
 * the current node, if there is one
*/
	private PDIFNode node = null;


/**
 * creates a filtered enumeration based on an enumeration of nodes.
 * @param hashEnum the enumeration to be filtered (elements must be PDIFNode objects)
 * @param type the type to filter for
 * 
*/
NodeHashtableEnumerator(Enumeration hashEnum, String type) {
	this.hashEnum = hashEnum;
	this.type = type;
}
public boolean hasMoreElements() {
	if (node != null)
		return true;
		
	while (hashEnum.hasMoreElements()) {
		node = (PDIFNode)hashEnum.nextElement();
		if (type.equals(node.type))
			return true;
	}
	node = null;	
	return false;
}
public Object nextElement() {
	PDIFNode tmpNode;
	if (hasMoreElements()) {
		tmpNode = node;
		node = null;
		return tmpNode;
	}	else
		throw new NoSuchElementException("NodeHashtableEnumerator");
}
}