/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */

package org.mitre.pam.util.pdifHashtables;

/**
 * a connection between at least two PDIFNode objects
 *
 * @version 2
 * @author Nazario Irizarry, Jr.
 */
public class PDIFConn implements Cloneable {
	public static final int FROM = 0;
	public static final int TO = 1;
/**
 * list of IDs (names) associated with the connection
*/
	public PDIFNode[] nodeList;
/**
 * list of roles the above named objects play
*/
	public String[] roleList;
/**
* The peer for a connector can be anything
*/
	Object peer;

/**
 * default to a size of 2
 */
	public PDIFConn() {
		this(2);
	}	
	public PDIFConn(int numElts) {
		if (numElts <= 0) 
			throw new IllegalArgumentException ("can't create connection with " + numElts + " spots.");
		nodeList = new PDIFNode[numElts];
		roleList = new String[numElts];
	}	
	public Object clone() {
		PDIFConn theClone;
		try {
			 theClone = (PDIFConn)super.clone();
		} catch (CloneNotSupportedException e) {
			throw new InternalError();
		}	
		
		theClone.nodeList = (PDIFNode[])nodeList.clone();
		theClone.roleList = (String[])roleList.clone();
		return theClone;
	}	
	public String toString() {
		String tmp = "[PDIFConn: ";
		for (int i = 0; i < nodeList.length; i++)
			tmp = tmp + "(" + roleList[i] + ":" + nodeList[i].getName() + ") ";
		return tmp + "]";
	}
	
	public int getIndexOfNodeWithEqualsRole(String role) {
		for (int i = 0; i < nodeList.length; i++)
			if (roleList[i].equals(role)) {
				return i;
			}	
		return -1;
	}

	public int getIndexOfNodeWithExactRole(String role) {
		for (int i = 0; i < nodeList.length; i++)
			if (roleList[i] == role) {
				return i;
			}	
		return -1;
	}

	public void setPeer(Object peer) {
		this.peer = peer;
	}

	public Object getPeer() {
		return this.peer;
	}
	
	public PDIFNode getFrom() {
		return nodeList[FROM];
	}
	public PDIFNode getTo() {
		return nodeList[TO];
	}
	public void setFrom(PDIFNode from) {
		nodeList[FROM] = from;
	}
	public void setTp(PDIFNode to) {
		nodeList[TO] = to;
	}
/**
* @return true if the two nodes are equal in that
* they have endpoints with the same names and roles
*/
	public boolean equalsBasedOnNameAndRole(PDIFConn otherConn) {
		if (nodeList.length != otherConn.nodeList.length)
			return false;
		if (nodeList.length == 0)
			return false;
			
		for (int i = nodeList.length-1; i>=0; i-- ) {
			String endName = nodeList[i].name;
			String endRole = roleList[i];
			boolean thisMatch = false;
			for (int j = otherConn.nodeList.length-1; j>=0; j--) {
				// can do == because PDIFRead.makeNewNode ensures same String object
				if (endRole == otherConn.roleList[j] &&
						endName == otherConn.nodeList[j].name) {
					thisMatch = true;
					break;
				}	
			}
			if (!thisMatch)
				return false;
		}
		return true;
	}
	
}