/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util.smtpIn;

import org.mitre.pam.util.Codecs;
import java.io.BufferedReader;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import java.text.ParsePosition;

import java.util.Date;
import java.util.StringTokenizer;
import java.util.Vector;


/**
* This class decomposes MIME formatted mail messages.
*
* If you are unfamiliar with the Multipurpose Internet Mail Extensions (MIME)
* please see all relavent RFC's.
* These documents may be found at "http://www.oac.uci.edu/indiv/ehood/MIME/MIME.html".
*
* An example of a MIME formatted mail message:
*
* Received: from scuba-atm (scubadiver.mitre.org [129.83.66.153])
* by mbunix.mitre.org (8.8.8/8.8.8/mitre.0) with SMTP id KAA29689
* for <test1@jleung-pc.mitre.org>; Mon, 13 Apr 1998 10:16:25 -0400 (EDT)
* Sender: operator@mbunix.mitre.org
* Message-ID: <35321DD3.2820@mitre.org>
* Date: Mon, 13 Apr 1998 14:14:43 +0000
* From: GBS user <jfantry@mitre.org>
* X-Mailer: Mozilla 2.02 (X11; I; SunOS 5.5.1 sun4u)
* MIME-Version: 1.0
* To: test1@jleung-pc.mitre.org
* Subject: test1-2
* Content-Type: multipart/mixed; boundary="------------32797D0F664F"
*
* This is a multi-part message in MIME format.
*
* --------------32797D0F664F
* Content-Type: text/plain; charset=us-ascii
* Content-Transfer-Encoding: 7bit
*
* test1-2
* pam.zip
*
* --------------32797D0F664F
* Content-Type: application/zip
* Content-Transfer-Encoding: base64
* Content-Disposition: inline; filename="PAMClassFiles22i1.zip"
*
* UEsDBBQHAAAAADFGLCMAAAAAAAAAAAAAAAAGAAAAbWl0cmUvUEsDBBQHAAAAADFGLCMAAAAA
* AAAAAAAAAAAKAAAAbWl0cmUvcGFtL1BLAwQUBwAAAAAxRiwjAAAAAAAAAAAAAAAAFQAAAG1p
* dHJlL3BhbS9hZ2dyZWdhdG9yL1BLAwQUBwAAAAAxRiwjE6ZhHJQCAACUAgAAKQAAAG1pdHJl
* L3BhbS9hZ2dyZWdhdG9yL1NpZ25hdHVyZUVudHJ5LmNsYXNzyv66vgADAC0AJgcAJAcAIQcA
* HAcAIAcAHwkAAgALCgADAAoKAAUADAkAAgANDAAeACUMACMAFQwAGQAdDAAaABMBAAR0aGlz
* ZW4uY2xhc3NQSwECFAcUBwAAAAAyRiwjNLLx0/oNAAD6DQAALAAAAAAAAAAAAAAAAACNlQkA
* bWl0cmUvcGFtL3Byb2R1Y3QvdmlzaXQvQXNMb2NhbFByb2R1Y3QuY2xhc3NQSwUGAAAAAM4A
* zgDfQAAA0aMJAAAA
* --------------32797D0F664F--
*
* @author John R. Fantry
* @author Jones Leung, fix null datestamp exception 
*/
public class MIMEd extends GenericMaild {

  protected static final String CONTENT_TYPE = "content-type";
  protected static final String MULTIPART_CONTENT = "multipart";
  protected static final String TEXT_CONTENT = "text";
  protected static final String APPLICATION_TYPE = "application";
  protected static final String OCTET_STREAM_TYPE = "octet-stream";
  protected static final String POSTSCRIPT_TYPE = "postscript";
  protected static final String IMAGE_TYPE = "image";
  protected static final String AUDIO_TYPE = "audio";
  protected static final String VIDEO_TYPE = "video";
  protected static final String MESSAGE_TYPE = "message";
  protected static final String RFC822_TYPE = "rfc822";
  protected static final String PARTIAL_TYPE = "partial";
  protected static final String EXTERNAL_BODY_TYPE = "external-body";
  protected static final String PLAIN_TYPE = "plain";
  protected static final String CONTENT_TRANSFER_ENCODING = "content-transfer-encoding";
  protected static final String BASE64 = "base64";
  protected static final String SEVEN_BIT = "7bit";
  protected static final String CONTENT_DISPOSITION = "content-disposition";
  protected static final String MIME_VERSION = "mime-version";

  protected BufferedReader in;

  protected File MIMEmessage;

  protected String contentType = new String();
  protected String contentSubType = new String();
  protected String boundary = new String();


  /**
  * Constructor.
  *
  * @param mailMessage  The mail message to decompose.
  *
  * @exception IOException  Thrown if there is an error reading the mail message.
  */
  public MIMEd(File MIMEmessage) throws IOException {
    this.MIMEmessage = MIMEmessage;
    processHeader();
    if(contentType.equalsIgnoreCase(MULTIPART_CONTENT)) {
      processMultipart();
    }
  }

  /**
  * This function extracts fields out of the header of the message.
  *
  * @exception IOException  Thrown if there is an error reading the mail message.
  */
  protected void processHeader() throws IOException {
    String line;
    in = new BufferedReader(new FileReader(MIMEmessage));

    // first blank line represents end of header
    while((line = in.readLine()).length() > 0) {
      if(containsField(line)) {
        if(field.equalsIgnoreCase(DATE_FIELD)) {
// JL: Set the date to current date in case it is null            
          date = dateFormatter.parse(value, new ParsePosition(0));
          if (date == null) {
            date = new Date();
          }
        }
        else if(field.equalsIgnoreCase(FROM_FIELD)) {
          from = value;
        }
        else if(field.equalsIgnoreCase(SUBJECT_FIELD)) {
          subject = value;
        }
        else if(field.equalsIgnoreCase(TO_FIELD)) {
          if(value.indexOf(",") < 0) {
            to.addElement(value);
          }
          else {
            StringTokenizer st = new StringTokenizer(value, ",");
            while(st.hasMoreElements()) {
              to.addElement(trim(st.nextToken()));
            }
            in.mark(256);
            line = in.readLine();
            while(line.indexOf(":") < 0) {
              st = new StringTokenizer(line, ",");
              while(st.hasMoreElements()) {
                to.addElement(trim(st.nextToken()));
              }
              in.mark(256);
              line = in.readLine();
            }
            in.reset();
          }
        }
        else if(field.equalsIgnoreCase(CONTENT_TYPE)) {
          contentType = value.substring(0, value.indexOf("/"));
          if(value.indexOf(";") > -1) {
            contentSubType = value.substring(value.indexOf("/") + 1, value.indexOf(";"));
          } else {
            contentSubType = value.substring(value.indexOf("/") + 1, value.length());
          }
          if(contentType.equalsIgnoreCase(MULTIPART_CONTENT)) {
            boundary = value.substring(value.indexOf("\"") + 1, value.length() - 1);
          }
        }
      }
    }
    in.close();
  }

  /**
  * This function will extract the name of an attachment and store it.
  */
  protected void catalogAttachment() {
    int indexOfName = value.indexOf("filename");
    if(indexOfName > -1) {
      int indexOfFirstQuote = value.indexOf("\"", indexOfName);
      int indexOfSecondQuote = value.indexOf("\"", indexOfFirstQuote + 1);
      String attachmentName = value.substring(indexOfFirstQuote + 1, indexOfSecondQuote);
      attachmentNames.addElement(attachmentName);
    } else {
      attachmentNames.addElement(null);
    }
  }

  /**
  * This function parses through a mail message extracting the names of any
  * attachments.
  *
  * @exception IOException  Thrown if there is an error reading the mail message.
  */
  protected void processMultipart() throws IOException {
    String line;
    String type = new String();
    String subType = new String();
    in = new BufferedReader(new FileReader(MIMEmessage));

    // advance past the header and the message body
    advanceAttachment(2);

    // read until we get to the end of the file
    while(((line = in.readLine()) != null) && (line.length() > 0)) {
      if(containsField(line)) {
        if(field.equalsIgnoreCase(CONTENT_TYPE)) {
          type = value.substring(0, value.indexOf("/"));
          if(value.indexOf(";") > -1) {
            subType = value.substring(value.indexOf("/") + 1, value.indexOf(";"));
          }
          else {
            subType = value.substring(value.indexOf("/") + 1);
          }
        }
        else if(field.equalsIgnoreCase(CONTENT_DISPOSITION)) {
          if((type.equalsIgnoreCase(APPLICATION_TYPE)) && (subType.equalsIgnoreCase(OCTET_STREAM_TYPE))) {
            catalogAttachment();
            advanceAttachment(1);
          }
          else if((type.equalsIgnoreCase(APPLICATION_TYPE)) && (subType.equalsIgnoreCase(POSTSCRIPT_TYPE))) {
            catalogAttachment();
            advanceAttachment(1);
          }
          else if((type.equalsIgnoreCase(IMAGE_TYPE))) {
            catalogAttachment();
            advanceAttachment(1);
          }
          else if((type.equalsIgnoreCase(AUDIO_TYPE))) {
            catalogAttachment();
            advanceAttachment(1);
          }
          else if((type.equalsIgnoreCase(VIDEO_TYPE))) {
            catalogAttachment();
            advanceAttachment(1);
          }
          else if((type.equalsIgnoreCase(MESSAGE_TYPE)) && (subType.equalsIgnoreCase(RFC822_TYPE))) {
            catalogAttachment();
            advanceAttachment(1);
          }
          else if((type.equalsIgnoreCase(MESSAGE_TYPE)) && (subType.equalsIgnoreCase(PARTIAL_TYPE))) {
            catalogAttachment();
            advanceAttachment(1);
          }
          else if((type.equalsIgnoreCase(MESSAGE_TYPE)) && (subType.equalsIgnoreCase(EXTERNAL_BODY_TYPE))) {
            catalogAttachment();
            advanceAttachment(1);
          }
          else if((type.equalsIgnoreCase(TEXT_CONTENT)) && (subType.equalsIgnoreCase(PLAIN_TYPE))) {
            catalogAttachment();
            advanceAttachment(1);
          }
          else {
            catalogAttachment();
            advanceAttachment(1);
          }
        }
      }
    }
    in.close();
  }

  /**
  * This function extracts the message body out of the mail message.
  *
  * @return  The body of the mail message.
  *
  * @exception IOException  Thrown if there is an error reading the mail message.
  */
  public String getBody() throws IOException {
    String line;
    StringBuffer body = new StringBuffer();
    in = new BufferedReader(new FileReader(MIMEmessage));

    if(contentType.equalsIgnoreCase(TEXT_CONTENT)) {
      while((line = in.readLine()) != null && line.length() > 0) {
      }
      while((line = in.readLine()) != null) {
        body.append(line + "\n");
      }
    }
    else if(contentType.equalsIgnoreCase(MULTIPART_CONTENT)) {
      int blankLines = 0;
      while(blankLines < 2) {
      	line = in.readLine();
      	if (line == null)
      		break;
        if(line.length() == 0) {
          blankLines++;
        }
      }
      while((line = in.readLine()) != null &&
      		!line.startsWith("--" + boundary)) {
        body.append(line + "\n");
      }
    }
    in.close();
    return body.toString();
  }

  /**
  * This function attempts to extract an attachment that is identified within
  * the mail message by index.  The index is zero based.
  * The attachment is saved into the dst file.
  *
  * @return A String representing the MIME type of the attachment that
  * is extracted.
  *
  * @param attachmentNum  The index of the attachment to extract.
  * @param dst            The destination file for the attachment.
  *
  * @exception IOException  Thrown if there is an error reading the file.
  */
  public String extractFileAttachment(int attachmentNum, File dst) throws IOException {
    String line;
    String encoding = new String();
    String MIMEtype = new String();
    in = new BufferedReader(new FileReader(MIMEmessage));
    advanceAttachment(attachmentNum + 2);
    while((line = in.readLine()).length() > 0) {
      if(containsField(line)) {
        if(field.equalsIgnoreCase(CONTENT_TRANSFER_ENCODING)) {
          encoding = value;
        }
        else if(field.equalsIgnoreCase(CONTENT_TYPE)) {
          if(value.indexOf(";") > -1) {
            MIMEtype = value.substring(0, value.indexOf(";"));
          }
          else {
            MIMEtype = value.substring(0);
          }
        }
      }
    }
    if(encoding.equalsIgnoreCase(BASE64)) {
      BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(dst));
      while(!(line = in.readLine()).startsWith("--" + boundary) && (line.length() > 0)) {
        byte data[] = Codecs.base64Decode(line.getBytes());
        out.write(data);
      }
      out.flush();
      out.close();
    }
    else if(encoding.equalsIgnoreCase(SEVEN_BIT)) {
      BufferedWriter out = new BufferedWriter(new FileWriter(dst));
      while(!(line = in.readLine()).startsWith("--" + boundary)) {
        out.write(line, 0, line.length());
        out.newLine();
      }
      out.flush();
      out.close();
    }
    in.close();
    return MIMEtype;
  }

  /**
  * This function will advance through a given number of attachments.
  *
  * @param num  The number of attachments to advance.
  *
  * @exception IOException  Thrown if there is an error reading the file.
  */
  protected void advanceAttachment(int num) throws IOException {
    while(num > 0) {
        
      String line = in.readLine();
      
// JL: Prevent NullPointerException      
      if (line == null) {
        break;
      }
        
      if(line != null && line.startsWith("--" + boundary)) {
        num--;
      }
    }
  }

  /**
  * This function determines whether or not a given mail message is MIME
  * formatted.
  *
  * @return  True if the mail message is MIME formatted, false otherwise.
  *
  * @param mailMessage  The mail message to check.
  *
  * @exception IOException  Thrown if there is an error reading the file.
  */
  public static boolean isMIMEencoded(File mailMessage) throws IOException {
    String line;
    BufferedReader reader = new BufferedReader(new FileReader(mailMessage));

    // first blank line represents end of header
    while((line = reader.readLine()).length() > 0) {
      if(line.indexOf(":") > -1) {
        String s = line.substring(0, line.indexOf(":"));
        s = trim(s);

        if(s.equalsIgnoreCase(MIME_VERSION)) {
          reader.close();
          return true;
        }
      }
    }
    reader.close();
    return false;
  }
}