/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util.smtpIn;

import java.io.File;
import java.io.IOException;

import java.util.Date;
import java.util.Vector;


/**
* This interface must be implemented by any class that desires to be a
* mail decomposer.
*
* @author John R. Fantry
*/
public interface MailDecomposer {

  /**
  * @return  The date of the mail message.
  */
  public Date getDate();

  /**
  * @return  The from field of the mail message.
  */
  public String getFrom();

  /**
  * @return  The subject line of the mail message.
  */
  public String getSubject();

  /**
  * @return  A list of addresses to whom the message is addressed.  If the
  *          message is only addressed to one person then the list will only
  *          contain one entry.
  */
  public Vector getTo();

  /**
  * @return  True if the message contains attachments, false otherwise.
  */
  public boolean attachmentsExist();

  /**
  * @return  A list of attachment names contained in the message.
  */
  public Vector getAttachmentNames();

  /**
  * @return  The number of attachments contained in the message.
  */
  public int numOfAttachments();

  /**
  * @return  The message body.
  *
  * @exception IOException  Thrown if there is an error reading the mail message.
  */
  public String getBody() throws IOException;

  /**
  * This function attempts to extract an attachment with the given name.
  * The attachment is saved into the dst file.
  *
  * @return  A String representing the MIME type of the attachment that
  * is extracted.
  *
  * @param attachmentName  The name of the attachment to extract.
  * @param dst             The destination file for the attachment.
  *
  * @thorws IOException  Thrown if there is an error reading the file.
  */
  public String extractFileAttachment(String attachmentName, File dst) throws IOException;

  /**
  * This function attempts to extract an attachment that is identified within
  * the mail message by index.  The index is zero based.  The attachment is saved
  * into the dst file.
  *
  * @return  A String representing the MIME type of the attachment that
  * is extracted.
  *
  * @param attachmentNum  The index of the attachment to extract.
  * @param dst            The destination file for the attachment.
  *
  * @thorws IOException  Thrown if there is an error reading the file.
  */
  public String extractFileAttachment(int attachmentNum, File dst) throws IOException;
}