/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pdif.eventParser;

import java.util.Vector;

/** 
* An attribute for a PDIF node may have a single value or it may have
* an ordered sequence of values.
*
* @version  1, 12 Dec 1997
* @author   Nazario Irizarry, Jr.
*/

public class Attribute {
/**
* An attributes name.
*/
	public String name;
/**
* This is the value of an attribute
*/
	ValueList value = ValueList.EMPTY_VALUE_LIST;
/**
* Build an attribute
*
* @param name may not be null
* @param multipleValues may be null or of size == 0; 
* 		if multipleValues.size() == 0 then it is stored as a ValueList of size 0
*/
	Attribute(String name, Vector multipleValues) {
		if (name == null)
			throw new IllegalArgumentException("Attribute name can not be null");
		this.name = name;
		if (multipleValues == null || multipleValues.size() == 0 || multipleValues.elementAt(0) == null) 
			return;
		this.value = new ValueList(multipleValues);
	}
/**
* Build an attribute
* @param name may not be null
* @param singleValue may be ""; if singleValue==null it is stored
* as a ValueList of size 0
*/
	Attribute(String name, String singleValue) {
		this.name = name;
		if (singleValue == null) 
			return;
		this.value = new ValueList(singleValue);
	}
/**
* Gets a single value.  If there were multiple values
* only the first is returned.
*
* @return never null
*/
	public String getValue(){
		try {
			return value.elementAt(0);
		}
		catch (RuntimeException e) {
			return "";
		}
	}
/**
* Gets the multiple values
*
* @return the ValueList, never null; the size of ValueList can be 0
*/
	public ValueList getValueList(){
		return value;
	}
}