/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pdif.eventParser;

/** 
* An end of a connector.  
*
* @version  1, 12 Dec 1997
* @author   Nazario Irizarry, Jr.
*/

public class ConnectorEnd {
/**
* All instances of PDIF information nodes are typed (of a specific kind)
*/
	public String kind = "<unspecified>";
/**
* All instances of PDIF information nodes have a unique id.  The uniqueness
* is at least within nodes of similar kind, but in general should probably
* be unique for all nodes of any kind.
*/
	public String id = "";
/**
* When a node is an endpoint of a relation is has a role that it plays
* in the relation.  In general there can be any number of endpoints in a 
* single relation.  However, the most common is two endpoints.
*/
	public String role;
/**
* A dot concatenated string used in sorting endpoints in a list.
*/
	public String kindRole = "";

/**
* Designed to be used by a higher level PDIF parser.  An example
* input would be "(point.from 39a)".  In this case the 
* kind is "point".  The role is "from". The id is "39a".
* <p>
* On input the call would look like : new ConnectorEnd("point.from", "39a", "from"),
* assuming that "from" is the default role.  Note that the first argument
* is scanned for the LAST dot in the string in case the dot is part of the
* kind name.
* <p>
*
*
* @param kindRole dot concatentated kind-role string
* @param id string identifies the id of the connector end
* @param defaultRole the default role in case kindRole has no '.' in it
*/
	ConnectorEnd(String kindRole, String id, String defaultRole) {
		this.id = id;
		
		int dot = kindRole.lastIndexOf('.');
		if (dot < 0) {
			role = defaultRole;
			kind = kindRole;
		}
		else if (dot == 0) {
			role = kindRole.substring(1);
		}
		else {
			kind = kindRole.substring(0, dot);
			role = kindRole.substring(dot+1);
		}
		
		this.kindRole = kind + '.' + role;
	}
}