/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package ORG.mitre.pdif;
//import ORG.mitre.pdif.*;
import java.io.*;

public class TestPDIF {

	public static void main(String args[]) throws Exception{
		File file = new File("pdif.txt");
		PrintWriter p = new PrintWriter(new FileOutputStream(file));
		createPDIF2(p);
		p.close();
		
		InputStream s = new FileInputStream(file);
		
//		testParse2(s);
		testParse3(s);
	}
	
	private static void testParse3(InputStream streamIn) throws Exception {
		ParseEvent3 parse = new ParseEvent3(streamIn);
		int event;
		
		event = parse.next();
		while (event != ParseEvent3.EOF) {
			switch (event) {
				case ParseEvent3.EOF :
					println("EOF");
					return;

				case ParseEvent3.END_OF_PDIF_BLOCK :
					println("END_OF_PDIF_BLOCK\n");
					break;

				case ParseEvent3.NODE_ENTER :
					print("NODE ENTER ");
					print(" ["+parse.node.kind);
					println(", "+parse.node.id+"]");
					break;

				case ParseEvent3.NODE_ATTRIBUTE :
					print("    NODE ATTRIBUTE ");
					print(parse.attribute.name);
					print(" = "+parse.attribute.value);
					print(" for node ");
					print(" ["+parse.node.kind);
					println(", "+parse.node.id+"]");
					if (parse.attribute.multipleValues.size() > 1) {
						printLevel(3);
						print( "    multvalue = ");
						for (int i = 0; i< parse.attribute.multipleValues.size(); i++) {
							print((String)parse.attribute.multipleValues.elementAt(i));
							print(" ");
						}
						println("");
					}
					break;

				case ParseEvent3.HEADER_ATTRIBUTE :
					print("HEADER ATTRIBUTE ");
					print(parse.attribute.name);
					println(" = "+parse.attribute.value);
					if (parse.attribute.multipleValues.size() > 1) {
						printLevel(3);
						print( "    multvalue = ");
						for (int i = 0; i< parse.attribute.multipleValues.size(); i++) {
							print((String)parse.attribute.multipleValues.elementAt(i));
							print(" ");
						}
						println("");
					}
					break;
				
				case ParseEvent3.NODE_LEAVE :
					print(" NODE LEAVE ");
					print(" kind = "+parse.node.kind);
					println("  id = "+parse.node.id);
					break;
				
				case ParseEvent3.NON_NODE :
					print("NON NODE ");
					print(" kind = "+parse.node.kind);
					println("  id = "+parse.node.id);
					break;

				case ParseEvent3.NON_CONNECTOR :
					print("NON CONNECTOR ");
					print(" kind = "+parse.connectorEnds[0].kind);
					print(" role = "+parse.connectorEnds[0].role);
					print(" kindRole = "+parse.connectorEnds[0].kindRole);
					println("  id = "+parse.connectorEnds[0].id);
					print("              ");
					print(" kind = "+parse.connectorEnds[1].kind);
					print(" role = "+parse.connectorEnds[1].role);
					print(" kindRole = "+parse.connectorEnds[1].kindRole);
					println("  id = "+parse.connectorEnds[1].id);
					break;

				case ParseEvent3.CONNECTOR :
					print("CONNECTOR ");
					print(" kind = "+parse.connectorEnds[0].kind);
					print(" role = "+parse.connectorEnds[0].role);
					print(" kindRole = "+parse.connectorEnds[0].kindRole);
					println("  id = "+parse.connectorEnds[0].id);
					print("          ");
					print(" kind = "+parse.connectorEnds[1].kind);
					print(" role = "+parse.connectorEnds[1].role);
					print(" kindRole = "+parse.connectorEnds[1].kindRole);
					println("  id = "+parse.connectorEnds[1].id);
					break;
			}
			event = parse.next();
		}
	}

	private static void testParse2(InputStream streamIn) throws Exception {
		ParseEvent2 parse = new ParseEvent2(streamIn);
		int event;
		
		event = parse.next();
		while (event != ParseEvent2.EOF) {
			switch (event) {
				case ParseEvent2.EOF :
					println("EOF");
					return;

				case ParseEvent2.ENTER :
					printLevel(parse.depth);
					print("ENTER ");
					print(" kind = "+parse.arg1);
					println("  value = "+parse.arg2);
					if (parse.multiValues.size() > 1) {
						printLevel(parse.depth);
						print( "    multvalue = ");
						for (int i = 0; i< parse.multiValues.size(); i++) {
							print((String)parse.multiValues.elementAt(i));
							print(" ");
						}
						println("");
					}
					break;
				
				case ParseEvent2.LEAVE :
					printLevel(parse.depth);
					print("LEAVE ");
					print(" kind = "+parse.arg1);
					println("  value = "+parse.arg2);
					if (parse.multiValues.size() > 1) {
						printLevel(parse.depth);
						print( "    multvalue = ");
						for (int i = 0; i< parse.multiValues.size(); i++) {
							print((String)parse.multiValues.elementAt(i));
							print(" ");
						}
						println("");
					}
					break;
				
			}
			event = parse.next();
		}
	}

	private static void createPDIF2(PrintWriter p) {
		p.println("(pdif (header (source naz)(date 1997121208234506))");
		p.println("(node ");
		p.println("(-point a (height 3)(displayAttributes bold pink \"blue\")) ");
		p.println("(point b (displayBars j k %%the%20rain%20in%20spain)) ");
		p.println(")) ");

		p.println("(foobar this section is not a pdif section (noheader (source john))");
		p.println("(node ");
		p.println("(bus c12 (height 3)(displayAttributes silver metallic \"orange\")) ");
		p.println("(bus d34 (owner john@host.org)) ");
		p.println(")) ");

		p.println("(pdif (header (source john))");
		p.println("(node ");
		p.println("(car c12 (height 3)(displayAttributes silver metallic \"orange\")) ");
		p.println("(car d34 (owner john@host.org)) ");
		p.println("(car e89 (owner mary@host.org)) ");
		p.println("(garage john (owner john@host.org)) ");
		p.println("(garage mary (owner john@host.org)) ");
		p.println(") ");
		p.println("(connector ");
		p.println("(garage.container john (car.contained d34)(car.contained c12)) ");
		p.println("(garage.container mary (car.contained e89)) ");
		p.println(")) ");
	}
	
	private static void println(String s) {
		System.out.println(s);
	}
	private static void print(String s) {
		System.out.print(s);
	}
	private static void printLevel(int n) {
		print("["+n+"]");
		for(int i = -1; i<=n; i++)
			print("   ");
	}

}