package org.mitre.pam.daemon;

/**
* This class creates an alarm Daemon that delays setting a property,
* which can trigger any agent waiting for this event to occur.
* When fired, the <code>setString</code> property is copied
* into the <code>property</code>, which can be watched by other
* agents/modules to perform a given action.
*
* Copyright (c)1998 The Mitre Corporation, All rights reserved
*
* @author Jason Mathews
*/
public class DelayedSetProperty extends DaemonBase {
  protected final static String SET_TIME = "setTime";
  protected final static String SET_STRING = "setString";
  protected final static String TARGET = "target";

  public DelayedSetProperty() {
    setProperty(SET_TIME, "");
    setProperty(SET_STRING, "");
    setProperty(TARGET, "");
    setPeriod(60);
  }

  public void doDaemonTask() {

    String timeStr = stateMemento.getProperty(SET_TIME);
    if (timeStr == null || timeStr.length() == 0) {
      log("DelayedSetProperty: Missing alarm time");
      return;
    }

    long time;
    try {
      time = Long.parseLong(timeStr);
    }
    catch (NumberFormatException nfe) {
      reportException(nfe);
      return;
    }
    
    long current = System.currentTimeMillis();

    // Check if alarm has triggered

    if (current >= time) {
	// copy the set string into the target property
	String setString = stateMemento.getProperty(SET_STRING);
	stateMemento.setProperty(TARGET, setString);

	// alarm triggered, payload delivered, now it's time to go to sleep
	setProperty("enabled", "false");
    }
  }

}
