/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.daemon.rules;

import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.interfaces.ExecToMonitor;
import org.mitre.pam.interfaces.Reporter;

import jess.Context;
import jess.Funcall;
import jess.ReteException;
import jess.RU;
import jess.Userfunction;
import jess.Value;
import jess.ValueVector;

/**
* Jess userfunction to determine if a PAM agent exists by its id
*
*    (agentExists <agent-id>)
*
* argument <agent-id> must be an atom or a string
*
* @returns TRUE if the agent exists, otherwise FALSE.
*
* @author Jason Mathews
*/

class AgentExists implements Userfunction {

    protected ExecToMonitor exec;
    protected Reporter reporter;

    AgentExists(Reporter reporter, ExecToMonitor exec) {
		this.exec = exec;
		this.reporter = reporter;
    }

    public String name() { 
	return "agentExists"; 
    }

    public Value call(ValueVector vv, Context context)
    {
	//    System.out.println("agentExists size=" + vv.size() + " call=" + vv);
	if (vv.size() != 2) {
	    return Funcall.FALSE();
	}

	try {

	  // convert argument to string
	  String name = vv.get(1).stringValue();

	  if (exec.getActiveAgent(name) != null) {
	      //System.out.println("agent "+name+" is defined");
	      return Funcall.TRUE();
	  } else {
	      //System.out.println("agent "+name+" is NOT defined");
	      return Funcall.FALSE();
	  }

	} catch (ReteException re) {
	    reporter.reportException( re);
	    return Funcall.FALSE();
	}
    }
}
