package org.mitre.pam.daemon.rules;

import java.util.Enumeration;
import java.util.Vector;

import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.DIM;
import org.mitre.pam.interfaces.Juggler;
import org.mitre.pam.interfaces.PIM;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.ExecToEditor;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.interfaces.Reporter;

import jess.Context;
import jess.Funcall;
import jess.Rete;
import jess.ReteException;
import jess.RU;
import jess.Value;
import jess.ValueVector;
import jess.Userfunction;


/**
* This JESS extension will assert a given agent and all relavent connections.  If you
* assert a Juggler than all of its PIM's and DIM's will also be asserted.  If you assert
* a PIM or a DIM then all the Juggler's that are connected to that agent will be
* asserted along with all the additional PIM's and DIM's connected to the newly
* asserted Juggler.
*
* @author John R. Fantry
*/
class AssertAgentAndConnections implements Userfunction  {

	private final static String m_name = "assertAgentAndConnections";
	ExecToEditor exec;
	Reporter reporter;
	AssertAgent assertAgent;

  AssertAgentAndConnections(AssertAgent assertAgent, Reporter reporter, ExecToEditor exec) {
	this.assertAgent = assertAgent;
	this.exec = exec;
	this.reporter = reporter;
  }

  public Value call(ValueVector vv, Context context) throws ReteException {

    if (vv.size() < 2) {
	reporter.warning(m_name + " called with invalid parameters");
	return Funcall.FALSE();
    }

    String agentId = vv.get(1).stringValue();

    Agent agent = exec.getActiveAgent(agentId);

    // Make sure agent exists
    if (agent == null)
      return Funcall.FALSE();

    Vector assertList = buildAssertList(agent);

    for (int i=0; i < assertList.size(); i++) {
      assertAgent.assertOneAgent((Agent)assertList.elementAt(i), context);
    }

    return Funcall.TRUE();
  }

  public String name() {
    return m_name;
  }

  protected Vector buildAssertList(Agent agent) {
    Vector assertList = new Vector();
    assertList.addElement(agent);

    if(agent instanceof PIM) {
      Enumeration enum = exec.getListOfAgentSinks(agent.getId());
      while(enum.hasMoreElements()) {
        Vector temp = buildAssertList((Agent)enum.nextElement());
        for(int i=0; i < temp.size(); i++) {
          Agent a = (Agent)temp.elementAt(i);
          if(!assertList.contains(a)) {
            assertList.addElement(a);
          }
        }
      }
    }

    else if(agent instanceof DIM) {
      Enumeration enum = exec.getListOfAgentSources(agent.getId());
      while(enum.hasMoreElements()) {
        Vector temp = buildAssertList((Agent)enum.nextElement());
        for(int i=0; i < temp.size(); i++) {
          Agent a = (Agent)temp.elementAt(i);
          if(!assertList.contains(a)) {
            assertList.addElement(a);
          }
        }
      }
    }

    else if(agent instanceof Juggler) {
      Enumeration LHS = exec.getListOfAgentSources(agent.getId());
      Enumeration RHS = exec.getListOfAgentSinks(agent.getId());
      while(LHS.hasMoreElements()) {
        assertList.addElement((Agent)LHS.nextElement());
      }
      while(RHS.hasMoreElements()) {
        assertList.addElement((Agent)RHS.nextElement());
      }
    }

    return assertList;
  }
}