/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.daemon.rules;

import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;

import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.ExecToMonitor;

import jess.Context;
import jess.Funcall;
import jess.Rete;
import jess.ReteException;
import jess.RU;
import jess.Value;
import jess.ValueVector;
import jess.Userfunction;
import jess.Resetable;

import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.ExecToEditor;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.interfaces.Reporter;

/**
 * This JESS extension will assert all active modules that have
 * have certain specified attributes (e.g. creator -> "mathews")
 *
 * @author Jason Mathews
 */
class AssertAgentAttributes implements Userfunction {

    private final static String m_name = "assertAgentWithAttribute";
    ExecToEditor exec;
    Reporter reporter;
    AssertAgent assertAgent;

    AssertAgentAttributes(AssertAgent assertAgent, Reporter reporter, ExecToEditor exec) {
		this.assertAgent = assertAgent;
		this.exec = exec;
		this.reporter = reporter;
    }

    // (assertAgentWithAttribute attrname attrvalue [attrname attrvalue]* )

    public Value call(ValueVector vv, Context context) throws ReteException {

	// size >= 3 and odd number
	if (vv.size() < 3 || vv.size() %2 == 0) {
	    reporter.warning(m_name + " called with invalid parameters");
	    return Funcall.FALSE();
	}

	Hashtable props = new Hashtable();

	for (int i=1; i < vv.size(); i += 2) {
	    String property = vv.get(i).stringValue();
		
	    if (property == null || property.length()==0) {
		reporter.log("invalid attribute name");
		 continue;
	    }

	    String value = vv.get(i+1).stringValue();

	    if (value == null) {
		reporter.log("invalid attribute value for " + property);
		 continue;
	    }
 reporter.debugMsg("property ", property, ": ", value, m_name);
	    props.put(property, value);
	}

	if (props.size() != 0)
	    addProperties(props, context);
	
	return Funcall.TRUE();
    }

    private int addProperties(Hashtable props, Context context) throws ReteException {
	Vector assertList = buildAssertList(props);
		
	for(int i=0; i < assertList.size(); i++) {
	    Agent agent = (Agent) assertList.elementAt(i);
reporter.debugMsg(m_name , agent,null,null,m_name); // debug
	    assertAgent.assertOneAgent(agent, context);
	}

	return assertList.size();
    }

    /*
      private int addAgentsOnReset(Hashtable props) throws ReteException {
      Vector assertList = buildAssertList(props);
		
      for(int i=0; i < assertList.size(); i++) {
       Agent agent = (Agent) assertList.elementAt(i);
       assertAgent.assertOneAgent(agent, false);
      }
      return assertList.size();
      }
    */
	
    public String name() {
	return m_name;
    }

    protected Vector buildAssertList(Hashtable props) {
	Vector assertList = new Vector();

	int types[] = new int[] { ExecToMonitor.PIM, ExecToMonitor.DIM,
				  ExecToMonitor.JUGGLER, ExecToMonitor.DAEMON };

 reporter.debugMsg("build assert list ", props,null,null,m_name);
	for(int i=0; i < types.length; i++) {
	    Enumeration enum = exec.getListOfActiveModules(types[i]);
	    while (enum.hasMoreElements()) {
		Agent agent = (Agent) enum.nextElement();
		boolean addToList = true;
		for (Enumeration e=props.keys(); e.hasMoreElements(); ) {
		    String property = (String) e.nextElement();
		    String val = (String) props.get(property);
		    // check if product matches
		    String agentval = agent.getProperty(property);
		    if (agentval == null || !agentval.equals(val)) {
			// reporter.log(agent + " " + property + ":" + val + ":" + agentval);
			addToList = false;
			break;
		    }
		}
		if (addToList) {
			// reporter.log(agent + " added");
		    assertList.addElement(agent);
		}
	    }
	}

	return assertList;
    }

/*
       public void reset() throws ReteException
      {
      Enumeration e = assertAgent.acceptablepropertys.elements();
      while (e.hasMoreElements())
      {
      String property = (String) e.nextElement();
      addAgentsOnReset(property);
      }

      }
*/

}
