package org.mitre.pam.daemon.rules;

import java.util.Enumeration;
import java.util.Vector;

import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.ExecToMonitor;


import jess.Context;
import jess.Funcall;
import jess.Rete;
import jess.ReteException;
import jess.RU;
import jess.Value;
import jess.ValueVector;
import jess.Userfunction;
import jess.Resetable;

import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.ExecToEditor;
import org.mitre.pam.executive.ExecGlobal;


/**
* This JESS extension will assert all active modules of the given type (i.e. passThru).
*
* @author John R. Fantry
*/
class AssertKind implements Userfunction, Resetable {

	private final static String m_name = "assertAgentOfKind";
	ExecToEditor exec;
	AssertAgent assertAgent;

  AssertKind(AssertAgent assertAgent) {
		this.assertAgent = assertAgent;
		exec = ExecGlobal.getExec();
  }

	public Value call(ValueVector vv, Context context) throws ReteException {
	
		if (vv.size() < 2)
			return Funcall.FALSE();
	
		String kind = vv.get(1).stringValue();
		
		if (kind == null)
			return Funcall.FALSE();
		
		assertAgent.addAcceptableKind(kind);
		
		addKind(kind, context);
	
		return Funcall.TRUE();
	}

	private int addKind(String kind, Context context) throws ReteException {
		Vector assertList = buildAssertList(kind);
		
		for(int i=0; i<assertList.size(); i++) {
			Agent agent = (Agent)assertList.elementAt(i);
			assertAgent.assertOneAgent(agent, context);
		}
		return assertList.size();
	}

	private int addKindOnReset(String kind) throws ReteException{
		Vector assertList = buildAssertList(kind);
		
		for(int i=0; i<assertList.size(); i++) {
			Agent agent = (Agent)assertList.elementAt(i);
			assertAgent.assertOneAgent(agent,false);
		}
		return assertList.size();
	}
	
  public String name() {
    return m_name;
  }

  protected Vector buildAssertList(String kind) {
    Vector assertList = new Vector();

    Enumeration enums[] = new Enumeration[4];
    enums[0] = exec.getListOfActiveModules(ExecToMonitor.PIM);
    enums[1] = exec.getListOfActiveModules(ExecToMonitor.DIM);
    enums[2] = exec.getListOfActiveModules(ExecToMonitor.JUGGLER);
    enums[3] = exec.getListOfActiveModules(ExecToMonitor.DAEMON);

    for(int i=0; i<enums.length; i++) {
      Enumeration enum = enums[i];
      while(enum.hasMoreElements()) {
        Agent agent = (Agent)enum.nextElement();
        if(agent.getKind().equals(kind)) {
          assertList.addElement(agent);
        }
      }
    }
    return assertList;
  }

	public void reset() throws ReteException
	{
		Enumeration e = assertAgent.acceptableKinds.elements();
		while (e.hasMoreElements())
		{
			String kind = (String) e.nextElement();
			addKindOnReset(kind);
		}
	}
}