package org.mitre.pam.daemon.rules;

import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.interfaces.Agent;
import jess.Rete;
import jess.ReteException;

/**
* A thread that waits a small time from the time of notification
* and the runs jess.  Used so that we wait until some changes
* have accumulated before running Jess.
*
* @author Nazario Irizarry, Jr. 3/21/98
*/
class AutoRun extends Thread {
	SyncSemaphore sync;
	Rete rete;
	int exceptionCount;
	int errorThreshold = 5;
	Agent parent;
	static final String JESS_ERRORS = "jessErrors";
	static final String ERROR_THRESHOLD = "errorThreshold";
	static final String USE_AUTORUN = "useAutoRun";

	AutoRun(SyncSemaphore sync, Rete rete, Agent parent) {
		this.sync = sync;
		this.rete = rete;
		this.parent = parent;
	}
	
	public void run() {
		while(true) {
			try {
				sync.waitForTrigger();
			}
			catch (InterruptedException ignore) {}

			try {
				sleep(5000);
			}
			catch (InterruptedException ignore) {}
			
			try {
				sync.clear(); // clear again in case triggered during sleep
				rete.run();
			}
			catch (ReteException re) {
				exceptionCount++;
				parent.getReporter().reportException(re);
				parent.setProperty(JESS_ERRORS, Integer.toString(exceptionCount));
				if (exceptionCount > errorThreshold) {
					sync.setAutorun(false);
					parent.setProperty(JESS_ERRORS, Integer.toString(exceptionCount));
					parent.getReporter().log("error threshold exceeded, stopping autorun");
				}
			}
		}
	}

	int getErrorCount() {
		return exceptionCount;
	}
	void clearErrorCount() {
		exceptionCount = 0;
		parent.setProperty(JESS_ERRORS, "0");
	}
	void setErrorThreshold(int n) {
		errorThreshold = n;
	}
}