/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */

package org.mitre.pam.daemon.rules;

import java.util.Vector;
import java.util.Hashtable;

import org.mitre.pam.daemon.rules.AssertAgent;
import org.mitre.pam.interfaces.ExecToEditor;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pdif.eventParser.ValueList;
import org.mitre.pam.interfaces.Daemon;
import org.mitre.pam.interfaces.Memento;

import jess.Context;
import jess.Funcall;
import jess.RU;
import jess.ReteException;
import jess.Userfunction;
import jess.Value;
import jess.ValueVector;

/**
* Implements PAM agent editing functions as a single Jess userfunction.
*
* @author Nazario Irizarry
* @author Jason Mathews Fixed size check in setProperty which makes setting setProperty
* enabled true/false use setPropertyValues not setProperty and is not passed down to
* the setEnabled method
*/

class EditAgent implements Userfunction
{
	private String m_name = "editAgent";
	protected ExecToEditor exec;
	private static Hashtable defaultAttributes;

	EditAgent() {
		exec = ExecGlobal.getExec();
//		defaultAttributes = exec.makeNewMemento(this);
//		defaultAttributes.setProperty("enabled","false");
	}

	public String name() { 
		return m_name;
	}

	private String getAsString(Value value) throws ReteException {
		if ((value.type() & RU.STRING) != 0) {
			return value.stringValue();
		}
		return value.toString();
	}

	public static Hashtable getDefaultAttributes() {
		if (defaultAttributes == null) {
			defaultAttributes = new Hashtable();
			defaultAttributes.put("enabled","false");
		}
		return defaultAttributes;
	}

/**
* <pre>
* syntax:  editAgent <agent-id> removeProperty <propertyName> 
* syntax:  editAgent <agent-id> setProperty <propertyName> <propertyValue>
* syntax:  editAgent <agent-id> createOfType <agentType>
* syntax:  editAgent <agent-id> destroy
* syntax:  editAgent <agent-id> connectOutputTo <other-agent-id>
* syntax:  editAgent <agent-id> disconnectOutputTo <other-agent-id>
* syntax:  editAgent <agent-id> activateASAP
* </pre>
*/

	public Value call(ValueVector vv, Context context) throws ReteException
	{
		int vvSize = vv.size();
		
		if (vvSize < 3) {
			throw new ReteException("editAgent","there must be at least 2 arguments to editAgent","");
		}
	    String agentId = vv.get(1).stringValue();
	    String command = vv.get(2).stringValue();
	    
		Agent agent = null;
		if (!command.equals("createOfType")) {
	        agent =  exec.getActiveAgent(agentId);
		    if (agent == null)
		    	throw new ReteException("editAgent","no such existing agent: ",agentId);
		}

		try {
			if (command.equals("removeProperty")) {
				if (vvSize < 4) {
					throw new ReteException("editAgent removeProperty","arg 3, <propertyName>, must not be null","");
				}
				String property = vv.get(3).stringValue();
				agent.removeProperty(property);
			}
			else if (command.equals("setProperty")) {
				if (vvSize < 5) {
					throw new ReteException("editAgent setProperty","args 3&4, <propertyName> <value>, must not be null","");
				}
				String property = vv.get(3).stringValue();
				// e.g. (editAgent testAgent setProperty enabled true) :: size = 5
				if (vvSize == 5) {
					agent.setProperty(property, getAsString(vv.get(4)));
				}
				else {
				// e.g. (editAgent testAgent setProperty list 1 2 3) :: size = 7
					Vector vectorList = new Vector(vvSize-3);
					for (int i = 4; i < vvSize; i++) {
						vectorList.addElement(getAsString(vv.get(i)));
					}
					agent.setPropertyValues(property, new ValueList(vectorList));
				}
			}
			else if (command.equals("createOfType")) {
				if (vvSize < 4) {
					throw new ReteException("editAgent createOfType","arg 3, <type>, must not be null","");
				}
				String type = vv.get(3).stringValue();
				exec.declareNode(type, agentId, getDefaultAttributes());
			}
			else if (command.equals("destroy")) {
				exec.deleteNode(agentId);
			}
			else if (command.equals("activateASAP")) {
				if (agent instanceof Daemon) {
					((Daemon)agent).activateASAP();
				}
				else {
					throw new ReteException("editAgent activateASAP",agentId,"is not of a type that can be activated");
				}
			}
			else if (command.equals("connectOutputTo")) {
				if (vvSize < 4) {
					throw new ReteException("editAgent connectOutputTo","arg 3, <otherAgentId>, must not be null","");
				}
				String otherAgent = vv.get(3).stringValue();
				exec.connectNodes(agentId, otherAgent);
			}
			else if (command.equals("disconnectOutputTo")) {
				if (vvSize < 4) {
					throw new ReteException("editAgent disconnectOutputTo","arg 3, <otherAgentId>, must not be null","");
				}
				String otherAgent = vv.get(3).stringValue();
				exec.disconnectNodes(agentId, otherAgent);
			}
			else {
				throw new ReteException("editAgent","no such command:",command);
			}
		}
		catch (org.mitre.pam.store.IntegrityException e) {
			return Funcall.FALSE();
		}

		
		return Funcall.TRUE();
	}
	
	public String getKind(){
		return "EditAgent";
	}
	public String getId(){
		return "EditAgentGenericIdShouldNeverShowUp";
	}
}
