/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */


import java.io.File;
import java.io.IOException;
import javax.servlet.*;


import org.mitre.pam.interfaces.IPamPlugin;
import org.mitre.pam.interfaces.ExecToController;
import org.mitre.pam.interfaces.ExecToEditor;
import org.mitre.pam.interfaces.ExecCallbacks;
import org.mitre.pam.interfaces.Reporter;


import org.mitre.pam.executive.Exec;
import org.mitre.pam.executive.ExecGlobal;


import org.mitre.pam.monitor.*;

import org.mitre.pam.aggregator.approve.SrvProductApprove;
import org.mitre.pam.daemon.BackWebSubscriber;
import javax.servlet.http.HttpServlet;
import Acme.Serve.FileServlet;
import org.mitre.pam.wizard.StartWizard;
import org.mitre.pam.util.StreamCopy;

/**
* This module follows the iPAM plugin interface for defining
* pam agents.  The name (including case) must match the name
* of the .zip or .jar file that contains it.  
* isOkToLoad() will always be called first.
*
* @author Nazario Irizarry, Jr.
*/
public class PAMStandard implements IPamPlugin{
	static int BUILD=107;

/**
* Define the mapping between module aliases and java classes.  
* This is for the standard extensions above and beyond any already declared by iPAM core.
* There may be additional aliases to the same java classes here or in the startup file.
* <p>
* This method is optional.
*
*/
	public static void defineModules(ExecToController exec) {		
	/*
	* Test PIMs
	*/
		exec.defineModuleAlias("getPIM1", "org.mitre.pam.getter.TestPIM1");
		exec.defineModuleAlias("getPIMComplex", "org.mitre.pam.getter.TestPIMComplex");
		exec.defineModuleAlias("getPIMExpired", "org.mitre.pam.getter.TestPIMExpired");
		exec.defineModuleAlias("getPIMIndexed", "org.mitre.pam.getter.TestPIMIndexed");
		exec.defineModuleAlias("getPIMByteArray", "org.mitre.pam.getter.TestPIMByteArray");
		exec.defineModuleAlias("getPIMContinuous", "org.mitre.pam.getter.TestAsyncSource");
		exec.defineModuleAlias("getPIMDictionary", "org.mitre.pam.getter.TestPIMDict");
		exec.defineModuleAlias("getPIMURL", "org.mitre.pam.getter.TestPIMUrl");
		exec.defineModuleAlias("acceptAIM", "org.mitre.pam.service.aim.AIMPIM");
	/*
	* PIMs
	*/
		exec.defineModuleAlias("getOnePage", "org.mitre.pam.getter.webcopy.WebCopyPIM0");
		exec.defineModuleAlias("getRootPlusReferences", "org.mitre.pam.getter.webcopy.WebCopyPIM1");
		exec.defineModuleAlias("getDirectory", "org.mitre.pam.getter.GetDirectory");
		exec.defineModuleAlias("getFTPDirectory", "org.mitre.pam.getter.GetFTPDirectoryList");
		exec.defineModuleAlias("getAltavista", "org.mitre.pam.getter.search.altavista.AltavistaPIM2");
		exec.defineModuleAlias("getPOPMail", "org.mitre.pam.getter.GetPOPMail");
		exec.defineModuleAlias("getURL", "org.mitre.pam.getter.GetURL");
		exec.defineModuleAlias("getExciteByKeyword", "org.mitre.pam.getter.search.excite.ExciteKeywordPIM");
		exec.defineModuleAlias("getExciteNewstracker", "org.mitre.pam.getter.search.excite.ExciteNewstrackerPIM");
		exec.defineModuleAlias("getExcite", "org.mitre.pam.getter.search.excite.ExciteNewstrackerPIM"); // old name
		exec.defineModuleAlias("getStockQuote", "org.mitre.pam.getter.GetQuotePIM");
		exec.defineModuleAlias("getBNN", "org.mitre.pam.getter.search.bnn.BNNPIM3");
		exec.defineModuleAlias("getSmarTraveler", "org.mitre.pam.getter.search.smarTraveler.TravelerPIM");

	/*
	* Asynchronous sources
	*/
		exec.defineModuleAlias("acceptSMTPMail", "org.mitre.pam.getter.GetAsyncMail");
		exec.defineModuleAlias("acceptTableOfContents", "org.mitre.pam.getter.TableOfContents"); //ni42
		exec.defineModuleAlias("tableOfContents", "org.mitre.pam.getter.TableOfContents"); // old name
		exec.defineModuleAlias("acceptRemoteConnector", "org.mitre.pam.getter.remote.Connector"); //ni42
		exec.defineModuleAlias("remoteConnector", "org.mitre.pam.getter.remote.Connector"); // old name
		exec.defineModuleAlias("acceptUDP", "org.mitre.pam.getter.GetAsyncUDP");
		exec.defineModuleAlias("acceptSerializedProduct", "org.mitre.pam.getter.remote.AcceptSerializedProduct");

	/*
	* jugglers
	*/
		exec.defineModuleAlias("notifyOfChange", "org.mitre.pam.aggregator.detectChange.NotifyOfChange2");
		exec.defineModuleAlias("notifyOfDateChange", "org.mitre.pam.aggregator.detectChange.NotifyOfDateChange");
		exec.defineModuleAlias("notifyOfFTPDateChange", "org.mitre.pam.aggregator.detectChange.NotifyOfDateChangeFTP");
		exec.defineModuleAlias("passThru", "org.mitre.pam.aggregator.PassThru"); // lowercase first letter in passThru
		exec.defineModuleAlias("PassThru", "org.mitre.pam.aggregator.PassThru"); // uppercase first letter in PassThru
		exec.defineModuleAlias("approve", "org.mitre.pam.aggregator.approve.Approve");
		exec.defineModuleAlias("wrapAsZip", "org.mitre.pam.aggregator.WrapAsZip");
		exec.defineModuleAlias("unwrapFromZip", "org.mitre.pam.aggregator.UnwrapFromZip");
		exec.defineModuleAlias("expireOldProducts", "org.mitre.pam.aggregator.CheckExpiration");
		exec.defineModuleAlias("archive", "org.mitre.pam.aggregator.Archive");
		exec.defineModuleAlias("headlinesOnly", "org.mitre.pam.aggregator.HeadlinesOnly");
		exec.defineModuleAlias("switch", "org.mitre.pam.aggregator.Switch");
		exec.defineModuleAlias("passOneRandomProduct", "org.mitre.pam.aggregator.PassOneRandomProduct");
		exec.defineModuleAlias("passIfKeywordsMatch", "org.mitre.pam.aggregator.KeyMatch");
		exec.defineModuleAlias("forwardAsURL", "org.mitre.pam.aggregator.ForwardAsUrl");
		exec.defineModuleAlias("flattenProductHierarchy", "org.mitre.pam.aggregator.FlattenHierarchy");
		exec.defineModuleAlias("filterSubproductByKeyword", "org.mitre.pam.aggregator.PassSubIfKeyMatch");
		exec.defineModuleAlias("appendDateToId", "org.mitre.pam.aggregator.AppendDateToId");
		exec.defineModuleAlias("passIfProductAttributesMatch", "org.mitre.pam.aggregator.PassProductAttributes");

		exec.defineModuleAlias("passAsHTMLIndex", "org.mitre.pam.aggregator.PassAsHTMLIndex");
		exec.defineModuleAlias("notifyOfProduct", "org.mitre.pam.aggregator.NotifyOfProduct");
		exec.defineModuleAlias("streamThru", "org.mitre.pam.aggregator.StreamThru"); 
		exec.defineModuleAlias("countProducts", "org.mitre.pam.aggregator.ProductCounter"); 
		exec.defineModuleAlias("passIfPointInsidePolygon", "org.mitre.pam.aggregator.PointInsidePolygon"); 
		exec.defineModuleAlias("generateTrigger", "org.mitre.pam.aggregator.Trigger"); 
		exec.defineModuleAlias("queue", "org.mitre.pam.aggregator.QueueThru"); 
		exec.defineModuleAlias("replaceWords", "org.mitre.pam.aggregator.ReplaceWords"); 
		exec.defineModuleAlias("extractProperties", "org.mitre.pam.aggregator.ExtractProperties"); 

	/*
	* DIMs
	*/
		exec.defineModuleAlias("putQueuedDIM", "org.mitre.pam.putter.TestQueuedDIM");
		exec.defineModuleAlias("putMail", "org.mitre.pam.putter.PutMail");
		exec.defineModuleAlias("putBackWebLocal", "org.mitre.pam.putter.BackWebLocalDIM");
		exec.defineModuleAlias("putDIM1", "org.mitre.pam.putter.TestDIM1");
		exec.defineModuleAlias("putFTP", "org.mitre.pam.putter.FTPFileDIM");
		exec.defineModuleAlias("putProductToDirectory", "org.mitre.pam.putter.PutProductToDirectory"); // prefered name
		exec.defineModuleAlias("putDirectory", "org.mitre.pam.putter.PutProductToDirectory"); // old name
		exec.defineModuleAlias("putLocalPublisher", "org.mitre.pam.putter.PutLocalPublisher");
		exec.defineModuleAlias("putStarBurstCLI", "org.mitre.pam.putter.PutStarBurstCLI");
		exec.defineModuleAlias("putToRemoteConnector", "org.mitre.pam.putter.RemoteConnectorDIM");
		exec.defineModuleAlias("putGBSFTP", "org.mitre.pam.putter.GBSFTPFileDIM");
		exec.defineModuleAlias("putCDF", "org.mitre.pam.putter.PutCDF");
		exec.defineModuleAlias("putAIM", "org.mitre.pam.service.aim.AIMDIM");
		exec.defineModuleAlias("putUDP", "org.mitre.pam.putter.PutUDP");
		exec.defineModuleAlias("putSerializedProduct", "org.mitre.pam.putter.PutSerializedProduct");

	/*
	* Daemons
	*/
		exec.defineModuleAlias("makeBackWebSubscriptionPage", "org.mitre.pam.daemon.BackWebSubscriber");
	/*
	* Other bundles
	*/
		exec.defineModuleAlias("bytesPerMinuteResourcePool", "org.mitre.pam.service.resource.BytesPerMinuteRscPool");
		exec.defineModuleAlias("throttleBytesPerMinute", "org.mitre.pam.service.resource.ThrottleBytesPerMinute");

		try {
			Object tibcoExists = Class.forName("COM.TIBCO.rv.RvDataCallback");
			if (tibcoExists != null) {
				exec.defineModuleAlias("acceptTIBCO", "org.mitre.pam.getter.GetAsyncTIBCO");
				exec.defineModuleAlias("putTIBCO", "org.mitre.pam.putter.PutTIBCO");
			}
		}
		catch ( ClassNotFoundException dontUseTIBCO ) {}
	/*
	* The follow are pre-defined by core
	*
	*	defineModuleAlias("checkpoint", "org.mitre.pam.daemon.CheckpointDaemon");
	*	defineModuleAlias("logFilter", "org.mitre.pam.daemon.LogFilter");
	*	defineModuleAlias("exceptionChecker", "org.mitre.pam.daemon.ExceptionChecker");
	*	defineModuleAlias("remoteAgentEditor", "org.mitre.pam.rmi.editor.EditorDaemon");
	*	defineModuleAlias("memo", "org.mitre.pam.daemon.UserInfo");
	*	defineModuleAlias("dictionary", "org.mitre.pam.daemon.Dict");
	*	defineModuleAlias("deleteExpiredAgents", "org.mitre.pam.daemon.DeleteOnceAgents");
	*/


	}
/**
* Bring up the standard gui
* <p>
* This method is optional.
*/
	public static void defineGUIs(ExecToController exec) {
		exec.addAdminTool("org.mitre.pam.monitor.MonitorFrame","Administration"); 
		exec.addAdminTool("org.mitre.pam.bert.Bert","Channels (Bert)"); 
		exec.addAdminTool("org.mitre.pam.monitor.SimpleAdminFrame","Property Change"); 
		exec.addAdminTool("org.mitre.pam.monitor.AgentStateFrame","Agent enables"); 
		exec.addAdminTool("org.mitre.pam.monitor.watcher.Watcher","Dynamic monitor");
	}

/**
* Starts the web service and defines all "servlets"
* Some of these servlets take arguments in the constructor
* so they would not work outside of PAM
* <p>
* This method is optional.
*/
	public static void defineServlets(ExecToController exec){

		try {
			// Add servlets
			HttpServlet gState = new SrvGetExecState();
	 		HttpServlet backWebSubscribe = new SrvBackWebSub();
	 		HttpServlet backWebSubLive = new SrvBackWebSubLive();
			HttpServlet generalStatus = new SrvPAMServlet();
	 		HttpServlet approve = new SrvProductApprove();
	 		HttpServlet bwInfoPage = new SrvServeOneFile(new File(ExecGlobal.getSupportDir(BackWebSubscriber.SUB_PAGE_DIR), "index.html"));
			HttpServlet agentEditor = new SrvAgentEditor();



			exec.addServlet(SrvAgentEditor.QUERY_BASE, agentEditor);
	 		exec.addServlet(SrvProductApprove.APPROVE_QUERY_BASE, approve); 
			exec.addServlet(SrvGetExecState.PAM_STATE_POST, gState);
			exec.addServlet(SrvGetExecState.PAM_STATE_QUERY, gState);
			exec.addServlet(SrvControl.QUERY_URL, new SrvControl() );
		/*
		* Backweb support
		*/
			FileServlet bwSubDir = new FileServlet();
			bwSubDir.relativeLocalRoot = ExecGlobal.getSupportDir("BackWebSubscribe");
			bwSubDir.alias = new String("/PAM/BackWeb");
	 		exec.addServlet( SrvBackWebSub.allAvailableURL, backWebSubscribe);
	 		exec.addServlet( SrvBackWebSubLive.postURL, backWebSubLive);
	 		exec.addServlet( SrvBackWebSubLive.infoPageURL, bwInfoPage);
	 		exec.addServlet( SrvBackWebSubLive.infoPageURL+"/", bwInfoPage);
	 		exec.addServlet( SrvBackWebSubLive.infoPageURL+"/index.html", bwInfoPage);

			exec.addServlet( SrvPAMServlet.QUERY_URL, generalStatus );
			
			HttpServlet pamLogInfo = 
							new SrvOneFileValidated(new File(ExecGlobal.getCheckPointDir(), "log.txt"), 
							"exec", exec.getActiveAgent("exec").getPropertyValues("pamAccess")); 
			exec.addServlet( "/PAM/Log.query", pamLogInfo );
	  
			HttpServlet wds = new SrvWorkingDir();
			String wdName = ExecGlobal.getWorkingDirURLRoot();
			exec.addServlet( wdName+"/", wds);
			exec.addServlet( wdName+"/*", wds);
			
			HttpServlet sds = new SrvSupportDir();
			String sptName = ExecGlobal.getSupportDirURLRoot();
			exec.addServlet( sptName+"/", sds);
			exec.addServlet( sptName+"/*", sds);
			exec.addServlet( "/PAM/BackWeb/*", bwSubDir );
		}

		catch (Exception e) {

			exec.getExecAsAgent().getReporter().reportException(e);

		}

	}
/**
* Checks all the known support directories to ensure that
* they exist and have some minimum number of files.
* isOkToLoad() will always be called first.
* <p>
* This method is mandatory.
*
* @return true if this module can be loaded
*/
	static public boolean isOkToLoad(ExecToController exec, String[] cmdLineArgs) {
		if (exec.getBuildNumber(0) != BUILD) {
			exec.getExecAsAgent().getReporter().warning( "PAMCore build "+exec.getBuildNumber(0)+" does not match PAMStandard build "+BUILD);
		}
		return true;
	}

/**
* Wizards are run after all modules are defined but
* before any servlets are defined. 
* <p>
* This method is optional.
*/
	static public void runWizards(ExecToController exec) {
		String startConfigFile = exec.getExecAsAgent().getStringProperty(ExecGlobal.EXEC_STARTED_FROM_FILE,"");
		if (startConfigFile.length() == 0) {
			StartWizard startWizard = new StartWizard();
			startWizard.buildScript(exec);
			startWizard.runScript(exec);
		}

	}

/**
* An optional plugin definition method to create any needed support directories.
* This method is called before defineModules() is called
*/
	public static void makeSupportDirIfNeeded(ExecToController exec, File zipFile){
		StreamCopy.makeASupportDir(exec, SrvAgentEditor.TEMPLATEDIR,	"2",	"version.txt", zipFile);		
		StreamCopy.makeASupportDir(exec, "BackWeb",	"1",	"version.txt", zipFile);		
		StreamCopy.makeASupportDir(exec, "BackWebSubscribe",	"1",	"version.txt", zipFile);		
		StreamCopy.makeASupportDir(exec, "CastanetNotifier",	"1",	"version.txt", zipFile);		
		StreamCopy.makeASupportDir(exec, "CDFDIM",	"1",	"version.txt", zipFile);		
		StreamCopy.makeASupportDir(exec, "CDFsubscriptions",	"1",	"version.txt", zipFile);		
		StreamCopy.makeASupportDir(exec, "Gui",	"1",	"version.txt", zipFile);		
		StreamCopy.makeASupportDir(exec, "help",	"1",	"version.txt", zipFile);		
		StreamCopy.makeASupportDir(exec, "putProductToDirectory",	"1",	"version.txt", zipFile);		
		StreamCopy.makeASupportDir(exec, "putStarBurstCLI",	"1",	"version.txt", zipFile);		
		StreamCopy.makeASupportDir(exec, "smtp",	"1",	"version.txt", zipFile);	
	}


/**
* This is modeled after a java.lang.Package call by same name
*/
	public static String getImplementationVersion() {
		return "2.5."+BUILD;
	}

}

