/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam;

import java.io.File;
import java.io.IOException;
import javax.servlet.*;


import org.mitre.pam.interfaces.ExecToController;
import org.mitre.pam.interfaces.ExecCallbacks;
import org.mitre.pam.executive.Exec;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.executive.VersionInfo;

//import org.mitre.pam.monitor.*;

import javax.servlet.http.HttpServlet;
import Acme.Serve.FileServlet;


/**
* A main for PAM, creates exec, web service, and GUI.
*
* @author Nazario Irizarry, Jr.
*/
public class IPAM {
	
	static String versionString;

	public static void main(String[] args) {
		boolean faceless = false;
		String startFile = null;
		
		if (args != null) {
			for (int i = 0; i < args.length; i++) {
				if (args[i] != null) {
					if (args[i].equals("-?")) {
						echoHelp();
						return;
					}
					else if (args[i].equals("-version")) {
						echoVersionString();
						return;
					}
					else if (args[i].equals("-help")) {
						echoHelp();
						return;
					}
					else if (args[i].equals("-nogui")) {
						faceless = true;
					}
					else if (!args[i].startsWith("-")) {
						startFile = args[i];
						break;
					}
				}
			}
		}		

		ExecToController exec = ExecGlobal.getExec();
		if (exec == null)
			exec = new Exec();

		try {
			if (startFile == null) {
				File configFile = exec.getDefaultCheckPointFile();
				if (configFile.exists()) {
					exec.startFromConfig(configFile, !faceless, args);
				}
				else {
					exec.startFromConfig(null,!faceless, args);
				}
			}
			else {
				exec.startFromConfig(new File(startFile), !faceless, args);
			}
		}
		catch (Exception e) {
			System.err.println(e.getMessage());
			System.err.println("Could not create initial configuration");
			e.printStackTrace(System.err);
			Runtime.getRuntime().exit(1);
		}

		if (!faceless)
			exec.setToolPickerVisibility(true);


		try {
			exec.listenForWebConnections();
		}
		catch (IOException e) {
			exec.getExecAsAgent().getReporter().reportException(e);
			System.exit(1);
		}

	}

	protected static void echoHelp() {
		System.out.println("jre -classpath <classpath> org.mitre.pam.IPAM [-help] [-nogui] [startfile]");
	}
	protected static void echoVersionString() {
		int major = VersionInfo.getBuildNumber(1);
		int minor = VersionInfo.getBuildNumber(2);
		int build = VersionInfo.getBuildNumber(0);
		String versionString = "iPAM "+Integer.toString(major)+"."+Integer.toString(minor)+"."+Integer.toString(build);
		System.out.println(versionString);
	}
	
}

