/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.aggregator;

import java.util.Hashtable;
import java.util.Enumeration;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.BasicProductAttributes;
import org.mitre.pdif.eventParser.ValueList;

/**
* A Handler that filters out selected products with
* matching logicalChannelId
*
* @author Nazario Irizarry, Jr.
*/
public class StreamThru extends NoSchedBase {  //ni42 - new
	final static String LOGICAL_ID = BasicProductAttributes.TRACK_IDENTIFIER;
	final static String[] DEF_ID = new String[0];
	Hashtable ids = new Hashtable(100);
/*
* 
*/
	public StreamThru () {
		setProperty(LOGICAL_ID, "");
		setProperty(LOG_ACTIVITY,"false");
		metaMultiStringInfo(LOGICAL_ID, new ValueList(DEF_ID), "list of ids to pass (or empty to pass all)");
	}

/**
* Accept a single id to pass
*/
	public void setProperty(String property, String value) {
		if (property.equals(LOGICAL_ID)) {
			ids.clear();
			if (value != null && value.length() > 0)
				ids.put(value,value);
		}
		super.setProperty(property, value);
	}

/**
* Accept multiple ids to pass
*/
	public void setPropertyValues(String property, ValueList value) {
		if (property.equals(LOGICAL_ID)) {
			ids.clear();
			Enumeration e = value.elements();
			while (e.hasMoreElements()) {
				String id = (String) e.nextElement();
				if (id.length() > 0)
					ids.put(id, id);
			}
		}
		super.setPropertyValues(property, value);
	}
/**
* Accept the input and pass only if the ids checkout
*/
	protected Product buildProduct(Product p) {
	/*
	* If we are passing everything just do so
	*/
		if (ids.size() == 0) {
			return p;
		}

	/*
	* If the input product has no id then we reject it at
	* this point since we know we are filtering
	*/
		ValueList productIds = p.getPropertyValues(LOGICAL_ID);
		if (productIds.size() == 0) {
			return null;
		}
	/*
	* Here we must test all the input ids against our list
	*/
		Enumeration prodIdEnum = productIds.elements();
		while (prodIdEnum.hasMoreElements()) {
			Object pid = prodIdEnum.nextElement();
			if (ids.containsKey(pid))
				return p;
		}
	/*
	* nothing matched
	*/
		return null;
	}
}
