/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.aggregator;

//import java.util.*;
import java.io.File;
import java.io.IOException;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.store.FileRef;
import org.mitre.pam.productVisitor.AsZipFile;
import org.mitre.pam.product.view.BasicProduct;



/**
* A basic Juggler that converts its input(s) into a single zip file.
*
* @author Nazario Irizarry, Jr.
*/
public class WrapAsZip extends NoSchedBase {

	public static final String WRAP_PREFIX = "wrp";
	public static final String WRAP_SUFFIX = "zip";
	
	public WrapAsZip () {
//        setProperty(ScheduleAttributes.SCHEDULE_VERB, ScheduleAttributes.SCHEDULE_NONE);
		setDescription("Converts product into a single zip file");
	}
/**
* This channel builds a composite product from the product that it's sources provide.
* It augments the topic vector of the source product -- which is wrong when two channels
* end up using the same product, but this will have to do for now.
*/
	protected Product buildProduct(Product jugglersProduct) {
		if (jugglersProduct == null)
			return null;
		
		String zipNameRoot = FileRef.getNewFileId(WRAP_PREFIX);
		File zipFile = new File(getWorkingDir(), zipNameRoot+'.'+WRAP_SUFFIX);
		File psFile = new File(getWorkingDir(), zipNameRoot+".ps");
/*
		System.out.println("zip name root "+zipNameRoot);
		System.out.println("zip file  "+zipFile.getAbsolutePath());
		System.out.println("ps  file "+psFile.getAbsolutePath());
*/
		AsZipFile zipit = new AsZipFile(zipFile, psFile, reporter);
		zipit.visit(jugglersProduct);
		
		File resultingZip;
		try {
			resultingZip = zipit.getZipFile();
		}
		catch (IOException e) {
			reporter.reportException(e);
			return null;
		}
		
		if (resultingZip != null) {
			FileRef fr = new FileRef(resultingZip, resultingZip.lastModified(), resultingZip.getName());
			BasicProduct bp = new BasicProduct(getId(), getDescription());
			bp.setContent(fr);
			return bp;
		}
		else {
			zipFile.delete();
			reporter.assert(false, "Unable to create zip file",null,null,null);
			return null;
		}
	}
}
