/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter;

import java.io.File;
import java.util.Enumeration;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.product.view.BasicProduct;
import org.mitre.pam.product.view.VectorProduct;
import org.mitre.pam.store.FileRef;
import org.mitre.pdif.eventParser.ValueList;

/**
* Gets a list of files and builds a product with references to those files.
*
* @author Jason Mathews
*/

public class GetFile extends PIMShell1 {

  private static final String FILE_LIST = "fileList";
  private static final String KEEP_SOURCE_FILES = "keepSourceFiles";

  public GetFile() {
    setDescription("Get list of files");
    metaStringInfo(FILE_LIST, "", "List of sources files");
    metaBooleanInfo(KEEP_SOURCE_FILES, true, "If set then source files will not be deleted once they have been scanned");
  }

  public Product doFetch() {
    ValueList fileList = getPropertyValues(FILE_LIST);
    
    if (fileList.size() == 0)
      return null;

    boolean keepSourceFiles = getBooleanProperty(KEEP_SOURCE_FILES, true);

    VectorProduct vp = new VectorProduct(getId(), getDescription());

    for (Enumeration e = fileList.elements(); e.hasMoreElements(); ) {
      
	  File file = new File((String) e.nextElement());
	  if (!file.exists()) {
	    log(file + " does not exist");
	    continue;
	  }
	  /*
	    if (file.isDirectory()) {
	    log(file + " is a directory");
	    continue;
	    }
	  */
      FileRef f = new FileRef(file, file.lastModified(), file.getName());
      if(keepSourceFiles)
	f.setKeepFile(true);
      else
	f.setKeepFile(false);
      BasicProduct bp = new BasicProduct(file.getName(),"Referenced by "+getId());
      bp.setContent(f);
      vp.addElement(bp);
    }

    if (vp.size() == 0)
      return null;
    else if (vp.size() == 1) {
      BasicProduct bp = (BasicProduct) vp.elementAt(0);
      bp.setDescription(getDescription());
      return bp;
    }
    else
      return vp;
  }
}

