/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter;

import java.util.Hashtable;
import java.net.URL;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ProductFactory;
import org.mitre.pam.store.FileRef;

/**
* Simple PIM that takes a little time to execute and then
* returns a fixed product.
* For test purposes.  <P>
*
* @author Nazario Irizarry, Jr.
* @author Jason Mathews. 7/12/99 Added productID property.
*                        8/24/99 Added setContent method for subclasses to set different content
*/
public class TestPIM1 extends PIMShell1 {

	public final static String CONTENT = "content";
	public final static String DELAY = "delayInSeconds";
	public final static String MODIFICATION_DATE = "modificationDateInMilliseconds";
	public final static String PRODUCT_ID = "productID";
	public final static String TEST_CATEGORY = "Test Source";

	public TestPIM1 () {
		setDescription("Creates a string product");
		metaStringInfo(CONTENT, "<content>", "This becomes the content of the product");

		String id = getId();
		metaStringInfo(PRODUCT_ID, "", "Optinal ID of the product");

		metaIntegerInfo(DELAY, 2, "Amount of time to delay before returning a product");
		
		metaStringInfo(MODIFICATION_DATE, "", "Optional modification date for the product");
		
		setProperty(META_GUI_GROUP, TEST_CATEGORY);
	}

/**
* Here we build a "canned" product with an id and description that
* match the id and description of this agent.
* We return a BasicProduct with String content.  The content is taken
* from the "content" attribute of this agent.
*/
	public Product doFetch(){
		
	/*
	* Delay a little bit.
	*/
		int delay = getIntegerProperty(DELAY, 2); // default 2 seconds
		if (delay > 0)
		try {
			Thread.sleep(1000 * delay);
		}
		catch (Exception e) {}
		
	/*
	* Specify the id and description of the product when we first create it.
	* id can not be null or empty, but description may be empty.
	*/
	String id = getProperty(PRODUCT_ID);
	if (id == null || id.length() == 0)
		id = getId();
		Object content = makeContent();
		Product prod = null;
		if (content instanceof FileRef) {
			prod = exec.getProductFactory().makeProduct(id, getDescription(), (FileRef) content);
		}
		else if (content instanceof URL) {
			prod = exec.getProductFactory().makeProduct(id, getDescription(), (URL) content);
		}
		else {
			prod = exec.getProductFactory().makeProduct(id, getDescription(), content.toString());
		}

	/*
	* We set the creation time on the product. Normally the creation
	* time is simply the current time, but we change it here in case
	* we need a specific time for test purposes.
	*/
		String dateString = getProperty(MODIFICATION_DATE);
		if (dateString != null && dateString.length() != 0) {
		  try {
		    prod.setModificationDate(Long.parseLong(dateString));
		  } catch (NumberFormatException e) {}
		}

		return prod;		
	}

/** 
* Make String, URL, or FileRef content
* @return content if successful otherwise null
*/
  protected Object makeContent() {
    return getStringProperty("content","");
  }
}
