/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter;

import java.util.StringTokenizer;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.product.view.BasicProduct;


/**
* Test PIM that takes a string of unsigned byte values separated by whitespace
* and then returns a byte array of those byte values as a product.
* For test purposes.  <P>
*
* @author Henry C. Callihan, Jr.
*/
public class TestPIMByteArray extends PIMShell1 {

	private final static String CONTENT = "content";
	private final static String RADIX = "radix";
	public TestPIMByteArray () {
		setDescription("A byte array producer for test purposes");

		setProperty(META_GUI_GROUP, TestPIM1.TEST_CATEGORY);
		metaStringInfo(CONTENT, "05 A8 FF 16", "The product content");
		metaIntegerInfo(RADIX, 16, "The radix of the product");
	}

/**
* Callback to receive a command.  We ignore the
* command for now since we can only do one thing - fetch.
*/
	public Product doFetch(){
	    int radix;


		BasicProduct prod = new BasicProduct(getId(),getDescription());
        StringTokenizer st = new StringTokenizer(getProperty(CONTENT));
	    radix = getIntegerProperty(RADIX,16);
		int byte_size = st.countTokens();
       	int holder;
       	byte[] ba = new byte[byte_size];
		String s;

		for (int i = 0; i < byte_size; i++)
		{
		    s = new String(st.nextToken());
		    try
		    {
		        holder = Integer.parseInt(s,radix);
		        if (holder < 0 || holder > 255)
		            throw new NumberFormatException("Not an unsigned byte format");
		        ba[i] = (byte) holder;
		    }
		    catch (NumberFormatException nfe)
		    {
		        reportException(nfe);
		    }
		}

		prod.setContent(ba);

		return prod;

	}
}
