/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.net.MalformedURLException;
import java.net.URL;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.product.view.BasicProduct;
import org.mitre.pam.product.view.HashtableProduct;
import org.mitre.pam.interfaces.ContainerProduct;
import org.mitre.pam.interfaces.ProductFactory;
import org.mitre.pam.store.FileRef;
import org.mitre.pam.util.PAMDate;


/**
* Generates a complex product for test purposes.
*
* @author Tim Taylor
*/

public class TestPIMComplex extends PIMShell1 {
	private static String DEBUG_KEYWORD = "TestPIMComplex";
	protected Product bp1, bp2, bp3, bp4;	// allows subclass references
	private static String KEEP = "setKeepFlag";
	
  public TestPIMComplex() {
    setDescription("Create a canned complex product (for testing)");
	setProperty(META_GUI_GROUP, TestPIM1.TEST_CATEGORY);
	metaBooleanInfo(KEEP, true, "Keep files when no longer referenced");
  }

  /**
   * Callback to receive a command.  We ignore the
   * command for now since we can only do one thing - fetch.
   */
	public Product doFetch(){
		
		try {
			Thread.sleep(3000);
		}
		catch (Exception e) {
		
		}
		
		boolean keep = getBooleanProperty(KEEP, true);
    // Build up a dummy product...
    ProductFactory fab = exec.getProductFactory();
    ContainerProduct prod = fab.makeVectorProduct(name + " Output",
      "Test data from \"" + name + "\" using multiple products");

    // Add a String product
    Product bp = fab.makeProduct("String Product #1", "This is the first element", "This first element uses a String object as its content.");
    bp1 = bp;
    prod.addElement(bp);

    // Now a URL product
    try {
      bp = fab.makeProduct("URL Product #1", "Tim Taylor's home page", new URL("http://nts-bart.mitre.org/"));
      bp2 = bp;
      prod.addElement(bp);
    } catch(MalformedURLException e) {
      reportException(e);
    }  

    // Now a Vector product
    try {
      ContainerProduct vp = fab.makeVectorProduct("Vector Product #1", "Tim's Favorite URLs");
      bp = fab.makeProduct("Cup O' Joe Java Shop", "Cool Site for Java Developers",new URL("http://www.cupojoe.com/"));
      bp3 = bp;
      vp.addElement(bp);
      bp = fab.makeProduct("Black Coffee", "Java Apps, resources and Source Code",new URL("http://www.km-cd.com/black_coffee/"));
      vp.addElement(bp);
      bp = fab.makeProduct("JARS", "Java Review Service",new URL("http://www.jars.com/"));
      vp.addElement(bp);
      bp = fab.makeProduct("Languages for Java VM", "The machine can run more than Java",new URL("http://grunge.cs.tu-berlin.de/~tolk/vmlanguages.html"));
      vp.addElement(bp);
      prod.addElement(vp);
    } catch(MalformedURLException e) {
      reportException(e);
    }

    // Now a single fileRef...
    File fooFile = makeFile(FileRef.getNewFileId("foo"), "This is a test of the ...");
    FileRef f = new FileRef(fooFile, fooFile.lastModified(), fooFile.getAbsolutePath());
    f.setKeepFile(keep);                // Keep the file around...
    bp = fab.makeProduct("FileRef object #1", "A link to the file foo",f);
    bp4 = bp;
    prod.addElement(bp);

    // More FileRefs
    ContainerProduct hi = fab.makeVectorProduct("Hash Index Product #1", "Collection of FileRefs");
    File file = makeFile("abc.html", "<html><body><ul><li><a href=\"def.html\">def.html</a><li><a href=\"ghi.html\">ghi.html</a></ul></body></html>");
    f = new FileRef(file, file.lastModified(), file.getAbsolutePath());
    f.setKeepFile(keep);                // Keep the file around...
    bp = fab.makeProduct("FileRef object #2", "abc.html",f);
    hi.addRootElement(bp);
    
    file = makeFile("def.html", "<html><body>This is file def.html...</body></html>");
    f = new FileRef(file, file.lastModified(), file.getAbsolutePath());
    f.setKeepFile(keep);                // Keep the file around...
    bp = fab.makeProduct("FileRef object #3", "def.html",f);
    hi.addElement(bp);
    
    file = makeFile("ghi.html", "<html><body>This is file ghi.html... created on " + new PAMDate() + "</body></html>");
    f = new FileRef(file, file.lastModified(), file.getAbsolutePath());
    f.setKeepFile(keep);                // Keep the file around...
    bp = fab.makeProduct("FileRef object #4", "ghi.html",f);
    hi.addElement(bp);
    prod.addElement(hi);

	return prod;
		
	}

  protected File makeFile(String filename, String contents) {
    File file = new File(getWorkingDir(), filename);
    PrintStream out = null;

    try {
      out = new PrintStream(new FileOutputStream(file), true);
    } catch(java.io.IOException e) {
      reporter.assert(false, "Unable to create the output file '" , file.getAbsolutePath() , "'",null);
      return null;
    }
    out.println(contents);
    out.close();
    return file;
  }  

}