package org.mitre.pam.getter;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.PropertyMetaData;
import org.mitre.pam.product.view.DictionaryProduct;
import org.mitre.pam.interfaces.IDictionary;
import org.mitre.pam.util.AttributeKey;

/**
* Simple PIM that takes a little time to execute and then
* returns a fixed DictionaryProduct.
* For test purposes.  <P>
*
* @author Nazario Irizarry, Jr.
* @author Jones Leung, molded into DictionaryProduct
*/
public class TestPIMDict extends PIMShell1 {


    AttributeKey personIndex;
    AttributeKey latitudeIndex;
    AttributeKey longitudeIndex;
    AttributeKey timeIndex;
    AttributeKey observationSourceIndex;
    AttributeKey headingIndex;
    AttributeKey speedIndex;
    AttributeKey positiveIntIndex;

    AttributeKey negativeDoubleIndex;
    AttributeKey negativeLongIndex;
    AttributeKey negativeFloatIndex;
    AttributeKey negativeIntIndex;
    
    IDictionary dict;

	public TestPIMDict () {
		setDescription("provides a canned answer for development purposes");
		setProperty("content","<content goes here>");
		setProperty(META_GUI_GROUP, TestPIM1.TEST_CATEGORY);
	/*
	* do not define the dictionary in the constructor, wait to make sure that
	* the instance of the agent will actually be used
	*/
	}

/**
* Define dictionary entries
*/
	private void defineDictionaryIfNeeded() {
		if (dict != null)
			return;
		dict = exec.getDictionary("testDictionary", this);
	    personIndex = dict.makeAttributeKey("personName", PropertyMetaData.STRING);
	    latitudeIndex = dict.makeAttributeKey("latitude", PropertyMetaData.DOUBLE);
	    longitudeIndex = dict.makeAttributeKey("longitude", PropertyMetaData.DOUBLE);
	    timeIndex = dict.makeAttributeKey("time", PropertyMetaData.LONG);
	    observationSourceIndex = dict.makeAttributeKey("observation_source", PropertyMetaData.STRING);
	    headingIndex = dict.makeAttributeKey("heading", PropertyMetaData.STRING);
	    speedIndex = dict.makeAttributeKey("speed", PropertyMetaData.FLOAT);
	    positiveIntIndex = dict.makeAttributeKey("positiveInt", PropertyMetaData.INTEGER);
	
	    negativeDoubleIndex = dict.makeAttributeKey("negativeDouble", PropertyMetaData.DOUBLE);
	    negativeLongIndex = dict.makeAttributeKey("negativeLong", PropertyMetaData.LONG);
	    negativeFloatIndex = dict.makeAttributeKey("negativeFloat", PropertyMetaData.FLOAT);
	    negativeIntIndex = dict.makeAttributeKey("negativeInt", PropertyMetaData.INTEGER);
	}
/**
* Callback to receive a command.  We ignore the
* command for now since we can only do one thing - fetch.
*/
  public Product doFetch(){
	/*
	* define the dictionary here using lazy instantiation
	* so we don't burden the system with definitions for agents that are unused
	* or always disabled
	*/
		defineDictionaryIfNeeded();
		
		try {
			Thread.sleep(3000);
		}
		catch (Exception e) {

		}

        DictionaryProduct dp = new DictionaryProduct(getId(), getDescription(), dict);
        String personName = "Jones";
        double latitude = 123.4567;
        double longitude = 100.57;
        long time = System.currentTimeMillis();
        String observationSourceName = "eyes";
        String heading = "North";
        float speed = 5;
        int positiveInt = 100;

        double negativeDouble = -123.4567;
        long negativeLong = -System.currentTimeMillis();
        float negativeFloat = -5;
        int negativeInt = -100;


        dp.setProperty (personIndex, personName);
        dp.setProperty (latitudeIndex, latitude);
        dp.setProperty (longitudeIndex, longitude);
        dp.setProperty (timeIndex, time);
        dp.setProperty (observationSourceIndex, observationSourceName);
        dp.setProperty (headingIndex, heading);
        dp.setProperty (speedIndex, speed);

        dp.setProperty (positiveIntIndex, positiveInt);
        dp.setProperty (negativeDoubleIndex, negativeDouble);
        dp.setProperty (negativeLongIndex, negativeLong);
        dp.setProperty (negativeFloatIndex, negativeFloat);
        dp.setProperty (negativeIntIndex, negativeInt);
        return dp;
	}
}