/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter.remote;

import java.io.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.rmi.Naming;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.store.FileRef;
import org.mitre.pam.util.StreamCopy;
import org.mitre.pam.productVisitor.AsZipFile;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.executive.Command;

/**
* ConnectorServer is an RMI Server class which is instantiated with a reference
*  to a Connector object.  When the notify method is invoked remotely, a 
*  zipped product file is retrieved, reconstructed and passed up to the Connector.
*  The Connector then passes this product on to any of its clients.
*
* @author Srinivas Inguva
*/

public class ConnectorServer extends UnicastRemoteObject implements ConnectorInterface {
  public static String prodFile = PRODUCT_FILE_NAME; // value is set, this default doesn't matter

  private Connector myConnector;
  private File myDir;
  private Reporter reporter;

  public ConnectorServer(Connector j) throws RemoteException {
    myConnector = j;
    myDir = j.getWorkingDir();
    this.reporter = j.getReporter();
  }

  public int notify(URL location) throws RemoteException {
    try { 
      //System.out.println("Opening connection to: "+location);
      InputStream URLinput = location.openStream();
      myDir.mkdirs();
      int c = location.getFile().lastIndexOf('/');
      File temp = new File(myDir,location.getFile().substring(c+1));
      FileOutputStream fout = new FileOutputStream(temp);
      StreamCopy.copyFully(URLinput,fout);
      FileRef prodZipFile = new FileRef(temp,temp.lastModified(),temp.getName());
      //System.out.println("Reconstructing product from file");
      Product p = AsZipFile.buildProduct(myDir,temp,prodFile,reporter);
      if (p == null) {
	//System.out.println("null product");
	return -1;
      } else {
	    myConnector.pushProduct(p);
      }
      //Note: doCommand(SOURCES_DONE) calls Exec.notifyMyClients()
      //System.out.println("Executing Command SOURCES_DONE");
//      myConnector.doCommand(Command.SOURCES_DONE,null);
      
      return 1;
    } catch (MalformedURLException e) {
      return -2;
    } catch (IOException e) {
      return -1;
    }
  }

  public void setProdFile(String pf) {
    prodFile = pf;
  }

}
