/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter.search.altavista;

import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;
import java.net.URL;
import java.io.IOException;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.product.view.BasicProduct;
import org.mitre.pam.product.view.VectorProduct;
import org.mitre.pam.getter.PIMShell1;
import org.mitre.pdif.eventParser.ValueList;
import org.mitre.pam.getter.search.ThreePart;

/**
* Do a search at altavista.  Get one page worth of info
*
* @author Nazario Irizarry, Jr.
* @author Jason Mathews 7/27/99 [iPAM 2.3.72]
*                       Normalize have and have-not vectors such that empty strings are removed
*                       and string are trimmed. Added check for null query.
* @author Jason Mathews 9/23/99 [iPAM 2.4.84]
*			Added Altavista99a for AltaVista Search Intranet 2.3A
*/
public class AltavistaPIM2 extends PIMShell1 {

	private static final String MUST_HAVE = "mustHaveWords";
	private static final String MUST_NOT_HAVE = "mustNotHaveWords";
	private static final String TIME_BACK = "nDaysToSearchBack";
	private static final String FORMAT_YEAR = "formatNumber";
	private static final String URL_P0 = "urlBase";
	private static final String URL_P1 = "urlModifier";
	private static final String[] DEFAULT_NOT_HAVE = {"sex","nude","erotic","horny"};
	private static final String[] DEFAULT_HAVE = {"word1","word2"};
	private static final String[] FORMATS = {"1997","1998","1999","1999a"};

	// settable by static set methods at startup
	private static String DEFAULT_FORMAT = "1999";
    private static String DEF_URL_P0 = "http://www.altavista.digital.com/";
    private static String DEF_URL_P1 = "cgi-bin/query?pg=aq&what=web&kl=en";
	
	public AltavistaPIM2 () { 
		ValueList must = new ValueList(DEFAULT_HAVE);
		ValueList mustNot = new ValueList(DEFAULT_NOT_HAVE);

		setDescription("Dated keyword-based search");
		
		metaEnumerationInfo(FORMAT_YEAR,DEFAULT_FORMAT==null?"1999":DEFAULT_FORMAT, "A variant of the result format to parse", FORMATS);

		metaMultiStringInfo(MUST_HAVE, must,"search for documents with all of these words");
		metaMultiStringInfo(MUST_NOT_HAVE, mustNot,"resulting documents must not have any of these words");
		metaStringInfo(TIME_BACK, "7", "number of days to search (back from now)");
		metaStringInfo(URL_P0, DEF_URL_P0, "should not need to change this");
		metaStringInfo(URL_P1, DEF_URL_P1, "should not need to change this");
	}

    public static void setURL(String part1, String part2) {
	if (part1 != null && part1.length() != 0)
	    DEF_URL_P0 = part1;
	if (part2 != null && part2.length() != 0)
	    DEF_URL_P1 = part2;
    }

    public static void setDefaultFormat(String formatYear) {
	DEFAULT_FORMAT = formatYear;
    }

/**
* Callback to receive a command.  We ignore the
* command for now since we can only do one thing - fetch.
*/
	public Product doFetch(){
		
	/*
	* How many days to look back
	*/
		long nDays = 7;
		try {
			nDays = Long.parseLong(getProperty(TIME_BACK));
		}
		catch (NumberFormatException e) {
		}
		long date = System.currentTimeMillis() - nDays*1000*60*60*24;
		if (date <= 0) {
			throw new IllegalArgumentException("Value for " + TIME_BACK + " is out of range");
		}
		Date startDate = new Date(date);
		
	/*
	* Get the url parts
	*/
		String urlp0 = getProperty(URL_P0);
		if (urlp0 == null || urlp0.length() == 0)
			throw new IllegalArgumentException("Attribute "+URL_P0+" must be supplied ");
		String urlp1 = getProperty(URL_P1);
		if (urlp1 == null || urlp1.length() == 0)
			throw new IllegalArgumentException("Attribute "+URL_P1+" must be supplied ");

		Vector words = getStringVector(MUST_HAVE);
		if (words==null) {
			reporter.warning("Value for " + MUST_HAVE + " must be supplied");
			return null;
		}
		Vector xWords = getStringVector(MUST_NOT_HAVE);

	/*
	* Invoke the right version of altavista
	*/
		GetOnePageBase getter = null;
		String format = getStringProperty(FORMAT_YEAR, "");
		if (format.length() != 0) {
		  if (format.endsWith("97"))
			getter = new GetOnePage97(reporter, getId(), urlp0, urlp1);
		  else if (format.endsWith("98"))
			getter = new GetOnePage98(reporter, getId(), urlp0, urlp1);
		  else if (format.endsWith("99a"))
			getter = new GetOnePage99a(reporter, getId(), urlp0, urlp1);
		}
		if (getter == null)
			getter = new GetOnePage99(reporter, getId(), urlp0, urlp1);
		try {
			setProgress("Trying altavista (1)");
			getter.getNow(words, xWords, startDate);
		}
		catch (IOException ignore) {
//			reportException(e);
			try {
				Thread.sleep(60000);
				setProgress("Retrying altavista (2)");
				getter.getNow(words, xWords, startDate);
			}
			catch (IOException ignore2) {
//				log("Unable to get altavista on second try");
				try {
					Thread.sleep(120000);
					setProgress("Retrying altavista (3)");
					getter.getNow(words, xWords, startDate);
				}
				catch (IOException ignore3) {
					log("Unable to get altavista after three tries for "+name);
				}
				catch (Exception ie3) {}
			}
			catch (Exception ie2) {}
		}
		catch (Exception ignoreParseException) {}

		Vector list = getter.getResults();
/*		for (int i=0; i<list.size(); i++) {
		     ThreePart it = (ThreePart) list.elementAt(i);
		     System.out.println(" 0 "+it.id);
		     System.out.println(" 1 "+it.description);
		     System.out.println(" 2 "+it.content);
		     System.out.println();

		}
*/
		reporter.debugMsg("Altavista query for ",name," --> ",getter.getLastQuery(), LOG_QUERIES);
		if (list == null || list.size() == 0) {
			return null;
		}
/*
		else if (list.size() == 1) {
			ThreePart it = (ThreePart) list.elementAt(0);
			BasicProduct prod = new BasicProduct(it.id, it.description);
			try {
				prod.setContent(new URL(it.content));
			}
			catch (Exception e) {
				prod.setContent(it.content);
			}
			pimsProduct = prod;
		}
*/
		else {
			VectorProduct container = new VectorProduct(getId(),getDescription());
			Enumeration e = list.elements();
			while (e.hasMoreElements()) {
				ThreePart it = (ThreePart) e.nextElement();
				BasicProduct prod = new BasicProduct(it.id, it.description);
				try {
					prod.setContent(new URL(it.url));
				}
				catch (Exception e2) {
					prod.setContent(it.url);
				}
				container.addElement(prod);
			}
			return container;
		}
		
	}
	
  /* Returns a normalized vector of string properties;
   *
   *  1) Leading and trailing whitespace are first removed from the strings
   *  2) null and empty strings are also removed
   *
   *  Returns null or a vector of non-null/non-empty strings
   */
  private Vector getStringVector(String name) {

    ValueList list = getPropertyValues(name);
    if (list.size() == 0) return null;
    Vector words = new Vector();
    Enumeration e=list.elements();
    while (e.hasMoreElements()) {
      String s = (String) e.nextElement();
      if (s != null) {
	s = s.trim();
	if (s.length() != 0)
	  words.addElement(s);
      }
    }
    
    if (words.size() == 0) words = null;
    return words;
  }


/**
* Build a summary of the search keywords
*/
/*
	private String buildSearchPhraseSummary() {
		StringBuffer sum = new StringBuffer(80);
		sum.append("Search at www.altavista.digital.com for \"");
		String word;

		word = getProperty(KEY1);
		sum.append(word);

		word = getProperty(KEY2);
		if (word != null && word.length() != 0) {
			sum.append(", ");
			sum.append(word);
		}

		word = getProperty(KEY3);
		if (word != null && word.length() != 0) {
			sum.append(", ");
			sum.append(word);
		}

		word = getProperty(KEY4);
		if (word != null && word.length() != 0) {
			sum.append(", ");
			sum.append(word);
		}

		sum.append("\" going back ");
		String length = getProperty(TIME_BACK);
		sum.append(length);
		if (length.equals("1")) {
			sum.append(" day");
		}
		else {
			sum.append(" days");
		}
		
		return sum.toString();
	}
*/
}
