package org.mitre.pam.getter.search.altavista;

import java.util.Date;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;
import java.io.*;
import java.net.*;

import org.mitre.pam.getter.search.ThreePart;
import org.mitre.pam.getter.search.parse.LoggingHTMLParser;
import org.mitre.pam.getter.search.parse.ParseException;
import org.mitre.pam.interfaces.Reporter;

/**
* Parse results from altavista.
*
* @author Nazario Irizarry, Jr. version 2
* @author Jason Mathews 7/27/99 [iPAM 2.3.72]
*                       URL-encode have and have-not words and quote as appropriate
*			extend common parent class GetOnePageBase
*/

public class GetOnePage97 extends GetOnePageBase {

	// private static String urlPart0 = "http://www.altavista.digital.com/";
	// private static String urlPart1 = "cgi-bin/query?pg=aq&what=web&kl=en";
	
/*
	public static void main(String args[]) {
		main();
	}

	public static void main() {
		Vector words = new Vector();
		Vector xwords = new Vector();
		words.addElement("mariner");
		words.addElement("mariner");
		xwords.addElement("sex");
		xwords.addElement("nude");
		
		ExecCallbacks exec = org.mitre.pam.executive.ExecGlobal.getExec();
		System.out.println("findings...");
		GetOnePage getter = new GetOnePage(exec,"test");
		try {
			File f = new File("testdata:altavista answer");

			getter.getNow(words, xwords, f);
			Vector list = getter.getResults();
			for (int i=0; i<list.size(); i++) {
			     ThreePart it = (ThreePart) list.elementAt(i);
			     System.out.println(" 0 "+it.id);
			     System.out.println(" 1 "+it.description);
			     System.out.println(" 2 "+it.url);
			     System.out.println();

			}
		}
		catch (Exception e) {
			e.printStackTrace();			
			System.out.println("items in result "+getter.getResults().size());
		}
		System.out.println(" ... end of findings ...");

		
	}

	private GetOnePage97(ExecCallbacks exec, String id) {
		this.exec = exec;
		this.id = id;
	}
*/
	public GetOnePage97(Reporter reporter, String id, String urlPart0, String urlPart1) {
		super(reporter, id, urlPart0, urlPart1);
	}
	
	protected void scan(LoggingHTMLParser parser) throws ParseException {
	      while (parser.SkipToAfterTag("<dt>")) {
			ThreePart s = new ThreePart();

			parser.SkipToOpenTag(LoggingHTMLParser.A);
			Hashtable params = parser.ProcessParameters();
			s.url = (String) params.get("HREF");         // get link
			parser.SkipToEndOfTag();
			s.id = parser.GetUntilTag("</a>");          // get title
			s.id = LoggingHTMLParser.removeString(s.id, "<strong>");
			s.id = LoggingHTMLParser.removeString(s.id, "</strong>");
			s.id = LoggingHTMLParser.removeString(s.id, "</b>");
		
			parser.SkipToAfterTag("<dd>");
			s.description = parser.GetUntilTag("<br>");       // get source
			s.description = LoggingHTMLParser.removeAllStrings(s.description, "&nbsp;");
		
			results.addElement(s);
		}
	}

  protected String makeUrlString(Vector words, Vector excludeWords, Date startDate)
  {
		// empty query -> do nothing
		if (words == null || words.size() == 0) return null;

		StringBuffer queryString = new StringBuffer(80);

		queryString.append(urlPart0);
		queryString.append(urlPart1);
		
	/*
	* Build the AND part of the string
	*/
		boolean added = false;
		Enumeration includes = words.elements();
		while (includes.hasMoreElements()) {
			String aWord = (String) includes.nextElement();
			if (aWord != null && aWord.length() > 0) {
				if (!added) {
					queryString.append("&q=");
					added = true;
				}
				else
					queryString.append("+AND+");
				addQuotedWord(queryString, aWord);
			}
		}
		
		if (!added)
			return null;
	/*
	* Build the exclude part of the string
	*/
		if (excludeWords != null) {
		Enumeration excludes = excludeWords.elements();
		while (excludes.hasMoreElements()) {
			String aWord = (String)excludes.nextElement();
			if (aWord != null && aWord.length() > 0) {
				// +AND+(NOT+aWord)
				queryString.append("+AND+%28NOT+");
				addQuotedWord(queryString, aWord);
				queryString.append("%29");
			}
		}
		}
	/*
	* Build the ranking part of the string
	*/
		added = false;
		includes = words.elements();
		while (includes.hasMoreElements()) {
			String aWord = (String)includes.nextElement();
			if (aWord != null && aWord.length() > 0) {
				if (!added) {
					queryString.append("&r=");
					added = true;
				}
				else {
					queryString.append('+');
				}
				addQuotedWord(queryString, aWord);
			}
		}
	/*
	* Add the starting date
	*/
	
		queryString.append("&d0=");
		queryString.append(startDate.getDate());
		queryString.append("%2F");
		queryString.append(monthName[startDate.getMonth()]);
		queryString.append("%2F");
		queryString.append(startDate.getYear()+1900);
	
		return queryString.toString();
	}

}
