package org.mitre.pam.getter.search.altavista;

import java.util.Date;
import java.util.Vector;
import java.net.URLEncoder;
import java.net.URL;
import java.io.InputStream;
import java.io.IOException;

import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.getter.search.parse.ParseException;
import org.mitre.pam.getter.search.parse.LoggingHTMLParser;

/**
* Parse results from altavista.
*
* @author Nazario Irizarry, Jr. version 1
* @author Jason Mathews 7/27/99 [iPAM 2.3.72]                   
*		-made this a common parent class instead of an interface
*		-added addQuotedWord to URL-encode have and have-not words and quote as appropriate
*/

public abstract class GetOnePageBase {

	public static final String monthName[] = {
		"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec" 
		};

	protected String urlPart0; // "http://www.altavista.digital.com/";
	protected String urlPart1; // "cgi-bin/query?pg=aq&what=web&kl=en";

	protected Vector results = new Vector(10);
	protected String altavistaURL;
	protected Reporter reporter;
	protected String id;

	protected final static String DEBUG_KEY = "GetAltavista";

	public GetOnePageBase(Reporter reporter, String id, String urlPart0, String urlPart1) {
		this.reporter = reporter;
		this.id = id;
		this.urlPart0 = urlPart0;
		this.urlPart1 = urlPart1;
	}

	/**
	* implemented by altavista subclasses to handle custom page parsing
	*/
	protected abstract void scan(LoggingHTMLParser parser) throws ParseException;

	/**
	* implemented by altavista subclasses to handle custom URL generation
	*/
        protected abstract String makeUrlString(Vector words, Vector excludeWords, Date startDate);

/**
* for testing
*/
	protected void getNow(Vector words, Vector excludeWords, java.io.File fin) throws IOException, ParseException {
		
		InputStream in = new java.io.FileInputStream(fin);

		try {
		 LoggingHTMLParser parser = new LoggingHTMLParser(in, reporter);
		 scan(parser);
		} finally {
		  if (in != null)
		    try {
		      in.close();
		    } catch (IOException ignore) {}
		}
	}

/**
* builds altavista search string, opens URL, parses findings
*/
	public void getNow(Vector words, Vector excludeWords, Date startDate) throws IOException, ParseException {

		results.setSize(0); // start from scratch
		
		String queryString = makeUrlString(words, excludeWords, startDate);
		if (queryString == null)
			return;
		
		URL url = new URL(queryString);
		altavistaURL = queryString;

		InputStream in = url.openStream();
		try {
		  LoggingHTMLParser parser = new LoggingHTMLParser(in, reporter);
		  scan(parser);
		} catch (ParseException e) {
		  assert(false,"The following error occured on this url - "+altavistaURL);
		  reporter.reportException(e);
		  // go on if found parse error.
		} catch (Error e) {
		  // javacc parser throws Error instead of Exception
		  assert(false,"The following error occured on this url - "+altavistaURL);
		  reporter.reportException(e);
		  // go on if found parse error.
		}
		finally {
		  if (in != null)
		    try {
		      in.close();
		    } catch (IOException ignore) {}
		}
	}

	public String getLastQuery() {
		return altavistaURL;
	}

/**
* Returns the results of the search
*/
	public Vector getResults() {
		return results;
	}

	protected void assert(boolean cond, String s) {
		if (cond)
			return;
		reporter.assert(cond, s, null, null, null);
	}

  /*
   * Properly escape and quote a CGI parameter value for a URL
   *
   * If word contains a space then double quote the URL encoded string
   */
  protected static void addQuotedWord(StringBuffer buf, String word)
  {
    boolean quoted = false;
      for (int i=0; i < word.length(); i++)
	if (word.charAt(i) == ' ') {
	  quoted = true;
    	  buf.append("%22");
	  break;
	}
    buf.append(URLEncoder.encode(word));
    if (quoted) buf.append("%22");
  }
}
