/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter.search.excite;

import java.util.Hashtable;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;
import java.net.URL;
import java.io.IOException;
import java.io.File;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.product.view.BasicProduct;
import org.mitre.pam.product.view.VectorProduct;
import org.mitre.pam.getter.PIMShell1;
import org.mitre.pam.store.FileRef;

/**
* This PIM fetches a vector of stories from a custom excite channel whose
* URL is specified in the sourceSpec.  
*
* 1997 The MITRE Corporation
*
* @author Michael DeCorpo
*/

public class ExciteNewstrackerPIM extends PIMShell1 {

	private static final String SOURCE_URL = "completeExciteURL";
	private static final String PARSER = "parserVersion";
	private static final String[] VERSION_OPTIONS = {"98","99"};

  public ExciteNewstrackerPIM () { 
    setDescription("Newtracker info from nt.excite.com");
   
    setProperty(SOURCE_URL,"<URL for nt.excite.com>");
    metaStringInfo(SOURCE_URL,"<URL for nt.excite.com>","the nt.excite.com url (probably copied from your web browser)");

    setProperty(PARSER,"99");
    metaEnumerationInfo(PARSER,"99","Which version of newstracker server",VERSION_OPTIONS);
  }


/**
* Callback to receive a command.  We ignore the
* command for now since we can only do one thing - fetch.
*/
  public Product doFetch(){

    String urlString = null;
    try {
      urlString = getStringProperty(SOURCE_URL, "");
      URL newstrackerURL = new URL(urlString);
      if (!newstrackerURL.getHost().toLowerCase().equals("nt.excite.com")) {
    	reporter.log("The specified source spec must be from nt.excite.com, "+getId()+" can not activate");
    	return null;
      }
    }
    catch (Exception e) {
    	reportException(e);
    	return null;
    }

	GetNewsTracker getexcite=null;
	int whichParser = getIntegerProperty(PARSER,99);
	if (whichParser == 99) {
		getexcite = new GetNewsTracker2(urlString, reporter);
	}
	else if (whichParser == 98) {
		getexcite = new GetNewsTracker1(urlString, reporter);
	}
	else {
		reporter.error("Parser version must be 98 or 99");
		return null;
	}
	
    try {
      setProgress("Trying Excite");
      getexcite.getNow();
    }
    catch (IOException ignore) {
      try {
	Thread.sleep(60000);
	setProgress("Retrying Excite (2)");
	getexcite.getNow();
      }
      catch (IOException ignore2) {
	try {
	  Thread.sleep(120000);
	  setProgress("Retrying Excite (3)");
	  getexcite.getNow();
	}
	catch (IOException ignore3) {
	  log("Unable to get Excite after three tries for "+getId());
	}
	catch (Exception ie3) {}
      }
      catch (Exception ie2) {}
    }
    catch (Exception ignoreParseException) { 
    }    
    
    Vector list = getexcite.getStories();
    reporter.debugMsg("Excite query for ",name," --> ",urlString, LOG_QUERIES);

	if (list.size() == 0) {
		return null;
	}
	else {
		VectorProduct container = new VectorProduct(getId(),getDescription());
		
		Enumeration e = list.elements();
		while (e.hasMoreElements()) {
			ExciteStory it = (ExciteStory) e.nextElement();
			
			/* Make each story a BasicProduct with id=story title,
			* description=the story, content=the URL of the full story.
			*/
			BasicProduct prod = new BasicProduct(it.title,  it.source + ": " + it.story);
			/*
			*  set the products modification date to the date found
			*/
			if (it.date != null)
			prod.setModificationDate(Date.parse(it.date));
			try {
				prod.setContent(new URL(it.url));
			}
			catch (Exception exc) {
				reporter.assert(false, "Could not resolve a url for story ",it.title," query=",urlString);
				prod.setContent("<no url>");
			}
			container.addElement(prod);
		}
		
		return container;
	}
    
  }
  
  
}



