/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter.search.excite;

import java.util.Date;
import java.util.Vector;
import java.util.Hashtable;
import java.io.*;
import java.lang.*;
import java.net.*;
import org.mitre.pam.getter.search.parse.LoggingHTMLParser;
import org.mitre.pam.interfaces.Reporter;

/**
 * This class connects to a custom Excite Channel and puts all of the stories 
 * found into a Vector.
 *  
 *
 * @author Michael DeCorpo
 * @author Nazario Irizarry, Jr (updates)
 */


public class GetNewsTracker1 implements GetNewsTracker {
  
  Vector stories = new Vector(20);  
  private String exciteURLString;
  Reporter reporter;

/** 
 * The constructor 
 */
  public GetNewsTracker1(String urlString, Reporter reporter) 
  {
  	this.reporter = reporter;
  	
    if (urlString == null)
      exciteURLString = "";
    else
      exciteURLString = urlString;
  }

  

/**
 * Return a Vector of Excite Stories
 *
 */
  public Vector getStories()
  {
    return stories;
  }

/**
 * Connect to Excite, filter and parse the output stream from the Excite URL
 * Note that the returned date on hits may be null if the date can't be parsed.
 *
 */
  public void getNow() throws IOException, Exception
  {
    URL exciteURL = new URL(exciteURLString);

    InputStream urlStream = exciteURL.openStream();
    LoggingHTMLParser parser = new LoggingHTMLParser(urlStream, reporter);
	int storyCount = 0;
    try {
		while (parser.SkipToAfterTag("<table border=0 cellpadding=10 cellspacing=0 width=100%>")) 
		{
			storyCount++;
			ExciteStory s = new ExciteStory();
			
			parser.SkipToOpenTag(LoggingHTMLParser.A);
			Hashtable params = parser.ProcessParameters();
			s.url = (String) params.get("HREF");         // get link
			if (s.url == null) {
				reporter.log("null story detection based on URL on query "+exciteURLString+" story index = "+storyCount);
				break;
			}
			
			parser.SkipToEndOfTag();
			s.title = parser.GetUntilNextTag();          // get title
			if (s.title == null) {
				reporter.log("null story detection based on TITLE on query "+exciteURLString+" story index = "+storyCount);
				break;
			}
			parser.SkipToAfterOpenTag(LoggingHTMLParser.NOBR);
			parser.SkipToAfterString("(");
			s.source = parser.GetUntilString(")");       // get source
			
			parser.SkipToAfterTag("<br>");
			s.story = parser.GetUntilNextTag();          // get story
			if (s.story == null) {
				reporter.log("null story detection based on STORY on query "+exciteURLString+" story index = "+storyCount);
				break;
			}
			s.story = parser.removeAllStrings(s.story, "&nbsp;");
			
			parser.SkipToAfterString("First found: ");
			s.date = null;
			String dateStr = parser.GetUntilNextTag();           // get date
			/*
			* remove "Universal time"
			*/
			if (dateStr != null && dateStr.length() > 4) {
				int i = dateStr.indexOf(" Universal");
				if (i >= 0) {
					s.date = dateStr.substring(0,i);
				}
				else
					s.date = dateStr;
			}
		stories.addElement(s);
		}
    }
    catch (Exception e) {
    	reporter.assert(false,"The following error occured on this url - ",exciteURLString," story index = ",Integer.toString(storyCount));
    	reporter.reportException(e);
    }   // go on if found parse error.
    catch (Error e) {
    	reporter.assert(false,"The following error occured on this url - ",exciteURLString," story index = ",Integer.toString(storyCount));
    	reporter.reportException(e);
    }
    finally {
    	urlStream.close(); 
    }
  
  }
		

}



