/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter.search.parse;

import org.mitre.pam.interfaces.Reporter;

/**
* This is intended to be a proxy for HTMLParser so we can 
* properly log any parse errors.  For now we simply
* subclass HTMLParser.
*/
public class LoggingHTMLParser extends HTMLParser {
	Reporter reporter;

  public LoggingHTMLParser(java.io.InputStream stream, Reporter reporter) {
  	/*
  	* Use seven bit filter because sometimes javacc gets confused if
  	* eigth bit is used
  	*/
  	super(new HTMLParserTokenManager(new ASCII_CharStream(new SevenBitFilter(stream), 1, 1)));
  	this.reporter = reporter;
  }

/**
* Deletes all .html tags in the string
*/
 public static String stripHTML(String s) {
	int i = 0, j;
	int lastpos = -1;
	StringBuffer buf = new StringBuffer(256);
//	System.out.println("Check text: " + s);

	while ((i = s.indexOf('<', lastpos+1)) > lastpos && (j = s.indexOf('>', i+1)) > i) {
		if (i - lastpos > 1) {
//			System.out.println("Append text: " + i + " " + j + " [" +
//				s.substring(lastpos+1, i) +"]");

			buf.append(s.substring(lastpos+1, i));
		}
		lastpos = j;
	}

	if (lastpos+1 < s.length()) {
		buf.append(s.substring(lastpos+1));
//		System.out.println("pos="+lastpos + " -> " + s.substring(lastpos+1));
	}

	return buf.toString().trim();
 }

 
 /**
 * Allows removal of a substring if it is found.
 * Helpful for removing optional HTML formatting tags
 */
 	public static String removeString(String inStr, String remStr) {
		int ind = inStr.indexOf(remStr);
		
		if (ind < 0) 
			return inStr;
		
		int strLen = inStr.length();
		int remLen = remStr.length();
		
		if (ind == 0)
			return inStr.substring(remLen);

		if (ind + remLen >= strLen)
			return inStr.substring(0, ind);
		
		return inStr.substring(0, ind) + inStr.substring(ind+remLen);
	}
 /**
 * Allows removal of a substring however many times they may be found.
 * Helpful for removing optional HTML formatting tags
 */
 	public static String removeAllStrings(String inStr, String remStr) {
 		String before = inStr;
 		
 		do {
	 		String after = removeString(before, remStr);
	 		
	 		if (before == after)
	 			return after;
	 			
	 		before = after;
	 	}
	 	while (before.length() > 0);
	 	
	 	return before;
	}

}