/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter.search.smarTraveler;

import java.util.Date;
import java.util.Vector;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.io.*;
import java.net.*;
import org.mitre.pam.getter.search.parse.LoggingHTMLParser;
import org.mitre.pam.getter.search.parse.ParseException;
import org.mitre.pam.getter.search.parse.Token;
import org.mitre.pam.interfaces.Reporter;

/**
 * This class connects to a specfic SmarTraveler web page and puts the traffic info
 * into a vector with each paragraph in a separate BasicProduct.
 *
 *
 * @author Jason Mathews
 */

public class GetTravelerWash extends GetTraveler {

  protected String key;
  protected int method;
  private String id;


/**
 * The constructor
 */
  public GetTravelerWash(String urlString, Reporter reporter, String key, int method, String id)
  {
    super(reporter, urlString);
    this.key = key;
    this.method = method;
    this.id = id;
// System.out.println("Traveler Method: " + method + " " + urlString);
  }


  public Vector getStories() {
	return stories;
  }


  public String getTitle() {
	return title;
  }


/**
 * Connect to Traveler, filter and parse the output stream from the Traveler URL
 * Note that the returned date on hits may be null if the date can't be parsed.
 *
 */
  public void getNow() throws IOException, ParseException
  {
//      System.out.println("getTraveler.getNow()"); // DEBUG

    URL travelerURL = new URL(travelerURLString);
    InputStream urlStream = travelerURL.openStream();

//    FileInputStream urlStream = new FileInputStream("boston.asp");    // DEBUG

    try {
	LoggingHTMLParser parser = new LoggingHTMLParser(urlStream, reporter);

	if (!parser.SkipToAfterOpenTag(LoggingHTMLParser.CENTER) ||
	    !parser.SkipToAfterOpenTag(LoggingHTMLParser.FONT) ||
	    !parser.SkipToAfterOpenTag(LoggingHTMLParser.B) ||
	    (title = parser.GetUntilClosedTagOfType(LoggingHTMLParser.B)) == null)
	{
		throw new ParseException("Unable to parse smarTraveler content");
	}

	// Get Date --> mm/dd/yy hh:mm:ss pm
	// expected date format: "3/21/98 1:12:19 PM" minimum 17 char length
	parser.SkipToAfterOpenTag(LoggingHTMLParser.FONT);
	String date = parser.GetUntilNextTag();
	if (date != null && date.length() < 17)
		date = null;

	// Parse body of message into paragraphs
	String text = parser.GetUntilClosedTagOfType(LoggingHTMLParser.BLOCKQUOTE);
	if (text == null)
		throw new ParseException("Missing end of blockquote in body");

	StringTokenizer st = new StringTokenizer(text, "<");
	String tag;
	String subtitle = "";
	int i;
	while (st.hasMoreElements()) {	
		String sub = (String) st.nextElement();
		int p = sub.indexOf(">");
		if (p < 0) break;
		tag = sub.substring(0, p+1);
		text = sub.substring(p+1).trim();
		if (tag.equals("B>")) {
			subtitle = text; // New category found; e.g. current conditions
		} else if (text.length() > 0) {						
			// ignore sentence: .. viewed/monitored ... by smarTraveler camera(s)
			if ((i=text.indexOf("smarTraveler")) >= 0 && text.indexOf("camera", i+1) > 0)
				continue;
			int endpos = text.length() - 1;
			if (endpos > 40) endpos = 40; // stop after 40 characters
			String topic = "";
			String storyTitle = subtitle;
			// Find topic terminated by a ':' -- e.g. I-495/Inner Loop:
			for (i=0; i < endpos; i++) {
				char c = text.charAt(i);
				if (c == ':') {
					topic = text.substring(0,i);
					storyTitle = subtitle + " " + topic;
					text = text.substring(i+1).trim();
					break;
				}
				// stop at any punctuation which means there is no topic
				else if (c=='.' || c==',' || c == ';')
					break;
			}

			// Check if story matches keyword for given method
			if (method == TravelerStory.EXACT && !topic.equals(key) ||
			   method == TravelerStory.SUBSTRING && topic.indexOf(key) == -1) {
// System.out.println("*** story does not match method: " + key + " : " + topic);
				continue;
			}

			TravelerStory s = new TravelerStory();
			s.date  = date;
			s.title = storyTitle;
			s.story = text;
			stories.addElement(s);
		}
	}
    }
    catch (ParseException e) {
      // log it and rethrow it allowing caller to try again in case server has temporary problems
      reporter.assert(false, "The following error occured on this url - ",travelerURLString,null,null);
      reporter.reportException( e );
      throw e;
    }
    finally {
	try {
	  urlStream.close();
	}
	catch (IOException ignore) {}
    }
  }

}
