/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter.webcopy;

import java.util.*;
import java.net.*;
import java.io.*;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.product.view.BasicProduct;
import org.mitre.pam.product.view.VectorProduct;
import org.mitre.pam.store.FileRef;

import org.mitre.pam.getter.PIMShell1;
import org.mitre.pam.util.PAMDate;

/**
* Gets A single page from a web site. <P>
* Has two areas that are not complete: 1.  What if url does not actually exist
* 2 what if a sub url times out or does not exist?
*
* @author Nazario Irizarry, Jr.
*/
public class WebCopyPIM1 extends PIMShell1 {
	protected static final String URL = "url";
	protected static final String NAME_OF_INDEX_FILE = "nameOfIndexFile";
	protected static final String DEFAULT_INDEX_NAME = "index.html";
	protected static final String REQUEST_PROPERTY_ROOT = "requestProperty_";
	protected static final String REQUEST_PROPERTY = REQUEST_PROPERTY_ROOT+"example!";

	protected int maxDepth = 1;
	protected String defaultDescription = "From an HTTP source";

	public WebCopyPIM1 () {
		setDescription(defaultDescription);
		setProperty(URL,"");
		setProperty(NAME_OF_INDEX_FILE, DEFAULT_INDEX_NAME);
		setProperty(REQUEST_PROPERTY, "");
		maxDepth = 1;
		metaStringInfo(URL, "", "URL of the source");
		metaStringInfo(NAME_OF_INDEX_FILE, DEFAULT_INDEX_NAME, "name to use for index file");
		metaStringInfo(REQUEST_PROPERTY, "", "add optional HTTP request header property of this form");
	}
/**
* Do the fetch.
*/
	public synchronized Product doFetch(){
		FileOutputStream fout = null;
//		TimeOutInputStream timer;

		try {
			WebCopy wc = new WebCopy(reporter);
		/*
		* Request properties?
		*/
			Enumeration ep = getProperties();
			while (ep.hasMoreElements()) {
				String prop = (String) ep.nextElement();
				if (prop.startsWith(REQUEST_PROPERTY_ROOT) && !prop.equals(REQUEST_PROPERTY)) {
					String name = prop.substring(REQUEST_PROPERTY_ROOT.length());
					String value = getStringProperty(prop,"");
					if (value.length() > 0)
						wc.addRequestProperty(name, value);
				}
			}
		/*
		* Note how we build a working directory
		*/
			wc.setLocalDir (getWorkingDir());
			wc.setTimeout (30000);
			wc.setMaxDepth (maxDepth);

			URL link = new URL (getSourceSpec());
//			timer = new TimeOutInputStream(wc, 120000);	// 2 minute timeout
			wc.setIndexFile(getProperty(NAME_OF_INDEX_FILE));
			wc.startCopy (link);
//			timer.stop();
//			timer = null;

			String now = (new PAMDate()).toString();
			Enumeration enum = wc.getResults();
			int nItems = wc.getResultCount();
		/*
		* We found no items
		*/
			if ( nItems == 0  ) {
/*
				BasicProduct prod = new BasicProduct(getSourceSpec(),"Nothing fetched by "+getId());
				prod.setContent("Last fetch on "+now);
				pimsProduct = prod;
*/
				return null;
			}
		/*
		* We found one item
		*/
			else if ( nItems == 1  ) {
				BasicProduct oneProduct;
				FileRef f = (FileRef) enum.nextElement();

				if (getDescription().equals(defaultDescription))
					oneProduct = new BasicProduct(getId(), "From "+getSourceSpec());
				else
					oneProduct = new BasicProduct(getId(), getDescription());

				oneProduct.setContent(f);
				return oneProduct;
			}
		/*
		* We found more than one item
		*/
			else {
				VectorProduct prod;

				if (getDescription().equals(defaultDescription))
					prod = new VectorProduct(getId(), "From "+getSourceSpec());
				else
					prod = new VectorProduct(getId(), getDescription());

				while (enum.hasMoreElements()) {
					LinkObject f = (LinkObject) enum.nextElement();
					if (f.isRootComponent) {
						BasicProduct fileProduct = new BasicProduct( getId(), "From "+f.getSourceName());
						fileProduct.setContent(f);
						prod.addRootElement(fileProduct);
					}
					else {
						BasicProduct fileProduct = new BasicProduct( f.getSourceName(), "From "+f.getSourceName());
						fileProduct.setContent(f);
						if (f.isSupportComponent) {
							fileProduct.setProperty(Product.PROPERTY_SUPPORT_COMPONENT, Product.PROPERTY_SUPPORT_COMPONENT);
						}
						prod.addElement(fileProduct);
					}
				}
				link = null;
				wc = null;

				return prod;
			}

		}
		catch (IOException e) {
			reportException(e);
			return null;
		}
		finally {
      if (fout != null)
        try {
            fout.close();
        }
        catch (IOException e) {
          reportException(e);
        }
		}
	}

	private String getSourceSpec() {
		return getStringProperty(URL,"");
	}

	public void setProperty(String attribute, String value) {
		if (attribute.equals("sourceSpec"))
			attribute = URL;
		super.setProperty(attribute, value);
	}
}
