/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.monitor;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.net.URLEncoder;
import org.mitre.pam.interfaces.ExecToMonitor;
import org.mitre.pam.interfaces.Schedulable;
import org.mitre.pam.aggregator.ChanSchedBase;
import org.mitre.pam.monitor.SrvAgentEditor;
import org.mitre.pam.monitor.SrvGetExecState;
import org.mitre.pam.util.AuthContext;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.executive.ExecGlobal;

/**
 * A servlet that allows remote control of iPAM agents [edit/activate]
 * and ability to checkpoint or shutdown the service.
 *
 * @author Jason Mathews 12/16/1999
 * @since iPAM 2.5
 */

public class SrvControl extends SrvPAMBase implements Runnable {

  public static String QUERY_URL = "/PAM/Control.query";
  public static final String TEMPLATEDIR = SrvAgentEditor.TEMPLATEDIR;

  public void doPost(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException
  {
    doGet(req, res);
  }

  public void doGet(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException
  {

    /*
     * Begin access check +++++++++
     */
    AuthContext context = getAuthContext(req, res);

    if (!context.verifyAccess(reporter, this, QUERY_URL))
      return;
    /*
     * End access check ------------
     */

    ServletOutputStream out = res.getOutputStream();

    res.setContentType("text/html");

    String uid = req.getParameter("uid");
    if (uid == null) {
      makeList(out); // default action
    } else {
      // all other actions require "uid" parameter
      String agent = req.getParameter("agent");
      if (agent != null && agent.length() > 0)
	listAgent(req, res, agent);
      else if (req.getParameter("ckpt") != null)
	checkpoint(req, res);
      else {
	String shutdown = req.getParameter("shutdown");
	if (shutdown != null)
	  shutDown(req, res, shutdown);       // shutdown cmd
	else
	  printErrorMessage(out, "Parameter \"uid\" not specified.");
      }
    }

/*
    // debug
    Enumeration params = req.getParameterNames();
    out.print("<PRE>");
    while(params.hasMoreElements()) {
      String param = (String)params.nextElement();
      String val = req.getParameter(param);
      out.println(param + ": " + val);
    }
    out.println("</PRE>");
    // debug
*/

    out.println("</body></html>");
    out.flush();
  }

  private void checkpoint(HttpServletRequest req, HttpServletResponse res)  throws ServletException, IOException {
    res.setHeader("Pragma","no-cache");  // so the browser won't cache the page
    ServletOutputStream out = res.getOutputStream();
    out.println("<html><head><title>Checkpoint iPAM Request");
    out.println("</title></head>");
    out.println("<body bgcolor=\"#FFFFFF\">");
    out.println("<P><B>Checkpoint request issued</B>");
    out.println("<P><A HREF=\"" + QUERY_URL + "\">Back</a>");
    exec.checkPoint();
  }

  private void shutDown(HttpServletRequest req, HttpServletResponse res, String state) throws ServletException, IOException {
    Agent agent = null;

    ServletOutputStream out = res.getOutputStream();

    res.setHeader("Pragma","no-cache");  // so the browser won't cache the page

    if (state.equals("2")) {
      reporter.log("Shutdown requested");
      out.println("<html><head><title>Shutdown iPAM Request");
      out.println("</title></head>");
      out.println("<body bgcolor=\"#FFFFFF\">");
      out.println("<P><B>Shutdown request issued</B>");
      out.println("<P>Shutdown should be completed after several seconds.");
      Thread t = new Thread(this);
      t.setDaemon(true);
      t.start();
    } else if (state.equals("1")) {
      out.println("<html><head><title>Confirm Shutdown iPAM Request");
      out.println("</title></head>");
      out.println("<body bgcolor=\"#FFFFFF\">");
      out.println("<table border=1><tr><td><center>");
      shutDownForm(out, "2");
      out.println("</center></table>");
    } else {
      printErrorMessage(out, "invalid command");
    }
  }

  public void run() {
    try {
      Thread.sleep(2000); // delay 2 seconds to let connection close
    } catch (InterruptedException e) {}
    exec.halt();
  }

  private void shutDownForm(ServletOutputStream out, String state) throws IOException {

    if (state == null) state = "";

    StringBuffer buf = new StringBuffer();
    String peerProtocolHostAndPort = ExecGlobal.getPeerProtocolHostAndPort();

    if (peerProtocolHostAndPort != null && peerProtocolHostAndPort.length() != 0) {
      String protocolHostAndPort = ExecGlobal.getProtocolHostAndPort();
      if (!peerProtocolHostAndPort.equals(protocolHostAndPort))
	buf.append(peerProtocolHostAndPort);
    }

    buf.append(ExecGlobal.getSupportDirURLRoot());
    buf.append('/');
    buf.append(TEMPLATEDIR);
    buf.append('/');
    buf.append("images/Stop.jpg");

    String imageUrl = buf.toString();

    out.println("<form METHOD=POST>");
    if (state.equals("1"))
      out.println("<input type=submit name=ckpt value=\"Checkpoint\">");
    else
      out.println("<h1>Confirm Shutdown iPAM Request</h1>");
    out.println("<input type=hidden name=uid value=\"1\">");
    out.print("<input type=hidden name=shutdown value=\"");
    out.print(state);
    out.println("\">");

    out.println("<input align=absbottom border=0 type=image src=\"" + imageUrl + "\">");
    //      out.println("<input type=submit value=\"Shutdown\"");
    out.println("</form>");
  }

  private void listAgent(HttpServletRequest req, HttpServletResponse res, String id) throws ServletException, IOException {
    reporter.log("id=" + id);

    String feedback = null;
    Agent agent = null;

    agent = exec.getActiveAgent(id);
    if (agent == null) {
      feedback  = "Agent: <b>" + id + "</b> doesn't exist.";
    }
    else if (! (agent instanceof Schedulable)) {
      feedback  = "Agent: <b>" + id + "</b> is not of a Schedable type";
      agent = null;
    }

    ServletOutputStream out = res.getOutputStream();

    if (feedback != null) {
      printErrorMessage(out, feedback);
      return;
    }

    res.setHeader("Pragma","no-cache");  // so the browser won't cache the page

    out.println("<html>");
    out.print("<head><title>iPAM Agent " + id);
    out.println("</title></head>");
    out.println("<body bgcolor=\"#FFFFFF\">");

    ((Schedulable)agent).activateASAP();

    out.println("<B>Agent " + agent.getId() + " activated</B>");
    out.println("<P><A HREF=\"" + QUERY_URL + "\">Back</a>");
  }

  private void makeList(ServletOutputStream out) throws ServletException, IOException {

    out.println("<html>");
    out.print("<head><title>iPAM Control Panel");

    String id = exec.getExecAsAgent().getProperty("id");

    if (id != null && id.length() > 0 && ! id.equals("unnamed"))
      out.print( " on " + id);
    else id = "iPAM";

    out.println("</title></head>");
    out.println("<body bgcolor=\"#FFFFFF\">");

    out.println("<h1>" + id + " Control Panel</h1>");

    out.println("<table>");

    out.println("<TR><th>Agent<th>Type<th>Edit<th>Activate");

    listAgents(out, ExecToMonitor.DAEMON);
    listAgents(out, ExecToMonitor.JUGGLER);

    out.println("</table>");

    /* ---------------------------------------------------------------------- */

    /*
      out.println("<form METHOD=POST>");
      out.println("<input type=hidden name=uid value=\"1\">");
      out.println("<input type=submit name=ckpt value=\"Checkpoint\"");
      out.println("</form>");
    */

    shutDownForm(out, "1");

    out.println("<hr>");

    /* ---------------------------------------------------------------------- */

    out.println("<table>");
    //  out.println("<TR><th>Short cuts");
    out.println("<tr><td>");
    out.println("<a href=\"" + SrvGetExecState.PAM_STATE_QUERY + "\">" +
		SrvGetExecState.PAM_STATE_QUERY + "</a>");

    out.println("<tr><td>");
    out.println("<a href=\"/PAM/Log.query\">/PAM/Log.query</a>");

    out.println("<tr><td>");
    out.println("<a href=\"/PAM/List.query\">/PAM/List.query</a>");

    //    out.println("<TD bgcolor=\"#ff0000\">");
    //    out.println("<a href=\"" + QUERY_URL + "?shutdown=1&uid=1\">Shutdown PAM Service</a>");
    out.println("</table>");
  }

  public void listAgents(ServletOutputStream out, int type) throws IOException {

    Enumeration e = exec.getListOfActiveModules(type); 
    while (e.hasMoreElements()) {
      Agent agent = (Agent)e.nextElement();
      if (type == ExecToMonitor.JUGGLER && !( agent instanceof ChanSchedBase)) continue;
      out.println("<TR>");
      out.print("<TD><B>");
      out.print(agent.getId());
      out.println("</B>");

      out.print("<TD>");
      String kind = agent.getStringProperty("mementoKind", "");
      if (kind.length() == 0)
	kind = agent.getClass().toString();
      out.println(kind);

      boolean enabled = agent.isEnabled();
      /*
	if (enabled)
	out.print("<TD>Enabled");
	else
	out.print("<TD BGCOLOR=\"#FF0000\">Disabled");
      */

      String query = "?agent=" + URLEncoder.encode(agent.getId()) + "&uid=1";
      out.println("<TD><a href=\"" + SrvAgentEditor.QUERY_BASE + query + "\">Edit</a>");
      if (enabled)
	out.println("<TD><a href=\"" + QUERY_URL + query + "\">Activate</a>");
      else
	out.println("<TD><STRIKE>Activate</STRIKE>");

    }
  }

  /**
   * This method prints the error message sent in a nice format.
   */
  protected void printErrorMessage(ServletOutputStream out, String msg) throws IOException
  {
    out.println( "<HTML><HEAD>" );
    out.println("<title>iPAM Control Error</title></head>");
    out.println("<body bgcolor=\"#FFFFFF\">");
    out.println("<h2>Servlet Error</h2>");
    out.println(msg);
    reporter.log(msg);
  }
}
