/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.monitor;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.mitre.pam.util.Codecs;
import org.mitre.pam.executive.Exec;
import org.mitre.pam.interfaces.ExecToMonitor;
import org.mitre.pam.interfaces.ExecToController;

/**
* Serves a file as a servlet under the javax servlet definition.
*
* @author Nazario Irizarry
*
* 10/18/99 URL encode the links and HTML encode filenames
*/

abstract public class SrvDir extends SrvPAMBase {


	protected File relativeLocalRoot;
	protected String logicalURLRoot;

	protected File fileToGet;
	protected int fileGetError;

    public SrvDir() {
	}


    abstract public String getServletInfo();

/**
* Intercept the service call to reset our private attributes.
* Invokes javax.servlet.http.HttpServlet.service
*/
	protected void service (HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException {
		fileToGet = null;
		fileGetError = 0;
		super.service(req, res);
	}

/**
* Examines the requested path to try to resolve it to an existing file
* or directory.
*/
	protected boolean resolvePath( HttpServletRequest req) {
	
		getExec();
		
		if (fileToGet != null)
			return true;
		
		String path = req.getServletPath();

//		System.out.println("WorkDirServlet request 2 for "+path);
	/*
	* Assume that we are proxied at logicalURLRoot
	*/
		int wdi = path.indexOf(logicalURLRoot);
		if (wdi < 0) {
		    fileGetError = HttpServletResponse.SC_BAD_REQUEST;
		    return false;
		}
		path = path.substring(wdi+logicalURLRoot.length());
		
//		System.out.println(" - sub request for "+path);
	/*
	* Check for obvious errors
	*/
		if ( path == null || path.charAt( 0 ) != '/' ) {
		    fileGetError = HttpServletResponse.SC_BAD_REQUEST;
		    return false;
		}
	/*
	* Make sure user isn't trying to snoop our system
	*/
		if ( path.indexOf( "/../" ) != -1 || path.endsWith( "/.." ) ) {
			fileGetError = HttpServletResponse.SC_FORBIDDEN;
		    return false;
		}

	/*
	* Check to see if we are looking for the base dir
	*/
		if (path.equals("/")) {
			fileToGet = relativeLocalRoot;
			return true;
		}
	/*
	* Strip off leading /
	*/
		if (path.charAt(0) == '/')
			path = path.substring( 1 );

	/*
	* Strip off trailing /
	*/		
		if ( path.endsWith("/") )
		    path = path.substring( 0, path.length() - 1 );

	/*
	* Create a file reference for what is left using relative local root
	*/
		String filename = path.replace( '/', File.separatorChar );

//		filename = getServletContext().getRealPath( filename );
		fileToGet = new File(relativeLocalRoot, filename );
		
		if (fileToGet.exists()) {
//			System.out.println(" file is "+fileToGet.getAbsolutePath());
			return true;
		}
		

		log(getClass().getName()+" cant find - "+filename);

		fileGetError = HttpServletResponse.SC_NOT_FOUND;
		return false;
	}
/**
* Implement the GET method
*/
	protected void doGet (HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException {

		if (!resolvePath( req)) {
			res.sendError(fileGetError);
			return;
		}

	    if ( fileToGet.isDirectory() ) {
	    	doGetDirectory( req, res, false, fileToGet );
	    	}
	    else 
			doGetFile( req, res, false, fileToGet );
	}



    private void doGetFile( 
    		HttpServletRequest req, 
    		HttpServletResponse res, 
    		boolean headOnly, 
    		File file ) throws IOException
	{
//	log( "getting " + req.getRequestURI() );
	if ( ! file.canRead() )
	    {
	    res.sendError( HttpServletResponse.SC_FORBIDDEN );
	    return;
	    }

	res.setStatus( HttpServletResponse.SC_OK );

	res.setContentType( getServletContext().getMimeType( file.getName() ) );
	res.setContentLength( (int) file.length() );
	res.setDateHeader( "Last-modified", file.lastModified() );
	OutputStream out = res.getOutputStream();
	if ( ! headOnly ) {
	    InputStream in = new FileInputStream( file );
	    copyStream( in, out );
	    in.close();
	    }
	}
    
    private void copyStream( InputStream in, OutputStream out ) throws IOException
	{
		byte[] buf = new byte[4096];
		int len;
		while ( ( len = in.read( buf ) ) != -1 ) {
		    if (len > 0)
		    	out.write( buf, 0, len );
		}
	}


    private void doGetDirectory( 
    			HttpServletRequest req, 
    			HttpServletResponse res, 
    			boolean headOnly, 
    			File file ) throws IOException
	{
	String pathStr = req.getRequestURI();
	int t1 = pathStr.indexOf("//");
	int t2 = pathStr.indexOf('/',t1);
	if (t2 >= 0)
		pathStr = pathStr.substring(t2);
		
//	log( "get dir " + pathStr );
	if ( ! file.canRead() )
	    {
	    res.sendError( HttpServletResponse.SC_FORBIDDEN );
	    return;
	    }
	res.setStatus( HttpServletResponse.SC_OK );
	res.setContentType( "text/html" );
	OutputStream out = res.getOutputStream();
	if ( ! headOnly )
	    {
	    PrintStream p = new PrintStream(  out );
	    p.println( "<HTML><HEAD>" );
	    p.println( "<TITLE>Index of " + pathStr + "</TITLE>" );
	    p.println( "</HEAD><BODY BGCOLOR=\"#a0c0a0\">" );
	    p.println( "<H2>Index of " + pathStr + "</H2>" );
	    p.println( "<PRE>" );
	    p.println( "mode     bytes  last-changed  name" );
	    p.println( "<HR>" );
	    String[] names = file.list();
	    Acme.Utils.sortStrings( names );
		for ( int i = 0; i < names.length; ++i ) {
			File aFile = new File( file.getAbsolutePath()+File.separatorChar+ names[i] );
			String aFileType;
			if ( aFile.isDirectory() )
			    aFileType = "d";
			else if ( aFile.isFile() )
			    aFileType = "-";
			else
			    aFileType = "?";
			String aFileRead = ( aFile.canRead() ? "r" : "-" );
			String aFileWrite = ( aFile.canWrite() ? "w" : "-" );
			String aFileExe = "-";
			String aFileSize = Acme.Fmt.fmt( aFile.length(), 8 );
			String aFileDate =
			    Acme.Utils.lsDateStr( new Date( aFile.lastModified() ) );
			String aFileDirsuf = ( aFile.isDirectory() ? "/" : "" );
			String aFileSuf = ( aFile.isDirectory() ? "/" : "" );
			String qualifiedFileName = names[i];
			p.println(
			    aFileType + aFileRead + aFileWrite + aFileExe +
			    "  " + aFileSize + "  " + aFileDate + "  " +
			    "<A HREF=\"" + Codecs.urlPathEncode(qualifiedFileName) + aFileDirsuf + "\">" +
			    Codecs.HTMLEncode(names[i]) + aFileSuf + "</A>" );
			}
	    p.println( "</PRE>" );
	    p.println( "<HR>" );
	    p.println( "</BODY></HTML>" );
	    p.flush();
	    }
	    
	}
    
}
