/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.monitor.watcher;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.awt.Point;
import java.awt.Dimension;
import java.util.Random;

import org.mitre.pam.interfaces.PIM;
import org.mitre.pam.interfaces.DIM;
import org.mitre.pam.interfaces.ConnectedAgent;


/**
* An island is kind of like a cluster but we only use it to test
* for the independence of two subnetworks.  Islands can be promoted
* into clusters.
*
* @author Nazario Irizarry, Jr.
*/
class Island {
	private static final NoSuchAgentException NSE = new NoSuchAgentException();
	Hashtable allNodes;
	Hashtable items;
	public static final String DEBUG_NAME = "watcher";


/*
	Island() {
	}
*/

	Island(Hashtable allNodes) {
		this.allNodes = allNodes;

		items = new Hashtable(16);
	}
/**
* enumeration of nodes
*/
	final Enumeration nodeEnumeration() {
		return items.elements();
	}
/*
* Does this island contain the specified agent?
*/
	final boolean contains(ConnectedAgent agent) {
		return items.containsKey(agent);
	}
/*
* Does this island contain the node?
*/
	final boolean contains(NodeData node) {
		return items.containsKey(node.agent);
	}
/*
* Grow a cluster that contains a specified agent
*/
	final void growAroundR(ConnectedAgent agent) {
		NodeData node = (NodeData)allNodes.get(agent);
		growAroundR(node);
	}
/*
* Grow a cluster that contains a specified node
*/
	final void growAroundR(NodeData node) {
		ConnectedAgent agent = node.agent;
		
		items.put(node.agent, node);
		if (!(agent instanceof PIM)) {
			Enumeration e = agent.getSourceList().elements();
			while (e.hasMoreElements()) {
				ConnectedAgent source = (ConnectedAgent) e.nextElement();
				if (!contains(source)) {
					growAroundR(source);
				}
			}
		}
		if (!(agent instanceof DIM)) {
			Enumeration e = agent.getSinkList().elements();
			while (e.hasMoreElements()) {
				ConnectedAgent sink = (ConnectedAgent) e.nextElement();
				if (!contains(sink)) {
					growAroundR(sink);
				}
			}
		}
	}
/*
* The size of the island is the number of members
*/

	final int size() {
		return items.size();
	}
/**
* Get list of node inputs for the specified node
*/
	protected Enumeration inputNodesOn(NodeData node) {
		return inputNodesOn(node.agent);
	}
/**
* Get list of node inputs for the specified agent
*/
	protected Enumeration inputNodesOn(ConnectedAgent agent) {
		Vector list = new Vector(5);
		if (!(agent instanceof PIM)) {
			Enumeration e = agent.getSourceList().elements();
			while (e.hasMoreElements()) {
				ConnectedAgent source = (ConnectedAgent) e.nextElement();
				try {
					list.addElement(nodeForAgent(source));
				}
				catch (NoSuchAgentException ignore) {
				}
			}
		}
		return list.elements();	
	}
/**
* Get list of node outputs for the specified node
*/
	protected Enumeration outputNodesOn(NodeData node) {
		return outputNodesOn(node.agent);
	}
/**
* Get list of node outputs for the specified agent
*/
	protected Enumeration outputNodesOn(ConnectedAgent agent) {
		Vector list = new Vector(5);
		if (!(agent instanceof DIM)) {
			Enumeration e = agent.getSinkList().elements();
			while (e.hasMoreElements()) {
				ConnectedAgent sink = (ConnectedAgent) e.nextElement();
				try {
					list.addElement(nodeForAgent(sink));
				}
				catch (NoSuchAgentException ignore) {
				}
			}
		}
		return list.elements();	
	}
/*
* Lookup an agents node
*/
	protected NodeData nodeForAgent(ConnectedAgent agent) throws NoSuchAgentException {
		NodeData node = (NodeData) items.get(agent);
		if (node != null)
			return node;
		throw NSE;
	}

}