/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.monitor.watcher;

import java.awt.Color;
import java.awt.Graphics;
import org.mitre.pam.util.draw.Block;
import org.mitre.pam.util.draw.BlockGroup;
import org.mitre.pam.executive.ExecGlobal;

import org.mitre.pam.interfaces.Memento;
import org.mitre.pam.interfaces.MementoListener;
import org.mitre.pam.interfaces.MementoEvent;
import org.mitre.pam.interfaces.Handler;

/**
* A simple extension of Block to allow
* "animation".
*
* @author Nazario Irizarry, Jr.
*/
final class WatchBlock extends Block implements MementoListener{

	static BlockGroup defaultBlockGroup = new BlockGroup();

	private static Color idleColor = defaultBlockGroup.normalBackgroundColor;
	private static Color waitColor = Color.yellow.brighter();
	private static Color runColor = Color.green.brighter();
	private static final int IDLE = 0;
	private static final int WAIT = 1;
	private static final int RUN = 2;
	private static final int NSTEPS = 4;
	private static final int NCOLORS = 3;
	
	private static Color[] basicColor = {idleColor, waitColor, runColor}; // NCOLORS
	private int transitionIndex = IDLE;
	private int targetIndex = IDLE;
	
	Color [] activeTransition;
	private int transitionStep;
	private static Color transitionMatrix[][][];
	
	boolean okToPaint = true;
	
	

/**
* This static code creates the transition sequences.
*/
	static {
		transitionMatrix = new Color[NCOLORS][NCOLORS][];
		for (int from = 0; from < NCOLORS; from++)
			for (int to = 0; to < NCOLORS; to++)
				transitionMatrix[from][to] = makeTransitionSequence(from,to);
	}
	
	static Color[] makeTransitionSequence(int from, int to) {
		Color[] cm = new Color[NSTEPS];
		for (int i = 0; i < NSTEPS; i++) {
			cm[i] = blend(from, to, i);
		}
		return cm;
	}
	
	static Color blend(int from, int to, int step) {
		if (from == to)
			return basicColor[to];
		step++; // the first step should move away from the starting color
		if (step == NSTEPS)
			return basicColor[to]; // end sequence on an exact color
		
		Color fromColor = basicColor[from];
		Color toColor = basicColor[to];
		int red = fromColor.getRed() + (toColor.getRed() - fromColor.getRed())*step/NSTEPS;
		int green = fromColor.getGreen() + (toColor.getGreen() - fromColor.getGreen())*step/NSTEPS;
		int blue = fromColor.getBlue() + (toColor.getBlue() - fromColor.getBlue())*step/NSTEPS;
		return new Color(red,green,blue);
	}
/**
* Override
*/
	protected Color getBackgroundColor() {
		if (activeTransition == null)
			return basicColor[transitionIndex];

		Color returnColor = activeTransition[transitionStep++];
		if (transitionStep >= NSTEPS) {
			if (transitionIndex == targetIndex) {
				doneTransition();
			}
			else {
				setTransition(targetIndex);
			}
			
		}
			
		return returnColor;
	}
/*
	private int reverseLookup(Color color) {
		if (color == idleColor)
			return IDLE;
		if (color == waitColor)
			return WAIT;
		if (color == runColor)
			return RUN;
		System.err.println(" not a known color");
		return IDLE;
	}
*/
	private void setTransition(int to) {
//		System.out.println("x to "+this+"   "+transitionIndex+" -> "+to);
		activeTransition = transitionMatrix[transitionIndex][to];
		transitionStep = 0;
		transitionIndex = to;
	}
	
	private void doneTransition() {
		activeTransition = null;
//		System.out.println("done "+this+"   "+transitionIndex);
	}
/**
* Set a new color state
*/
	void setTargetState(int to) {
		if (targetIndex == to)
			return;
		if (activeTransition == null)
			setTransition( to);
		targetIndex = to;
	}
/**
* Change shape or color if needed
*/
	void morph() {
		if (activeTransition == null)
			return;
		refresh();		
	}
/**
* We want to watch for agents that enable or disable
*/
    public void propertyChange(MementoEvent evt){
    	String attribute = evt.getPropertyName();
    	if (attribute == ExecGlobal.STATE) {
    		Object value = evt.getNewValue();
    		if (value == ExecGlobal.IDLE) {
    			setTargetState(IDLE);
    		}
    		else if (value == Handler.STATE_PIMS_OUTSTANDING) {
    			setTargetState(WAIT);
    		} 
    		else if (value == Handler.STATE_DIMS_OUTSTANDING) {
//    			setTargetState(WAIT);
    			setTargetState(RUN); // makes handlers look like they are doing something
    		}
    		else if (value == ExecGlobal.WORKING) {
    			setTargetState(RUN);
    		}
    		else  {
    			setTargetState(WAIT);
    		}
    	}
    }

/**
* Override to implement invisibility
*/
	public void paint(Graphics g) {
		if (okToPaint) 
			super.paint(g);
	}
}
