/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.monitor.watcher;

import org.mitre.pam.interfaces.AdminTool;

/**
* A display to watch PAM
* dynamic interconnections.
*
* @version 1, 15 February 1999
* @version 2, 28 July 1999
* @author Nazario Irizarry, Jr.
*/
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

import java.util.Random;
import java.awt.event.WindowEvent;
import java.awt.Point;
import java.awt.Dimension;
import java.awt.event.WindowListener;


import org.mitre.pam.interfaces.HasProperties;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.Memento;
import org.mitre.pam.interfaces.MementoListener;
import org.mitre.pam.interfaces.MementoEvent;

import org.mitre.pam.interfaces.ConfigurationListener;
import org.mitre.pam.interfaces.ConfigurationEvent;
import org.mitre.pam.interfaces.AdminTool;
import org.mitre.pam.interfaces.ExecToController;
import org.mitre.pam.interfaces.ExecToEditor;
import org.mitre.pam.interfaces.ExecToMonitor;
import org.mitre.pam.interfaces.ConnectedAgent;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.PIM;
import org.mitre.pam.interfaces.DIM;

import org.mitre.pam.util.draw.DrawPanel;
import org.mitre.pam.util.draw.DrawListener;
import org.mitre.pam.util.draw.DrawParent;
import org.mitre.pam.util.draw.BlockConnector;
import org.mitre.pam.util.draw.GenericConnector;
import org.mitre.pam.util.draw.Block;
import org.mitre.pam.util.draw.DrawEvent;
import org.mitre.pam.util.alarm.AlarmReporter;

public final class Watcher extends DrawPanel implements WindowListener, AdminTool, ConfigurationListener, MementoListener, Runnable  {
	public static final String DEBUG_NAME = "watcher";
	private static final NoSuchAgentException NSE = new NoSuchAgentException();
	private static Random random = new Random(System.currentTimeMillis());

	ExecToEditor exec;
	Thread animationThread;
	
	private AlarmReporter reporter = new AlarmReporter(DEBUG_NAME);
	DrawParent drawParent;
	private final int leftEdge = 60; // left position of cluster on screen
	

	/*
	* a hashtable of ConnectedAgents
	*/

	Hashtable nodes = new Hashtable(200);
	Vector clusters = new Vector(20);
	boolean initialized;
	
	SinglesCluster singlesCluster = new SinglesCluster(nodes, reporter);

	public Watcher() {
		setTitle(DEBUG_NAME);
		drawParent = getDrawParent();
		addWindowListener(this);
		buttonDoFunc.setEnabled(false);
		buttonSubmit.setEnabled(false);
		buttonReload.setEnabled(true);
		buttonReload.addActionListener(new SymAction());
		setReporter(reporter);
	}
	
    public void setVisible(boolean b)
	{
		if(b)
		{
			if (!initialized) {
				if (exec != null) {
					exec.addConfigurationListener( this );
					initialized = true;
					loadEnabledAgents();
					animationThread = new Thread(this);
					animationThread.setPriority(4);
					animationThread.start();
				}
				else {
					System.err.println("Watcher -- exec is null(1)");
				}
			}
		}
		else {
			if (initialized) {
				if (exec != null) {
					exec.removeConfigurationListener( this );
					watchNoAgents();
					initialized = false;
				}
				else {
					System.err.println("Watcher -- exec is null(2)");
				}
			}
		}
		super.setVisible(b);
	}


	private void watchNoAgents() {
		Enumeration enum = exec.getListOfActiveAgents(null,null);

		while (enum.hasMoreElements()) {
			Object next =  enum.nextElement();
			if (next instanceof ConnectedAgent) {
				ConnectedAgent agent = (ConnectedAgent) next;
				agent.getMemento().removeMementoListener(this);
			}

		}
	}

	private void setNodesToOffscreen() {
		Enumeration enum = nodes.elements();
		while (enum.hasMoreElements()) {
			NodeData node =  (NodeData) enum.nextElement();
			node.setOnScreen(false);
		}
	}
	
	private void loadEnabledAgents() {
		drawParent.removeAll();
		setNodesToOffscreen();
		nodes.clear();
		clusters.setSize(0);
		singlesCluster.clear();
		Enumeration enum = exec.getListOfActiveAgents(null, null); // gets all agents

		while (enum.hasMoreElements()) {
			Object next =  enum.nextElement();
			if (next instanceof ConnectedAgent) {
				ConnectedAgent agent = (ConnectedAgent) next; 
				if (agentIsLoadable(agent)) {
					addAgent(agent);
				}
				else {
					watchAgent(agent);
				}
			}
		}
	}

	private void watchAgent(ConnectedAgent agent) {
		agent.getMemento().addMementoListener(this);
	}

	private boolean agentIsLoadable(ConnectedAgent agent) {
		return agent.isEnabled();
	}

	/**
	* A node has been created 
	*/
	public void nodeCreated(ConfigurationEvent e) {
		Agent agent = e.getAgent();
		if (agent instanceof ConnectedAgent) {
			ConnectedAgent cAgent = (ConnectedAgent)agent;
			if (agentIsLoadable(cAgent)) {
				addAgent(cAgent);
			}
			else {
				watchAgent(cAgent);
			}
		}
	}
			
	/**
	* A node deleted
	*/
	public void nodeDeleted(ConfigurationEvent e) {
		removeAgent(e.getAgent());
	}

	private void removeAgent(Agent agent) {
		NodeData nodeData = (NodeData)nodes.remove(agent);
		if (nodeData != null) {
			reporter.debugMsg("remove agent",agent,null,null,DEBUG_NAME);
			drawParent.remove(nodeData.block);
			nodeData.setOnScreen(false);
			Cluster cluster = nodeData.getParentCluster();
			if (cluster != null) {
				cluster.removeNode(nodeData);
				cluster.kill(); // flag for further action
			}
		}
	}
/**
* Add a new node to an appropriate cluster
*/
	private void addNodeToExistingClusters(NodeData node) {
		ConnectedAgent agent = node.agent;
		reporter.debugMsg("Add node to existing clusters :",node,null,null,DEBUG_NAME);
		Vector addFromSingles = new Vector(10);
	/*
	* Create a list of clusters that are the parents of input or
	* output agents
	*/
		Vector clusterList = new Vector(5);
		if (!(agent instanceof PIM)) {
			Enumeration e = inputNodesOn(agent);
			while (e.hasMoreElements()) {
				NodeData sourceNode = (NodeData) e.nextElement();
				Cluster parentCluster = sourceNode.getParentCluster();
				if (parentCluster == singlesCluster) {
					addFromSingles.addElement(sourceNode);
				}
				else if (parentCluster != null && 
					!parentCluster.isDead() && 
					!clusterList.contains(parentCluster)) {
					clusterList.addElement(parentCluster);
				}
			}
		}
		if (!(agent instanceof DIM)) {
			Enumeration e = outputNodesOn(agent);
			while (e.hasMoreElements()) {
				NodeData sinkNode = (NodeData)  e.nextElement();
				Cluster parentCluster = sinkNode.getParentCluster();
				if (parentCluster == singlesCluster) {
					addFromSingles.addElement(sinkNode);
				}
				else if (parentCluster != null && 
					!parentCluster.isDead() && 
					!clusterList.contains(parentCluster)) {
					clusterList.addElement(parentCluster);
				}
			}
		}
	/*
	* If there were no existing clusters then we need a new one
	*/
		if (clusterList.size() == 0) {
			Cluster cluster = new Cluster(nodes, reporter);
			reporter.debugMsg("new cluster: ",cluster,null,null,DEBUG_NAME);
			cluster.addNode(node);
			clusters.addElement(cluster);
		}
	/*
	* If there was one cluster then add the node to the existing cluster
	*/
		else if (clusterList.size() == 1) {
			Cluster cluster = (Cluster) clusterList.elementAt(0);
			reporter.debugMsg("existing cluster: ",cluster,null,null,DEBUG_NAME);
			cluster.addNode(node);
		}
	/*
	* If there was more than one cluster then we need to merge clusters
	*/
		else {
			reporter.debugMsg("merge clusters",null,null,null,DEBUG_NAME);
			Cluster cluster = (Cluster) clusterList.elementAt(0);
			cluster.addNode(node);
			for (int i = 1; i<clusterList.size(); i++) {
				Cluster consumedCluster = (Cluster) clusterList.elementAt(i);
				reporter.debugMsg(" merging ",consumedCluster," --> ",cluster,DEBUG_NAME);
				cluster.mergeWith(consumedCluster);
				clusters.removeElement(consumedCluster);
			}
		}
	/*
	* Add any orphaned nodes that we are connected to that used to be in the singles cluster
	*/
		Cluster cluster = node.getParentCluster();
		for (int i = 0; i< addFromSingles.size(); i++) {
			cluster.addNode((NodeData)addFromSingles.elementAt(i));
		}
	}
/**
* Add a new node to database
*/
	private NodeData addAgent(ConnectedAgent agent) {
		NodeData node;
	/*
	* Don't add if we have it already
	*/
		if ((node = (NodeData) nodes.get(agent)) != null) {
			return node;
		}
		agent.getMemento().addMementoListener(this);
		reporter.debugMsg("add agent ",agent,null,null,DEBUG_NAME);
	/*
	* Add a new node
	*/
		node = new NodeData(agent, new WatchBlock());
		nodes.put(agent, node);
		drawParent.add(node.block);
		node.setOnScreen(true);
	/*
	* Add a node's connections
	*/
		Enumeration e = inputNodesOn(agent);
		while (e.hasMoreElements()) {
			NodeData sourceNode = (NodeData) e.nextElement();
			sourceNode.updateConnectionInfo(); // in case node just enabled, then its inputs have a bad connection count
			BlockConnector conn = new BlockConnector(sourceNode.block, node.block);
			drawParent.add(conn);
		}

		e = outputNodesOn(agent);
		while (e.hasMoreElements()) {
			NodeData sinkNode = (NodeData) e.nextElement();
			sinkNode.updateConnectionInfo(); // in case node just enabled, then its outputs have a bad connection count
			BlockConnector conn = new BlockConnector(node.block, sinkNode.block);
			drawParent.add(conn);
		}

		return node;
	}
	/**
	* A connection made
	*/
	public void connectionMade(ConfigurationEvent e){
		try {
			NodeData fromNode = nodeForAgent(e.getAgentFrom());
			NodeData toNode = nodeForAgent(e.getAgentTo());
			if (fromNode == null || toNode == null)
				return; // connection is from or to a disabled node
	
			fromNode.updateConnectionInfo();
			toNode.updateConnectionInfo();
			BlockConnector conn = new BlockConnector(fromNode.block, toNode.block);
			drawParent.add(conn);
		/*
		* If either the from cluster or the to cluster are null
		* then don't worry about anything because
		* the main maintenance loop will catch the unassigned
		* node(s) and assign the to clusters
		* But if there are two existing clusters then we have to
		* do something because we may need to merge two different clusters
		*/
			Cluster fromCluster = fromNode.getParentCluster();
			if (fromCluster == null)
				return;
			Cluster toCluster = toNode.getParentCluster();
			if (toCluster == null)
				return;
		/*
		* if the two cluster references are really the same
		* cluster then we just need to layout the cluster
		*/
			if(fromCluster == toCluster) {
//		reporter.debugMsg("existing cluster: ",fromCluster,null,null,DEBUG_NAME);
				fromCluster.layoutNeeded();
			}
		/*
		* if the two clusters are different
		* then one needs to die
		*/
			else {
				if (fromCluster.items.size() > toCluster.items.size()) {
					toCluster.kill();
//					reporter.debugMsg(" killed ",toCluster," for a merge with ",cluster,DEBUG_NAME);
				}
				else {
					fromCluster.kill();
//					reporter.debugMsg(" killed ",fromCluster," for a merge with ",cluster,DEBUG_NAME);
				}
			}
		}
		catch (NoSuchAgentException ignore) {}
	}
	
	/**
	* A connection broken
	*/
	public void connectionBroken(ConfigurationEvent e){
		try {
			NodeData fromNode = nodeForAgent(e.getAgentFrom());
			NodeData toNode = nodeForAgent(e.getAgentTo());
			fromNode.updateConnectionInfo();
			toNode.updateConnectionInfo();
			BlockConnector conn = new BlockConnector(fromNode.block, toNode.block);
			drawParent.remove(conn);
			Cluster fromCluster = fromNode.getParentCluster();
			Cluster toCluster = toNode.getParentCluster();
		/*
		* Since update of the clustering is delayed it may be possible
		* that the from and to clusters are different.  If that
		* is the case then there is nothing to do.
		*/
			if (fromCluster != toCluster) {
				return;
//				throw new RuntimeException(" From and To clusters not same, "+fromCluster+ ",     "+toCluster);
			}
		/*
		* do we get two clusters now?
		*/
			Island testIsland = new Island(nodes);
			testIsland.growAroundR(fromNode);
	reporter.debugMsg("test island size = ",Integer.toString(testIsland.size()),", ",toCluster,DEBUG_NAME);
	reporter.debugMsg(" size = ",Integer.toString(toCluster.size()),null,null,DEBUG_NAME);
		/*
		* Split into two clusters
		*/
			if (testIsland.size() != toCluster.size()) {
	reporter.debugMsg(toCluster," will be split",null,null,DEBUG_NAME);
				Cluster newCluster = new Cluster(testIsland, reporter);
				newCluster.layout();
				newCluster.setOrigin(toCluster.getOriginX(), toCluster.getOriginY() + 20);
				clusters.addElement(newCluster);
			}
		/*
		* Still have one cluster
		*/
			else {
	reporter.debugMsg(toCluster," remains intact",null,null,DEBUG_NAME);
				toCluster.layoutNeeded();
			}
			
		}
		catch (NoSuchAgentException ignore) {}
	}

	
/**
* Close the window.
*/
	public void windowClosing(WindowEvent e) {
		setVisible(false);		 // hide the Frame
	}
	

	public boolean isDebugSession() {
		return true;
	}


/**
* Tell the tool to become active and to work -- from interface AdminTool
*/
	public void setEnabled(boolean enabled){
	}
/**
* Tell tool to display state for named agent -- from interface AdminTool.

* Not implemented
*/
	public void displayAgent(String agent){
	}
/**
* Give the tool a handle to the exec -- from interface AdminTool.
*/
	public void setExec(ExecToController exec){
		this.exec = exec;
	}
/**
* Allocate nodes to clusters.
* Some clusters may already exist.
*/
	private void verifyClustering() {
//		reporter.debugMsg(" Verify clusters",null,null,null,DEBUG_NAME);
	/*
	* Find and remove dead clusters
	*/
		for (int i = clusters.size()-1; i>=0; i--) {
			Cluster cluster = (Cluster) clusters.elementAt(i);
			if (cluster.size() == 0 || cluster.isDead()) {
				reporter.debugMsg("- empty or dead cluster removed",null,null,null,DEBUG_NAME);
				clusters.removeElementAt(i);
			}
		}
	/*
	* Update node connection info for nodes in the singles cluster
	*/
//		singlesCluster.scanMembersConnections();
	/*
	* Find nodes that are not in a cluster and allocate to a cluster
	*/
		Enumeration e = nodes.elements();
		while (e.hasMoreElements()) {
			NodeData node = (NodeData) e.nextElement();
			Cluster parentCluster = node.getParentCluster();
			if (node.hasNoConnections() ) {
				if (parentCluster != singlesCluster)
					singlesCluster.addNode(node);
			}
			else {
				if (parentCluster == null || 
						parentCluster.isDead() || 
						(parentCluster == singlesCluster)) {
					reporter.debugMsg("looking for new cluster for",node,null,null,DEBUG_NAME);
					addNodeToExistingClusters(node);
				}
			}
		}
		
	/*
	* Find and remove new empty clusters
	*/
		for (int i = clusters.size()-1; i>=0; i--) {
			Cluster cluster = (Cluster) clusters.elementAt(i);
			if (cluster.size() == 0) {
		reporter.debugMsg("- empty cluster removed",null,null,null,DEBUG_NAME);
					clusters.removeElementAt(i);
			}
		}

		if (clusters.size() > 0) {	

	/*
	* Allocate order to unordered nodes
	*/
			for (int i = clusters.size()-1; i>=0; i--) {
				Cluster cluster = (Cluster) clusters.elementAt(i);
				if (!cluster.hasGoodLayout())
					cluster.layout();
			}
			if (!singlesCluster.hasGoodLayout())
				singlesCluster.layout();
	/*
	* Allocate order to unordered clusters.
	*/
			assignPositionsToClusters();

		}
	}

/**
* Assign initial position to clusters
*/
	private void assignPositionsToClusters() {
		Enumeration e = clusters.elements();
		if (reporter.isDebugKey(DEBUG_NAME)) {
//			reporter.log("** Assign positions to clusters :");
			while (e.hasMoreElements()){
				Cluster cluster = (Cluster) e.nextElement();
				cluster.debugListMembers();
			}
		}

	/*
	* Assign each cluster a position
	*/
		e = clusters.elements();
		int posY = 40;
		int maxWidth = 40;
		while (e.hasMoreElements()){
			Cluster cluster = (Cluster) e.nextElement();
			Dimension d = cluster.getMass();
			cluster.setOrigin(leftEdge, posY);			
			posY += 15 + cluster.height; // pos for next cluster
			if (cluster.width + leftEdge > maxWidth)
				maxWidth = cluster.width + leftEdge;
		}

		Dimension d = singlesCluster.getMass();
		singlesCluster.setOrigin(leftEdge, posY);			
		posY += 15 + singlesCluster.height; // pos for next cluster
		if (singlesCluster.width + leftEdge > maxWidth)
			maxWidth = singlesCluster.width;

		
		checkWindowSize(maxWidth+50, posY+50);
	} 
/*
* Adjust window size if it is too small
*/
	private void checkWindowSize(int width, int height) {
		Dimension d = drawParent.getSize();
		if ((d.width  < (width + 50)) ||
				(d.height < (height + 50))) {
			drawParent.setSize(width+150,height+150);
			drawParent.invalidate();
			validate();
		}
	}
/*
* Lookup an agents node
*/
	private NodeData nodeForAgent(ConnectedAgent agent) throws NoSuchAgentException {
		NodeData node = (NodeData) nodes.get(agent);
		if (node != null)
			return node;
		throw NSE;
	}

	public void setTitle(String title) {
		super.setTitle("Watcher, "+title);
	}
/**
* We want to watch for agents that enable or disable
*/
    public void propertyChange(MementoEvent evt){
    	String attribute = evt.getPropertyName();
    	if (attribute == Agent.ENABLED) {
    		Agent agent = (Agent) exec.getActiveAgent(((Memento)evt.getSource()).getParentId());
    		if ( !( agent instanceof ConnectedAgent) )
    			return;
    			
    		if (evt.getNewValue().equals("true") && agentIsLoadable((ConnectedAgent)agent)) {
    			addAgent((ConnectedAgent)agent);
    		}
    		else {
    			removeAgent(agent);
    		}
    	}
    }

/**
* Get list of node inputs for the specified node
*/
	protected Enumeration inputNodesOn(NodeData node) {
		return inputNodesOn(node.agent);
	}
/**
* Get list of node inputs for the specified agent
*/
	protected Enumeration inputNodesOn(ConnectedAgent agent) {
		Vector list = new Vector(5);
		if (!(agent instanceof PIM)) {
			Enumeration e = agent.getSourceList().elements();
			while (e.hasMoreElements()) {
				ConnectedAgent source = (ConnectedAgent) e.nextElement();
				try {
					list.addElement(nodeForAgent(source));
				}
				catch (NoSuchAgentException ignore) {
				}
			}
		}
		return list.elements();	
	}
/**
* Get list of node outputs for the specified node
*/
	protected Enumeration outputNodesOn(NodeData node) {
		return outputNodesOn(node.agent);
	}
/**
* Get list of node outputs for the specified agent
*/
	protected Enumeration outputNodesOn(ConnectedAgent agent) {
		Vector list = new Vector(5);
		if (!(agent instanceof DIM)) {
			Enumeration e = agent.getSinkList().elements();
			while (e.hasMoreElements()) {
				ConnectedAgent sink = (ConnectedAgent) e.nextElement();
				try {
					list.addElement(nodeForAgent(sink));
				}
				catch (NoSuchAgentException ignore) {
				}
			}
		}
		return list.elements();	
	}

/**
* The body of the animation thread
*/
	private long lastClusterVerify;
	public void run() {
		do {
			try {
				Thread.sleep(200);
				lastClusterVerify -= 200;
			}
			catch (InterruptedException ignore) {}
			
			if (lastClusterVerify < 0) {
				lastClusterVerify = 10000;
				verifyClustering();
			}
			
			synchronized(drawParent.manipulateLines) {
				Enumeration clusterList = clusters.elements();
				while (clusterList.hasMoreElements()) {
					Cluster cluster = (Cluster) clusterList.nextElement();
					cluster.slide();
				}
				
				singlesCluster.slide();
			}
		} while (isVisible());
	}
/**
* We don't do anything with this event
*/
    public void windowOpened(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowClosed(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowIconified(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowDeiconified(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowActivated(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowDeactivated(WindowEvent e) {}

	void reload(java.awt.event.ActionEvent event) {
		watchNoAgents();
		loadEnabledAgents();
	}

	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
//			if (object == buttonReload)
				reload(event);
		}
	}
}