/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.productVisitor;


import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.io.PrintWriter;
import java.net.URL;

import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ProductVisitor;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.store.FileRef;
import org.mitre.pam.interfaces.ContainerProduct;

/**
* Modified from AsPlainText
*
* @author Michael DeCorpo
*/

public class AsBWInfoFlash {
	protected String summaryFileName;
	protected Reporter reporter;
	
	protected Product inputProduct;
	
	private static int count = 1;
/**
* 
*/
	public AsBWInfoFlash (String summaryFileName, Reporter reporter) {
		if (summaryFileName == null || summaryFileName.length()==1 || reporter == null)
			throw new IllegalArgumentException("argument must not be empty");
		this.summaryFileName = summaryFileName;
		this.reporter = reporter;
	}
	
	public void visit(Product inputProduct) {
		this.inputProduct = inputProduct;
	}

	public void writeResults(PrintWriter out) {
		process(out, inputProduct, 0);			
	}
	

	protected void process(PrintWriter out, Object product) {
		process(out, product, 0);			
	}

	protected void process(PrintWriter out, Object product, int level) {
		if (product == null)
			return;

		if (product instanceof Product)
			process(out, (Product) product, level);
	}
	
	protected void process(PrintWriter out, Product product, int level) {
		if (product == null)
			return;

		// count is 1 or 2
		if ((count < 1) || (count > 2))
		  count = 1;
	/*
	* "name, date"
	*/

		String pid = product.getId();

		if (level == 0) {
			doPreamble(out, pid + " - " + product.getModificationDate().toString());
		}

		Object content = product.getContent();
		/*
		* enumeration contents
		*/
		if (content instanceof Enumeration) {
			Product root = ((ContainerProduct)product).getRoot();
			if (root != null) {
				process(out, root, level+1);
			}
			else {
				Enumeration enum = (Enumeration) content;
				while (enum.hasMoreElements()) {
				  process(out, enum.nextElement(), level+1);
				}
			}
		}
		/*
		* String contents
		*/
		else if (content instanceof String) {
		  printOneProduct(out, (String)content, null);
		}
		/*
		* FileRef contents
		*/
		else if (content instanceof FileRef) {
		  String name = ((FileRef) content).getFile().getName();
//		  String name = product.getId();
		  printOneProduct(out, pid, null);

		}
		/*
		* URL contents
		*/
		else if (content instanceof URL) {
		  printOneProduct(out,pid,((URL)content).toString());

		}
		/*
		* unknown contents
		*/
		else {
			reporter.warning("unrecognized content type");
//			throw new IllegalArgumentException(" AsBWInfoFlash, unrecognized content "+content);
		}
	}

  // print the Script text for one product
  private void printOneProduct(PrintWriter out, String text, String link)
  {
    out.print("    prepareText text=\"" + sanitize(text));
    for (int i=0; i<220-text.length(); i++)
      out.print(" ");
    out.println("\"");
    out.println("        rect(114,48,391,97) sequence=" + count);
    out.println("        font=\"Helvetica\" size=10");
    out.println("        fgColor=darkred");
    out.println("        bgColor=white");
    out.println("    onClick {");
    if (link != null && !link.equals("")) {
    	out.println("        rect(113,48,392,98) does { accessWeb URL=\"" + link + "\" }");
    }
    else {
    	out.println("        rect(113,48,392,98) does { accessWeb URL=\"" + summaryFileName + "\" }");
    }
    out.println("        rect (386,5,399,18) does { end all }");
    out.println("        rect (106,7,170,44) does { accessWeb URL=\"" + summaryFileName + "\" }");
    out.println("    }\n");
    out.println("    show sequence=" + count);
    nextcount();
    out.println("    pause for 3 seconds \n");
  }

	private void doPreamble(PrintWriter out, String title) {

	  // prepare the "Full Page" and Product Description links text
	  // on the two text sequences.  The third is just the image
	  out.println();
	  out.println("    prepareText text=\"Full Page\"");
	  out.println("        rect (106,7,170,44) sequence=1");
	  out.println("        font=\"TimesRoman\" size=10 underline");
	  out.println("        fgColor=darkblue transparent");
	  out.println();

	  out.print("    prepareText text=\"");
	  out.print(title);
	  out.println("\"");
	  out.println("        rect (175,7,384,44) sequence=1");
	  out.println("        font=\"TimesRoman\" size=8");
	  out.println("        fgColor=black transparent");
	  out.println();

	  out.println("    prepareText text=\"Full Page\"");
	  out.println("        rect (106,7,170,44) sequence=2");
	  out.println("        font=\"TimesRoman\" size=10 underline");
	  out.println("        fgColor=darkblue transparent");
	  out.println();

	  out.print("    prepareText text=\"");
	  out.print(title);
	  out.println("\"");
	  out.println("        rect (175,7,384,44) sequence=2");
	  out.println("        font=\"TimesRoman\" size=8");
	  out.println("        fgColor=black transparent");
	  out.println("\n");

	}
	
	private String sanitize(String s) {
		StringBuffer sb = new StringBuffer(s);
		for (int i=0; i<s.length(); i++) {
			char c = sb.charAt(i);
			if (c == '\"')
				sb.setCharAt(i,' ');
			else if (c < 0x20) {		// nonprinting chars
				sb.setCharAt(i,' ');
			}
		}
		
		return sb.toString();
	}

  // increment the count to 2 or 1 depending on the current sequence
  private void nextcount()
  {
    if (count == 1)
      count = 2;
    else
      count = 1;

  }

	
}
