/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.productVisitor;

import org.mitre.pam.interfaces.ProductVisitor;
import org.mitre.pam.interfaces.Product;
import java.util.Vector;
import java.util.Enumeration;

/**
 * Get various information about a product's contents, by viewing
 * them as a tree:  (nonempty) Vectors being internal nodes and 
 * other types being leaf nodes.
 *
 * @author Srinivas Inguva
 */

public class AsTree implements ProductVisitor {
  protected Product myProduct;

  public AsTree() {
  }

  public void visit(Product product) {
    myProduct = product;
  }

  public int getDepth() {
    if (myProduct == null) 
      return 0;

    return findDepth(myProduct);
  }

  private int findDepth(Product product) {
    Object content;
    content = product.getContent();

    int depth, newDepth;
    depth = 0;

    if (content instanceof Enumeration) {
      Enumeration cv = (Enumeration) content;
//      System.out.println("AsTree.findDepth(), enumeration content, size ");
      while (cv.hasMoreElements()) {
	newDepth = 1 + findDepth(cv.nextElement());
	if (newDepth > depth)
	  depth = newDepth;
      }
    } 
    else if (content instanceof Vector) {
      Vector cv = (Vector) content;
//      System.out.println("AsTree.findDepth(), vector content, size "+cv.size());
      for (int i = 0; i< cv.size(); i++) {
	newDepth = 1 + findDepth(cv.elementAt(i));
	if (newDepth > depth)
	  depth = newDepth;
      }
    } 
    
    return depth;
  }//end findDepth(Product)

  private int findDepth(Object product) {
    if (product instanceof Product)
      return findDepth((Product) product);

    return 0;
  }//end findDepth(Object)

  public int getLeafCount() {
    if (myProduct == null) 
      return 0;
    
    return findLeaves(myProduct);
  }

  private int findLeaves(Product product) {

    Object content;
    content = product.getContent();

    int leafCount = 0;

    if (content instanceof Enumeration) {
      Enumeration cv = (Enumeration) content;
//      System.out.println("AsTree.findLeaves(), enumeration ");
      while (cv.hasMoreElements()) {
	leafCount += findLeaves(cv.nextElement());
      }
      //if, for some reason, we have a Vector with no contents,
      // treat it as a leaf node:
      if (leafCount == 0)
	leafCount = 1;

    }
    else {
      leafCount = 1;
    }
    return leafCount;

  }//end findLeaves(Product)

  private int findLeaves(Object product) {
    if (product instanceof Product)
      return findLeaves((Product) product);

    return 0;
  }//end findLeaves(Object)

}//end class AsTree
