package org.mitre.pam.productVisitor;

import org.mitre.pam.interfaces.Product;

import org.mitre.pam.store.FileRef;

import java.io.*;

/**
* This class contains functions which can be used to serialize and unserialize Products. <p>
*
* @author Jones Leung
* @author John R. Fantry - modified to support Java serialization
*/
public class Serializer {
/**
* Recreate the object using Java serialization
* @param in  the object stream to read the object from
* @param rootDir this directory will become the root for all files contained in the
*                restored object
* @return   the object which has been unserialized, or null
*/
    public static Object unSerialize(ObjectInputStream in, File rootDir) throws IOException, ClassNotFoundException {

        Object obj = null;

        FileRef.addSerialRootDir(in, rootDir);

        obj = in.readObject();

        FileRef.removeSerialRootDir(in);
        FileRef.removeDeserializedFiles(in);

        return obj;
    }

    /**
    * Serialize an object using Java serialization
    * @param obj  the object to serialize
    * @param out  the stream to write to
    */
    public static void serialize(Object obj, ObjectOutputStream out) throws IOException {
      if(!(obj instanceof Serializable)) {
        return;
      }

      if(obj instanceof Product) {
        FileRef.addSerialFileList(out, AsZipFile.buildFiles((Product)obj, null));
      }

      out.writeObject(obj);

      FileRef.removeSerialFileList(out);
      FileRef.removeSerializedFiles(out);

      out.flush();
    }
}