/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.putter;

import org.mitre.pam.util.FTP;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.Source;

import org.mitre.pam.product.view.BasicProduct;
import org.mitre.pam.product.view.VectorProduct;
import org.mitre.pam.productVisitor.AsFlatFileList;

import org.mitre.pam.store.FileRef;

import java.io.IOException;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Date;
import java.util.Enumeration;

/**
* This is a special DIM to interface to the Global Broadcast Service (Phase I).
*
* @author John Fantry, Technical Co-op (D510)
* @author GBS modifications by Michael DeCorpo
* @author Jason Mathews, many changes (D510)
*/
public class GBSFTPFileDIM extends DIMShell1 {
  protected final static String USER = "user";
  protected final static String PASSWORD = "password";
  protected final static String NOTIFY = "notifyDir";
  protected final static String FTP_URL = "ftpUrl";

  public GBSFTPFileDIM() {
    setDescription("Saves files only to specified FTP directory");
    setProperty(FTP_URL, "ftp://x.y.z/rootDir");
    setProperty(PASSWORD, "<ftp password here>");
    setProperty(USER, "anonymous");
    setProperty(NOTIFY, "/home/user/unwrapped/notify");

    metaStringInfo(FTP_URL, "ftp://x.y.z/rootDir","Destination directory in URL form");
    metaStringInfo(PASSWORD, "<ftp password here>","Password for FTP account");
    metaStringInfo(USER, "anonymous","FTP account name");
    metaStringInfo(NOTIFY, "/home/user/unwrapped/notify","Notify directory for Global Broadcast Service");
  }

  public void putProduct(Product p) {
      String url = getStringProperty(FTP_URL,"");

      if (!url.startsWith("ftp://") ) {
	  throw new IllegalArgumentException("Missing or invalid FTP URL");
      }

      String user = getStringProperty(USER,"anonymous");
      String password = getStringProperty(PASSWORD,"");

      if (user.length()==0 || password.length()==0) {
	  throw new IllegalArgumentException("Missing login information for ftp");
      }

      String notifyDir = getStringProperty(NOTIFY,"");

      if ( notifyDir.length()==0) {
	  throw new IllegalArgumentException("Missing notify directory");
      }

      if (p == null) return;

      String host, path;
      int pos = url.indexOf('/', 6);
      if (pos == -1) {
	  host = url.substring(6);
	  path = "/";
      }
      else {
	  host = url.substring(6, pos);
	  // extract path from url
	if (url.endsWith("/"))
	      path = url.substring(pos, url.length()-1);
	  else path = url.substring(pos);
      }

      FTP ftp = new FTP(host, user, password, reporter);

      try {
	ftp.connect();

	AsFlatFileList asFile = new AsFlatFileList();  // extract all FileRef's
	asFile.visit(p);
	Enumeration enum = asFile.getFileProductList();

	while (enum.hasMoreElements()) {
	  FileRef f = ((FileRef)((Product) enum.nextElement()).getContent());
	  File source = f.getFile();
	  if (source == null) {
	      reporter.assert(false, "FileRef ", f.getSourceName() , " has null file content.",null);
	      continue;
	  }

	  String[] responses = ftp.sendCommand(FTP.CWD, path);
	  if (responses[0].charAt(0) != '2') {
	      throw new IOException(responses[0]);
	  }

	  // send source file
	  ftp.sendCommand(FTP.STOR, source);

	  // create a new local notify file
	  File notifyFile = new File(getWorkingDir(), (new Long(((new Date()).getTime())).toString()));

	  PrintWriter out = new PrintWriter(new FileWriter(notifyFile));
	  // RandomAccessFile out = new RandomAccessFile(notifyFile, "rw");
	  try {
	    //out.writeBytes();
	    out.print(path + "/" + source.getName());
	    out.close();
	    // ftp it to remote notify directory
	    responses = ftp.sendCommand(FTP.CWD, notifyDir);
	    if (responses[0].charAt(0) != '2') {
		throw new IOException(responses[0]);
	    }
	    ftp.sendCommand(FTP.STOR, notifyFile);
	  } finally {	
	      notifyFile.delete();	
	  }
	} //while
      // catch IOExceptions, UnknownHostExceptions, etc.
      } catch (Exception e) {
	  reportException(e);
      } finally {
	  if (ftp != null && ftp.isConnected()) {
	      try {
		  ftp.close();
	      } catch (IOException ignore) {}
	  }
      }
  }

 /*
 * for backward compatibility
 */
	public void setProperty(String attribute, String value) {
		if (attribute.equalsIgnoreCase("targetSpec")) {
			attribute = FTP_URL;
		}
		super.setProperty(attribute, value);
	}
}
