/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.putter;

import java.util.*;
import java.io.*;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ProductWriter;
import org.mitre.pam.interfaces.ContainerProduct;
import org.mitre.pam.product.view.BasicProduct;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.store.FileRef;
import org.mitre.pam.aggregator.ChanSchedBase;
import org.mitre.pam.productVisitor.AsPlainText;
import org.mitre.pam.productVisitor.AsPlainTextNoRootId;
import org.mitre.pam.productVisitor.AsTickerText;
import org.mitre.pam.productVisitor.AsTickerTextNoRootId;
import org.mitre.pam.productVisitor.AsFlatFileList;
import org.mitre.pam.util.PrintCRLFWriter;
import org.mitre.pam.util.nntp.NNTPClient;
import org.mitre.pam.util.nntp.NewsWriter;
import org.mitre.pdif.eventParser.ValueList;

/**
* This DIM posts a news article to a newsgroup. <p>
*
* @author Jones Leung
*/
public class PutNews extends DIMShell1 {  
    protected static final String DEBUG_KEY = "PutNews";
    protected static final String OUTPUT_STYLE = "outputStyle";
    protected static final String NEWS_SERVER = "newsServer";
    protected static final String NEWS_GROUPS = "newsGroups";
    
    protected String outputStyle;
    protected NNTPClient sender = null;
    private static final String[] OUTPUT_STYLES = {"plainText","plainTextNoRootId","tickerText","tickerTextNoRootId"};
/**
* Constructor for PutNews DIM.
*/
    public PutNews () {
        setDescription ("Posts a news article to a newsgroup");		
        setProperty (OUTPUT_STYLE, "");
        setProperty (NEWS_SERVER, "<your news server here>");
        setProperty (NEWS_GROUPS, "<news group here>");
        
        metaStringInfo(NEWS_SERVER, "<your news server here>", "host name of news server");
        metaStringInfo(NEWS_GROUPS, "<news group here>", "name of news group to post to");
        metaEnumerationInfo(OUTPUT_STYLE, "plainText", "the output style", OUTPUT_STYLES);
    }    
/**
* Post the news article to the newsgroup.
* @param    arg             The source for this DIM
*/
    public void putProduct (Product p) {                 
        try {	    
            setProgress("starting");


            setProgress("creating instance of NNTPClient");
            String host = getStringProperty (NEWS_SERVER,"");
            sender = new NNTPClient(host, getWorkingDir(), reporter);                

            sender.connect();
            
            // Attempt to find a "from" address
            String from = getStringProperty(ChanSchedBase.CREATOR_EMAIL,"");
            if( from.length() == 0 || from.equals("<user@hostname>") || from.equals("user@hostname")) {
                from = new String("<unspecified>");
            }

            reporter.debugMsg("News article from: ", from,null,null, DEBUG_KEY);

            setProgress("getting product");

            String subject = p.getId();

            setProgress("rendering product");

            ProductWriter pw;
            outputStyle = getStringProperty(OUTPUT_STYLE,"");
            if (outputStyle.equalsIgnoreCase("plainTextNoRootId")) {
                pw = new AsPlainTextNoRootId();
            } else if (outputStyle.equalsIgnoreCase("tickerTextNoRootId")) {
                pw = new AsTickerTextNoRootId();
            } else if (outputStyle.equalsIgnoreCase("tickerText")) {
                pw = new AsTickerText();
            } else { // use default visitor
                pw = new AsPlainText();
            } 

            StringWriter sw = new StringWriter();
            PrintWriter ps = new PrintCRLFWriter(sw);

            // If a root product exists then we should render it instead of the container
            if(p instanceof ContainerProduct) {
                ContainerProduct cp = (ContainerProduct)p;
                Product root = cp.getRoot();

                if(root != null) {
                    // We have a valid root product
                    pw.visit(root);
                } else {
                    // Root product does not exist, render container instead
                    pw.visit(p);
                }
            } else {
                // Product is not a container, it does not have a root
                pw.visit(p);
            }

            pw.writeResults(ps);
            String body = sw.toString();
//            debugMsg("Message body: " + body, DEBUG_KEY);
            
            NewsWriter message = new NewsWriter();
            message.setFrom(from);
            message.setBody(body);
            message.setSubject(subject);
            
            Enumeration enum = getNewsGroups();
            while (enum.hasMoreElements()) {
                message.addNewsgroup ((String)enum.nextElement());
            }           
            
            AsFlatFileList aflat = new AsFlatFileList();
            aflat.visit(p);
            Enumeration e = aflat.getFileProductList();

            while(e.hasMoreElements()) {
                FileRef fileref = ((FileRef)((Product)e.nextElement()).getContent());
                File attachment = fileref.getFile();

                message.addAttachment(fileref.getMIMEType(), attachment, fileref.getSourceName());
            }           
            
            setProgress ("posting news article");
            sender.post (message);            
            
            setProgress("closing NNTP connection");
            sender.quit();
            setProgress("done");        
        } catch (Exception e) {
            reportException (e);
        }
    }
/**
* Gets an Enumeration of newsgroups from the NEWS_GROUPS attribute (used internally).
* @return   an Enumeration of newsgroups
*/
    protected Enumeration getNewsGroups () {
        ValueList vl = getPropertyValues (NEWS_GROUPS);
        return vl.elements();
    }
}