/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.service.aim;

import java.util.Hashtable;
import org.mitre.pam.interfaces.Reporter;
/**
* Allows us to have multiple agents that use shared AIM connections.
* The exposed public methods are the same as those
* on the AIMDriver so that we can use the proxy in place of the driver.
*/
final public class AIMDriverProxy {

/*
* A indexed by AOL login name, has references to AIMDriver
*/
	private static Hashtable drivers = new Hashtable();
/**
* A reference to that actual driver that this is a proxy for
*/
	AIMDriver actualDriver;
/**
* A reference to AIMAgent using our proxy service
*/
	AIMAgent aimAgent;
/**
* The name used as the key in the driver table
*/
	String proxyKeyName;

	Reporter reporter;
	
	static String DEBUG = "aolim";
	
	public AIMDriverProxy(AIMAgent agent, Reporter reporter) {
		this.aimAgent = agent;
		this.reporter = reporter;
	}

/**
* Create instance of AIMDriver to establish connection to a server.
* If the driver with the specified login name exists then
* we just add one more listener to it.
*/
	public void startService(String AIMTOCServer, int AIMTOCServerPort, String loginName, String password, String authHost, int authPort) {
		if (actualDriver != null)
			stopService();
		synchronized (drivers) {
			proxyKeyName = loginName.toLowerCase();
			actualDriver = (AIMDriver) drivers.get(proxyKeyName);
			if (actualDriver == null) {
				actualDriver = new AIMDriver(aimAgent, reporter);
				drivers.put(proxyKeyName, actualDriver);
				reporter.debugMsg("new service for ",aimAgent,"login name:",loginName,DEBUG);
				actualDriver.startService(AIMTOCServer, AIMTOCServerPort, loginName, password, authHost, authPort);
			}
			else {
				actualDriver.addAIMListener(aimAgent);
				reporter.debugMsg("add listener ",aimAgent,"login name:",loginName,DEBUG);
			}
		}
	}

/**
* Disconnect from server
*/
	public void stopService() {
		if (actualDriver == null)
			return;
		synchronized (drivers) {
			actualDriver.removeAIMListener(aimAgent);
			reporter.debugMsg("remove listener ",aimAgent,null,null,DEBUG);
			if (actualDriver.getAIMListenerCount() <= 0) {
				actualDriver.stopService();
				drivers.remove(proxyKeyName);
				reporter.debugMsg("close service for ",aimAgent,null,null,DEBUG);
			}
		}
		actualDriver = null;
	}	
/**
* Send an Instant Message by specifying the reciever and the message.
*/
	public void sendIM(String userName, String msg) {
		actualDriver.sendIM(userName, msg);
	}
/**
* Are we connected?
*/
	public boolean isConnected() {
		return (actualDriver != null) && (actualDriver.isConnected());
	}
}