/* Copyright 2000 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.service.resource;

import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.Reporter;
/**
* This class works closely with the BytesPerMinuteRscPool class to manage
* the available bandwidth for the various video agents.
*
* @author	Henry C. Callihan, Jr.
*/
public class BandwidthOnlyConsumer extends RateConsumer {
/**
* BandwidthConsumer constructor comment.
*/
	public BandwidthOnlyConsumer(AccessibleAgent agent, String debugKey) {
		super(agent, debugKey);
	}
/**
* This method will always return no delay.
*
* @param time long
* @return 0 always
*
* @author	Henry C. Callihan, Jr.
*/
	protected synchronized long calculateDelay(long time) {
	if (budget <= 0.0)
		budget = 1.0;
	return 0;
}
/**
* This method is not used.
*
* @param time long
*
* @author	Henry C. Callihan, Jr.
*/
	protected void calculateMetrics(long currentTime) {
		/*
		* Do nothing because the BandwdithOnlyConsumer just serves
		* as a controller that accepts the bandwidth parameter and
		* passes it on to the actual consumer.
		* We have no information as to what the consumer is actually doing.
		*/
	}
/**
* This method will use the desired bandwidth to update the
* historical and overage properties.
*
* @param bandwidth int
*
* @author	Henry C. Callihan, Jr.
*/
	public void setDesiredBandwidth(int bandwidth) {
		setConsumption(bandwidth);
	}
/**
* This method will set whether to use the pool weight to divide up the pool's
* budget or not.
*
* @param value boolean
*
* @author	Henry C. Callihan, Jr.
*/
	public void setUsePoolWeight(boolean value) {
		/*
		* For now will always use the pool weight regardless of how it is set
		*/
		//usePoolWeight = value;
		//agent.superSetProperty(USE_POOL_WEIGHT, value ? "true" : "false");
		usePoolWeight = true;
		agent.superSetProperty(USE_POOL_WEIGHT, "true");
	}
/**
* This method will always return no delay.
*
* @return long
* @param requestor org.mitre.pam.interfaces.AccessibleAgent
* @param time long
* @param byteCount double
*
* @author	Henry C. Callihan, Jr.
*/
	protected long updateAndCalculateDelay(AccessibleAgent requestor, long time, double byteCount) {
		updateHistory(requestor, time, byteCount);
		return 0;
	}
}