/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.service.resource;

/**
* A resource consumer is somewhat analogous to an AWT
* Component. <P>
*
* The algorithm and precise meaning of consumption is vague by design.
*
* @author Nazario Irizarry, Jr.
*/
public interface ResourceConsumer  {

/**
* The consumer is queried for how much it has actually consumed.  This is
* analogous to AWT current size info.  <p>
* If the resource represents a level
* (like number-of-bytes moved), then the answer
* would be the accumulated level. <p>
* If the resource represents a rate (like number-of-bytes-per-minute), then the
* answer is some historical average over some
* time window. <p>
* 
* @return Number whos actual type will depend on the resource.  
* Java can deal abstractly with numbers.
*
*/
	public Number getConsumption(String resourceDimension);
/**
* The id of the consumer<p>
* 
* @return A string that is a unique identifier.
*/
	public String getId();
/**
* Set the constraint on this consumer.  There
* must be a default set of constraints available for each manager,
* so that the pool does not have to explicitly have its constraints
* set by an external controller. 
* Simple pools will probably ignore this method.
*
* @param constraintName is a tag that has meaning to the manager
* @param constraintValue is a value on the tag that has meaning to the manager and
* is used by the manager in its algorithm for managing budgets and actual consumption
*/
	public void setConsumerConstraint(String constraintName, Object constaintValue);
/**
* Instructs the consumer
* to reset its tally of consumption. 
* Rate related resources (such as bytes-per-minute) should
* reset their history to this value. Level related resources
* should set their tally to this number.
* 
*
*/
	public void setConsumption(String resourceDimension, Number value);
}