/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.service.resource;

/**
* A resource controller implements a particular resource
* pool's policy to control pool usage.<P>
*
* @author Nazario Irizarry, Jr.
*/
public interface ResourceController  {
/**
* Add a consumer to the list of controlled agents
*/
	public void addConsumer(ResourceConsumer consumer);
/**
* Allows the "children" or "member" consumers
* to request an estimate of how long until the pool is within budget.
*
* @return 0 if within budget, or delay time (milliseconds) as estimated wait
*/
	public long estimateDelayToSend(AccessibleAgent requestor, long time);
/**
* The id of the consumer<p>
* 
* @return A string that is a unique identifier.
*/
	public String getId();
/**
* Tell the controller that a consumer constraint/allocation
* has changed and request the pool to redelegate budgets.
*/
	public void needDelegationRecompute();
/**
* Remove a consumer from the list of controlled agents
*/
	public void removeConsumer(ResourceConsumer consumer);
/**
* Allows the "children" or "member" consumers
* to request permission
*
* @return 0 if ok to send, or delay time (milliseconds) if need to wait
*/
	public  long requestToSend(AccessibleAgent requestor, long time, double sizeRequested);
/**
* Retract the request to send
*/
	public  void retractRequestToSend(AccessibleAgent requestor);

}