/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */

package org.mitre.pam.wizard;
/*
	A basic extension of the java.awt.Window class
 */

import java.awt.*;
import java.util.Stack;
import org.mitre.pam.interfaces.ExecToController;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.store.IntegrityException;

public class StartWizard extends WizardBase1
{
	public StartWizard()
	{
		super("Start Wizard");
	}

	
	
	private DisplayText intro;
	private EnterTextLine id;
	private EnterTextLine thisHost;
	private EnterTextLine thisPort;
	private EnterYesNo pamNeedsFirewall;
	private EnterTextLine fireWallHost;
	private EnterTextLine fireWallPort;
	private EnterTextLine mailHost;
	private EnterYesNo createTest;
	private DisplayText infoTest;
	private DisplayText finale;
	private DisplayText finale2;
		
	protected WizardFrame[] buildScriptBody(ExecToController exec) {

		intro = new DisplayText("iPAM can't find a startup file.  This wizard will create one.");
		id = new EnterTextLine("Enter an informational id or name for this iPAM.", "unnamed");
		thisHost = new EnterTextLine("Enter this machines host name.", "a.b.c");
		thisPort = new EnterTextLine("Enter this iPAM's TCP/IP port number.", "19090");
		pamNeedsFirewall = new EnterYesNo("Does iPAM need to reach through an Http proxy?", false, 5, 7);
		fireWallHost = new EnterTextLine("Enter the host name of the firewall.", "x.y.z");
		fireWallPort = new EnterTextLine("Enter the TCP/IP port number for the firewall.", "80");
		mailHost = new EnterTextLine("Enter the host name for a default SMTP outgoing mail server, such as: localSMTPserver.mitre.org\n"
								+ "(You may leave this blank)", "");
		createTest = new EnterYesNo("Would you like the wizard to create a simple test configuration using a local source, and local output?",true, 9, 10);
		infoTest = new DisplayText("The test configuration will create a fixed input every minute and display it to a local window.");
		finale = new DisplayText("This completes the initial setup.\n"
								 + "If you would like to change these settings in the future use a text editor to edit 0checkpts/ckpt.txt\n"
								 );
		finale2 = new DisplayText("This configuration will be saved within the next few seconds.\n"
									+ "You can verify that it has been saved to a checkpoint file by looking at the log window.\n"	
								);

		WizardFrame[] script = new WizardFrame[12];
		script[0] = intro;
		script[1] = id;
		script[2] = thisHost;
		script[3] = thisPort;
		script[4] = pamNeedsFirewall;
		script[5] = fireWallHost;
		script[6] = fireWallPort;
		script[7] = mailHost;
		script[8] = createTest;
		script[9] = infoTest;
		script[10] = finale;
		script[11] = finale2;
		
		return script;
	}
		
	protected void implementWizardResults(ExecToController exec) {
		/*
		* test case
		*/
		if (exec == null) {
			System.out.println("pam need firewall: "+pamNeedsFirewall.getValue());
			System.out.println(" firewall host: "+fireWallHost.getValue());
			System.out.println(" firewall port: "+fireWallPort.getValue());
			System.out.println("create test case: "+createTest.getValue());
			System.out.println("mail host: "+mailHost.getValue());
		}
		/*
		* real case
		*/
		else {

			Agent theExec = exec.getExecAsAgent();
			theExec.setProperty(ExecGlobal.EXEC_ID, (String)id.getValue());
			theExec.setProperty(ExecGlobal.PROTOCOL_AND_HOST, "http://"+(String)thisHost.getValue());
			theExec.setProperty(ExecGlobal.SERVER_PORT_NUMBER, (String)thisPort.getValue());
			theExec.setProperty(ExecGlobal.PEER_PROTOCOL_HOST_PORT, "http://"+(String)thisHost.getValue()+":"+(String)thisPort.getValue());
			theExec.setProperty(ExecGlobal.DEBUG_KEY, "logQueries");
			
			String mailHostName = (String) mailHost.getValue();
			if (mailHostName != null && mailHostName.length()>0) {
				theExec.setProperty(ExecGlobal.DEFAULT_SMTP_HOST_NAME, mailHostName);
			}
			
			
			Boolean needsFireWall = (Boolean) pamNeedsFirewall.getValue();
			if (needsFireWall.booleanValue()) {
				theExec.setProperty(ExecGlobal.PROXY_HOST, (String)fireWallHost.getValue());
				theExec.setProperty(ExecGlobal.PROXY_PORT, (String)fireWallPort.getValue());
			}

			Boolean wantTest = (Boolean) createTest.getValue();
			try {
				if (wantTest.booleanValue()) {
					String pim = "Test Source";
					String juggler = "test activator";
					String dim = "Test Output";
					exec.declareNode("getPIM1",pim,null);
					exec.declareNode("putDIM1",dim,null);
					exec.declareNode("passThru",juggler,null);
					exec.connectNodes(juggler,dim);
					exec.connectNodes(pim,juggler);
					Agent via = exec.getActiveAgent(juggler);
					via.setProperty("period","60");
				}			
				if (true) {
					exec.declareNode("checkpoint","ckpt",null);
					Agent via = exec.getActiveAgent("ckpt");
					via.setProperty("period","600");
				}
				if (true) {
					exec.declareNode("logFilter","filterTheLogFile",null);
					Agent filter = exec.getActiveAgent("filterTheLogFile");
					filter.setEnabled(true);
					filter.setProperty("schedule","daily");
					filter.setProperty("hourOfDay","1");
					filter.setProperty("saveExceptionsOnlyAfterNDays","2");
					filter.setProperty("deleteAllAfterNDays","7");
					filter.setProperty("mailLogTo","");
				}
				if (true) {
					exec.declareNode("exceptionChecker","checkTheLogFile",null);
					Agent filter = exec.getActiveAgent("checkTheLogFile");
					filter.setEnabled(false);
					filter.setProperty("schedule","hourly");
					filter.setProperty("minuteOfHour","1");
					filter.setProperty("mailExceptionsTo","");
				}
				if (true) {
					exec.checkPoint();
				}
			}
			catch (IntegrityException e) {
				e.printStackTrace(System.err);
			}	
		}
	}


	public static void main (String args[]) {
		StartWizard wiz = new StartWizard();
		wiz.buildScript(null);
		wiz.runScript(null);
	}
	//{{DECLARE_MENUS
	//}}
}
