/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
import java.util.Properties;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.FileReader;

import org.mitre.pam.interfaces.IPamPlugin;
import org.mitre.pam.interfaces.ExecToController;
import org.mitre.pam.interfaces.ExecToEditor;
import org.mitre.pam.interfaces.ExecCallbacks;
import org.mitre.pam.executive.Exec;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.util.StreamCopy;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.aggregator.ProcessViaXSL;
import org.mitre.pam.aggregator.ProcessViaECMA;

/**
* An iPAM plugin with XML handlers.  
* This code is under development
* and may not function!!!!!
*
* @author Nazario Irizarry, Jr.
*/
public class PAMXML implements IPamPlugin {
	private static boolean yesXSLP=true;
	private static boolean yesXML4j=true;
	private static boolean yesFESI=true;
	
/**
* Define the mapping between module aliases and java classes.  
*
*/
	public static void defineModules(ExecToController exec) {
		try {
			exec.defineModuleAlias("forwardAsXMLFile", "org.mitre.pam.aggregator.ForwardAsXMLFile");

			if (yesXML4j && yesXSLP) {
				exec.defineModuleAlias("processViaXSL", "org.mitre.pam.aggregator.ProcessViaXSL");
			}
			else {
				exec.getExecAsAgent().getReporter().warning("Can't load processViaXSL");
			}

			if (yesFESI) {
				exec.defineModuleAlias("processViaECMAScript", "org.mitre.pam.aggregator.ProcessViaECMA");
			}
			else {
				exec.getExecAsAgent().getReporter().warning("Can't load processViaECMAScript");
			}

		}
		catch (Exception e) {
			exec.getExecAsAgent().getReporter().reportException(e);
		}
	}

/**
* Look for the existence of required class libraries
*/
	static public boolean isOkToLoad(ExecToController exec, String[] cmdLineArgs) {
		try {
			try {
				Class exists = Class.forName("com.ibm.xml.parser.Parser");
			}
			catch (ClassNotFoundException ignore) {
				yesXML4j = false;
				exec.getExecAsAgent().getReporter().warning("Can't find xml parser; Is xml4j in the classpath?");
			}
	
			try {
				Class exists = Class.forName("FESI.Interpreter.Evaluator");
			}
			catch (ClassNotFoundException ignore) {
				yesFESI = false;
				exec.getExecAsAgent().getReporter().warning("Can't find ECMAScript interpreter; Is FESI in the classpath?");
			}

			try {
				Class exists = Class.forName("com.kav.xsl.XSLProcessor");
			}
			catch (ClassNotFoundException ignore) {
				yesXSLP = false;
				exec.getExecAsAgent().getReporter().warning("Can't find xsl:p interpreter; Is xslp in the classpath?");
			}
			
		}
		catch (Error e) {
			exec.getExecAsAgent().getReporter().reportException(e);
			return false;
		}

		return true;
	}

/**
* An optional plugin definition method to create any needed support directories.
* This method is called before defineModules() is called
*/
	public static void makeSupportDirIfNeeded(ExecToController exec, File zipFile){
		StreamCopy.makeASupportDir(exec, ProcessViaXSL.SUPPORT_SUB_DIR, "3", "version.txt", zipFile);
		StreamCopy.makeASupportDir(exec, ProcessViaECMA.SUPPORT_SUB_DIR, "2", "version.txt", zipFile);
	}

/**
* This is modeled after java.lang.Package call by same name
*/
	public static String getImplementationVersion() {
		return "1.0.3";
	}
}
