/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
import java.util.Properties;
import java.io.File;

import org.mitre.pam.interfaces.IPamPlugin;
import org.mitre.pam.interfaces.ExecToController;
import org.mitre.pam.interfaces.ExecToEditor;
import org.mitre.pam.interfaces.ExecCallbacks;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.executive.Exec;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.util.StreamCopy;

/**
* An iPAM plugin to define an output agent that can display
* "track" data on a map.  Tracks are usually real-time reports on
* things that have a position in, on, or above the earth.
* In iPAM lingo this output agent displays a simple icon
* for products that have lattitude, longitude and several other attributes.
* The positions for these objects is updated when a newer report with the
* same id is delivered. <p>
*
* This plugin is based on the Object/FX map product
* SpatialX.  @see http://www.objectfx.com/.
* You must obtain the jar file ofx.spatialx.jar separately from
* Object/FX in order to run this code.
*
* @author Nazario Irizarry, Jr.
*/
public class SpatialXMap implements IPamPlugin {

	static final String SUPPORT_DIR = "localofxdim";

/**
* Define the mapping between module aliases and java classes.  
* You must obtain the jar file ofx.spatialx.jar separately from
* Object/FX in order to run this code.
*
*/
	public static void defineModules(ExecToController exec) {		

		try {
			File ofxSupportDirFile = ExecGlobal.getSupportDir(SUPPORT_DIR);
			Properties sysProps = System.getProperties();
			String classPath = sysProps.getProperty("java.class.path","")+File.pathSeparator+ofxSupportDirFile.getAbsolutePath();
			sysProps.put("java.class.path",classPath);
			exec.defineModuleAlias("putOnLocalMap", "org.mitre.pam.putter.mapdim.MapDIM");
		}
		catch (Exception e) {
			exec.getExecAsAgent().getReporter().reportException(e);
		}
	}


/**
* There are no special preload conditions
*/
	static public boolean isOkToLoad(ExecToController exec, String[] cmdLineArgs) {
		try {
			Class exists = Class.forName("com.ofx.geometry.SxGeometryInterface");
			return true;
		}
		catch (ClassNotFoundException ignore) {
			exec.getExecAsAgent().getReporter().log("Map output dim not loaded, can't find ofx.spatialx.jar");
		}
		catch (Error e) {
			exec.getExecAsAgent().getReporter().reportException(e);
		}
		return false;
	}


/**
* Here we create the runtime directory with sample "maps" and "map data".
*/
	public static void makeSupportDirIfNeeded(ExecToController exec, File zipFile){
		StreamCopy.makeASupportDir(exec, SUPPORT_DIR,	"1",	"version.txt", zipFile);		
	}
/**
* This is modeled after java.lang.Package call by same name
*/
	public static String getImplementationVersion() {
		return "1.0.0";
	}
}

